@@ -1,3 +1,28 @@
+/*
+ * CodonPartitionedRobustCounting.java
+ *
+ * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.app.beagle.evomodel.substmodel;
 
 import dr.app.beagle.evomodel.sitemodel.SiteRateModel;
@@ -35,10 +60,14 @@ import java.util.List;
 
 public class CodonPartitionedRobustCounting extends AbstractModel implements TreeTraitProvider, Loggable {
 
+    private static final boolean DEBUG = false;
+
     public static final String UNCONDITIONED_PREFIX = "u_";
     public static final String SITE_SPECIFIC_PREFIX = "c_";
     public static final String TOTAL_PREFIX = "total_";
     public static final String BASE_TRAIT_PREFIX = "base_";
+    public static final String COMPLETE_HISTORY_PREFIX = "all_";
+    public static final String UNCONDITIONED_PER_BRANCH_PREFIX = "b_u_";
 
 //    public CodonPartitionedRobustCounting(String name, TreeModel tree,
 //                                          AncestralStateBeagleTreeLikelihood[] partition,
@@ -57,6 +86,8 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
                                           boolean useUniformization,
                                           boolean includeExternalBranches,
                                           boolean includeInternalBranches,
+                                          boolean doUnconditionalPerBranch,
+                                          boolean saveCompleteHistory,
                                           StratifiedTraitOutputFormat branchFormat,
                                           StratifiedTraitOutputFormat logFormat) {
         super(name);
@@ -85,6 +116,8 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
             }
         }
 
+        this.saveCompleteHistory = saveCompleteHistory;
+
         productChainModel =
                 new ProductChainSubstitutionModel("codonLabeling", substModelsList, siteRateModelsList);
         addModel(productChainModel);
@@ -92,6 +125,7 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
         this.useUniformization = useUniformization;
         if (useUniformization) {
             markovJumps = new UniformizedSubstitutionModel(productChainModel);
+            ((UniformizedSubstitutionModel) markovJumps).setSaveCompleteHistory(saveCompleteHistory);
         } else {
             markovJumps = new MarkovJumpsSubstitutionModel(productChainModel);
         }
@@ -107,10 +141,19 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
 
         this.includeExternalBranches = includeExternalBranches;
         this.includeInternalBranches = includeInternalBranches;
+        this.doUnconditionedPerBranch = doUnconditionalPerBranch;
 
         setupTraits();
     }
 
+    public double[] getUnconditionalCountsForBranch(NodeRef child) {
+        if (!unconditionsPerBranchKnown) {
+            computeAllUnconditionalCountsPerBranch();
+            unconditionsPerBranchKnown = true;
+        }
+        return unconditionedCountsPerBranch[child.getNumber()];
+    }
+
     public double[] getExpectedCountsForBranch(NodeRef child) { // TODO This function will implement TraitProvider
         if (!countsKnown) {
             computeAllExpectedCounts();
@@ -175,14 +218,39 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
                 );
             }
 
-            if (DEBUG) {
 
-                System.err.println("Computing robust counts for " +
-                        parentSeq0[i] + parentSeq1[i] + parentSeq2[i] + " -> " +
-                        childSeq0[i] + childSeq1[i] + childSeq2[i] + " : " +
-//                        vParentState + " -> " + vChildState + " = " + codonCount);
-                        parentState + " -> " + childState + " = " + codonCount);
-                System.exit(-1);
+            if (useUniformization && saveCompleteHistory) {
+                UniformizedSubstitutionModel usModel = (UniformizedSubstitutionModel) markovJumps;
+
+                StateHistory history = usModel.getStateHistory();
+
+                // Only report syn or nonsyn changes
+                double[] register = usModel.getRegistration();
+                history = history.filterChanges(register);
+
+                int historyCount = history.getNumberOfJumps();
+                if (historyCount > 0) {
+                    double parentTime = tree.getNodeHeight(tree.getParent(child));
+                    double childTime = tree.getNodeHeight(child);
+                    history.rescaleTimesOfEvents(parentTime, childTime);
+
+                    int n = history.getNumberOfJumps();
+                    // MAS may have broken the next line
+                    String hstring = "{" + (i + 1) + "," + history.toStringChanges(usModel.dataType) + "}";
+            if (DEBUG) {
+                        System.err.println("site " + (i + 1) + " : "
+                                + history.getNumberOfJumps()
+                                + " : "
+                                + history.toStringChanges(usModel.dataType)
+                                + " " + codonLabeling.getText());
+                    }
+
+                    if (completeHistoryPerNode == null) {
+                        completeHistoryPerNode = new String[tree.getNodeCount()][numCodons];
+            }
+//                    completeHistoryPerNode[child.getNumber()][i] = usModel.getCompleteHistory(parentTime, childTime); // TODO Should reformat
+                    completeHistoryPerNode[child.getNumber()][i] = hstring;
+                }
             }
 
             count[i] = codonCount;
@@ -212,6 +280,37 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
             }
         };
 
+        if (saveCompleteHistory) {
+            TreeTrait stringTrait = new TreeTrait.SA() {
+
+                public String getTraitName() {
+                    return COMPLETE_HISTORY_PREFIX + codonLabeling.getText();
+                }
+
+                public Intent getIntent() {
+                    return Intent.BRANCH;
+                }
+
+                public String[] getTrait(Tree tree, NodeRef node) {
+                    getExpectedCountsForBranch(node); // Lazy simulation of complete histories
+                    List<String> events = new ArrayList<String>();
+                    for (int i = 0; i < numCodons; i++) {
+                        if (completeHistoryPerNode[node.getNumber()][i] != null) {
+                            events.add(completeHistoryPerNode[node.getNumber()][i]);
+                        }
+                    }
+                    String[] array = new String[events.size()];
+                    events.toArray(array);
+                    return array;
+                }
+
+                public boolean getLoggable() {
+                    return true;
+                }
+            };
+            treeTraits.addTrait(stringTrait);
+        }
+
         TreeTrait unconditionedSum;
         if (!TRIAL) {
             unconditionedSum = new TreeTrait.D() {
@@ -296,6 +395,38 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
         treeTraits.addTrait(sumOverTreeTrait);
         treeTraits.addTrait(sumOverSitesAndTreeTrait);
 
+        if (doUnconditionedPerBranch) {
+            TreeTrait unconditionedBase = new TreeTrait.DA() {
+
+                public String getTraitName() {
+                    return UNCONDITIONED_PER_BRANCH_PREFIX + codonLabeling.getText();
+                }
+
+                public Intent getIntent() {
+                    return Intent.BRANCH;
+                }
+
+                public double[] getTrait(Tree tree, NodeRef node) {
+                    return getUnconditionalCountsForBranch(node);
+                }
+
+                public boolean getLoggable() {
+                    return false;   // TODO Should be switched to true to log unconditioned values per branch
+                }
+            };
+
+            TreeTrait sumUnconditionedOverSitesTrait = new TreeTrait.SumAcrossArrayD(
+                    UNCONDITIONED_PER_BRANCH_PREFIX + codonLabeling.getText(),
+                    unconditionedBase) {
+                @Override
+                public boolean getLoggable() {
+                    return true;
+                }
+            };
+
+            treeTraits.addTrait(unconditionedBase);
+            treeTraits.addTrait(sumUnconditionedOverSitesTrait);
+        }
     }
 
     public TreeTrait[] getTreeTraits() {
@@ -331,26 +462,57 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
         return numCodons;
     }
 
+    private void computeAllUnconditionalCountsPerBranch() {
+        if (unconditionedCountsPerBranch == null) {
+            unconditionedCountsPerBranch = new double[tree.getNodeCount()][numCodons];
+        }
+        double[] rootDistribution = productChainModel.getFrequencyModel().getFrequencies();
+        for (int i = 0; i < tree.getNodeCount(); i++) {
+            NodeRef node = tree.getNode(i);
+            if (!tree.isRoot(node)) {
+                final double expectedLength = getExpectedBranchLength(node);
+                fillInUnconditionalTraitValues(expectedLength, rootDistribution, unconditionedCountsPerBranch[node.getNumber()]);
+            }
+        }
+    }
+
     private void computeUnconditionedTraitValues() {
         if (unconditionedCounts == null) {
             unconditionedCounts = new double[numCodons];
         }
         final double treeLength = getExpectedTreeLength();
         double[] rootDistribution = productChainModel.getFrequencyModel().getFrequencies();
+//        final int stateCount = 64;
+//        double[] lambda = new double[stateCount * stateCount];
+//        productChainModel.getInfinitesimalMatrix(lambda);
+//        for (int i = 0; i < numCodons; i++) {
+//            final int startingState = MathUtils.randomChoicePDF(rootDistribution);
+//            StateHistory history = StateHistory.simulateUnconditionalOnEndingState(
+//                    0.0,
+//                    startingState,
+//                    treeLength,
+//                    lambda,
+//                    stateCount
+//            );
+//            unconditionedCounts[i] = markovJumps.getProcessForSimulant(history);
+//        }
+        fillInUnconditionalTraitValues(treeLength, rootDistribution, unconditionedCounts);
+    }
+
+    private void fillInUnconditionalTraitValues(double expectedLength, double[] freq, double[] out) {
         final int stateCount = 64;
         double[] lambda = new double[stateCount * stateCount];
         productChainModel.getInfinitesimalMatrix(lambda);
         for (int i = 0; i < numCodons; i++) {
-            final int startingState = MathUtils.randomChoicePDF(rootDistribution);
+            final int startingState = MathUtils.randomChoicePDF(freq);
             StateHistory history = StateHistory.simulateUnconditionalOnEndingState(
                     0.0,
                     startingState,
-                    treeLength,
+                    expectedLength,
                     lambda,
                     stateCount
             );
-            unconditionedCounts[i] = markovJumps.getProcessForSimulant(history);
-
+            out[i] = markovJumps.getProcessForSimulant(history);
         }
     }
 
@@ -409,6 +571,7 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
     protected void handleModelChangedEvent(Model model, Object object, int index) {
         countsKnown = false;
         unconditionsKnown = false;
+        unconditionsPerBranchKnown = false;
     }
 
     protected void handleVariableChangedEvent(Variable variable, int index, Parameter.ChangeType type) {
@@ -423,6 +586,7 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
     protected void restoreState() {
         countsKnown = false;
         unconditionsKnown = false;
+        unconditionsPerBranchKnown = false;
     }
 
     protected void acceptState() {
@@ -445,18 +609,24 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
 
     private int numCodons;
 
-    private static final boolean DEBUG = false;
-
     private boolean countsKnown = false;
     private boolean unconditionsKnown = false;
+    private boolean unconditionsPerBranchKnown = false;
     private double[] unconditionedCounts;
+    private double[][] unconditionedCountsPerBranch;
     private double[][] computedCounts; // TODO Temporary storage until generic TreeTraitProvider/Helpers are finished
 
+    private String[][] completeHistoryPerNode;
+
     protected Helper treeTraits = new Helper();
     protected TreeTraitLogger treeTraitLogger;
 
     private final boolean includeExternalBranches;
     private final boolean includeInternalBranches;
+    private final boolean doUnconditionedPerBranch;
 
     private static final boolean TRIAL = true;
+
+    private boolean saveCompleteHistory = false;
+
 }
