@@ -1,7 +1,7 @@
 /*
- * TreeLikelihood.java
+ * BeagleTreeLikelihood.java
  *
- * Copyright (C) 2002-2006 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -27,19 +27,24 @@ package dr.app.beagle.evomodel.treelikelihood;
 
 import beagle.*;
 import dr.app.beagle.evomodel.parsers.TreeLikelihoodParser;
-import dr.app.beagle.evomodel.sitemodel.BranchSiteModel;
+import dr.app.beagle.evomodel.sitemodel.BranchSubstitutionModel;
+import dr.app.beagle.evomodel.sitemodel.EpochBranchSubstitutionModel;
+import dr.app.beagle.evomodel.sitemodel.HomogenousBranchSubstitutionModel;
 import dr.app.beagle.evomodel.sitemodel.SiteRateModel;
 import dr.app.beagle.evomodel.substmodel.EigenDecomposition;
 import dr.evolution.alignment.AscertainedSitePatterns;
 import dr.evolution.alignment.PatternList;
 import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
+import dr.evolution.util.Taxon;
 import dr.evolution.util.TaxonList;
 import dr.evomodel.branchratemodel.BranchRateModel;
 import dr.evomodel.branchratemodel.DefaultBranchRateModel;
 import dr.evomodel.tree.TreeModel;
+import dr.evomodel.treelikelihood.TipStatesModel;
 import dr.inference.model.Model;
 import dr.inference.model.Parameter;
+import dr.util.Citable;
 
 import java.util.*;
 import java.util.logging.Logger;
@@ -53,6 +58,7 @@ import java.util.logging.Logger;
  * @version $Id$
  */
 
+@SuppressWarnings("serial")
 public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
 
     // This property is a comma-delimited list of resource numbers (0 == CPU) to
@@ -62,30 +68,39 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
     private static final String PREFERRED_FLAGS_PROPERTY = "beagle.preferred.flags";
     private static final String REQUIRED_FLAGS_PROPERTY = "beagle.required.flags";
     private static final String SCALING_PROPERTY = "beagle.scaling";
+    private static final String RESCALE_FREQUENCY_PROPERTY = "beagle.rescale";
+
+    // Which scheme to use if choice not specified (or 'default' is selected):
+    private static final PartialsRescalingScheme DEFAULT_RESCALING_SCHEME = PartialsRescalingScheme.DELAYED;
 
     private static int instanceCount = 0;
     private static List<Integer> resourceOrder = null;
+    private static List<Integer> preferredOrder = null;
+    private static List<Integer> requiredOrder = null;
+    private static List<String> scalingOrder = null;
 
     private static final int RESCALE_FREQUENCY = 10000;
     private static final int RESCALE_TIMES = 1;
 
     public BeagleTreeLikelihood(PatternList patternList,
                                 TreeModel treeModel,
-                                BranchSiteModel branchSiteModel,
+                                BranchSubstitutionModel branchSubstitutionModel,
                                 SiteRateModel siteRateModel,
                                 BranchRateModel branchRateModel,
+                                TipStatesModel tipStatesModel,
                                 boolean useAmbiguities,
                                 PartialsRescalingScheme rescalingScheme) {
 
-        this(patternList, treeModel, branchSiteModel, siteRateModel, branchRateModel, useAmbiguities, rescalingScheme,
+        this(patternList, treeModel, branchSubstitutionModel, siteRateModel, branchRateModel, tipStatesModel, useAmbiguities, rescalingScheme,
                 null);
     }
 
     public BeagleTreeLikelihood(PatternList patternList,
                                 TreeModel treeModel,
-                                BranchSiteModel branchSiteModel,
+                                BranchSubstitutionModel branchSubstitutionModel,
                                 SiteRateModel siteRateModel,
                                 BranchRateModel branchRateModel,
+                                TipStatesModel tipStatesModel,
                                 boolean useAmbiguities,
                                 PartialsRescalingScheme rescalingScheme,
                                 Map<Set<String>, Parameter> partialsRestrictions) {
@@ -100,8 +115,15 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             this.siteRateModel = siteRateModel;
             addModel(this.siteRateModel);
 
-            this.branchSiteModel = branchSiteModel;
-            addModel(branchSiteModel);
+            this.branchSubstitutionModel = branchSubstitutionModel;
+            if (!(branchSubstitutionModel instanceof HomogenousBranchSubstitutionModel)) {
+                logger.info("  Branch site model used: " + branchSubstitutionModel.getModelName());
+                if (branchSubstitutionModel instanceof Citable) {
+                    logger.info("      Please cite: " + Citable.Utils.getCitationString((Citable) branchSubstitutionModel, "", ""));
+                }
+            }
+            eigenCount = this.branchSubstitutionModel.getEigenCount();
+            addModel(branchSubstitutionModel);
 
             if (branchRateModel != null) {
                 this.branchRateModel = branchRateModel;
@@ -111,6 +133,8 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             }
             addModel(this.branchRateModel);
 
+            this.tipStatesModel = tipStatesModel;
+
             this.categoryCount = this.siteRateModel.getCategoryCount();
 
             this.tipCount = treeModel.getExternalNodeCount();
@@ -126,8 +150,8 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             // one partials buffer for each tip and two for each internal node (for store restore)
             partialBufferHelper = new BufferIndexHelper(nodeCount, tipCount);
 
-            // two eigen buffers: for store and restore.
-            eigenBufferHelper = new BufferIndexHelper(1, 0);
+            // two eigen buffers for each decomposition for store and restore.
+            eigenBufferHelper = new BufferIndexHelper(eigenCount, 0);
 
             // two matrices for each node less the root
             matrixBufferHelper = new BufferIndexHelper(nodeCount, 0);
@@ -135,35 +159,33 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             // one scaling buffer for each internal node plus an extra for the accumulation, then doubled for store/restore
             scaleBufferHelper = new BufferIndexHelper(getScaleBufferCount(), 0);
 
+            this.branchSubstitutionModel.setFirstBuffer(matrixBufferHelper.getBufferCount());
+
             // Attempt to get the resource order from the System Property
             if (resourceOrder == null) {
-                resourceOrder = new ArrayList<Integer>();
-                String r = System.getProperty(RESOURCE_ORDER_PROPERTY);
-                if (r != null) {
-                    String[] parts = r.split(",");
-                    for (String part : parts) {
-                        try {
-                            int n = Integer.parseInt(part.trim());
-                            resourceOrder.add(n);
-                        } catch (NumberFormatException nfe) {
-                            System.err.println("Invalid entry '" + part + "' in " + RESOURCE_ORDER_PROPERTY);
-                        }
-                    }
-                }
+                resourceOrder = parseSystemPropertyIntegerArray(RESOURCE_ORDER_PROPERTY);
+            }
+            if (preferredOrder == null) {
+                preferredOrder = parseSystemPropertyIntegerArray(PREFERRED_FLAGS_PROPERTY);
+            }
+            if (requiredOrder == null) {
+                requiredOrder = parseSystemPropertyIntegerArray(REQUIRED_FLAGS_PROPERTY);
+            }
+            if (scalingOrder == null) {
+                scalingOrder = parseSystemPropertyStringArray(SCALING_PROPERTY);
             }
 
             // first set the rescaling scheme to use from the parser
             this.rescalingScheme = rescalingScheme;
-
-            // then allow it to be overriden from the command line
-            if (System.getProperty(SCALING_PROPERTY) != null) {
-                this.rescalingScheme = PartialsRescalingScheme.parseFromString(System.getProperty(SCALING_PROPERTY));
-            }
-
             int[] resourceList = null;
             long preferenceFlags = 0;
             long requirementFlags = 0;
 
+            if (scalingOrder.size() > 0) {
+                this.rescalingScheme = PartialsRescalingScheme.parseFromString(
+                        scalingOrder.get(instanceCount % scalingOrder.size()));
+            }
+
             if (resourceOrder.size() > 0) {
                 // added the zero on the end so that a CPU is selected if requested resource fails
                 resourceList = new int[]{resourceOrder.get(instanceCount % resourceOrder.size()), 0};
@@ -172,23 +194,23 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
                 }
             }
 
-            if (System.getProperty(PREFERRED_FLAGS_PROPERTY) != null) {
-                preferenceFlags = Long.valueOf(System.getProperty(PREFERRED_FLAGS_PROPERTY));
+            if (preferredOrder.size() > 0) {
+                preferenceFlags = preferredOrder.get(instanceCount % preferredOrder.size());
             }
 
-            if (System.getProperty(REQUIRED_FLAGS_PROPERTY) != null) {
-                requirementFlags = Long.valueOf(System.getProperty(REQUIRED_FLAGS_PROPERTY));
+            if (requiredOrder.size() > 0) {
+                requirementFlags = requiredOrder.get(instanceCount % requiredOrder.size());
             }
 
             // Define default behaviour here
             if (this.rescalingScheme == PartialsRescalingScheme.DEFAULT) {
                 //if GPU: the default is dynamic scaling in BEAST
                 if (resourceList != null && resourceList[0] > 1) {
-                    this.rescalingScheme = PartialsRescalingScheme.DYNAMIC;
+                    this.rescalingScheme = DEFAULT_RESCALING_SCHEME;
                 } else { // if CPU: just run as fast as possible
 //                    this.rescalingScheme = PartialsRescalingScheme.NONE;
                     // Dynamic should run as fast as none until first underflow
-                    this.rescalingScheme = PartialsRescalingScheme.DYNAMIC;
+                    this.rescalingScheme = DEFAULT_RESCALING_SCHEME;
                 }
             }
 
@@ -198,13 +220,31 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             } else {
 //                preferenceFlags |= BeagleFlag.SCALING_MANUAL.getMask();
             }
+            String r = System.getProperty(RESCALE_FREQUENCY_PROPERTY);
+            if (r != null) {
+                rescalingFrequency = Integer.parseInt(r);
+                if (rescalingFrequency < 1) {
+                    rescalingFrequency = RESCALE_FREQUENCY;
+                }
+            }
 
             if (preferenceFlags == 0 && resourceList == null) { // else determine dataset characteristics
                 if (stateCount == 4 && patternList.getPatternCount() < 10000) // TODO determine good cut-off
                     preferenceFlags |= BeagleFlag.PROCESSOR_CPU.getMask();
             }
 
-            if (branchSiteModel.canReturnComplexDiagonalization()) {
+            if (BeagleFlag.VECTOR_SSE.isSet(preferenceFlags) && stateCount != 4) {
+                // @todo SSE doesn't seem to work for larger state spaces so for now we override the
+                // SSE option.
+                preferenceFlags &= ~BeagleFlag.VECTOR_SSE.getMask();
+                preferenceFlags |= BeagleFlag.VECTOR_NONE.getMask();
+
+                if (stateCount > 4 && this.rescalingScheme == PartialsRescalingScheme.DYNAMIC) {
+                    this.rescalingScheme = PartialsRescalingScheme.DELAYED;
+                }
+            }
+
+            if (branchSubstitutionModel.canReturnComplexDiagonalization()) {
                 requirementFlags |= BeagleFlag.EIGEN_COMPLEX.getMask();
             }
 
@@ -216,8 +256,8 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
                     compactPartialsCount,
                     stateCount,
                     patternCount,
-                    eigenBufferHelper.getBufferCount(),            // eigenBufferCount
-                    matrixBufferHelper.getBufferCount(),
+                    eigenBufferHelper.getBufferCount(), // eigenBufferCount
+                    matrixBufferHelper.getBufferCount() + this.branchSubstitutionModel.getExtraBufferCount(treeModel),
                     categoryCount,
                     scaleBufferHelper.getBufferCount(), // Always allocate; they may become necessary
                     resourceList,
@@ -253,6 +293,18 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             logger.info("  " + (useAmbiguities ? "Using" : "Ignoring") + " ambiguities in tree likelihood.");
             logger.info("  With " + patternList.getPatternCount() + " unique site patterns.");
 
+            if (tipStatesModel != null) {
+                tipStatesModel.setTree(treeModel);
+
+                if (tipStatesModel.getModelType() == TipStatesModel.Type.PARTIALS) {
+                    tipPartials = new double[patternCount * stateCount];
+                } else {
+                    tipStates = new int[patternCount];
+                }
+
+                addModel(tipStatesModel);
+            }
+
             for (int i = 0; i < tipCount; i++) {
                 // Find the id of tip i in the patternList
                 String id = treeModel.getTaxonId(i);
@@ -262,10 +314,26 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
                     throw new TaxonList.MissingTaxonException("Taxon, " + id + ", in tree, " + treeModel.getId() +
                             ", is not found in patternList, " + patternList.getId());
                 } else {
-                    if (useAmbiguities) {
-                        setPartials(beagle, patternList, index, i);
+                    if (tipStatesModel != null) {
+                        // using a tipPartials model.
+                        // First set the observed states:
+                        tipStatesModel.setStates(patternList, index, i, id);
+
+                        if (tipStatesModel.getModelType() == TipStatesModel.Type.PARTIALS) {
+                            // Then set the tip partials as determined by the model:
+                            setPartials(beagle, tipStatesModel, i);
+                        } else {
+                            // or the tip states:
+                            tipStatesModel.getTipStates(i, tipStates);
+                            beagle.setTipStates(i, tipStates);
+                        }
+
                     } else {
-                        setStates(beagle, patternList, index, i);
+                        if (useAmbiguities) {
+                            setPartials(beagle, patternList, index, i);
+                        } else {
+                            setStates(beagle, patternList, index, i);
+                        }
                     }
                 }
             }
@@ -288,18 +356,21 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
 
             beagle.setPatternWeights(patternWeights);
 
+            String rescaleMessage = "  Using rescaling scheme : " + this.rescalingScheme.getText();
             if (this.rescalingScheme == PartialsRescalingScheme.AUTO &&
                     resourceDetails != null &&
                     (resourceDetails.getFlags() & BeagleFlag.SCALING_AUTO.getMask()) == 0) {
                 // If auto scaling in BEAGLE is not supported then do it here
                 this.rescalingScheme = PartialsRescalingScheme.DYNAMIC;
-                logger.info("  Auto rescaling not supported in BEAGLE, using : " + this.rescalingScheme.getText());
-            } else {
-                logger.info("  Using rescaling scheme : " + this.rescalingScheme.getText());
+                rescaleMessage = "  Auto rescaling not supported in BEAGLE, using : " + this.rescalingScheme.getText();
             }
+            if (this.rescalingScheme == PartialsRescalingScheme.DYNAMIC) {
+                rescaleMessage += " (rescaling every " + rescalingFrequency + " evaluations)";
+            }
+            logger.info(rescaleMessage);
 
             if (this.rescalingScheme == PartialsRescalingScheme.DYNAMIC) {
-                everUnderflowed = true; // If commented out, BEAST does not rescale until first under-/over-flow.
+                everUnderflowed = false; // If false, BEAST does not rescale until first under-/over-flow.
             }
 
             updateSubstitutionModel = true;
@@ -311,6 +382,42 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
         hasInitialized = true;
     }
 
+    private static List<Integer> parseSystemPropertyIntegerArray(String propertyName) {
+        List<Integer> order = new ArrayList<Integer>();
+        String r = System.getProperty(propertyName);
+        if (r != null) {
+            String[] parts = r.split(",");
+            for (String part : parts) {
+                try {
+                    int n = Integer.parseInt(part.trim());
+                    order.add(n);
+                } catch (NumberFormatException nfe) {
+                    System.err.println("Invalid entry '" + part + "' in " + propertyName);
+                }
+            }
+        }
+        return order;
+    }
+
+    private static List<String> parseSystemPropertyStringArray(String propertyName) {
+
+        List<String> order = new ArrayList<String>();
+
+        String r = System.getProperty(propertyName);
+        if (r != null) {
+            String[] parts = r.split(",");
+            for (String part : parts) {
+                try {
+                    String s = part.trim();
+                    order.add(s);
+                } catch (NumberFormatException nfe) {
+                    System.err.println("Invalid entry '" + part + "' in " + propertyName);
+                }
+            }
+        }
+        return order;
+    }
+
     public TreeModel getTreeModel() {
         return treeModel;
     }
@@ -370,6 +477,27 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
         beagle.setPartials(nodeIndex, partials);
     }
 
+    /**
+     * Sets the partials from a sequence in an alignment.
+     */
+    protected final void setPartials(Beagle beagle,
+                                     TipStatesModel tipStatesModel,
+                                     int nodeIndex) {
+        double[] partials = new double[patternCount * stateCount * categoryCount];
+
+        tipStatesModel.getTipPartials(nodeIndex, partials);
+
+        // if there is more than one category then replicate the partials for each
+        int n = patternCount * stateCount;
+        int k = n;
+        for (int i = 1; i < categoryCount; i++) {
+            System.arraycopy(partials, 0, partials, k, n);
+            k += n;
+        }
+
+        beagle.setPartials(nodeIndex, partials);
+    }
+
     public int getPatternCount() {
         return patternCount;
     }
@@ -419,6 +547,8 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
      */
     protected void handleModelChangedEvent(Model model, Object object, int index) {
 
+        fireModelChanged();
+
         if (model == treeModel) {
             if (object instanceof TreeModel.TreeChangedEvent) {
 
@@ -455,7 +585,7 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
                 updateNode(treeModel.getNode(index));
             }
 
-        } else if (model == branchSiteModel) {
+        } else if (model == branchSubstitutionModel) {
 
             updateSubstitutionModel = true;
             updateAllNodes();
@@ -465,6 +595,13 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             updateSiteModel = true;
             updateAllNodes();
 
+        } else if (model == tipStatesModel) {
+            if (object instanceof Taxon) {
+                for (int i = 0; i < treeModel.getNodeCount(); i++)
+                    if (treeModel.getNodeTaxon(treeModel.getNode(i)) != null && treeModel.getNodeTaxon(treeModel.getNode(i)).getId().equalsIgnoreCase(((Taxon) object).getId()))
+                        updateNode(treeModel.getNode(i));
+            } else
+                updateAllNodes();
         } else {
 
             throw new RuntimeException("Unknown componentChangedEvent");
@@ -536,8 +673,9 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
         }
 
         if (matrixUpdateIndices == null) {
-            matrixUpdateIndices = new int[nodeCount];
-            branchLengths = new double[nodeCount];
+            matrixUpdateIndices = new int[eigenCount][nodeCount];
+            branchLengths = new double[eigenCount][nodeCount];
+            branchUpdateCount = new int[eigenCount];
             scaleBufferIndices = new int[internalNodeCount];
             storedScaleBufferIndices = new int[internalNodeCount];
         }
@@ -562,13 +700,34 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
 
             rescalingCountInner++;
             rescalingCount++;
-            if (rescalingCount > RESCALE_FREQUENCY) {
+            if (rescalingCount > rescalingFrequency) {
                 rescalingCount = 0;
                 rescalingCountInner = 0;
             }
+        } else if (this.rescalingScheme == PartialsRescalingScheme.DELAYED && everUnderflowed) {
+            useScaleFactors = true;
+            recomputeScaleFactors = true;
+            rescalingCount++;
         }
 
-        branchUpdateCount = 0;
+        if (tipStatesModel != null) {
+            int tipCount = treeModel.getExternalNodeCount();
+            for (int index = 0; index < tipCount; index++) {
+                if (updateNode[index]) {
+                    if (tipStatesModel.getModelType() == TipStatesModel.Type.PARTIALS) {
+                        tipStatesModel.getTipPartials(index, tipPartials);
+                        beagle.setTipPartials(index, tipPartials);
+                    } else {
+                        tipStatesModel.getTipStates(index, tipStates);
+                        beagle.setTipStates(index, tipStates);
+                    }
+                }
+            }
+        }
+
+        for (int i = 0; i < eigenCount; i++) {
+            branchUpdateCount[i] = 0;
+        }
         operationListCount = 0;
 
         if (hasRestrictedPartials) {
@@ -582,17 +741,30 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
         final NodeRef root = treeModel.getRoot();
         traverse(treeModel, root, null, true);
 
-        if (updateSubstitutionModel) {
-            // we are currently assuming a homogeneous model...
-            EigenDecomposition ed = branchSiteModel.getEigenDecomposition(0, 0);
+        if (updateSubstitutionModel) { // TODO More efficient to update only the substitution model that changed, instead of all
+            // we are currently assuming a no-category model...
+            for (int i = 0; i < eigenCount; i++) {
+                if (EpochBranchSubstitutionModel.TRY_EPOCH) {
+            		  eigenBufferHelper.flipOffset(i);
+            		  
+            		  branchSubstitutionModel.setEigenDecomposition(
+            				  beagle, i, eigenBufferHelper, 0
+            				  );
+            		
+            		
+            	} else {
+            	
+                EigenDecomposition ed = branchSubstitutionModel.getEigenDecomposition(i, 0);
 
-            eigenBufferHelper.flipOffset(0);
+                eigenBufferHelper.flipOffset(i);
 
-            beagle.setEigenDecomposition(
-                    eigenBufferHelper.getOffsetIndex(0),
-                    ed.getEigenVectors(),
-                    ed.getInverseEigenVectors(),
-                    ed.getEigenValues());
+                beagle.setEigenDecomposition(
+                        eigenBufferHelper.getOffsetIndex(i),
+                        ed.getEigenVectors(),
+                        ed.getInverseEigenVectors(),
+                        ed.getEigenValues());
+            	}
+            }
         }
 
         if (updateSiteModel) {
@@ -600,19 +772,36 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             beagle.setCategoryRates(categoryRates);
         }
 
-        beagle.updateTransitionMatrices(
-                eigenBufferHelper.getOffsetIndex(0),
-                matrixUpdateIndices,
-                null,
-                null,
-                branchLengths,
-                branchUpdateCount);
+        for (int i = 0; i < eigenCount; i++) {
+            if (branchUpdateCount[i] > 0) {
 
-        if (COUNT_TOTAL_OPERATIONS) {
-            totalMatrixUpdateCount += branchUpdateCount;
+                if (EpochBranchSubstitutionModel.TRY_EPOCH) {
+                    branchSubstitutionModel.updateTransitionMatrices(
+                            beagle,
+                            i,
+                            eigenBufferHelper,
+                            matrixUpdateIndices[i],
+                            null,
+                            null,
+                            branchLengths[i],
+                            branchUpdateCount[i]);
+                } else {
+                    beagle.updateTransitionMatrices(
+                            eigenBufferHelper.getOffsetIndex(i),
+                            matrixUpdateIndices[i],
+                            null,
+                            null,
+                            branchLengths[i],
+                            branchUpdateCount[i]);
+                }
+            }
         }
 
         if (COUNT_TOTAL_OPERATIONS) {
+            for (int i = 0; i < eigenCount; i++) {
+                totalMatrixUpdateCount += branchUpdateCount[i];
+            }
+
             for (int i = 0; i <= numRestrictedPartials; i++) {
                 totalOperationCount += operationCount[i];
             }
@@ -638,7 +827,7 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             int rootIndex = partialBufferHelper.getOffsetIndex(root.getNumber());
 
             double[] categoryWeights = this.siteRateModel.getCategoryProportions();
-            double[] frequencies = branchSiteModel.getStateFrequencies(0);
+            double[] frequencies = branchSubstitutionModel.getStateFrequencies(0);
 
             int cumulateScaleBufferIndex = Beagle.NONE;
             if (useScaleFactors) {
@@ -677,12 +866,17 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
                 everUnderflowed = true;
                 logL = Double.NEGATIVE_INFINITY;
 
-                if (firstRescaleAttempt && rescalingScheme == PartialsRescalingScheme.DYNAMIC) {
+                if (firstRescaleAttempt && (rescalingScheme == PartialsRescalingScheme.DYNAMIC || rescalingScheme == PartialsRescalingScheme.DELAYED)) {
                     // we have had a potential under/over flow so attempt a rescaling
+                    if (rescalingScheme == PartialsRescalingScheme.DYNAMIC || (rescalingCount == 0)) {
+                        Logger.getLogger("dr.evomodel").info("Underflow calculating likelihood. Attempting a rescaling...");
+                    }
                     useScaleFactors = true;
                     recomputeScaleFactors = true;
 
-                    branchUpdateCount = 0;
+                    for (int i = 0; i < eigenCount; i++) {
+                        branchUpdateCount[i] = 0;
+                    }
 
                     if (hasRestrictedPartials) {
                         for (int i = 0; i <= numRestrictedPartials; i++) {
@@ -823,10 +1017,13 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             }
 
             // then set which matrix to update
-            matrixUpdateIndices[branchUpdateCount] = matrixBufferHelper.getOffsetIndex(nodeNum);
+            final int bufferIndex = matrixBufferHelper.getOffsetIndex(nodeNum);
+            final int eigenIndex = branchSubstitutionModel.getBranchIndex(tree, node, bufferIndex);
+            final int updateCount = branchUpdateCount[eigenIndex];
+            matrixUpdateIndices[eigenIndex][updateCount] = bufferIndex;
 
-            branchLengths[branchUpdateCount] = branchTime;
-            branchUpdateCount++;
+            branchLengths[eigenIndex][updateCount] = branchTime;
+            branchUpdateCount[eigenIndex]++;
 
             update = true;
         }
@@ -905,9 +1102,29 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
 
                     }
                 }
+                
+                // /////////////
+                // ---DEBUG---//
+                // /////////////
+
+//				double tmp[] = new double[stateCount * patternCount * categoryCount];
+//				System.out.println(nodeNum);
+//				getPartials(nodeNum, tmp);
+//
+//				for (int i = 0; i < 4; i++) {
+////					if (tmp[i] != 0) {
+//						System.out.println(Math.log(tmp[i]));
+////					}
+//				}
+
+				// //////////////////
+				// ---END: DEBUG---//
+				// //////////////////
+
             }
         }
 
+//        EpochBranchSubstitutionModel.printArray(branchUpdateCount, branchUpdateCount.length);
         return update;
 
     }
@@ -916,16 +1133,17 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
     // INSTANCE VARIABLES
     // **************************************************************
 
-    private int[] matrixUpdateIndices;
-    private double[] branchLengths;
-    private int branchUpdateCount;
+    private int eigenCount;
+    private int[][] matrixUpdateIndices;
+    private double[][] branchLengths;
+    private int[] branchUpdateCount;
     private int[] scaleBufferIndices;
     private int[] storedScaleBufferIndices;
 
     private int[][] operations;
     private int operationListCount;
     private int[] operationCount;
-//    private final boolean hasRestrictedPartials;
+    //    private final boolean hasRestrictedPartials;
     private static final boolean hasRestrictedPartials = false;
 
     private final int numRestrictedPartials;
@@ -944,6 +1162,8 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
     protected final int internalNodeCount;
 
     private PartialsRescalingScheme rescalingScheme;
+    private int rescalingFrequency = RESCALE_FREQUENCY;
+
     protected boolean useScaleFactors = false;
     private boolean useAutoScaling = false;
     private boolean recomputeScaleFactors = false;
@@ -955,17 +1175,23 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
     /**
      * the branch-site model for these sites
      */
-    protected final BranchSiteModel branchSiteModel;
+    protected final BranchSubstitutionModel branchSubstitutionModel;
 
     /**
      * the site model for these sites
      */
     protected final SiteRateModel siteRateModel;
+
     /**
      * the branch rate model
      */
     protected final BranchRateModel branchRateModel;
 
+    /**
+     * the tip partials model
+     */
+    private final TipStatesModel tipStatesModel;
+
     /**
      * the pattern likelihoods
      */
@@ -981,6 +1207,11 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
      */
     protected double[] tipPartials;
 
+    /**
+     * an array used to transfer tip states
+     */
+    protected int[] tipStates;
+
     /**
      * the BEAGLE library instance
      */
@@ -1010,60 +1241,60 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
 
     private boolean ascertainedSitePatterns = false;
 
-    protected class BufferIndexHelper {
-        /**
-         * @param maxIndexValue the number of possible input values for the index
-         * @param minIndexValue the minimum index value to have the mirrored buffers
-         */
-        BufferIndexHelper(int maxIndexValue, int minIndexValue) {
-            this.maxIndexValue = maxIndexValue;
-            this.minIndexValue = minIndexValue;
-
-            offsetCount = maxIndexValue - minIndexValue;
-            indexOffsets = new int[offsetCount];
-            storedIndexOffsets = new int[offsetCount];
-        }
-
-        public int getBufferCount() {
-            return 2 * offsetCount + minIndexValue;
-        }
-
-        void flipOffset(int i) {
-            if (i >= minIndexValue) {
-                indexOffsets[i - minIndexValue] = offsetCount - indexOffsets[i - minIndexValue];
-            } // else do nothing
-        }
-
-        int getOffsetIndex(int i) {
-            if (i < minIndexValue) {
-                return i;
-            }
-            return indexOffsets[i - minIndexValue] + i;
-        }
-
-        void getIndices(int[] outIndices) {
-            for (int i = 0; i < maxIndexValue; i++) {
-                outIndices[i] = getOffsetIndex(i);
-            }
-        }
-
-        void storeState() {
-            System.arraycopy(indexOffsets, 0, storedIndexOffsets, 0, indexOffsets.length);
-
-        }
-
-        void restoreState() {
-            int[] tmp = storedIndexOffsets;
-            storedIndexOffsets = indexOffsets;
-            indexOffsets = tmp;
-        }
-
-        private final int maxIndexValue;
-        private final int minIndexValue;
-        private final int offsetCount;
-
-        private int[] indexOffsets;
-        private int[] storedIndexOffsets;
-
-    }
+//    protected class BufferIndexHelper {
+//        /**
+//         * @param maxIndexValue the number of possible input values for the index
+//         * @param minIndexValue the minimum index value to have the mirrored buffers
+//         */
+//        BufferIndexHelper(int maxIndexValue, int minIndexValue) {
+//            this.maxIndexValue = maxIndexValue;
+//            this.minIndexValue = minIndexValue;
+//
+//            offsetCount = maxIndexValue - minIndexValue;
+//            indexOffsets = new int[offsetCount];
+//            storedIndexOffsets = new int[offsetCount];
+//        }
+//
+//        public int getBufferCount() {
+//            return 2 * offsetCount + minIndexValue;
+//        }
+//
+//        void flipOffset(int i) {
+//            if (i >= minIndexValue) {
+//                indexOffsets[i - minIndexValue] = offsetCount - indexOffsets[i - minIndexValue];
+//            } // else do nothing
+//        }
+//
+//        int getOffsetIndex(int i) {
+//            if (i < minIndexValue) {
+//                return i;
+//            }
+//            return indexOffsets[i - minIndexValue] + i;
+//        }
+//
+//        void getIndices(int[] outIndices) {
+//            for (int i = 0; i < maxIndexValue; i++) {
+//                outIndices[i] = getOffsetIndex(i);
+//            }
+//        }
+//
+//        void storeState() {
+//            System.arraycopy(indexOffsets, 0, storedIndexOffsets, 0, indexOffsets.length);
+//
+//        }
+//
+//        void restoreState() {
+//            int[] tmp = storedIndexOffsets;
+//            storedIndexOffsets = indexOffsets;
+//            indexOffsets = tmp;
+//        }
+//
+//        private final int maxIndexValue;
+//        private final int minIndexValue;
+//        private final int offsetCount;
+//
+//        private int[] indexOffsets;
+//        private int[] storedIndexOffsets;
+//
+//    }
 }
\ No newline at end of file
