@@ -1,7 +1,7 @@
 /*
  * BeastMain.java
  *
- * Copyright (C) 2002-2011 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -77,7 +77,7 @@ public class BeastMain {
         if (inputFile == null) {
             System.err.println();
             System.err.println("Error: no input file specified");
-            return;
+            System.exit(1);
         }
 
         String fileName = inputFile.getName();
@@ -146,6 +146,7 @@ public class BeastMain {
 
         } catch (java.io.IOException ioe) {
             infoLogger.severe("File error: " + ioe.getMessage());
+            System.exit(1);
         } catch (org.xml.sax.SAXParseException spe) {
             if (spe.getMessage() != null && spe.getMessage().equals("Content is not allowed in prolog")) {
                 infoLogger.severe("Parsing error - the input file, " + fileName + ", is not a valid XML file.");
@@ -154,10 +155,12 @@ public class BeastMain {
                 infoLogger.severe("Parsing error - poorly formed XML (possibly not an XML file):\n" +
                         spe.getMessage());
             }
+            System.exit(1);
         } catch (org.w3c.dom.DOMException dome) {
             infoLogger.severe("Error running file: " + fileName);
             infoLogger.severe("Parsing error - poorly formed XML:\n" +
                     dome.getMessage());
+            System.exit(1);
         } catch (dr.xml.XMLParseException pxe) {
             if (pxe.getMessage() != null && pxe.getMessage().equals("Unknown root document element, beauti")) {
                 infoLogger.severe("Error running file: " + fileName);
@@ -173,7 +176,7 @@ public class BeastMain {
                 infoLogger.severe("Parsing error - poorly formed BEAST file, " + fileName + ":\n" +
                         pxe.getMessage());
             }
-
+            System.exit(1);
         } catch (RuntimeException rex) {
             if (rex.getMessage() != null && rex.getMessage().startsWith("The initial posterior is zero")) {
                 infoLogger.warning("Error running file: " + fileName);
@@ -197,12 +200,13 @@ public class BeastMain {
                 System.err.println("Fatal exception: " + rex.getMessage());
                 rex.printStackTrace(System.err);
             }
-
+            System.exit(1);
         } catch (Exception ex) {
             infoLogger.warning("Error running file: " + fileName);
             infoLogger.severe("Fatal exception: " + ex.getMessage());
             System.err.println("Fatal exception: " + ex.getMessage());
             ex.printStackTrace(System.err);
+            System.exit(1);
         }
     }
 
@@ -236,6 +240,10 @@ public class BeastMain {
         System.out.println();
     }
 
+    private static long updateSeedByRank(long seed, int rank) {
+        return seed + 1000 * 1000 * rank;
+    }
+
     //Main method
     public static void main(String[] args) throws java.io.IOException {
 
@@ -270,13 +278,17 @@ public class BeastMain {
                         new Arguments.Option("beagle_SSE", "BEAGLE: use SSE extensions if available"),
                         new Arguments.Option("beagle_single", "BEAGLE: use single precision if available"),
                         new Arguments.Option("beagle_double", "BEAGLE: use double precision if available"),
-                        new Arguments.StringOption("beagle_scaling", new String[]{"default", "none", "dynamic", "always"},
+                        new Arguments.StringOption("beagle_scaling", new String[]{"default", "dynamic", "delayed", "always", "none"},
                                 false, "BEAGLE: specify scaling scheme to use"),
+                        new Arguments.IntegerOption("beagle_rescale", "BEAGLE: frequency of rescaling (dynamic scaling only)"),
+                        new Arguments.Option("mpi", "Use MPI rank to label output"),
                         new Arguments.Option("help", "Print this information and stop"),
                 });
 
+        int argumentCount = 0;
+
         try {
-            arguments.parseArguments(args);
+            argumentCount = arguments.parseArguments(args);
         } catch (Arguments.ArgumentException ae) {
             System.out.println();
             System.out.println(ae.getMessage());
@@ -285,8 +297,15 @@ public class BeastMain {
             System.exit(1);
         }
 
+        if (arguments.hasOption("version")) {
+            printTitle();
+        }
+
         if (arguments.hasOption("help")) {
             printUsage(arguments);
+        }
+
+        if (arguments.hasOption("version") || arguments.hasOption("help")) {
             System.exit(0);
         }
 
@@ -296,15 +315,16 @@ public class BeastMain {
         final boolean parserWarning = arguments.hasOption("warnings"); // if dev, then auto turn on, otherwise default to turn off
         final boolean strictXML = arguments.hasOption("strict");
         final boolean window = arguments.hasOption("window");
-        final boolean options = arguments.hasOption("options");
+        final boolean options = arguments.hasOption("options") || (argumentCount == 0);
         final boolean working = arguments.hasOption("working");
         String fileNamePrefix = null;
         boolean allowOverwrite = arguments.hasOption("overwrite");
+        boolean useMPI = arguments.hasOption("mpi");
 
         long seed = MathUtils.getSeed();
         boolean useJava = false;
 
-        int threadCount = 0;
+        int threadCount = -1;
 
         if (arguments.hasOption("java")) {
             useJava = true;
@@ -325,12 +345,9 @@ public class BeastMain {
                 arguments.hasOption("beagle_single") ||
                 arguments.hasOption("beagle_order") ||
                 arguments.hasOption("beagle_scaling") ||
+                arguments.hasOption("beagle_rescale") ||
                 arguments.hasOption("beagle_instances");
 
-        if (arguments.hasOption("beagle_scaling")) {
-            System.setProperty("beagle.scaling", arguments.getStringOption("beagle_scaling"));
-        }
-
         boolean beagleShowInfo = arguments.hasOption("beagle_info");
 
         if (arguments.hasOption("beagle_CPU")) {
@@ -362,7 +379,12 @@ public class BeastMain {
             System.setProperty("beagle.scaling", arguments.getStringOption("beagle_scaling"));
         }
 
+        if (arguments.hasOption("beagle_rescale")) {
+            System.setProperty("beagle.rescale", Integer.toString(arguments.getIntegerOption("beagle_rescale")));
+        }
+
         if (arguments.hasOption("threads")) {
+            // threadCount defaults to -1 unless the user specifies an option
             threadCount = arguments.getIntegerOption("threads");
             if (threadCount < 0) {
                 printTitle();
@@ -380,6 +402,24 @@ public class BeastMain {
             }
         }
 
+        if (useMPI) {
+            String[] nullArgs = new String[0];
+            try {
+                BeastMPI.Init(nullArgs);
+            } catch (Exception e) {
+                throw new RuntimeException("Unable to access MPI.");
+            }
+            int rank = BeastMPI.COMM_WORLD.Rank();
+            System.setProperty("mpi.rank.postfix", String.valueOf(rank));
+
+        }
+
+        String rankProp = System.getProperty("mpi.rank.postfix");
+        if (rankProp != null) {
+            int rank = Integer.valueOf(rankProp);
+            seed = updateSeedByRank(seed, rank);
+        }
+
         int maxErrorCount = 0;
         if (arguments.hasOption("errors")) {
             maxErrorCount = arguments.getIntegerOption("errors");
@@ -420,7 +460,22 @@ public class BeastMain {
 
             BeastDialog dialog = new BeastDialog(new JFrame(), titleString, icon);
 
-            if (!dialog.showDialog(nameString, seed)) {
+            dialog.setAllowOverwrite(allowOverwrite);
+            dialog.setSeed(seed);
+
+            dialog.setUseBeagle(useBeagle);
+
+            if (BeagleFlag.PROCESSOR_GPU.isSet(beagleFlags)) {
+                dialog.setPreferBeagleGPU();
+            }
+
+            dialog.setPreferBeagleSSE(BeagleFlag.VECTOR_SSE.isSet(beagleFlags));
+
+            if (BeagleFlag.PRECISION_SINGLE.isSet(beagleFlags)) {
+                dialog.setPreferBeagleSingle();
+            }
+
+            if (!dialog.showDialog(nameString)) {
                 return;
             }
 
@@ -449,6 +504,7 @@ public class BeastMain {
                 if (dialog.preferBeagleSingle()) {
                     beagleFlags |= BeagleFlag.PRECISION_SINGLE.getMask();
                 }
+                System.setProperty("beagle.scaling", dialog.scalingScheme());
             }
 
             inputFile = dialog.getInputFile();
@@ -536,17 +592,16 @@ public class BeastMain {
             new BeastMain(inputFile, consoleApp, maxErrorCount, verbose, parserWarning, strictXML, additionalParsers);
         } catch (RuntimeException rte) {
             if (window) {
-                // This sleep for 2 seconds is to ensure that the final message
-                // appears at the end of the console.
-                try {
-                    Thread.sleep(2000);
-                } catch (InterruptedException e) {
-                    e.printStackTrace();
-                }
                 System.out.println();
                 System.out.println("BEAST has terminated with an error. Please select QUIT from the menu.");
+                // logger.severe will throw a RTE but we want to keep the console visible
+            } else {
+                System.exit(1);
             }
-            // logger.severe will throw a RTE but we want to keep the console visible
+        }
+
+        if (useMPI) {
+            BeastMPI.Finalize();
         }
 
         if (!window) {
