@@ -8,10 +8,16 @@
  */
 package dr.app.beauti;
 
-import dr.app.beauti.clockModelsPanel.ClockModelsPanel;
+import dr.app.beauti.ancestralStatesPanel.AncestralStatesPanel;
+import dr.app.beauti.clockModelsPanel.OldClockModelsPanel;
 import dr.app.beauti.components.ComponentFactory;
-import dr.app.beauti.components.SequenceErrorModelComponentFactory;
-import dr.app.beauti.components.TipDateSamplingComponentFactory;
+import dr.app.beauti.components.ancestralstates.AncestralStatesComponentFactory;
+import dr.app.beauti.components.continuous.ContinuousComponentFactory;
+import dr.app.beauti.components.discrete.DiscreteTraitsComponentFactory;
+import dr.app.beauti.components.dollo.DolloComponentFactory;
+import dr.app.beauti.components.hpm.HierarchicalModelComponentFactory;
+import dr.app.beauti.components.sequenceerror.SequenceErrorModelComponentFactory;
+import dr.app.beauti.components.tipdatesampling.TipDateSamplingComponentFactory;
 import dr.app.beauti.datapanel.DataPanel;
 import dr.app.beauti.generator.BeastGenerator;
 import dr.app.beauti.generator.Generator;
@@ -24,15 +30,14 @@ import dr.app.beauti.options.TraitData;
 import dr.app.beauti.priorsPanel.DefaultPriorDialog;
 import dr.app.beauti.priorsPanel.PriorsPanel;
 import dr.app.beauti.siteModelsPanel.SiteModelsPanel;
-import dr.app.beauti.taxonsetspanel.TaxaPanel;
+import dr.app.beauti.taxonsetspanel.SpeciesSetPanel;
+import dr.app.beauti.taxonsetspanel.TaxonSetPanel;
 import dr.app.beauti.tipdatepanel.TipDatesPanel;
 import dr.app.beauti.traitspanel.TraitsPanel;
-import dr.app.beauti.treespanel.OldTreesPanel;
 import dr.app.beauti.treespanel.TreesPanel;
 import dr.app.beauti.util.BEAUTiImporter;
 import dr.app.beauti.util.TextUtil;
 import dr.app.gui.FileDrop;
-import dr.app.java16compat.FileNameExtensionFilter;
 import dr.app.util.OSType;
 import dr.app.util.Utils;
 import dr.evolution.io.Importer.ImportException;
@@ -40,19 +45,18 @@ import dr.evolution.io.NexusImporter.MissingBlockException;
 import jam.framework.DocumentFrame;
 import jam.framework.Exportable;
 import jam.util.IconUtils;
+import org.jdom.JDOMException;
 
 import javax.swing.*;
 import javax.swing.border.Border;
 import javax.swing.event.ChangeEvent;
 import javax.swing.event.ChangeListener;
+import javax.swing.filechooser.FileNameExtensionFilter;
 import javax.swing.plaf.BorderUIResource;
 import java.awt.*;
-import java.awt.event.ActionEvent;
 import java.awt.event.HierarchyBoundsListener;
 import java.awt.event.HierarchyEvent;
-import java.io.File;
-import java.io.FileNotFoundException;
-import java.io.IOException;
+import java.io.*;
 
 /**
  * @author Andrew Rambaut
@@ -63,8 +67,21 @@ public class BeautiFrame extends DocumentFrame {
 
     private static final long serialVersionUID = 2114148696789612509L;
 
-    private final BeautiOptions options;
-    private final BeastGenerator generator;
+    public final static String DATA_PARTITIONS = "Partitions";
+    public final static String TAXON_SETS = "Taxa";
+    public final static String TIP_DATES = "Tips";
+    public final static String TRAITS = "Traits";
+    public final static String SITE_MODELS = "Sites";
+    public final static String CLOCK_MODELS = "Clocks";
+    public final static String TREES = "Trees";
+    public final static String ANCESTRAL_STATES = "States";
+    public final static String PRIORS = "Priors";
+    public final static String OPERATORS = "Operators";
+    public final static String MCMC = "MCMC";
+
+    private BeautiOptions options;
+    private BeastGenerator generator;
+    private final ComponentFactory[] components;
 
     public final JTabbedPane tabbedPane = new JTabbedPane();
     public final JLabel statusLabel = new JLabel();
@@ -72,11 +89,11 @@ public class BeautiFrame extends DocumentFrame {
     private DataPanel dataPanel;
     private TipDatesPanel tipDatesPanel;
     private TraitsPanel traitsPanel;
-    private TaxaPanel taxaPanel;
+    private TaxonSetPanel taxonSetPanel;
+    private SpeciesSetPanel speciesSetPanel;
     private SiteModelsPanel siteModelsPanel;
-    private ClockModelsPanel clockModelsPanel;
-    private OldTreesPanel oldTreesPanel;
-//    private SpeciesTreesPanel speciesTreesPanel;
+    private AncestralStatesPanel ancestralStatesPanel;
+    private OldClockModelsPanel clockModelsPanel;
     private TreesPanel treesPanel;
     private PriorsPanel priorsPanel;
     private OperatorsPanel operatorsPanel;
@@ -98,38 +115,47 @@ public class BeautiFrame extends DocumentFrame {
         // after a user cancel or a failure in beast file generation
         setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
 
-        getOpenAction().setEnabled(false);
-        getSaveAction().setEnabled(false);
+//        getOpenAction().setEnabled(false);
+//        getSaveAction().setEnabled(false);
 
         getFindAction().setEnabled(false);
 
         getZoomWindowAction().setEnabled(false);
 
-        ComponentFactory[] components = {
+        components = new ComponentFactory[] {
+                AncestralStatesComponentFactory.INSTANCE,
+                ContinuousComponentFactory.INSTANCE,
+                DiscreteTraitsComponentFactory.INSTANCE,
+//                DnDsComponentFactory.INSTANCE,
+                DolloComponentFactory.INSTANCE,
+                HierarchicalModelComponentFactory.INSTANCE,
                 SequenceErrorModelComponentFactory.INSTANCE,
-                TipDateSamplingComponentFactory.INSTANCE,
-//                DiscreteTraitsComponentFactory.INSTANCE
+                TipDateSamplingComponentFactory.INSTANCE
         };
 
         options = new BeautiOptions(components);
         generator = new BeastGenerator(options, components);
 
-        this.getContentPane().addHierarchyBoundsListener(new HierarchyBoundsListener(){
-			public void ancestorMoved(HierarchyEvent e) {}
-			public void ancestorResized(HierarchyEvent e) {
-				setStatusMessage();
-			}
-		});
+        this.getContentPane().addHierarchyBoundsListener(new HierarchyBoundsListener() {
+            public void ancestorMoved(HierarchyEvent e) {
+            }
+
+            public void ancestorResized(HierarchyEvent e) {
+                setStatusMessage();
+            }
+        });
     }
 
     public void initializeComponents() {
 
         dataPanel = new DataPanel(this, getImportAction(), getDeleteAction()/*, getImportTraitsAction()*/);
         tipDatesPanel = new TipDatesPanel(this);
-        traitsPanel = new TraitsPanel(this, getImportTraitsAction());
-        taxaPanel = new TaxaPanel(this);
+        traitsPanel = new TraitsPanel(this, dataPanel, getImportTraitsAction());
+        taxonSetPanel = new TaxonSetPanel(this);
+        speciesSetPanel = new SpeciesSetPanel(this);
         siteModelsPanel = new SiteModelsPanel(this, getDeleteAction());
-        clockModelsPanel = new ClockModelsPanel(this);
+        ancestralStatesPanel = new AncestralStatesPanel(this);
+        clockModelsPanel = new OldClockModelsPanel(this);
 //        oldTreesPanel = new OldTreesPanel(this);
         treesPanel = new TreesPanel(this, getDeleteAction());
 //        speciesTreesPanel = new SpeciesTreesPanel(this);
@@ -137,16 +163,66 @@ public class BeautiFrame extends DocumentFrame {
         operatorsPanel = new OperatorsPanel(this);
         mcmcPanel = new MCMCPanel(this);
 
-        tabbedPane.addTab("Data Partitions", dataPanel);
-        tabbedPane.addTab("Taxon Sets", taxaPanel);
-        tabbedPane.addTab("Tip Dates", tipDatesPanel);
-        tabbedPane.addTab("Traits", traitsPanel);
-        tabbedPane.addTab("Site Models", siteModelsPanel);
-        tabbedPane.addTab("Clock Models", clockModelsPanel);
-        tabbedPane.addTab("Trees", treesPanel);
-        tabbedPane.addTab("Priors", priorsPanel);
-        tabbedPane.addTab("Operators", operatorsPanel);
-        tabbedPane.addTab("MCMC", mcmcPanel);
+        int index = 0;
+        tabbedPane.addTab(DATA_PARTITIONS, dataPanel);
+        tabbedPane.setToolTipTextAt(index++, "<html>" +
+                "Import sequence alignments, organize data partitions,<br>" +
+                "link models between partitions and select *BEAST</html>");
+
+        tabbedPane.addTab(TAXON_SETS, taxonSetPanel);
+        tabbedPane.setToolTipTextAt(index++, "<html>" +
+                "Create and edit sets of taxa which can be used to <br>" +
+                "define times of most recent common ancestors and <br>" +
+                "to keep groups monophyletic.</html>");
+//        tabbedPane.addTab("Species Sets", speciesSetPanel);
+        tabbedPane.addTab(TIP_DATES, tipDatesPanel);
+        tabbedPane.setToolTipTextAt(index++, "<html>" +
+                "Specify sampling dates of tips for use in temporal <br>" +
+                "analyses of measurably evolving populations.</html>");
+        tabbedPane.addTab(TRAITS, traitsPanel);
+        tabbedPane.setToolTipTextAt(index++, "<html>" +
+                "Import and organize continuous and discrete traits <br>" +
+                "for taxa, convert them into data partitions for evolutionary<br>" +
+                "analysis.</html>");
+        tabbedPane.addTab(SITE_MODELS, siteModelsPanel);
+        tabbedPane.setToolTipTextAt(index++, "<html>" +
+                "Select evolutionary models to be used for each data <br>" +
+                "partition including substitution models, codon partitioning<br>" +
+                "and trait evolution models.</html>");
+        tabbedPane.addTab(CLOCK_MODELS, clockModelsPanel);
+        tabbedPane.setToolTipTextAt(index++, "<html>" +
+                "Select relaxed molecular clock models to be used across <br>" +
+                "the tree. Specify sampling of rates.</html>");
+        tabbedPane.addTab(TREES, treesPanel);
+        tabbedPane.setToolTipTextAt(index++, "<html>" +
+                "Select the priors on trees including coalescent models<br>" +
+                "birth-death speciation models and the *BEAST gene tree,<br>" +
+                "species tree options.</html>");
+
+        tabbedPane.addTab(ANCESTRAL_STATES, ancestralStatesPanel);
+        tabbedPane.setToolTipTextAt(index++, "<html>" +
+                "Select options for sampling ancestral states at specific<br>" +
+                "or all common ancestors, models of counting state changes<br>" +
+                "and models of sequencing error for data partitions.</html>");
+
+        tabbedPane.addTab(PRIORS, priorsPanel);
+        tabbedPane.setToolTipTextAt(index++, "<html>" +
+                "Specify prior probability distributions on each and every<br>" +
+                "parameter of the current model.</html>");
+        tabbedPane.addTab(OPERATORS, operatorsPanel);
+        tabbedPane.setToolTipTextAt(index++, "<html>" +
+                "Select and adjust the menu of operators that will be used<br>" +
+                "to propose changes to the parameters. Switch off operators<br>" +
+                "on certain parameters to fix them to initial values.</html>");
+        tabbedPane.addTab(MCMC, mcmcPanel);
+        tabbedPane.setToolTipTextAt(index++, "<html>" +
+                "Specify the details of MCMC sampling. This includes chain<br>" +
+                "length, sampling frequencies, log file names and more.</html>");
+
+        for (int i = 1; i < tabbedPane.getTabCount(); i++) {
+            tabbedPane.setEnabledAt(i, false);
+        }
+
         currentPanel = (BeautiPanel) tabbedPane.getSelectedComponent();
 
         tabbedPane.addChangeListener(new ChangeListener() {
@@ -179,21 +255,12 @@ public class BeautiFrame extends DocumentFrame {
         basePanel.add(tabbedPane, BorderLayout.CENTER);
         basePanel.add(panel2, BorderLayout.SOUTH);
 
-//        if (OSType.isMac()) {
-//            getContentPane().add(panel, BorderLayout.CENTER);
-////            setMinimumSize(new java.awt.Dimension(800, 600));
-//        } else {
-////            JScrollPane scrollPane = new JScrollPane(panel);
-////            getContentPane().add(scrollPane, BorderLayout.CENTER);
-//            getContentPane().add(panel, BorderLayout.CENTER);
-//        }
-
         add(basePanel, BorderLayout.CENTER);
 
         Toolkit tk = Toolkit.getDefaultToolkit();
         Dimension d = tk.getScreenSize();
-        System.out.println("Screen width = " + d.width);
-        System.out.println("Screen height = " + d.height);
+//        System.out.println("Screen width = " + d.width);
+//        System.out.println("Screen height = " + d.height);
 
         if (d.width < 1000 || d.height < 700) {
             setSize(new java.awt.Dimension(700, 550));
@@ -201,10 +268,6 @@ public class BeautiFrame extends DocumentFrame {
             setSize(new java.awt.Dimension(1024, 768));
         }
 
-        // todo Andrew:
-        // It is really nasty on the Mac not to have a minimum window size (probably
-        // other platforms too?). We surely don't require a working interface less
-        // than this...
         if (OSType.isMac()) {
             setMinimumSize(new java.awt.Dimension(640, 480));
         }
@@ -220,6 +283,8 @@ public class BeautiFrame extends DocumentFrame {
         importChooser = new JFileChooser(Utils.getCWD());
 
         importChooser.setMultiSelectionEnabled(true);
+        importChooser.setFileFilter(new FileNameExtensionFilter(
+                "Microsatellite (tab-delimited *.txt) Files", "txt"));
         importChooser.setFileFilter(new FileNameExtensionFilter(
                 "NEXUS (*.nex) & BEAST (*.xml) Files", "nex", "nexus", "nx", "xml", "beast", "fa", "fasta", "afa"));
         importChooser.setDialogTitle("Import Aligment...");
@@ -234,7 +299,6 @@ public class BeautiFrame extends DocumentFrame {
         }); // end FileDrop.Listener
 
 
-
     }
 
     /**
@@ -245,10 +309,15 @@ public class BeautiFrame extends DocumentFrame {
             dataPanel.setOptions(options);
             tipDatesPanel.setOptions(options);
             traitsPanel.setOptions(options);
-            taxaPanel.setOptions(options);
+            if (options.useStarBEAST) {
+                speciesSetPanel.setOptions(options);
+            } else {
+                taxonSetPanel.setOptions(options);
+            }
             siteModelsPanel.setOptions(options);
             clockModelsPanel.setOptions(options);
             treesPanel.setOptions(options);
+            ancestralStatesPanel.setOptions(options);
             priorsPanel.setOptions(options);
             operatorsPanel.setOptions(options);
             mcmcPanel.setOptions(options);
@@ -256,25 +325,42 @@ public class BeautiFrame extends DocumentFrame {
             setStatusMessage();
         } catch (IllegalArgumentException illegEx) {
             JOptionPane.showMessageDialog(this, illegEx.getMessage(),
-                    "Illegal Argument Exception",
-                    JOptionPane.ERROR_MESSAGE);
+                    "Illegal Argument Exception", JOptionPane.ERROR_MESSAGE);
         }
+
+        // enable/disable the other tabs and generate option depending on whether any
+        // data has been loaded.
+        boolean enabled = options.getDataPartitions().size() > 0;
+        for (int i = 1; i < tabbedPane.getTabCount(); i++) {
+            tabbedPane.setEnabledAt(i, enabled);
+        }
+        getExportAction().setEnabled(enabled);
     }
 
     /**
      * get all the options for all panels
      */
     private void getAllOptions() {
-        dataPanel.getOptions(options);
-        tipDatesPanel.getOptions(options);
-        traitsPanel.getOptions(options);
-        taxaPanel.getOptions(options);
-        siteModelsPanel.getOptions(options);
-        clockModelsPanel.getOptions(options);
-        treesPanel.getOptions(options);
-        priorsPanel.getOptions(options);
-        operatorsPanel.getOptions(options);
-        mcmcPanel.getOptions(options);
+        try {
+            dataPanel.getOptions(options);
+            tipDatesPanel.getOptions(options);
+            traitsPanel.getOptions(options);
+            if (options.useStarBEAST) {
+                speciesSetPanel.getOptions(options);
+            } else {
+                taxonSetPanel.getOptions(options);
+            }
+            siteModelsPanel.getOptions(options);
+            clockModelsPanel.getOptions(options);
+            treesPanel.getOptions(options);
+            ancestralStatesPanel.getOptions(options);
+            priorsPanel.getOptions(options);
+            operatorsPanel.getOptions(options);
+            mcmcPanel.getOptions(options);
+        } catch (IllegalArgumentException illegEx) {
+            JOptionPane.showMessageDialog(this, illegEx.getMessage(),
+                    "Illegal Argument Exception", JOptionPane.ERROR_MESSAGE);
+        }
     }
 
     public void doSelectAll() {
@@ -316,7 +402,7 @@ public class BeautiFrame extends DocumentFrame {
                     JOptionPane.WARNING_MESSAGE);
 
             if (option == JOptionPane.YES_OPTION) {
-                return !doGenerate();
+                return doGenerate();
             } else if (option == JOptionPane.CANCEL_OPTION || option == JOptionPane.DEFAULT_OPTION) {
                 return false;
             }
@@ -347,7 +433,32 @@ public class BeautiFrame extends DocumentFrame {
     }
 
     protected boolean readFromFile(File file) throws IOException {
-        return false;
+        FileInputStream fileIn =
+                new FileInputStream(file);
+        try {
+            ObjectInputStream in = new ObjectInputStream(fileIn);
+            options = (BeautiOptions) in.readObject();
+            in.close();
+        } catch (IOException ioe) {
+            JOptionPane.showMessageDialog(this,
+                    "Unable to read BEAUti file. BEAUti can only read files\n" +
+                    "created by 'Saving' within BEAUti. It cannot read BEAST\n" +
+                    "XML files. To read data within BEAST XML files, use\n" +
+                            "the 'Import' option.",
+                    "Unable to read file",
+                    JOptionPane.ERROR_MESSAGE);
+        } catch (ClassNotFoundException cnfe) {
+            JOptionPane.showMessageDialog(this, "Unable to read BEAUti file: " + cnfe.getMessage(),
+                    "Unable to read file",
+                    JOptionPane.ERROR_MESSAGE);
+            cnfe.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
+            return false;
+        }
+        fileIn.close();
+
+        generator = new BeastGenerator(options, components);
+
+        return true;
     }
 
     public String getDefaultFileName() {
@@ -355,7 +466,13 @@ public class BeautiFrame extends DocumentFrame {
     }
 
     protected boolean writeToFile(File file) throws IOException {
-        return false;
+        OutputStream fileOut = new FileOutputStream(file);
+        ObjectOutputStream out =
+                new ObjectOutputStream(fileOut);
+        out.writeObject(options);
+        out.close();
+        fileOut.close();
+        return true;
     }
 
     public final void doImport() {
@@ -363,6 +480,7 @@ public class BeautiFrame extends DocumentFrame {
         if (returnVal == JFileChooser.APPROVE_OPTION) {
             File[] files = importChooser.getSelectedFiles();
             importFiles(files);
+            tabbedPane.setSelectedComponent(dataPanel);
         }
     }
 
@@ -398,46 +516,33 @@ public class BeautiFrame extends DocumentFrame {
                             "Error reading file",
                             JOptionPane.ERROR_MESSAGE);
                     ime.printStackTrace();
-                    // there may be other files in the list so don't return
-//                    return;
-                } catch (Exception ex) {
-                    JOptionPane.showMessageDialog(this, "Fatal exception: " + ex,
+                } catch (JDOMException jde) {
+                    JOptionPane.showMessageDialog(this, "Error parsing imported file: " + jde,
                             "Error reading file",
                             JOptionPane.ERROR_MESSAGE);
-                    ex.printStackTrace();
-                    return;
+                    jde.printStackTrace();
+//                } catch (IllegalArgumentException illegEx) {
+//                    JOptionPane.showMessageDialog(this, illegEx.getMessage(),
+//                            "Illegal Argument Exception", JOptionPane.ERROR_MESSAGE);
+//
+//                } catch (Exception ex) {
+//                    JOptionPane.showMessageDialog(this, "Fatal exception: " + ex,
+//                            "Error reading file",
+//                            JOptionPane.ERROR_MESSAGE);
+//                    ex.printStackTrace();
+//                    return;
                 }
             }
         }
 
-        if (options.allowDifferentTaxa) {
-            setAllOptions();
+        if (!options.hasIdenticalTaxa()) {
+            setAllOptions(); // need this to refresh panels otherwise it will throw exception
             dataPanel.selectAll();
             dataPanel.unlinkTrees();
         }
 
         setAllOptions();
 
-//          // @Todo templates are not implemented yet...
-////        getOpenAction().setEnabled(true);
-////        getSaveAction().setEnabled(true);
-        getExportAction().setEnabled(true);
-    }
-
-//    public int allowDifferentTaxaJOptionPane() {
-//        // AR - Yes and No are perfectly good answers to this question
-//        return JOptionPane.showOptionDialog(this, "This file contains different taxa from the previously loaded\n"
-//                + "data partitions. This may be because the taxa are mislabelled\n" + "and need correcting before reloading.\n\n"
-//                + "Would you like to allow different taxa for each partition?\n", "Validation of Non-matching Taxon Name(s)",
-//                JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, new String[]{"Yes", "No"}, "No");
-//    }
-
-    public boolean createImportTraits() {
-        return traitsPanel.addTrait();
-    }
-
-    public boolean createImportTraits(String traitName) {
-        return traitsPanel.addTrait(traitName);
     }
 
     public final boolean doImportTraits() {
@@ -476,6 +581,8 @@ public class BeautiFrame extends DocumentFrame {
 
             traitsPanel.fireTraitsChanged();
             setAllOptions();
+
+            tabbedPane.setSelectedComponent(traitsPanel);
             return true;
 
         } else {
@@ -503,8 +610,18 @@ public class BeautiFrame extends DocumentFrame {
             return false;
         }
 
+        if (options.useStarBEAST && traitName.equalsIgnoreCase(TraitData.TRAIT_SPECIES)) {
+            JOptionPane.showMessageDialog(this,
+                    "This trait name is already in used to denote species\n" +
+                            "for *BEAST. Please select a different name.",
+                    "Reserved trait name",
+                    JOptionPane.WARNING_MESSAGE);
+
+            return false;
+        }
+
         // check that the trait name doesn't exist
-        if (options.containTrait(traitName)) {
+        if (options.traitExists(traitName)) {
             int option = JOptionPane.showConfirmDialog(this,
                     "A trait of this name already exists. Do you wish to replace\n" +
                             "it with this new trait? This may result in the loss or change\n" +
@@ -521,27 +638,51 @@ public class BeautiFrame extends DocumentFrame {
         return true;
     }
 
-    public void setupStarBEAST(boolean useStarBEAST) {
+    /**
+     * Attempts to set up starBEAST - returns true if successful
+     * @param useStarBEAST
+     * @return
+     */
+    public boolean setupStarBEAST(boolean useStarBEAST) {
         if (useStarBEAST) {
+            if (!options.traitExists(TraitData.TRAIT_SPECIES)) {
+                if (!traitsPanel.addTrait(
+                        "<html><p>" +
+                                "StarBEAST requires a trait to give species designations<br>" +
+                                "for each taxon. Create or import a discrete trait<br>" +
+                                "labelled 'species'.</p></html>",
+                        TraitData.TRAIT_SPECIES,
+                        true /* isSpeciesTrait */
+                )) {
+                    return false;
+                }
+            } else if (options.getTraitPartitions(options.getTrait(TraitData.TRAIT_SPECIES)).size() > 0) {
+                int option = JOptionPane.showConfirmDialog(this,
+                        "The trait named '" + TraitData.TRAIT_SPECIES + "', used to denote species in *BEAST, is\n" +
+                                "already in use as a data partition. Do you wish to continue?",
+                        "Species trait already in use",
+                        JOptionPane.YES_NO_OPTION,
+                        JOptionPane.WARNING_MESSAGE);
+                if (option == JOptionPane.NO_OPTION) {
+                    return false;
+                }
+
+            }
+
             dataPanel.selectAll();
-
             dataPanel.unlinkAll();
 
-//        if (options.getPartitionClockModels().size() > 1) {
-//        	dataPanel.linkClocks();
-//        }
-
-//            treesPanel.updatePriorPanelForSpeciesAnalysis();
-
             options.starBEASTOptions = new STARBEASTOptions(options);
             options.fileNameStem = "StarBEASTLog";
 
-            if (!createImportTraits(TraitData.TRAIT_SPECIES)) {
-                dataPanel.useStarBEASTCheck.setSelected(false); // go back to unchecked
-                useStarBEAST = false;
-            }
+            tabbedPane.removeTabAt(1);
+            tabbedPane.insertTab("Species Sets", null, speciesSetPanel, null, 1);
+
         } else { // remove species
-            options.removeTrait(TraitData.TRAIT_SPECIES);
+            options.fileNameStem = MCMCPanel.DEFAULT_FILE_NAME_STEM;
+
+            tabbedPane.removeTabAt(1);
+            tabbedPane.insertTab("Taxon Sets", null, taxonSetPanel, null, 1);
         }
 
         options.useStarBEAST = useStarBEAST;
@@ -549,6 +690,8 @@ public class BeautiFrame extends DocumentFrame {
         treesPanel.updatePriorPanelForSpeciesAnalysis();
 
         setStatusMessage();
+
+        return true;
     }
 
     public void updateDiscreteTraitAnalysis() {
@@ -564,14 +707,10 @@ public class BeautiFrame extends DocumentFrame {
     }
 
     public PartitionTreePrior getCurrentPartitionTreePrior() {
+        treesPanel.setOptions(options); // need this to refresh the currentTreeModel
         return treesPanel.currentTreeModel.getPartitionTreePrior();
     }
 
-    public void removeSpecifiedTreePrior(boolean isChecked) { // TipDatesPanel usingTipDates
-        //TODO: wait for new implementation in BEAST
-        treesPanel.setCheckedTipDate(isChecked);
-    }
-
     public void setStatusMessage() {
         int width = this.getWidth() - 260; // minus generate button size
         if (width < 100) width = 100; // prevent too narrow
@@ -584,9 +723,12 @@ public class BeautiFrame extends DocumentFrame {
 
         try {
             generator.checkOptions();
-        } catch (IllegalArgumentException iae) {
-            JOptionPane.showMessageDialog(this, iae.getMessage(), "Invalid BEAUti setting : ",
+        } catch (Generator.GeneratorException ge) {
+            JOptionPane.showMessageDialog(this, ge.getMessage(), "Invalid BEAUti setting : ",
                     JOptionPane.ERROR_MESSAGE);
+            if (ge.getSwitchToPanel() != null) {
+                switchToPanel(ge.getSwitchToPanel());
+            }
             return false;
         }
 
@@ -636,6 +778,15 @@ public class BeautiFrame extends DocumentFrame {
         return true;
     }
 
+    public void switchToPanel(String panelName) {
+        for (int i = 0; i < tabbedPane.getTabCount(); i++) {
+            if (tabbedPane.getTitleAt(i).equals(panelName)) {
+                tabbedPane.setSelectedIndex(i);
+                break;
+            }
+        }
+    }
+
     public JComponent getExportableComponent() {
 
         JComponent exportable = null;
@@ -650,64 +801,64 @@ public class BeautiFrame extends DocumentFrame {
         return exportable;
     }
 
-    public boolean doSave() {
-        return doSaveAs();
-    }
+//    public boolean doSave() {
+//        return doSaveAs();
+//    }
+//
+//    public boolean doSaveAs() {
+//        FileDialog dialog = new FileDialog(this,
+//                "Save Template As...",
+//                FileDialog.SAVE);
+//
+//        dialog.setVisible(true);
+//        if (dialog.getFile() == null) {
+//            // the dialog was cancelled...
+//            return false;
+//        }
+//
+//        File file = new File(dialog.getDirectory(), dialog.getFile());
+//
+//        try {
+//            if (writeToFile(file)) {
+//
+//                clearDirty();
+//            }
+//        } catch (IOException ioe) {
+//            JOptionPane.showMessageDialog(this, "Unable to save file: " + ioe,
+//                    "Unable to save file",
+//                    JOptionPane.ERROR_MESSAGE);
+//        }
+//
+//        return true;
+//    }
 
-    public boolean doSaveAs() {
-        FileDialog dialog = new FileDialog(this,
-                "Save Template As...",
-                FileDialog.SAVE);
+//    public Action getOpenAction() {
+//        return openTemplateAction;
+//    }
+//
+//    private final AbstractAction openTemplateAction = new AbstractAction("Apply Template...") {
+//        private static final long serialVersionUID = 2450459627280385426L;
+//
+//        public void actionPerformed(ActionEvent ae) {
+//            doApplyTemplate();
+//        }
+//    };
 
-        dialog.setVisible(true);
-        if (dialog.getFile() == null) {
-            // the dialog was cancelled...
-            return false;
-        }
-
-        File file = new File(dialog.getDirectory(), dialog.getFile());
-
-        try {
-            if (writeToFile(file)) {
-
-                clearDirty();
-            }
-        } catch (IOException ioe) {
-            JOptionPane.showMessageDialog(this, "Unable to save file: " + ioe,
-                    "Unable to save file",
-                    JOptionPane.ERROR_MESSAGE);
-        }
-
-        return true;
-    }
-
-    public Action getOpenAction() {
-        return openTemplateAction;
-    }
-
-    private final AbstractAction openTemplateAction = new AbstractAction("Apply Template...") {
-        private static final long serialVersionUID = 2450459627280385426L;
-
-        public void actionPerformed(ActionEvent ae) {
-            doApplyTemplate();
-        }
-    };
-
-    public Action getSaveAction() {
-        return saveAsAction;
-    }
-
-    public Action getSaveAsAction() {
-        return saveAsAction;
-    }
-
-    private final AbstractAction saveAsAction = new AbstractAction("Save Template As...") {
-        private static final long serialVersionUID = 2424923366448459342L;
-
-        public void actionPerformed(ActionEvent ae) {
-            doSaveAs();
-        }
-    };
+//    public Action getSaveAction() {
+//        return saveAsAction;
+//    }
+//
+//    public Action getSaveAsAction() {
+//        return saveAsAction;
+//    }
+//
+//    private final AbstractAction saveAsAction = new AbstractAction("Save Template As...") {
+//        private static final long serialVersionUID = 2424923366448459342L;
+//
+//        public void actionPerformed(ActionEvent ae) {
+//            doSaveAs();
+//        }
+//    };
 
     public Action getImportAction() {
         return importAlignmentAction;
