@@ -30,16 +30,15 @@ import dr.app.beauti.alignmentviewer.AlignmentViewer;
 import dr.app.beauti.alignmentviewer.AminoAcidDecorator;
 import dr.app.beauti.alignmentviewer.NucleotideDecorator;
 import dr.app.beauti.alignmentviewer.StateCellDecorator;
-import dr.app.beauti.enumTypes.FixRateType;
 import dr.app.beauti.options.*;
 import dr.app.beauti.util.PanelUtils;
 import dr.app.gui.table.TableEditorStopper;
 import dr.evolution.alignment.Alignment;
 import dr.evolution.datatype.DataType;
+import dr.evolution.datatype.Microsatellite;
 import dr.evolution.util.Taxa;
 import jam.framework.Exportable;
 import jam.panels.ActionPanel;
-import jam.table.HeaderRenderer;
 
 import javax.swing.*;
 import javax.swing.event.ListSelectionEvent;
@@ -76,8 +75,8 @@ public class DataPanel extends BeautiPanel implements Exportable {
     UnlinkTreesAction unlinkTreesAction = new UnlinkTreesAction();
     LinkTreesAction linkTreesAction = new LinkTreesAction();
 
-    CreateAction createAction = new CreateAction();
-    JButton createImportTraitButton = new JButton(createAction);
+    CreateTraitPartitionAction createTraitPartitionAction = new CreateTraitPartitionAction();
+    JButton createTraitPartitionButton;
 //    ShowAction showAction = new ShowAction();
 
     public JCheckBox useStarBEASTCheck = new JCheckBox("Use species tree ancestral reconstruction (*BEAST) Heled & Drummond 2010 ");
@@ -86,6 +85,8 @@ public class DataPanel extends BeautiPanel implements Exportable {
     SelectClockDialog selectClockDialog = null;
     SelectTreeDialog selectTreeDialog = null;
 
+    SelectTraitDialog selectTraitDialog = null;
+
     BeautiFrame frame = null;
 
     BeautiOptions options = null;
@@ -98,8 +99,8 @@ public class DataPanel extends BeautiPanel implements Exportable {
         dataTable = new JTable(dataTableModel);
 
         dataTable.getTableHeader().setReorderingAllowed(false);
-        dataTable.getTableHeader().setDefaultRenderer(
-                new HeaderRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
+//        dataTable.getTableHeader().setDefaultRenderer(
+//                new HeaderRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
 
         TableColumn col = dataTable.getColumnModel().getColumn(5);
         ComboBoxRenderer comboBoxRenderer = new ComboBoxRenderer();
@@ -196,9 +197,11 @@ public class DataPanel extends BeautiPanel implements Exportable {
         controlPanel1.setOpaque(false);
         controlPanel1.add(actionPanel1);
 
+        createTraitPartitionButton = new JButton(createTraitPartitionAction);
+
         controlPanel1.add(new JLabel("   "));
-        PanelUtils.setupComponent(createImportTraitButton);
-        controlPanel1.add(createImportTraitButton);
+        PanelUtils.setupComponent(createTraitPartitionButton);
+        controlPanel1.add(createTraitPartitionButton);
 
 //        controlPanel1.add(new JLabel(" or "));
 //
@@ -222,10 +225,13 @@ public class DataPanel extends BeautiPanel implements Exportable {
         useStarBEASTCheck.setToolTipText(STARBEASTOptions.CITATION);
         useStarBEASTCheck.addActionListener(new java.awt.event.ActionListener() {
             public void actionPerformed(ActionEvent e) {// wrong listener Issue 397: *BEAST in BEAUti is broken
-                frame.setupStarBEAST(useStarBEASTCheck.isSelected());
+                if (frame.setupStarBEAST(useStarBEASTCheck.isSelected()) == false) {
+                    useStarBEASTCheck.setSelected(false); // go back to unchecked
+                }
+
                 dataTableModel.fireTableDataChanged();
             }
-        });       
+        });
 
     }
 
@@ -233,20 +239,22 @@ public class DataPanel extends BeautiPanel implements Exportable {
 
         int[] selRows = dataTable.getSelectedRows();
         for (int row : selRows) {
-            JFrame frame = new JFrame();
-            frame.setSize(800, 600);
+            AbstractPartitionData partition = options.dataPartitions.get(row);
+            Alignment alignment = null;
 
-            PartitionData partition = options.dataPartitions.get(row);
-            Alignment alignment = partition.getAlignment();
+            if (partition instanceof PartitionData) alignment = ((PartitionData) partition).getAlignment();
 
-            // alignment == null if partition is trait http://code.google.com/p/beast-mcmc/issues/detail?id=343
+            // alignment == null if partition is trait or microsat http://code.google.com/p/beast-mcmc/issues/detail?id=343
             if (alignment == null) {
-                JOptionPane.showMessageDialog(this, "Cannot display traits currently. Use the traits panel to view and edit these.",
+                JOptionPane.showMessageDialog(this, "Cannot display traits or microsatellite data currently.\nUse the traits panel to view and edit traits.",
                         "Illegal Argument Exception",
                         JOptionPane.ERROR_MESSAGE);
                 return;
             }
 
+            JFrame frame = new JFrame();
+            frame.setSize(800, 600);
+
             AlignmentViewer viewer = new AlignmentViewer();
             if (alignment.getDataType().getType() == DataType.NUCLEOTIDES) {
                 viewer.setCellDecorator(new StateCellDecorator(new NucleotideDecorator(), false));
@@ -274,14 +282,7 @@ public class DataPanel extends BeautiPanel implements Exportable {
     private void fireDataChanged() {
 //        options.updateLinksBetweenPDPCMPSMPTMPTPP();
         options.updatePartitionAllLinks();
-
-        if (!(options.clockModelOptions.getRateOptionClockModel() == FixRateType.TIP_CALIBRATED
-                || options.clockModelOptions.getRateOptionClockModel() == FixRateType.NODE_CALIBRATED
-                || options.clockModelOptions.getRateOptionClockModel() == FixRateType.RATE_CALIBRATED)) {
-            //TODO correct?
-            options.clockModelOptions.fixRateOfFirstClockPartition();
-        }
-
+//        frame.setAllOptions();
         frame.setDirty();
     }
 
@@ -293,7 +294,7 @@ public class DataPanel extends BeautiPanel implements Exportable {
         col.setCellEditor(new ComboBoxCellEditor());
 
         col = dataTable.getColumnModel().getColumn(7);
-        col.setCellEditor(new DefaultCellEditor(new JComboBox(options.getNonTraitPartitionTreeModels().toArray())));
+        col.setCellEditor(new DefaultCellEditor(new JComboBox(options.getPartitionTreeModels().toArray())));
 
     }
 
@@ -307,25 +308,14 @@ public class DataPanel extends BeautiPanel implements Exportable {
                                                      int row, int column) {
 
             ((JComboBox) editorComponent).removeAllItems();
-            if (options.containTrait(table.getValueAt(row, 0).toString())) {
-                if (column == 5) {
-                    for (Object ob : options.getPartitionTraitsSubstitutionModels()) {
-                        ((JComboBox) editorComponent).addItem(ob);
-                    }
-                } else if (column == 6) {
-                    for (Object ob : options.getPartitionTraitsClockModels()) {
-                        ((JComboBox) editorComponent).addItem(ob);
-                    }
+
+            if (column == 5) {
+                for (Object ob : options.getPartitionSubstitutionModels()) {
+                    ((JComboBox) editorComponent).addItem(ob);
                 }
-            } else {
-                if (column == 5) {
-                    for (Object ob : options.getPartitionNonTraitsSubstitutionModels()) {
-                        ((JComboBox) editorComponent).addItem(ob);
-                    }
-                } else if (column == 6) {
-                    for (Object ob : options.getPartitionNonTraitsClockModels()) {
-                        ((JComboBox) editorComponent).addItem(ob);
-                    }
+            } else if (column == 6) {
+                for (Object ob : options.getPartitionClockModels()) {
+                    ((JComboBox) editorComponent).addItem(ob);
                 }
             }
 
@@ -362,8 +352,12 @@ public class DataPanel extends BeautiPanel implements Exportable {
 
         modelsChanged();
 
-        useStarBEASTCheck.setEnabled(options.getNonTraitsDataList().size() > 0); // single partition is allowed
-        createImportTraitButton.setEnabled(options.getNonTraitsDataList().size() > 0);
+        boolean taxaAvailable = options.taxonList != null && options.taxonList.getTaxonCount() > 0;
+        boolean traitAvailable = options.traits != null && options.traits.size() > 0 && (!options.useStarBEAST);
+
+        useStarBEASTCheck.setEnabled(taxaAvailable);
+        createTraitPartitionButton.setEnabled(traitAvailable);
+
 
         dataTableModel.fireTableDataChanged();
     }
@@ -377,25 +371,26 @@ public class DataPanel extends BeautiPanel implements Exportable {
 
     public void removeSelection() {
         int[] selRows = dataTable.getSelectedRows();
-        Set<PartitionData> partitionsToRemove = new HashSet<PartitionData>();
+        Set<AbstractPartitionData> partitionsToRemove = new HashSet<AbstractPartitionData>();
         for (int row : selRows) {
             partitionsToRemove.add(options.dataPartitions.get(row));
         }
 
+        boolean hasIdenticalTaxa = options.hasIdenticalTaxa(); // need to check this before removing partitions
+
         // TODO: would probably be a good idea to check if the user wants to remove the last partition
         options.dataPartitions.removeAll(partitionsToRemove);
 
-//        if (options.allowDifferentTaxa && options.dataPartitions.size() < 2) {
-//            uncheckAllowDifferentTaxa();
-//        }
-
-        if (options.getNonTraitsDataList().size() == 0) {
+        if (options.dataPartitions.size() == 0) {
             // all data partitions removed so reset the taxa
             options.reset();
             useStarBEASTCheck.setSelected(false);
+            frame.setupStarBEAST(false);
             frame.statusLabel.setText("");
             frame.setAllOptions();
             frame.getExportAction().setEnabled(false);
+        } else if (!hasIdenticalTaxa) {
+            options.updateTaxonList();
         }
 
         dataTableModel.fireTableDataChanged();
@@ -407,19 +402,61 @@ public class DataPanel extends BeautiPanel implements Exportable {
         dataTable.selectAll();
     }
 
-    public void createFromTraits() {
-        frame.createImportTraits();
+    public boolean createFromTraits(List<TraitData> traits) {
+        int selRow = -1;
+
+        if (selectTraitDialog == null) {
+            selectTraitDialog = new SelectTraitDialog(frame);
+        }
+
+        if (traits==null || traits.size() == 0) {
+            int result = selectTraitDialog.showDialog(options.traits, null);
+            if (result != JOptionPane.CANCEL_OPTION) {
+                TraitData trait = selectTraitDialog.getTrait();
+                String name = trait.getName();
+                if (selectTraitDialog.getMakeCopy()) {
+                    name = selectTraitDialog.getName();
+                }
+
+                selRow = options.createPartitionForTraits(name, trait);
+            } else {
+                return false;
+            }
+        } else {
+            if (traits.size() > 1) {
+                // a set of traits have been passed to the function
+                int result = selectTraitDialog.showDialog(null, null);
+                if (result != JOptionPane.CANCEL_OPTION) {
+                    String name = selectTraitDialog.getName();
+                    selRow = options.createPartitionForTraits(name, traits);
+                }  else {
+                    return false;
+                }
+            } else {
+                selRow = options.createPartitionForTraits(traits.get(0).getName(), traits);
+            }
+        }
+
+        modelsChanged();
         dataTableModel.fireTableDataChanged();
+
+        if (selRow != -1) {
+            dataTable.getSelectionModel().setSelectionInterval(selRow, selRow);
+        }
+        fireDataChanged();
+        repaint();
+
+        return true;
     }
 
     public void unlinkModels() {
         int[] selRows = dataTable.getSelectedRows();
         for (int row : selRows) {
-            PartitionData partition = options.dataPartitions.get(row);
+            AbstractPartitionData partition = options.dataPartitions.get(row);
 
             PartitionSubstitutionModel model = partition.getPartitionSubstitutionModel();
             if (!model.getName().equals(partition.getName())) {
-                PartitionSubstitutionModel newModel = new PartitionSubstitutionModel(options, partition);
+                PartitionSubstitutionModel newModel = new PartitionSubstitutionModel(options, partition.getName(), model);
                 partition.setPartitionSubstitutionModel(newModel);
             }
         }
@@ -432,14 +469,14 @@ public class DataPanel extends BeautiPanel implements Exportable {
 
     public void linkModels() {
         int[] selRows = dataTable.getSelectedRows();
-        List<PartitionData> selectedPartitionData = new ArrayList<PartitionData>();
+        List<AbstractPartitionData> selectedPartitionData = new ArrayList<AbstractPartitionData>();
         DataType dateType = null;
         for (int row : selRows) {
-            PartitionData partition = options.dataPartitions.get(row);
+            AbstractPartitionData partition = options.dataPartitions.get(row);
             if (dateType == null) {
-                dateType = partition.getPartitionSubstitutionModel().getDataType();
+                dateType = partition.getDataType();
             } else {
-                if (partition.getPartitionSubstitutionModel().getDataType() != dateType) {
+                if (partition.getDataType() != dateType) {
                     JOptionPane.showMessageDialog(this, "Can only link the models for data partitions \n" +
                             "of the same data type (e.g., nucleotides)",
                             "Unable to link models",
@@ -452,7 +489,7 @@ public class DataPanel extends BeautiPanel implements Exportable {
                 selectedPartitionData.add(partition);
         }
 
-        Object[] modelArray = options.getPartitionSubstitutionModels(dateType, selectedPartitionData).toArray();
+        Object[] modelArray = options.getPartitionSubstitutionModels(selectedPartitionData).toArray();
 
         if (selectModelDialog == null) {
             selectModelDialog = new SelectModelDialog(frame);
@@ -465,11 +502,15 @@ public class DataPanel extends BeautiPanel implements Exportable {
                 model.setName(selectModelDialog.getName());
             }
 
-            for (PartitionData partition : selectedPartitionData) {
+            for (AbstractPartitionData partition : selectedPartitionData) {
                 partition.setPartitionSubstitutionModel(model);
             }
         }
 
+        if (options.getPartitionSubstitutionModels(Microsatellite.INSTANCE).size() <= 1) {
+            options.shareMicroSat = true;
+        }
+
         modelsChanged();
 
         fireDataChanged();
@@ -479,12 +520,13 @@ public class DataPanel extends BeautiPanel implements Exportable {
     public void unlinkClocks() { // reuse previous PartitionTreePrior
         int[] selRows = dataTable.getSelectedRows();
         for (int row : selRows) {
-            PartitionData partition = options.dataPartitions.get(row);
+            AbstractPartitionData partition = options.dataPartitions.get(row);
 
             PartitionClockModel model = partition.getPartitionClockModel();
             if (!model.getName().equals(partition.getName())) {
-                PartitionClockModel newModel = new PartitionClockModel(options, partition);
+                PartitionClockModel newModel = new PartitionClockModel(options, partition.getName(), model);
                 partition.setPartitionClockModel(newModel);
+                newModel.setClockModelGroup(model.getClockModelGroup()); // set clock model group
             }
         }
 
@@ -497,9 +539,9 @@ public class DataPanel extends BeautiPanel implements Exportable {
     public void linkClocks() { // keep previous PartitionTreePrior for reuse
         int[] selRows = dataTable.getSelectedRows();
 
-        List<PartitionData> selectedPartitionData = new ArrayList<PartitionData>();
+        List<AbstractPartitionData> selectedPartitionData = new ArrayList<AbstractPartitionData>();
         for (int row : selRows) {
-            PartitionData partition = options.dataPartitions.get(row);
+            AbstractPartitionData partition = options.dataPartitions.get(row);
 
             if (!selectedPartitionData.contains(partition))
                 selectedPartitionData.add(partition);
@@ -517,7 +559,7 @@ public class DataPanel extends BeautiPanel implements Exportable {
                 model.setName(selectClockDialog.getName());
             }
 
-            for (PartitionData partition : selectedPartitionData) {
+            for (AbstractPartitionData partition : selectedPartitionData) {
                 partition.setPartitionClockModel(model);
             }
         }
@@ -531,11 +573,11 @@ public class DataPanel extends BeautiPanel implements Exportable {
     public void unlinkTrees() { // reuse previous PartitionTreePrior
         int[] selRows = dataTable.getSelectedRows();
         for (int row : selRows) {
-            PartitionData partition = options.dataPartitions.get(row);
+            AbstractPartitionData partition = options.dataPartitions.get(row);
 
             PartitionTreeModel model = partition.getPartitionTreeModel();
-            if (!model.getName().equals(partition.getName()) && partition.getTraitType() == null) {// not a trait
-                PartitionTreeModel newTree = new PartitionTreeModel(options, partition);
+            if (!model.getName().equals(partition.getName()) && partition.getTraits() == null) {// not a trait
+                PartitionTreeModel newTree = new PartitionTreeModel(options, partition.getName(), model);
 
                 // this prevents partition not broken, and used for unsharing tree prior only,
                 // because sharing uses shareSameTreePrior, unsharing uses getPartitionTreePrior
@@ -556,22 +598,20 @@ public class DataPanel extends BeautiPanel implements Exportable {
     public void linkTrees() { // keep previous PartitionTreePrior for reuse
         int[] selRows = dataTable.getSelectedRows();
 
-        List<PartitionData> selectedPartitionData = new ArrayList<PartitionData>();
+        List<AbstractPartitionData> selectedPartitionData = new ArrayList<AbstractPartitionData>();
         for (int row : selRows) {
-            PartitionData partition = options.dataPartitions.get(row);
+            AbstractPartitionData partition = options.dataPartitions.get(row);
 
             if (!selectedPartitionData.contains(partition))
                 selectedPartitionData.add(partition);
         }
 
-        if (options.allowDifferentTaxa) {//BEAST cannot handle multi <taxa> ref for 1 tree
-            if (selectedPartitionData.size() > 1) {
-                if (!options.validateDiffTaxa(selectedPartitionData)) {
-                    JOptionPane.showMessageDialog(this, "To share a tree, partitions need to have identical taxa.",
-                            "Illegal Configuration",
-                            JOptionPane.ERROR_MESSAGE);
-                    return;
-                }
+        if (selectedPartitionData.size() > 1) {
+            if (!options.hasIdenticalTaxa(selectedPartitionData)) {
+                JOptionPane.showMessageDialog(this, "To share a tree, partitions need to have identical taxa.",
+                        "Illegal Configuration",
+                        JOptionPane.ERROR_MESSAGE);
+                return;
             }
         }
 
@@ -590,7 +630,7 @@ public class DataPanel extends BeautiPanel implements Exportable {
             PartitionTreePrior prior = model.getPartitionTreePrior();
             options.linkTreePriors(prior);
 
-            for (PartitionData partition : selectedPartitionData) {
+            for (AbstractPartitionData partition : selectedPartitionData) {
                 partition.setPartitionTreeModel(model);
             }
 
@@ -615,7 +655,7 @@ public class DataPanel extends BeautiPanel implements Exportable {
     class DataTableModel extends AbstractTableModel {
 
         private static final long serialVersionUID = -6707994233020715574L;
-        String[] columnNames = {"Partition Name", "File Name", "Taxa", "Sites", "Data Type", "Subst. Model", "Clock Model", "Partition Tree"};
+        String[] columnNames = {"Partition Name", "File Name", "Taxa", "Sites", "Data Type", "Site Model", "Clock Model", "Partition Tree"};
 
         public DataTableModel() {
         }
@@ -627,29 +667,29 @@ public class DataPanel extends BeautiPanel implements Exportable {
         public int getRowCount() {
             if (options == null) return 0;
 //            return options.getPartitionDataNoSpecies().size();
-            return options.getNonTraitsDataList().size();
+            return options.dataPartitions.size();
         }
 
         public Object getValueAt(int row, int col) {
 //            PartitionData partition = options.getPartitionDataNoSpecies().get(row);
-            PartitionData partition = options.getNonTraitsDataList().get(row);
+            AbstractPartitionData partition = options.dataPartitions.get(row);
             switch (col) {
                 case 0:
                     return partition.getName();
                 case 1:
                     return partition.getFileName();
                 case 2:
-                    return "" + (partition.getTaxaCount() >= 0 ? partition.getTaxaCount() : "-");
+                    return "" + (partition.getTaxonCount() >= 0 ? partition.getTaxonCount() : "-");
                 case 3:
                     return "" + (partition.getSiteCount() >= 0 ? partition.getSiteCount() : "-");
                 case 4:
-                    return partition.getDataType();
+                    return partition.getDataDescription();
                 case 5:
 //                    return partition.getPloidyType();
 //                case 6:
                     return partition.getPartitionSubstitutionModel().getName();
                 case 6:
-                    return partition.getPartitionClockModel().getName();
+                    return "" + (partition.getPartitionClockModel() != null ? partition.getPartitionClockModel().getName() : "-");
                 case 7:
                     return partition.getPartitionTreeModel().getName();
                 default:
@@ -658,14 +698,12 @@ public class DataPanel extends BeautiPanel implements Exportable {
         }
 
         public void setValueAt(Object aValue, int row, int col) {
-//            PartitionData partition = options.getPartitionDataNoSpecies().get(row);
-            PartitionData partition = options.getNonTraitsDataList().get(row);
+            AbstractPartitionData partition = options.dataPartitions.get(row);
             switch (col) {
                 case 0:
                     String name = ((String) aValue).trim();
                     if (options.hasPartitionData(name)) {
-                        JOptionPane.showMessageDialog(frame, "Partitions cannot have the same name :\n"
-                                + name + "\nRenaming is failed.",
+                        JOptionPane.showMessageDialog(frame, "Duplicate partition name.",
                                 "Illegal Argument Exception", JOptionPane.ERROR_MESSAGE);
                         return;
                     }
@@ -677,7 +715,9 @@ public class DataPanel extends BeautiPanel implements Exportable {
 //                    partition.setPloidyType((PloidyType) aValue);
 //                    break;
 //                case 6:
-                    partition.setPartitionSubstitutionModel((PartitionSubstitutionModel) aValue);
+                    if (((PartitionSubstitutionModel) aValue).getDataType().equals(partition.getDataType())) {
+                        partition.setPartitionSubstitutionModel((PartitionSubstitutionModel) aValue);
+                    }
                     break;
                 case 6:
                     partition.setPartitionClockModel((PartitionClockModel) aValue);
@@ -692,6 +732,8 @@ public class DataPanel extends BeautiPanel implements Exportable {
         public boolean isCellEditable(int row, int col) {
             boolean editable;
 
+            AbstractPartitionData partition = options.dataPartitions.get(row);
+
             switch (col) {
                 case 0:// name
                     editable = true;
@@ -699,11 +741,11 @@ public class DataPanel extends BeautiPanel implements Exportable {
 //                case 5:// ploidy type selection menu
 //                    editable = true;
 //                    break;
-                case 5:// subsitution model selection menu
-                    editable = true;
+                case 5:// substitution model selection menu
+                    editable = partition.getDataType().getType() != DataType.CONTINUOUS;
                     break;
                 case 6:// clock model selection menu
-                    editable = true;
+                    editable = partition.getDataType().getType() != DataType.CONTINUOUS;
                     break;
                 case 7:// tree selection menu
                     editable = true;
@@ -819,14 +861,14 @@ public class DataPanel extends BeautiPanel implements Exportable {
         }
     }
 
-    public class CreateAction extends AbstractAction {
-        public CreateAction() {
-            super("Create or Import Trait ...");
-            setToolTipText("Create a data partition from one or more traits, or import from a mapping file");
+    public class CreateTraitPartitionAction extends AbstractAction {
+        public CreateTraitPartitionAction() {
+            super("Create partition from trait ...");
+            setToolTipText("Create a data partition from a trait. Traits can be defined in the Traits panel.");
         }
 
         public void actionPerformed(ActionEvent ae) {
-            createFromTraits();
+            createFromTraits(null);
         }
     }
 
