@@ -24,8 +24,8 @@
 package dr.app.beauti.generator;
 
 import dr.app.beauti.components.ComponentFactory;
-import dr.app.beauti.enumTypes.ClockType;
 import dr.app.beauti.options.*;
+import dr.app.beauti.types.ClockType;
 import dr.app.beauti.util.XMLWriter;
 import dr.evomodel.branchratemodel.BranchRateModel;
 import dr.evomodel.clock.RateEvolutionLikelihood;
@@ -70,7 +70,7 @@ public class BranchRatesModelGenerator extends Generator {
         Attribute[] attributes;
         int categoryCount = 0;
         String treePrefix;
-        List<PartitionTreeModel> activeTrees = options.getPartitionTreeModels(options.getAllPartitionData(model));
+        List<PartitionTreeModel> activeTrees = options.getPartitionTreeModels(options.getDataPartitions(model));
 
         switch (model.getClockType()) {
             case STRICT_CLOCK:
@@ -85,9 +85,8 @@ public class BranchRatesModelGenerator extends Generator {
 
                 break;
 
-            case UNCORRELATED_EXPONENTIAL:
-            case UNCORRELATED_LOGNORMAL:
-                writer.writeComment("The uncorrelated relaxed clock (Drummond, Ho, Phillips & Rambaut, 2006)");
+            case UNCORRELATED:
+                writer.writeComment("The uncorrelated relaxed clock (Drummond, Ho, Phillips & Rambaut (2006) PLoS Biology 4, e88 )");
 
                 for (PartitionTreeModel tree : activeTrees) {
                     treePrefix = tree.getPrefix();
@@ -113,51 +112,56 @@ public class BranchRatesModelGenerator extends Generator {
 
                     writer.writeOpenTag("distribution");
 
-                    if (model.getClockType() == ClockType.UNCORRELATED_EXPONENTIAL) {
+                    switch (model.getClockDistributionType()) {
 
-                        writer.writeOpenTag(ExponentialDistributionModel.EXPONENTIAL_DISTRIBUTION_MODEL);
+                        case LOGNORMAL:
+                            writer.writeOpenTag(LogNormalDistributionModelParser.LOGNORMAL_DISTRIBUTION_MODEL,
+                                    new Attribute.Default<String>(LogNormalDistributionModelParser.MEAN_IN_REAL_SPACE, "true"));
 
-                        if (activeTrees.indexOf(tree) < 1) {
-                            writeParameter("mean", ClockType.UCED_MEAN, model, writer);
-                        } else {
-                            writeParameterRef("mean", modelPrefix + ClockType.UCED_MEAN, writer);
-                        }
+                            if (activeTrees.indexOf(tree) < 1) {
+                                writeParameter("mean", ClockType.UCLD_MEAN, model, writer);
+                                writeParameter("stdev", ClockType.UCLD_STDEV, model, writer);
+                            } else {
+                                writeParameterRef("mean", modelPrefix + ClockType.UCLD_MEAN, writer);
+                                writeParameterRef("stdev", modelPrefix + ClockType.UCLD_STDEV, writer);
+                            }
 
-                        writer.writeCloseTag(ExponentialDistributionModel.EXPONENTIAL_DISTRIBUTION_MODEL);
+                            writer.writeCloseTag(LogNormalDistributionModelParser.LOGNORMAL_DISTRIBUTION_MODEL);
+                            break;
+                        case GAMMA:
+                            throw new UnsupportedOperationException("Uncorrelated gamma model not implemented yet");
+//                            break;
+                        case CAUCHY:
+                            throw new UnsupportedOperationException("Uncorrelated Cauchy model not implemented yet");
+//                            break;
+                        case EXPONENTIAL:
+                            writer.writeOpenTag(ExponentialDistributionModel.EXPONENTIAL_DISTRIBUTION_MODEL);
 
-                    } else if (model.getClockType() == ClockType.UNCORRELATED_LOGNORMAL) {
+                            if (activeTrees.indexOf(tree) < 1) {
+                                writeParameter("mean", ClockType.UCED_MEAN, model, writer);
+                            } else {
+                                writeParameterRef("mean", modelPrefix + ClockType.UCED_MEAN, writer);
+                            }
 
-                        writer.writeOpenTag(LogNormalDistributionModelParser.LOGNORMAL_DISTRIBUTION_MODEL,
-                                new Attribute.Default<String>(LogNormalDistributionModelParser.MEAN_IN_REAL_SPACE, "true"));
-
-                        if (activeTrees.indexOf(tree) < 1) {
-                            writeParameter("mean", ClockType.UCLD_MEAN, model, writer);
-                            writeParameter("stdev", ClockType.UCLD_STDEV, model, writer);
-                        } else {
-                            writeParameterRef("mean", modelPrefix + ClockType.UCLD_MEAN, writer);
-                            writeParameterRef("stdev", modelPrefix + ClockType.UCLD_STDEV, writer);
-                        }
-
-                        writer.writeCloseTag(LogNormalDistributionModelParser.LOGNORMAL_DISTRIBUTION_MODEL);
-
-                    } else {
-                        throw new RuntimeException(
-                                "Unrecognised relaxed clock model");
+                            writer.writeCloseTag(ExponentialDistributionModel.EXPONENTIAL_DISTRIBUTION_MODEL);
+                            break;
                     }
 
                     writer.writeCloseTag("distribution");
 
                     writer.writeOpenTag(DiscretizedBranchRatesParser.RATE_CATEGORIES);
-                    if (options.allowDifferentTaxa) { // http://code.google.com/p/beast-mcmc/issues/detail?id=235
-                        for (PartitionData dataPartition : options.getNonTraitsDataList()) {
-                            if (dataPartition.getPartitionClockModel().equals(model)) {
-                                categoryCount = (dataPartition.getTaxaCount() - 1) * 2;
-                            }
-                        }
-                    } else {
-                        categoryCount = (options.taxonList.getTaxonCount() - 1) * 2;
-                    }
-                    writeParameter(clockTree.getParameter("branchRates.categories"), categoryCount, writer);
+                    // AR - this parameter will now set its dimension automatically when BEAST is run
+//                    if (!options.hasIdenticalTaxa()) {
+//                        for (AbstractPartitionData dataPartition : options.dataPartitions) {
+//                            if (dataPartition.getPartitionClockModel().equals(model)) {
+//                                categoryCount = (dataPartition.getTaxonCount() - 1) * 2;
+//                            }
+//                        }
+//                    } else {
+//                        categoryCount = (options.taxonList.getTaxonCount() - 1) * 2;
+//                    }
+//                    writeParameter(clockTree.getParameter("branchRates.categories"), categoryCount, writer);
+                    writeParameter(clockTree.getParameter("branchRates.categories"), -1, writer);
                     writer.writeCloseTag(DiscretizedBranchRatesParser.RATE_CATEGORIES);
                     writer.writeCloseTag(DiscretizedBranchRatesParser.DISCRETIZED_BRANCH_RATES);
 
@@ -208,7 +212,7 @@ public class BranchRatesModelGenerator extends Generator {
 
                 break;
 
-            case AUTOCORRELATED_LOGNORMAL:
+            case AUTOCORRELATED:
                 writer.writeComment("The autocorrelated relaxed clock (Rannala & Yang, 2007)");
 
                 for (PartitionTreeModel tree : activeTrees) {
@@ -323,7 +327,7 @@ public class BranchRatesModelGenerator extends Generator {
                 writer.writeComment("The random local clock model (Drummond & Suchard, 2010)");
 
                 if (activeTrees == null || activeTrees.size() != 1) {
-                    throw new IllegalArgumentException("One random local clock CANNOT have different tree models !!!");
+                    throw new IllegalArgumentException("A single random local clock cannot be applied to multiple trees.");
                 }
                 treePrefix = activeTrees.get(0).getPrefix();
 
@@ -428,13 +432,22 @@ public class BranchRatesModelGenerator extends Generator {
             case RANDOM_LOCAL_CLOCK:
                 return modelPrefix + "clock.rate";
 
-            case UNCORRELATED_EXPONENTIAL:
-                return modelPrefix + ClockType.UCED_MEAN;
+            case UNCORRELATED:
+                switch (model.getClockDistributionType()) {
 
-            case UNCORRELATED_LOGNORMAL:
-                return modelPrefix + ClockType.UCLD_MEAN;
+                    case LOGNORMAL:
+                        return modelPrefix + ClockType.UCLD_MEAN;
+                    case GAMMA:
+                        throw new UnsupportedOperationException("Uncorrelated gamma model not supported yet");
+//                        return modelPrefix + ClockType.UCGD_SCALE;
+                    case CAUCHY:
+                        throw new UnsupportedOperationException("Uncorrelated Cauchy model not supported yet");
+//                        return modelPrefix + ClockType.UCCD_MEAN;
+                    case EXPONENTIAL:
+                        return modelPrefix + ClockType.UCED_MEAN;
+                }
 
-            case AUTOCORRELATED_LOGNORMAL:
+            case AUTOCORRELATED:
                 //TODO
                 throw new IllegalArgumentException("Autocorrelated Relaxed Clock, writeAllClockRateRefs(PartitionClockModel model, XMLWriter writer)");
 //	        	break;
@@ -453,17 +466,23 @@ public class BranchRatesModelGenerator extends Generator {
                 writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + "clock.rate");
                 break;
 
-            case UNCORRELATED_EXPONENTIAL:
-                writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + ClockType.UCED_MEAN);
-                break;
+            case UNCORRELATED:
+                switch (model.getClockDistributionType()) {
+                    case LOGNORMAL:
+                        writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + ClockType.UCLD_MEAN);
+                        writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + ClockType.UCLD_STDEV);
+                        break;
+                    case GAMMA:
+                        throw new UnsupportedOperationException("Uncorrelated gamma model not supported yet");
+                    case CAUCHY:
+                        throw new UnsupportedOperationException("Uncorrelated Couchy model not supported yet");
+                    case EXPONENTIAL:
+                        writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + ClockType.UCED_MEAN);
+                        break;
+                }
 
-            case UNCORRELATED_LOGNORMAL:
-                writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + ClockType.UCLD_MEAN);
-                writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + ClockType.UCLD_STDEV);
-                break;
-
-            case AUTOCORRELATED_LOGNORMAL:
-// TODO                
+            case AUTOCORRELATED:
+// TODO
                 break;
 
             default:
@@ -479,18 +498,17 @@ public class BranchRatesModelGenerator extends Generator {
             case STRICT_CLOCK:
                 break;
 
-            case UNCORRELATED_EXPONENTIAL:
-            case UNCORRELATED_LOGNORMAL:
-                for (PartitionTreeModel tree : options.getPartitionTreeModels(options.getAllPartitionData(model))) {
+            case UNCORRELATED:
+                for (PartitionTreeModel tree : options.getPartitionTreeModels(options.getDataPartitions(model))) {
                     writer.writeIDref(RateStatisticParser.RATE_STATISTIC, options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + "meanRate");
                     writer.writeIDref(RateStatisticParser.RATE_STATISTIC, options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + RateStatisticParser.COEFFICIENT_OF_VARIATION);
                     writer.writeIDref(RateCovarianceStatisticParser.RATE_COVARIANCE_STATISTIC, options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + "covariance");
                 }
                 break;
 
-            case AUTOCORRELATED_LOGNORMAL:
+            case AUTOCORRELATED:
 // TODO
-                for (PartitionTreeModel tree : options.getPartitionTreeModels(options.getAllPartitionData(model))) {
+                for (PartitionTreeModel tree : options.getPartitionTreeModels(options.getDataPartitions(model))) {
                     writer.writeIDref(RateStatisticParser.RATE_STATISTIC, options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + "meanRate");
                     writer.writeIDref(RateStatisticParser.RATE_STATISTIC, options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + RateStatisticParser.COEFFICIENT_OF_VARIATION);
                     writer.writeIDref(RateCovarianceStatisticParser.RATE_COVARIANCE_STATISTIC, options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + "covariance");
@@ -501,7 +519,7 @@ public class BranchRatesModelGenerator extends Generator {
 
             case RANDOM_LOCAL_CLOCK:
                 writer.writeIDref(SumStatisticParser.SUM_STATISTIC, model.getPrefix() + "rateChanges");
-                for (PartitionTreeModel tree : options.getPartitionTreeModels(options.getAllPartitionData(model))) {
+                for (PartitionTreeModel tree : options.getPartitionTreeModels(options.getDataPartitions(model))) {
                     writer.writeIDref(RateStatisticParser.RATE_STATISTIC, options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + "meanRate");
                     writer.writeIDref(RateStatisticParser.RATE_STATISTIC, options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + RateStatisticParser.COEFFICIENT_OF_VARIATION);
                     writer.writeIDref(RateCovarianceStatisticParser.RATE_COVARIANCE_STATISTIC, options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + "covariance");
@@ -514,5 +532,15 @@ public class BranchRatesModelGenerator extends Generator {
 
     }
 
+    public void writeClockLikelihoodReferences(XMLWriter writer) {
+        for (AbstractPartitionData partition : options.dataPartitions) { // Each PD has one TreeLikelihood
+            PartitionClockModel clockModel = partition.getPartitionClockModel();
+
+            if (clockModel != null && clockModel.getClockType() == ClockType.AUTOCORRELATED) {
+                throw new UnsupportedOperationException("Autocorrelated relaxed clock model not implemented yet");
+//                writer.writeIDref(ACLikelihoodParser.AC_LIKELIHOOD, clockModel.getPrefix() + BranchRateModel.BRANCH_RATES);
+            }
+        }
+    }
 
 }
