@@ -26,11 +26,12 @@
 package dr.app.beauti.generator;
 
 import dr.app.beauti.components.ComponentFactory;
-import dr.app.beauti.enumTypes.ClockType;
-import dr.app.beauti.enumTypes.FixRateType;
-import dr.app.beauti.enumTypes.TreePriorType;
 import dr.app.beauti.options.*;
+import dr.app.beauti.types.ClockType;
+import dr.app.beauti.types.FixRateType;
+import dr.app.beauti.types.TreePriorType;
 import dr.app.beauti.util.XMLWriter;
+import dr.evolution.datatype.DataType;
 import dr.evolution.util.Taxa;
 import dr.evomodel.branchratemodel.BranchRateModel;
 import dr.evomodel.tree.TreeModel;
@@ -43,8 +44,6 @@ import dr.evomodelxml.speciation.*;
 import dr.evomodelxml.tree.TMRCAStatisticParser;
 import dr.evomodelxml.tree.TreeLoggerParser;
 import dr.evomodelxml.tree.TreeModelParser;
-import dr.evomodelxml.treelikelihood.AncestralStateTreeLikelihoodParser;
-import dr.evomodelxml.treelikelihood.TreeLikelihoodParser;
 import dr.inference.model.ParameterParser;
 import dr.inferencexml.distribution.MixedDistributionLikelihoodParser;
 import dr.inferencexml.loggers.ColumnsParser;
@@ -59,6 +58,7 @@ import java.util.List;
 
 /**
  * @author Alexei Drummond
+ * @author Andrew Rambaut
  * @author Walter Xie
  */
 public class LogGenerator extends Generator {
@@ -170,10 +170,9 @@ public class LogGenerator extends Generator {
             writer.writeCloseTag(ColumnsParser.COLUMN);
         }
 
-        if (options.hasDiscreteIntegerTraitsExcludeSpecies()) {
-            for (PartitionSubstitutionModel model : options.getPartitionTraitsSubstitutionModels()) {
-                substitutionModelGenerator.writeStatisticLog(model, writer);
-            }
+        for (PartitionSubstitutionModel model : options.getPartitionSubstitutionModels()) {
+            if (model.getDataType().getType() == DataType.MICRO_SAT)
+                substitutionModelGenerator.writeMicrosatSubstModelParameterRef(model, writer);
         }
 
         generateInsertionPoint(ComponentGenerator.InsertionPoint.IN_SCREEN_LOG, writer);
@@ -191,11 +190,12 @@ public class LogGenerator extends Generator {
      * @param branchRatesModelGenerator  BranchRatesModelGenerator
      * @param substitutionModelGenerator SubstitutionModelGenerator
      * @param treeLikelihoodGenerator    TreeLikelihoodGenerator
-     * @param generalTraitGenerator
      */
-    public void writeLogToFile(XMLWriter writer, TreePriorGenerator treePriorGenerator, BranchRatesModelGenerator branchRatesModelGenerator,
-                               SubstitutionModelGenerator substitutionModelGenerator, TreeLikelihoodGenerator treeLikelihoodGenerator,
-                               GeneralTraitGenerator generalTraitGenerator) {
+    public void writeLogToFile(XMLWriter writer,
+                               TreePriorGenerator treePriorGenerator,
+                               BranchRatesModelGenerator branchRatesModelGenerator,
+                               SubstitutionModelGenerator substitutionModelGenerator,
+                               TreeLikelihoodGenerator treeLikelihoodGenerator) {
         writer.writeComment("write log to file");
 
         if (options.logFileName == null) {
@@ -235,7 +235,8 @@ public class LogGenerator extends Generator {
             if (options.getPartitionTreePriors().get(0).getNodeHeightPrior() == TreePriorType.SPECIES_BIRTH_DEATH) {
                 writer.writeIDref(ParameterParser.PARAMETER, TraitData.TRAIT_SPECIES + "." + BirthDeathModelParser.MEAN_GROWTH_RATE_PARAM_NAME);
                 writer.writeIDref(ParameterParser.PARAMETER, TraitData.TRAIT_SPECIES + "." + BirthDeathModelParser.RELATIVE_DEATH_RATE_PARAM_NAME);
-            } else if (options.getPartitionTreePriors().get(0).getNodeHeightPrior() == TreePriorType.SPECIES_YULE) {
+            } else if (options.getPartitionTreePriors().get(0).getNodeHeightPrior() == TreePriorType.SPECIES_YULE ||
+                    options.getPartitionTreePriors().get(0).getNodeHeightPrior() == TreePriorType.SPECIES_YULE_CALIBRATION) {
                 writer.writeIDref(ParameterParser.PARAMETER, TraitData.TRAIT_SPECIES + "." + YuleModelParser.YULE + "." + YuleModelParser.BIRTH_RATE);
             } else {
                 throw new IllegalArgumentException("Get wrong species tree prior using *BEAST : " + options.getPartitionTreePriors().get(0).getNodeHeightPrior().toString());
@@ -249,10 +250,17 @@ public class LogGenerator extends Generator {
             writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + TreeModel.TREE_MODEL + "." + TreeModelParser.ROOT_HEIGHT);
         }
 
-        for (Taxa taxa : options.taxonSets) {
-            // make tmrca(tree.name) eay to read in log for Tracer
-            PartitionTreeModel treeModel = options.taxonSetsTreeModel.get(taxa);
-            writer.writeIDref(TMRCAStatisticParser.TMRCA_STATISTIC, "tmrca(" + treeModel.getPrefix() + taxa.getId() + ")");
+        if (options.useStarBEAST) {
+            for (Taxa taxa : options.speciesSets) {
+                // make tmrca(tree.name) eay to read in log for Tracer
+                writer.writeIDref(TMRCAStatisticParser.TMRCA_STATISTIC, "tmrca(" + taxa.getId() + ")");
+            }
+        } else {
+            for (Taxa taxa : options.taxonSets) {
+                // make tmrca(tree.name) eay to read in log for Tracer
+                PartitionTreeModel treeModel = options.taxonSetsTreeModel.get(taxa);
+                writer.writeIDref(TMRCAStatisticParser.TMRCA_STATISTIC, "tmrca(" + treeModel.getPrefix() + taxa.getId() + ")");
+            }
         }
 
 //        if ( options.shareSameTreePrior ) { // Share Same Tree Prior
@@ -266,21 +274,37 @@ public class LogGenerator extends Generator {
 //	    }
 
         for (PartitionSubstitutionModel model : options.getPartitionSubstitutionModels()) {
-            substitutionModelGenerator.writeLog(writer, model);
+            substitutionModelGenerator.writeLog(model, writer);
             if (model.hasCodon()) {
                 writer.writeIDref(CompoundParameterParser.COMPOUND_PARAMETER, model.getPrefix() + "allMus");
             }
         }
 
-        if (options.clockModelOptions.getRateOptionClockModel() == FixRateType.FIX_MEAN) {
-            writer.writeIDref(ParameterParser.PARAMETER, "allClockRates");
-            for (PartitionClockModel model : options.getPartitionClockModels()) {
-                if (model.getClockType() == ClockType.UNCORRELATED_LOGNORMAL)
-                    writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + ClockType.UCLD_STDEV);
-            }
-        } else {
-            for (PartitionClockModel model : options.getPartitionClockModels()) {
-                branchRatesModelGenerator.writeLog(model, writer);
+        for (ClockModelGroup clockModelGroup : options.clockModelOptions.getClockModelGroups()) {
+            if (clockModelGroup.getRateTypeOption() == FixRateType.FIX_MEAN) {
+                writer.writeIDref(ParameterParser.PARAMETER, clockModelGroup.getName());
+                for (PartitionClockModel model : options.getPartitionClockModels(clockModelGroup)) {
+                    if (model.getClockType() == ClockType.UNCORRELATED) {
+                        switch (model.getClockDistributionType()) {
+                            case LOGNORMAL:
+                                writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + ClockType.UCLD_STDEV);
+                                break;
+                            case GAMMA:
+                                throw new UnsupportedOperationException("Uncorrelated gamma model not implemented yet");
+//                            break;
+                            case CAUCHY:
+                                throw new UnsupportedOperationException("Uncorrelated Cauchy model not implemented yet");
+//                            break;
+                            case EXPONENTIAL:
+                                // nothing required
+                                break;
+                        }
+                    }
+                }
+            } else {
+                for (PartitionClockModel model : options.getPartitionClockModels(clockModelGroup)) {
+                    branchRatesModelGenerator.writeLog(model, writer);
+                }
             }
         }
 
@@ -290,9 +314,8 @@ public class LogGenerator extends Generator {
 
         generateInsertionPoint(ComponentGenerator.InsertionPoint.IN_FILE_LOG_PARAMETERS, writer);
 
-        if (options.hasData()) {
-            treeLikelihoodGenerator.writeTreeLikelihoodReferences(writer);
-        }
+        treeLikelihoodGenerator.writeTreeLikelihoodReferences(writer);
+        branchRatesModelGenerator.writeClockLikelihoodReferences(writer);
 
         generateInsertionPoint(ComponentGenerator.InsertionPoint.IN_FILE_LOG_LIKELIHOODS, writer);
 
@@ -308,10 +331,6 @@ public class LogGenerator extends Generator {
                 writer.writeIDref(CoalescentLikelihoodParser.COALESCENT_LIKELIHOOD, prior.getPrefix() + COALESCENT); // only 1 coalescent
         }
 
-        if (options.hasDiscreteIntegerTraitsExcludeSpecies()) {
-            generalTraitGenerator.writeAncestralTreeLikelihoodReferences(writer);
-        }
-
         writer.writeCloseTag(LoggerParser.LOG);
 
         generateInsertionPoint(ComponentGenerator.InsertionPoint.AFTER_FILE_LOG, writer);
@@ -387,52 +406,18 @@ public class LogGenerator extends Generator {
 
             writer.writeIDref(TreeModel.TREE_MODEL, tree.getPrefix() + TreeModel.TREE_MODEL);
 
-            for (PartitionClockModel model : options.getPartitionClockModels(options.getAllPartitionData(tree))) {
-                if (options.getAllPartitionData(model).get(0).getTraitType() == null) {
-                    switch (model.getClockType()) {
-                        case STRICT_CLOCK:
-                            writer.writeIDref(StrictClockBranchRatesParser.STRICT_CLOCK_BRANCH_RATES, model.getPrefix() + BranchRateModel.BRANCH_RATES);
-                            break;
-
-                        case UNCORRELATED_EXPONENTIAL:
-                        case UNCORRELATED_LOGNORMAL:
-                            writer.writeIDref(DiscretizedBranchRatesParser.DISCRETIZED_BRANCH_RATES, options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + BranchRateModel.BRANCH_RATES);
-                            break;
-
-                        case RANDOM_LOCAL_CLOCK:
-                            writer.writeIDref(RandomLocalClockModelParser.LOCAL_BRANCH_RATES, model.getPrefix() + BranchRateModel.BRANCH_RATES);
-                            break;
-
-                        case AUTOCORRELATED_LOGNORMAL:
-                            writer.writeIDref(ACLikelihoodParser.AC_LIKELIHOOD, options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + BranchRateModel.BRANCH_RATES);
-                            break;
-
-                        default:
-                            throw new IllegalArgumentException("Unknown clock model");
-                    }
-                }
-            }
+            writeTreeTraits(writer, tree);
 
             if (options.hasData()) {
                 // we have data...
                 writer.writeIDref("posterior", "posterior");
             }
 
-            if (options.hasDiscreteIntegerTraitsExcludeSpecies()) {
-                for (PartitionData partitionData : options.getAllPartitionData(tree)) { // Each TD except Species has one AncestralTreeLikelihood
-                    if (partitionData.getTraitType() != null && (!partitionData.getName().equalsIgnoreCase(TraitData.TRAIT_SPECIES.toString())) )
-                        writer.writeIDref(AncestralStateTreeLikelihoodParser.RECONSTRUCTING_TREE_LIKELIHOOD,
-                                partitionData.getPrefix() + TreeLikelihoodParser.TREE_LIKELIHOOD);
-                }
-            }
+            generateInsertionPoint(ComponentGenerator.InsertionPoint.IN_TREES_LOG, tree, writer);
 
             writer.writeCloseTag(TreeLoggerParser.LOG_TREE);
         } // end For loop
 
-
-        generateInsertionPoint(ComponentGenerator.InsertionPoint.IN_TREES_LOG, writer);
-
-
         if (options.substTreeLog) {
             if (options.useStarBEAST) { // species
                 //TODO: species sub tree
@@ -452,31 +437,37 @@ public class LogGenerator extends Generator {
                         });
                 writer.writeIDref(TreeModel.TREE_MODEL, tree.getPrefix() + TreeModel.TREE_MODEL);
 
-                for (PartitionClockModel model : options.getPartitionClockModels(options.getAllPartitionData(tree))) {
-                    if (options.getAllPartitionData(model).get(0).getTraitType() == null) {
-                        switch (model.getClockType()) {
-                            case STRICT_CLOCK:
-                                writer.writeIDref(StrictClockBranchRatesParser.STRICT_CLOCK_BRANCH_RATES, model.getPrefix() + BranchRateModel.BRANCH_RATES);
-                                break;
+                // assume the first clock model is the one... (not sure if this makes sense)
+                PartitionClockModel model = options.getPartitionClockModels(options.getDataPartitions(tree)).get(0);
+                String tag = "";
+                String id = "";
 
-                            case UNCORRELATED_EXPONENTIAL:
-                            case UNCORRELATED_LOGNORMAL:
-                                writer.writeIDref(DiscretizedBranchRatesParser.DISCRETIZED_BRANCH_RATES, options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + BranchRateModel.BRANCH_RATES);
-                                break;
+                switch (model.getClockType()) {
+                    case STRICT_CLOCK:
+                        tag = StrictClockBranchRatesParser.STRICT_CLOCK_BRANCH_RATES;
+                        id = model.getPrefix() + BranchRateModel.BRANCH_RATES;
+                        break;
 
-                            case RANDOM_LOCAL_CLOCK:
-                                writer.writeIDref(RandomLocalClockModelParser.LOCAL_BRANCH_RATES, model.getPrefix() + BranchRateModel.BRANCH_RATES);
-                                break;
+                    case UNCORRELATED:
+                        tag = DiscretizedBranchRatesParser.DISCRETIZED_BRANCH_RATES;
+                        id = model.getPrefix() + BranchRateModel.BRANCH_RATES;
+                        break;
 
-                            case AUTOCORRELATED_LOGNORMAL:
-                                writer.writeIDref(ACLikelihoodParser.AC_LIKELIHOOD, options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + BranchRateModel.BRANCH_RATES);
-                                break;
+                    case RANDOM_LOCAL_CLOCK:
+                        tag = RandomLocalClockModelParser.LOCAL_BRANCH_RATES;
+                        id = model.getPrefix() + BranchRateModel.BRANCH_RATES;
+                        break;
 
-                            default:
-                                throw new IllegalArgumentException("Unknown clock model");
-                        }
-                    }
+                    case AUTOCORRELATED:
+                        tag = ACLikelihoodParser.AC_LIKELIHOOD;
+                        id =  options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + BranchRateModel.BRANCH_RATES;
+                        break;
+
+                    default:
+                        throw new IllegalArgumentException("Unknown clock model");
                 }
+                writer.writeIDref(tag, id);
+                writeTreeTrait(writer, tag, id, BranchRateModel.RATE, model.getPrefix() + BranchRateModel.RATE);
 
                 writer.writeCloseTag(TreeLoggerParser.LOG_TREE);
             }
@@ -485,33 +476,52 @@ public class LogGenerator extends Generator {
         generateInsertionPoint(ComponentGenerator.InsertionPoint.AFTER_TREES_LOG, writer);
     }
 
-//    <parameter idref="locations.trait.rates"/>
-//    <parameter idref="locations.trait.indicators"/>
-//    <parameter idref="locations.clock.rate"/>
+    private void writeTreeTraits(XMLWriter writer, PartitionTreeModel tree) {
+        for (PartitionClockModel model : options.getPartitionClockModels(options.getDataPartitions(tree))) {
 
-    public void writeAdditionalLogToFile(XMLWriter writer, BranchRatesModelGenerator branchRatesModelGenerator,
-                                         SubstitutionModelGenerator substitutionModelGenerator) {
-        if (options.hasDiscreteIntegerTraitsExcludeSpecies()) {
-            writer.writeComment("write rate matrix log to file");
+            switch (model.getClockType()) {
+                case STRICT_CLOCK:
+                    writeTreeTrait(writer, StrictClockBranchRatesParser.STRICT_CLOCK_BRANCH_RATES,
+                            model.getPrefix() + BranchRateModel.BRANCH_RATES,
+                            BranchRateModel.RATE, model.getPrefix() + BranchRateModel.RATE);
+                    break;
 
-            String fileName = options.logFileName.substring(0, options.logFileName.indexOf(".log")) + "_rateMatrix.log";
-            writer.writeOpenTag(LoggerParser.LOG, new Attribute[]{
-                    new Attribute.Default<String>(XMLParser.ID, "rateMatrixLog"),
-                    new Attribute.Default<String>(LoggerParser.LOG_EVERY, options.logEvery + ""),
-                    new Attribute.Default<String>(LoggerParser.FILE_NAME, fileName)});
+                case UNCORRELATED:
+                    writeTreeTrait(writer, DiscretizedBranchRatesParser.DISCRETIZED_BRANCH_RATES,
+                            options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + BranchRateModel.BRANCH_RATES,
+                            BranchRateModel.RATE, model.getPrefix() + BranchRateModel.RATE);
+                    break;
 
-            for (PartitionSubstitutionModel model : options.getPartitionTraitsSubstitutionModels()) {
-                substitutionModelGenerator.writeRateLog(model, writer);
+                case RANDOM_LOCAL_CLOCK:
+                    writeTreeTrait(writer, RandomLocalClockModelParser.LOCAL_BRANCH_RATES,
+                            model.getPrefix() + BranchRateModel.BRANCH_RATES,
+                            BranchRateModel.RATE, model.getPrefix() + BranchRateModel.RATE);
+                    break;
+
+                case AUTOCORRELATED:
+                    writer.writeIDref(ACLikelihoodParser.AC_LIKELIHOOD,
+                            options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + BranchRateModel.BRANCH_RATES);
+                    writeTreeTrait(writer, ACLikelihoodParser.AC_LIKELIHOOD,
+                            options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + BranchRateModel.BRANCH_RATES,
+                            BranchRateModel.RATE, model.getPrefix() + BranchRateModel.RATE);
+                    break;
+
+                default:
+                    throw new IllegalArgumentException("Unknown clock model");
             }
-
-            for (PartitionClockModel model : options.getPartitionTraitsClockModels()) {
-                branchRatesModelGenerator.writeLog(model, writer);
-            }
-
-            writer.writeCloseTag(LoggerParser.LOG);
-
         }
-
     }
 
+    private void writeTreeTrait(XMLWriter writer, String treeTraitTag, String treeTraitID, String traitName, String traitTag) {
+
+        writer.writeOpenTag(TreeLoggerParser.TREE_TRAIT,
+                new Attribute[]{
+                        new Attribute.Default<String>(TreeLoggerParser.NAME, traitName),
+                        new Attribute.Default<String>(TreeLoggerParser.TAG, traitTag)
+                });
+        writer.writeIDref(treeTraitTag, treeTraitID);
+
+        writer.writeCloseTag(TreeLoggerParser.TREE_TRAIT);
+
+    }
 }
\ No newline at end of file
