@@ -1,31 +1,53 @@
+/*
+ * SubstitutionModelGenerator.java
+ *
+ * Copyright (c) 2002-2011 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.app.beauti.generator;
 
 import dr.app.beauti.components.ComponentFactory;
-import dr.app.beauti.enumTypes.FrequencyPolicyType;
-import dr.app.beauti.enumTypes.LocationSubstModelType;
+import dr.app.beauti.options.AbstractPartitionData;
 import dr.app.beauti.options.BeautiOptions;
 import dr.app.beauti.options.PartitionData;
 import dr.app.beauti.options.PartitionSubstitutionModel;
-import dr.app.beauti.options.TraitData;
+import dr.app.beauti.types.FrequencyPolicyType;
+import dr.app.beauti.types.MicroSatModelType;
 import dr.app.beauti.util.XMLWriter;
-import dr.evolution.alignment.Patterns;
+import dr.evolution.alignment.Alignment;
 import dr.evolution.datatype.DataType;
 import dr.evolution.datatype.Nucleotides;
 import dr.evomodel.sitemodel.GammaSiteModel;
 import dr.evomodel.sitemodel.SiteModel;
-import dr.evomodel.substmodel.AbstractSubstitutionModel;
+import dr.evomodel.substmodel.AsymmetricQuadraticModel;
+import dr.evomodel.substmodel.LinearBiasModel;
 import dr.evomodel.substmodel.NucModelType;
+import dr.evomodel.substmodel.TwoPhaseModel;
 import dr.evomodelxml.sitemodel.GammaSiteModelParser;
 import dr.evomodelxml.substmodel.*;
 import dr.evoxml.AlignmentParser;
-import dr.evoxml.GeneralDataTypeParser;
-import dr.evoxml.MergePatternsParser;
-import dr.evoxml.SitePatternsParser;
+import dr.evoxml.MicrosatelliteParser;
 import dr.inference.model.ParameterParser;
-import dr.inferencexml.loggers.ColumnsParser;
 import dr.inferencexml.model.CompoundParameterParser;
-import dr.inferencexml.model.ProductStatisticParser;
-import dr.inferencexml.model.SumStatisticParser;
 import dr.util.Attribute;
 import dr.xml.XMLParser;
 
@@ -33,6 +55,7 @@ import java.util.List;
 
 /**
  * @author Alexei Drummond
+ * @author Andrew Rambaut
  * @author Walter Xie
  */
 public class SubstitutionModelGenerator extends Generator {
@@ -73,7 +96,9 @@ public class SubstitutionModelGenerator extends Generator {
                             ParameterParser.PARAMETER,
                             new Attribute[]{
                                     new Attribute.Default<String>(XMLParser.ID, prefix + "frequencies"),
-                                    new Attribute.Default<String>(ParameterParser.VALUE, "0.25 0.25 0.25 0.25")
+                                    new Attribute.Default<String>(ParameterParser.VALUE, "0.25 0.25 0.25 0.25"),
+                                    new Attribute.Default<String>(ParameterParser.LOWER, "0.0"),
+                                    new Attribute.Default<String>(ParameterParser.UPPER, "1.0")
                             },
                             true
                     );
@@ -153,6 +178,8 @@ public class SubstitutionModelGenerator extends Generator {
             case DataType.COVARION:
 
                 switch (model.getBinarySubstitutionModel()) {
+                    case BIN_DOLLO:
+                        return;
                     case BIN_SIMPLE:
                         writeBinarySimpleModel(writer, model);
                         break;
@@ -166,9 +193,12 @@ public class SubstitutionModelGenerator extends Generator {
                 break;
 
             case DataType.GENERAL:
-                writeDiscreteTraitsSubstModel(model, writer);
-                //****************** Site Model *****************
-                writeDiscreteTraitsSiteModel(model, writer);
+            case DataType.CONTINUOUS:
+                //handled by component
+                break;
+
+            case DataType.MICRO_SAT:
+                writeMicrosatSubstModel(model, writer);
                 break;
 
             default:
@@ -264,7 +294,7 @@ public class SubstitutionModelGenerator extends Generator {
                 }
         );
 
-        writeAlignmentRefInFrequencies(writer, model, prefix);
+        writeAlignmentRefInFrequencies(writer, model, prefix, num);
 
         writer.writeOpenTag(FrequencyModelParser.FREQUENCIES);
         switch (model.getFrequencyPolicy()) {
@@ -274,7 +304,7 @@ public class SubstitutionModelGenerator extends Generator {
                     writer.writeTag(ParameterParser.PARAMETER, new Attribute[]{
                             new Attribute.Default<String>(XMLParser.ID, prefix + "frequencies"),
                             new Attribute.Default<String>(ParameterParser.VALUE, "0.25 0.25 0.25 0.25")}, true);
-                } else { // multiple partitions but linked frequency                    
+                } else { // multiple partitions but linked frequency
                     if (num == 1) {
                         writer.writeTag(ParameterParser.PARAMETER, new Attribute[]{
                                 new Attribute.Default<String>(XMLParser.ID, model.getPrefix() + "frequencies"),
@@ -288,7 +318,7 @@ public class SubstitutionModelGenerator extends Generator {
             case EMPIRICAL:
                 if (num == -1 || model.isUnlinkedFrequencyModel()) { // single partition, or multiple partitions unlinked frequency
                     writeParameter(prefix + "frequencies", 4, Double.NaN, Double.NaN, Double.NaN, writer);
-                } else { // multiple partitions but linked frequency                    
+                } else { // multiple partitions but linked frequency
                     if (num == 1) {
                         writeParameter(model.getPrefix() + "frequencies", 4, Double.NaN, Double.NaN, Double.NaN, writer);
                     } else {
@@ -303,15 +333,16 @@ public class SubstitutionModelGenerator extends Generator {
 
     // adding mergePatterns or alignment ref for EMPIRICAL
 
-    private void writeAlignmentRefInFrequencies(XMLWriter writer, PartitionSubstitutionModel model, String prefix) {
+    private void writeAlignmentRefInFrequencies(XMLWriter writer, PartitionSubstitutionModel model, String prefix, int num) {
         if (model.getFrequencyPolicy() == FrequencyPolicyType.EMPIRICAL) {
             if (model.getDataType() == Nucleotides.INSTANCE && model.getCodonPartitionCount() > 1 && model.isUnlinkedSubstitutionModel()) {
-                for (PartitionData partition : options.getAllPartitionData(model)) { //?
-                    writer.writeIDref(MergePatternsParser.MERGE_PATTERNS, prefix + partition.getPrefix() + SitePatternsParser.PATTERNS);
+                for (AbstractPartitionData partition : options.getDataPartitions(model)) { //?
+                    if (num >= 0)
+                        writeCodonPatternsRef(prefix + partition.getPrefix(), num, model.getCodonPartitionCount(), writer);
                 }
             } else {
-                for (PartitionData partition : options.getAllPartitionData(model)) { //?
-                    writer.writeIDref(AlignmentParser.ALIGNMENT, partition.getAlignment().getId());
+                for (AbstractPartitionData partition : options.getDataPartitions(model)) { //?
+                    writer.writeIDref(AlignmentParser.ALIGNMENT, partition.getTaxonList().getId());
                 }
             }
         }
@@ -341,7 +372,7 @@ public class SubstitutionModelGenerator extends Generator {
                 }
         );
 
-        writeAlignmentRefInFrequencies(writer, model, prefix);
+        writeAlignmentRefInFrequencies(writer, model, prefix, -1);
 
         writeFrequencyModelBinary(writer, model, prefix);
 
@@ -370,13 +401,13 @@ public class SubstitutionModelGenerator extends Generator {
         // merge patterns then get frequencies.
 
         if (model.getFrequencyPolicy() == FrequencyPolicyType.EMPIRICAL) {
-            List<PartitionData> partitions = options.getAllPartitionData(model);
-
-            Patterns patterns = new Patterns(partitions.get(0).getAlignment());
-            for (int i = 1; i < partitions.size(); i++) {
-                patterns.addPatterns(partitions.get(i).getAlignment());
-            }
-            double[] frequencies = patterns.getStateFrequencies();
+            List<AbstractPartitionData> partitions = options.getDataPartitions(model);
+            Alignment alignment = ((PartitionData) partitions.get(0)).getAlignment();
+//            Patterns patterns = new Patterns(partitions.get(0).getAlignment());
+//            for (int i = 1; i < partitions.size(); i++) {
+//                patterns.addPatterns(partitions.get(i).getAlignment());
+//            }
+            double[] frequencies = alignment.getStateFrequencies();
             writer.writeOpenTag(FrequencyModelParser.FREQUENCIES);
             writer.writeTag(ParameterParser.PARAMETER, new Attribute[]{
                     new Attribute.Default<String>(XMLParser.ID, prefix + "frequencies"),
@@ -416,177 +447,6 @@ public class SubstitutionModelGenerator extends Generator {
         writer.writeCloseTag(FrequencyModelParser.FREQUENCIES);
     }
 
-    /**
-     * Discrete Traits Subst Model
-     *
-     * @param model  PartitionSubstitutionModel
-     * @param writer XMLWriter
-     */
-    private void writeDiscreteTraitsSubstModel(PartitionSubstitutionModel model, XMLWriter writer) {
-        int numOfStates = TraitData.getStatesListOfTrait(options.taxonList, options.getAllPartitionData(model).get(0).getName()).size();
-
-        if (numOfStates < 1) throw new IllegalArgumentException("The number of states must be greater than 1");
-
-        for (PartitionData partition : options.getAllPartitionData(model)) {
-            if (numOfStates != TraitData.getStatesListOfTrait(options.taxonList, partition.getName()).size()) {
-                throw new IllegalArgumentException("Discrete Traits having different number of states " +
-                        "\n" + "cannot share the same substitution model");
-            }
-        }
-
-        if (model.getLocationSubstType() == LocationSubstModelType.SYM_SUBST) {
-            writer.writeComment("symmetric CTMC model for discrete state reconstructions");
-
-            writer.writeOpenTag(GeneralSubstitutionModelParser.GENERAL_SUBSTITUTION_MODEL, new Attribute[]{
-                    new Attribute.Default<String>(XMLParser.ID, model.getPrefix() + AbstractSubstitutionModel.MODEL)});
-
-            for (PartitionData partition : options.getAllPartitionData(model)) { //?
-                writer.writeIDref(GeneralDataTypeParser.GENERAL_DATA_TYPE, partition.getPrefix() + GeneralTraitGenerator.DATA);
-            }
-
-            writer.writeOpenTag(GeneralSubstitutionModelParser.FREQUENCIES);
-
-            writeFrequencyModel(model, numOfStates, true, writer);
-
-            writer.writeCloseTag(GeneralSubstitutionModelParser.FREQUENCIES);
-
-            //---------------- rates and indicators -----------------
-
-            // AR - we are trying to unify this setup. The only difference between BSSVS and not is the presence of indicators...
-//            if (!model.isActivateBSSVS()) {
-//                writer.writeComment("Rates parameter in " + GeneralSubstitutionModelParser.GENERAL_SUBSTITUTION_MODEL
-//                        + " element should have (" + (numOfStates * (numOfStates - 1) / 2) + " - 1) dimensions");
-//                writeRatesAndIndicators(model, (numOfStates * (numOfStates - 1) / 2) - 1, null, writer);
-//            } else {
-//                writeRatesAndIndicators(model, numOfStates * (numOfStates - 1) / 2, null, writer);
-//            }
-            writeRatesAndIndicators(model, numOfStates * (numOfStates - 1) / 2, null, writer);
-            writer.writeCloseTag(GeneralSubstitutionModelParser.GENERAL_SUBSTITUTION_MODEL);
-
-        } else if (model.getLocationSubstType() == LocationSubstModelType.ASYM_SUBST) {
-            writer.writeComment("asymmetric CTMC model for discrete state reconstructions");
-
-            writer.writeOpenTag(GeneralSubstitutionModelParser.GENERAL_SUBSTITUTION_MODEL, new Attribute[]{
-                    new Attribute.Default<String>(XMLParser.ID, model.getPrefix() + AbstractSubstitutionModel.MODEL),
-                    new Attribute.Default<Boolean>(ComplexSubstitutionModelParser.RANDOMIZE, false)});
-
-            for (PartitionData partition : options.getAllPartitionData(model)) { //?
-                writer.writeIDref(GeneralDataTypeParser.GENERAL_DATA_TYPE, partition.getPrefix() + GeneralTraitGenerator.DATA);
-            }
-
-            writer.writeOpenTag(GeneralSubstitutionModelParser.FREQUENCIES);
-
-            writeFrequencyModel(model, numOfStates, true, writer);
-
-            writer.writeCloseTag(GeneralSubstitutionModelParser.FREQUENCIES);
-
-            //---------------- rates and indicators -----------------
-            writeRatesAndIndicators(model, numOfStates * (numOfStates - 1), null, writer);
-
-            writer.writeCloseTag(GeneralSubstitutionModelParser.GENERAL_SUBSTITUTION_MODEL);
-
-        } else {
-
-        }
-
-        if (model.isActivateBSSVS()) // If "BSSVS" is not activated, rateIndicator should not be there.
-            writeStatisticModel(model, writer);
-    }
-
-    private void writeFrequencyModel(PartitionSubstitutionModel model, int numOfSates, Boolean normalize, XMLWriter writer) {
-        if (normalize == null) {
-            writer.writeOpenTag(FrequencyModelParser.FREQUENCY_MODEL, new Attribute[]{
-                    new Attribute.Default<String>(XMLParser.ID, model.getPrefix() + FrequencyModelParser.FREQUENCY_MODEL)});
-        } else {
-            writer.writeOpenTag(FrequencyModelParser.FREQUENCY_MODEL, new Attribute[]{
-                    new Attribute.Default<String>(XMLParser.ID, model.getPrefix() + FrequencyModelParser.FREQUENCY_MODEL),
-                    new Attribute.Default<Boolean>(FrequencyModelParser.NORMALIZE, normalize)});
-        }
-
-        for (PartitionData partition : options.getAllPartitionData(model)) { //?
-            writer.writeIDref(GeneralDataTypeParser.GENERAL_DATA_TYPE, partition.getPrefix() + GeneralTraitGenerator.DATA);
-        }
-
-        writer.writeOpenTag(FrequencyModelParser.FREQUENCIES);
-        writeParameter(model.getPrefix() + "trait.frequencies", numOfSates, Double.NaN, Double.NaN, Double.NaN, writer);
-        writer.writeCloseTag(FrequencyModelParser.FREQUENCIES);
-
-        writer.writeCloseTag(FrequencyModelParser.FREQUENCY_MODEL);
-    }
-
-    private void writeRatesAndIndicators(PartitionSubstitutionModel model, int dimension, Integer relativeTo, XMLWriter writer) {
-        writer.writeComment("rates and indicators");
-
-        if (relativeTo == null) {
-            writer.writeOpenTag(GeneralSubstitutionModelParser.RATES);
-        } else {
-            writer.writeOpenTag(GeneralSubstitutionModelParser.RATES, new Attribute[]{
-                    new Attribute.Default<Integer>(GeneralSubstitutionModelParser.RELATIVE_TO, relativeTo)});
-        }
-
-        model.getParameter("trait.rates").isFixed = true;
-        writeParameter(model.getParameter("trait.rates"), dimension, writer);
-
-        writer.writeCloseTag(GeneralSubstitutionModelParser.RATES);
-
-        if (model.isActivateBSSVS()) { //If "BSSVS" is not activated, rateIndicator should not be there.
-            writer.writeOpenTag(GeneralSubstitutionModelParser.INDICATOR);
-            model.getParameter("trait.indicators").isFixed = true;
-            writeParameter(model.getParameter("trait.indicators"), dimension, writer);
-            writer.writeCloseTag(GeneralSubstitutionModelParser.INDICATOR);
-        }
-
-    }
-
-    private void writeStatisticModel(PartitionSubstitutionModel model, XMLWriter writer) {
-        writer.writeOpenTag(SumStatisticParser.SUM_STATISTIC, new Attribute[]{
-                new Attribute.Default<String>(XMLParser.ID, model.getPrefix() + "trait.nonZeroRates"),
-                new Attribute.Default<Boolean>(SumStatisticParser.ELEMENTWISE, true)});
-        writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + "trait.indicators");
-        writer.writeCloseTag(SumStatisticParser.SUM_STATISTIC);
-
-        writer.writeOpenTag(ProductStatisticParser.PRODUCT_STATISTIC, new Attribute[]{
-                new Attribute.Default<String>(XMLParser.ID, model.getPrefix() + "actualRates"),
-                new Attribute.Default<Boolean>(SumStatisticParser.ELEMENTWISE, false)});
-        writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + "trait.indicators");
-        writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + "trait.rates");
-        writer.writeCloseTag(ProductStatisticParser.PRODUCT_STATISTIC);
-    }
-
-    /**
-     * Write the site model XML block.
-     *
-     * @param model            the partition model to write in BEAST XML
-     * @param writer           the writer
-     */
-//    public void writeSiteModel(PartitionSubstitutionModel model, XMLWriter writer) {
-//
-//        switch (model.getDataType().getType()) {
-//            case DataType.NUCLEOTIDES:
-//                if (model.getCodonPartitionCount() > 1) { //model.getCodonHeteroPattern() != null) {
-//                    for (int i = 1; i <= model.getCodonPartitionCount(); i++) {
-//                        writeNucSiteModel(i, writer, model);
-//                    }
-//                    writer.println();
-//                } else {
-//                    writeNucSiteModel(-1, writer, model);
-//                }
-//                break;
-//
-//            case DataType.AMINO_ACIDS:
-//                writeAASiteModel(writer, model);
-//                break;
-//
-//            case DataType.TWO_STATES:
-//            case DataType.COVARION:
-//                writeTwoStateSiteModel(writer, model);
-//                break;
-//
-//            default:
-//                throw new IllegalArgumentException("Unknown data type");
-//        }
-//    }
-
     /**
      * Write the allMus for each partition model.
      *
@@ -623,7 +483,7 @@ public class SubstitutionModelGenerator extends Generator {
 
     }
 
-    public void writeLog(XMLWriter writer, PartitionSubstitutionModel model) {
+    public void writeLog(PartitionSubstitutionModel model, XMLWriter writer) {
 
         int codonPartitionCount = model.getCodonPartitionCount();
 
@@ -695,19 +555,25 @@ public class SubstitutionModelGenerator extends Generator {
                 String prefix = model.getPrefix();
                 switch (model.getBinarySubstitutionModel()) {
                     case BIN_SIMPLE:
+                    case BIN_DOLLO:
                         break;
                     case BIN_COVARION:
-                        writer.writeIDref(ParameterParser.PARAMETER, prefix + "bcov.alpha");
-                        writer.writeIDref(ParameterParser.PARAMETER, prefix + "bcov.s");
-                        writer.writeIDref(ParameterParser.PARAMETER, prefix + "frequencies");
-                        writer.writeIDref(ParameterParser.PARAMETER, prefix + "hfrequencies");
+                        writeParameterRef(prefix + "bcov.alpha", writer);
+                        writeParameterRef(prefix + "bcov.s", writer);
+                        writeParameterRef(prefix + "frequencies", writer);
+                        writeParameterRef(prefix + "hfrequencies", writer);
                         break;
 
                 }
                 break;//BINARY
 
             case DataType.GENERAL:
-                //TODO
+            case DataType.CONTINUOUS:
+                // these datatypes are handled by components
+                break;
+
+            case DataType.MICRO_SAT:
+                writeMicrosatSubstModelParameterRef(model, writer);
                 break;
 
             default:
@@ -735,28 +601,29 @@ public class SubstitutionModelGenerator extends Generator {
         }
     }
 
-    public void writeRateLog(PartitionSubstitutionModel model, XMLWriter writer) {
-        writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + "trait.rates");
+    public void writeMicrosatSubstModelParameterRef(PartitionSubstitutionModel model, XMLWriter writer) {
+        if (model.getRatePorportion() == MicroSatModelType.RateProportionality.EQUAL_RATE) {
+
+        } else if (model.getRatePorportion() == MicroSatModelType.RateProportionality.PROPORTIONAL_RATE) {
+            writeParameterRef(model.getPrefix() + "propLinear", writer);
+        } else if (model.getRatePorportion() == MicroSatModelType.RateProportionality.ASYM_QUAD) {
 
-        if (model.isActivateBSSVS()) { //If "BSSVS" is not activated, rateIndicator should not be there.
-            writer.writeIDref(ParameterParser.PARAMETER, model.getPrefix() + "trait.indicators");
-            writer.writeIDref(SumStatisticParser.SUM_STATISTIC, model.getPrefix() + "trait.nonZeroRates");
         }
-    }
+        if (model.getMutationBias() == MicroSatModelType.MutationalBias.UNBIASED) {
 
-    public void writeStatisticLog(PartitionSubstitutionModel model, XMLWriter writer) {
-        if (model.isActivateBSSVS()) { //If "BSSVS" is not activated, rateIndicator should not be there.
-            writer.writeOpenTag(ColumnsParser.COLUMN,
-                    new Attribute[]{
-                            new Attribute.Default<String>(ColumnsParser.LABEL, model.getPrefix() + "nonZeroRates"),
-                            new Attribute.Default<String>(ColumnsParser.SIGNIFICANT_FIGURES, "6"),
-                            new Attribute.Default<String>(ColumnsParser.WIDTH, "12")
-                    }
-            );
+        } else if (model.getMutationBias() == MicroSatModelType.MutationalBias.CONSTANT_BIAS) {
+            writeParameterRef(model.getPrefix() + "biasConst", writer);
+        } else if (model.getMutationBias() == MicroSatModelType.MutationalBias.LINEAR_BIAS) {
+            writeParameterRef(model.getPrefix() + "biasConst", writer);
+            writeParameterRef(model.getPrefix() + "biasLinear", writer);
+        }
+        if (model.getPhase() == MicroSatModelType.Phase.ONE_PHASE) {
 
-            writer.writeIDref(SumStatisticParser.SUM_STATISTIC, model.getPrefix() + "trait.nonZeroRates");
-
-            writer.writeCloseTag(ColumnsParser.COLUMN);
+        } else if (model.getPhase() == MicroSatModelType.Phase.TWO_PHASE) {
+            writeParameterRef(model.getPrefix() + "geomDist", writer);
+        } else if (model.getPhase() == MicroSatModelType.Phase.TWO_PHASE_STAR) {
+            writeParameterRef(model.getPrefix() + "geomDist", writer);
+            writeParameterRef(model.getPrefix() + "onePhaseProb", writer);
         }
     }
 
@@ -888,6 +755,8 @@ public class SubstitutionModelGenerator extends Generator {
             case BIN_COVARION:
                 writer.writeIDref(BinaryCovarionModelParser.COVARION_MODEL, prefix + "bcov");
                 break;
+            case BIN_DOLLO:
+                break; // Handled by component
             default:
                 throw new IllegalArgumentException("Unknown substitution model.");
         }
@@ -951,18 +820,75 @@ public class SubstitutionModelGenerator extends Generator {
         writer.writeCloseTag(GammaSiteModel.SITE_MODEL);
     }
 
-    private void writeDiscreteTraitsSiteModel(PartitionSubstitutionModel model, XMLWriter writer) {
-        writer.writeOpenTag(SiteModel.SITE_MODEL, new Attribute[]{
-                new Attribute.Default<String>(XMLParser.ID, model.getPrefix() + SiteModel.SITE_MODEL)});
+    private void writeMicrosatSubstModel(PartitionSubstitutionModel model, XMLWriter writer) {
 
-        writer.writeOpenTag(GammaSiteModelParser.SUBSTITUTION_MODEL);
-        writer.writeIDref(GeneralTraitGenerator.getLocationSubstModelTag(model), model.getPrefix() + AbstractSubstitutionModel.MODEL);
-        writer.writeCloseTag(GammaSiteModelParser.SUBSTITUTION_MODEL);
+        writer.writeOpenTag(AsymmetricQuadraticModel.ASYMQUAD_MODEL, new Attribute[]{
+                new Attribute.Default<String>(XMLParser.ID, model.getPrefix() + AsymmetricQuadraticModel.ASYMQUAD_MODEL),
+                new Attribute.Default<Boolean>(AsymQuadModelParser.IS_SUBMODEL,
+                        !(model.getMutationBias() == MicroSatModelType.MutationalBias.UNBIASED
+                                && model.getPhase() == MicroSatModelType.Phase.ONE_PHASE)), // ?U1 is false
+        });
+        writer.writeIDref(MicrosatelliteParser.MICROSAT, model.getMicrosatellite().getName());
 
-//        writer.writeOpenTag(GammaSiteModelParser.MUTATION_RATE);
-//        writeParameter(model.getParameter("trait.mu"), -1, writer);
-//        writer.writeCloseTag(GammaSiteModelParser.MUTATION_RATE);
+//        if (model.getRatePorportion() == MicroSatModelType.RateProportionality.EQUAL_RATE) {
+//            // no xml
+//        } else
+        if (model.getRatePorportion() == MicroSatModelType.RateProportionality.PROPORTIONAL_RATE) {
+            writeParameter(AsymQuadModelParser.EXPANSION_LIN, "propLinear", model, writer);
+            writeParameterRef(AsymQuadModelParser.CONTRACTION_LIN, model.getPrefix() + "propLinear", writer);
+        } else if (model.getRatePorportion() == MicroSatModelType.RateProportionality.ASYM_QUAD) {
 
-        writer.writeCloseTag(SiteModel.SITE_MODEL);
+        }
+        writer.writeCloseTag(AsymmetricQuadraticModel.ASYMQUAD_MODEL);
+
+        //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+        if (model.getMutationBias() != MicroSatModelType.MutationalBias.UNBIASED) {
+            writer.writeOpenTag(LinearBiasModel.LINEAR_BIAS_MODEL, new Attribute[]{
+                    new Attribute.Default<String>(XMLParser.ID, model.getPrefix() + LinearBiasModel.LINEAR_BIAS_MODEL),
+                    new Attribute.Default<Boolean>(LinearBiasModelParser.LOGISTICS, true),
+                    new Attribute.Default<Boolean>(LinearBiasModelParser.ESTIMATE_SUBMODEL_PARAMS,
+                            model.getMutationBias() == MicroSatModelType.MutationalBias.LINEAR_BIAS),
+                    new Attribute.Default<Boolean>(LinearBiasModelParser.IS_SUBMODEL,
+                            model.getPhase() != MicroSatModelType.Phase.ONE_PHASE),
+            });
+            writer.writeIDref(MicrosatelliteParser.MICROSAT, model.getMicrosatellite().getName());
+
+//            if (model.getMutationBias() == MicroSatModelType.MutationalBias.CONSTANT_BIAS)
+            writeParameterRef(LinearBiasModelParser.SUBMODEL, model.getPrefix() + AsymmetricQuadraticModel.ASYMQUAD_MODEL, writer);
+
+            writeParameter(LinearBiasModelParser.BIAS_CONSTANT, "biasConst", model, writer);
+
+            if (model.getMutationBias() == MicroSatModelType.MutationalBias.LINEAR_BIAS) {
+                writeParameter(LinearBiasModelParser.BIAS_LINEAR, "biasLinear", model, writer);
+            }
+            writer.writeCloseTag(LinearBiasModel.LINEAR_BIAS_MODEL);
+        }
+
+        //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+        if (model.getPhase() != MicroSatModelType.Phase.ONE_PHASE) {
+            writer.writeOpenTag(TwoPhaseModel.TWO_PHASE_MODEL, new Attribute[]{
+                    new Attribute.Default<String>(XMLParser.ID, model.getPrefix() + TwoPhaseModel.TWO_PHASE_MODEL),
+                    new Attribute.Default<Boolean>(TwoPhaseModelParser.ESTIMATE_SUBMODEL_PARAMS, true),
+            });
+            writer.writeIDref(MicrosatelliteParser.MICROSAT, model.getMicrosatellite().getName());
+
+            if (model.getMutationBias() == MicroSatModelType.MutationalBias.UNBIASED) {
+                writeParameterRef(TwoPhaseModelParser.SUBMODEL, model.getPrefix() + AsymmetricQuadraticModel.ASYMQUAD_MODEL, writer);
+            } else {
+                writeParameterRef(TwoPhaseModelParser.SUBMODEL, model.getPrefix() + LinearBiasModel.LINEAR_BIAS_MODEL, writer);
+            }
+
+            if (model.getPhase() == MicroSatModelType.Phase.TWO_PHASE) {
+                writeParameter(TwoPhaseModelParser.GEO_PARAM, "geomDist", model, writer);
+                writer.writeOpenTag(TwoPhaseModelParser.ONEPHASEPR_PARAM);
+                writeParameter(model.getPrefix() + "onePhaseProb", 1, 0.0, Double.NaN, Double.NaN, writer);
+                writer.writeCloseTag(TwoPhaseModelParser.ONEPHASEPR_PARAM);
+            } else if (model.getPhase() == MicroSatModelType.Phase.TWO_PHASE_STAR) {
+                writeParameter(TwoPhaseModelParser.GEO_PARAM, "geomDist", model, writer);
+                writeParameter(TwoPhaseModelParser.ONEPHASEPR_PARAM, "onePhaseProb", model, writer);
+            }
+            writer.writeCloseTag(TwoPhaseModel.TWO_PHASE_MODEL);
+        }
     }
+
 }
\ No newline at end of file
