@@ -1,5 +1,7 @@
 /*
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * PartitionTreePrior.java
+ *
+ * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -23,9 +25,13 @@
 
 package dr.app.beauti.options;
 
-import dr.app.beauti.enumTypes.*;
+import dr.app.beauti.types.*;
 import dr.evomodel.coalescent.VariableDemographicModel;
+import dr.evomodel.speciation.CalibrationPoints;
+import dr.evomodelxml.speciation.BirthDeathEpidemiologyModelParser;
 import dr.evomodelxml.speciation.BirthDeathModelParser;
+import dr.evomodelxml.speciation.BirthDeathSerialSamplingModelParser;
+import dr.math.MathUtils;
 
 import java.util.List;
 
@@ -36,31 +42,19 @@ import java.util.List;
  */
 public class PartitionTreePrior extends PartitionOptions {
 
-    // Instance variables
-
-    private final BeautiOptions options;
-
-    private PartitionTreeModel treeModel; // only used when not sharing same prior
-
     private TreePriorType nodeHeightPrior = TreePriorType.CONSTANT;
     private TreePriorParameterizationType parameterization = TreePriorParameterizationType.GROWTH_RATE;
     private int skylineGroupCount = 10;
     private TreePriorParameterizationType skylineModel = TreePriorParameterizationType.CONSTANT_SKYLINE;
     private TreePriorParameterizationType skyrideSmoothing = TreePriorParameterizationType.TIME_AWARE_SKYRIDE;
-    // AR - this seems to be set to taxonCount - 1 so we don't need to
-    // have a settable variable...
-    // public int skyrideIntervalCount = 1;
     private VariableDemographicModel.Type extendedSkylineModel = VariableDemographicModel.Type.LINEAR;
     private double birthDeathSamplingProportion = 1.0;
     private PopulationSizeModelType populationSizeModel = PopulationSizeModelType.CONTINUOUS_CONSTANT;
+    private CalibrationPoints.CorrectionType calibCorrectionType = CalibrationPoints.CorrectionType.EXACT;
     private boolean fixedTree = false;
 
     public PartitionTreePrior(BeautiOptions options, PartitionTreeModel treeModel) {
-        this.options = options;
-        this.partitionName = treeModel.getName();
-        this.treeModel = treeModel;
-
-        initTreePriorParaAndOpers();
+        super(options, treeModel.getName());
     }
 
     /**
@@ -71,76 +65,115 @@ public class PartitionTreePrior extends PartitionOptions {
      * @param source  the source model
      */
     public PartitionTreePrior(BeautiOptions options, String name, PartitionTreePrior source) {
-        this.options = options;
-        this.partitionName = name;
-        this.treeModel = source.treeModel;
+        super(options, name);
 
         this.nodeHeightPrior = source.nodeHeightPrior;
         this.parameterization = source.parameterization;
         this.skylineGroupCount = source.skylineGroupCount;
         this.skylineModel = source.skylineModel;
         this.skyrideSmoothing = source.skyrideSmoothing;
+        this.extendedSkylineModel = source.extendedSkylineModel;
         this.birthDeathSamplingProportion = source.birthDeathSamplingProportion;
+        this.populationSizeModel = source.populationSizeModel;
+        this.calibCorrectionType = source.calibCorrectionType;
         this.fixedTree = source.fixedTree;
-
-        initTreePriorParaAndOpers();
     }
 
-    private void initTreePriorParaAndOpers() {
-       
-        createParameterJeffreysPrior("constant.popSize", "coalescent population size parameter",
-                PriorScaleType.TIME_SCALE, 1.0, 0.0, Double.POSITIVE_INFINITY);
+    protected void initModelParametersAndOpererators() {
 
-        createParameterJeffreysPrior("exponential.popSize", "coalescent population size parameter",
-                PriorScaleType.TIME_SCALE, 1.0, 0.0, Double.POSITIVE_INFINITY);
+        createParameterOneOverXPrior("constant.popSize", "coalescent population size parameter",
+                PriorScaleType.TIME_SCALE, 1.0);
+
+        createParameterOneOverXPrior("exponential.popSize", "coalescent population size parameter",
+                PriorScaleType.TIME_SCALE, 1.0);
         createParameterLaplacePrior("exponential.growthRate", "coalescent growth rate parameter",
-                PriorScaleType.GROWTH_RATE_SCALE, 0.0, 0.0, 1.0, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY);
+                PriorScaleType.GROWTH_RATE_SCALE, 0.0, 0.0, 1.0);
         createParameterGammaPrior("exponential.doublingTime", "coalescent doubling time parameter",
-                PriorScaleType.NONE, 100.0, 0.001, 1000, 0.0, Double.POSITIVE_INFINITY, true);
+                PriorScaleType.NONE, 100.0, 0.001, 1000, true);
 
-        createParameterJeffreysPrior("logistic.popSize", "coalescent population size parameter",
-                PriorScaleType.TIME_SCALE, 1.0, 0.0, Double.POSITIVE_INFINITY);
+        createParameterOneOverXPrior("logistic.popSize", "coalescent population size parameter",
+                PriorScaleType.TIME_SCALE, 1.0);
         createParameterLaplacePrior("logistic.growthRate", "coalescent logistic growth rate parameter",
-                PriorScaleType.GROWTH_RATE_SCALE, 0.0, 0.0, 1.0, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY);
+                PriorScaleType.GROWTH_RATE_SCALE, 0.0, 0.0, 1.0);
         createParameterGammaPrior("logistic.doublingTime", "coalescent doubling time parameter",
-                PriorScaleType.NONE, 100.0, 0.001, 1000, 0.0, Double.POSITIVE_INFINITY, true);
+                PriorScaleType.NONE, 100.0, 0.001, 1000, true);
         createParameterGammaPrior("logistic.t50", "logistic shape parameter",
-                PriorScaleType.NONE, 1.0, 0.001, 1000, 0.0, Double.POSITIVE_INFINITY, true);
+                PriorScaleType.NONE, 1.0, 0.001, 1000, true);
 
-        createParameterJeffreysPrior("expansion.popSize", "coalescent population size parameter",
-                PriorScaleType.TIME_SCALE, 1.0, 0.0, Double.POSITIVE_INFINITY);
+        createParameterOneOverXPrior("expansion.popSize", "coalescent population size parameter",
+                PriorScaleType.TIME_SCALE, 1.0);
         createParameterLaplacePrior("expansion.growthRate", "coalescent expansion growth rate parameter",
-                PriorScaleType.GROWTH_RATE_SCALE, 0.0, 0.0, 1.0, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY);
+                PriorScaleType.GROWTH_RATE_SCALE, 0.0, 0.0, 1.0);
         createParameterGammaPrior("expansion.doublingTime", "coalescent doubling time parameter",
-                PriorScaleType.NONE, 100.0, 0.001, 1000, 0.0, Double.POSITIVE_INFINITY, true);
-        createParameterUniformPrior("expansion.ancestralProportion", "ancestral population proportion",
-                PriorScaleType.NONE, 0.1, 0.0, 0.1, 0.0, 1.0);
+                PriorScaleType.NONE, 100.0, 0.001, 1000, true);
+        createZeroOneParameterUniformPrior("expansion.ancestralProportion", "ancestral population proportion", 0.1);
 
-        createParameterUniformPrior("skyline.popSize", "Bayesian Skyline population sizes",
-                PriorScaleType.TIME_SCALE, 1.0, 0.0, Double.POSITIVE_INFINITY, 0.0, Double.POSITIVE_INFINITY);
+        createNonNegativeParameterUniformPrior("skyline.popSize", "Bayesian Skyline population sizes",
+                PriorScaleType.TIME_SCALE, 1.0, 0.0, Parameter.UNIFORM_MAX_BOUND);
         createParameter("skyline.groupSize", "Bayesian Skyline group sizes");
         // skyride.logPopSize is log unit unlike other popSize
         createParameterUniformPrior("skyride.logPopSize", "GMRF Bayesian skyride population sizes (log unit)",
-                PriorScaleType.TIME_SCALE, 1.0, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY);
+                PriorScaleType.LOG_TIME_SCALE, 1.0, -Parameter.UNIFORM_MAX_BOUND, Parameter.UNIFORM_MAX_BOUND);
         createParameter("skyride.groupSize", "GMRF Bayesian skyride group sizes (for backward compatibility)");
         createParameterGammaPrior("skyride.precision", "GMRF Bayesian skyride precision",
-                PriorScaleType.NONE, 1.0, 0.001, 1000, 0.0, Double.POSITIVE_INFINITY, true);
+                PriorScaleType.NONE, 1.0, 0.001, 1000, true);
 
-        createParameterUniformPrior("demographic.popSize", "Extended Bayesian Skyline population sizes",
-                PriorScaleType.TIME_SCALE, 1.0, 0.0, Double.POSITIVE_INFINITY, 0.0, Double.POSITIVE_INFINITY);
+        createNonNegativeParameterUniformPrior("demographic.popSize", "Extended Bayesian Skyline population sizes",
+                PriorScaleType.TIME_SCALE, 1.0, 0.0, Parameter.UNIFORM_MAX_BOUND);
         createParameter("demographic.indicators", "Extended Bayesian Skyline population switch", 0.0);
-        createParameterJeffreysPrior("demographic.populationMean", "Extended Bayesian Skyline population prior mean",
-                PriorScaleType.TIME_SCALE, 1, 0, Double.POSITIVE_INFINITY);
+        createParameterOneOverXPrior("demographic.populationMean", "Extended Bayesian Skyline population prior mean",
+                PriorScaleType.TIME_SCALE, 1);
 
         createDiscreteStatistic("demographic.populationSizeChanges", "Average number of population change points"); // POISSON_PRIOR
 
-        createParameterUniformPrior("yule.birthRate", "Yule speciation process birth rate",
-                PriorScaleType.BIRTH_RATE_SCALE, 1.0, 0.0, Double.POSITIVE_INFINITY, 0.0, Double.POSITIVE_INFINITY);
+        createNonNegativeParameterUniformPrior("yule.birthRate", "Yule speciation process birth rate",
+                PriorScaleType.BIRTH_RATE_SCALE, 1.0, 0.0, Parameter.UNIFORM_MAX_BOUND);
 
-        createParameterUniformPrior(BirthDeathModelParser.MEAN_GROWTH_RATE_PARAM_NAME, "Birth-Death speciation process rate",
-                PriorScaleType.BIRTH_RATE_SCALE, 1.0, 0.0, Double.POSITIVE_INFINITY, 0.0, Double.POSITIVE_INFINITY);
-        createParameterUniformPrior(BirthDeathModelParser.RELATIVE_DEATH_RATE_PARAM_NAME, "Death/Birth speciation process relative death rate",
-                PriorScaleType.BIRTH_RATE_SCALE, 0.5, 0.0, Double.POSITIVE_INFINITY, 0.0, Double.POSITIVE_INFINITY);
+        createNonNegativeParameterUniformPrior(BirthDeathModelParser.MEAN_GROWTH_RATE_PARAM_NAME, "Birth-Death speciation process rate",
+                PriorScaleType.BIRTH_RATE_SCALE, 0.01, 0.0, 100000.0);
+        createNonNegativeParameterUniformPrior(BirthDeathModelParser.RELATIVE_DEATH_RATE_PARAM_NAME, "Birth-Death speciation process relative death rate",
+                PriorScaleType.NONE, 0.5, 0.0, 1.0);
+        createParameterBetaDistributionPrior(BirthDeathModelParser.BIRTH_DEATH + "." + BirthDeathModelParser.SAMPLE_PROB,
+                "Birth-Death the proportion of taxa sampled from birth-death tree",
+                0.01, 1.0, 1.0, 0.0);
+        createNonNegativeParameterUniformPrior(BirthDeathSerialSamplingModelParser.BDSS + "."
+                + BirthDeathSerialSamplingModelParser.LAMBDA,
+                "Birth-Death speciation process rate", PriorScaleType.BIRTH_RATE_SCALE,
+                2.0, 0.0, 100000.0);
+        createZeroOneParameterUniformPrior(BirthDeathSerialSamplingModelParser.BDSS + "."
+                + BirthDeathSerialSamplingModelParser.RELATIVE_MU,
+                "Birth-Death relative death rate", 0.5);
+        //Issue 656
+//        createParameterBetaDistributionPrior(BirthDeathSerialSamplingModelParser.BDSS + "."
+//                + BirthDeathSerialSamplingModelParser.SAMPLE_PROBABILITY,
+//                "Birth-Death the proportion of taxa sampled from birth death tree",
+//                0.01, 1.0, 1.0, 0.0);
+        createNonNegativeParameterUniformPrior(BirthDeathSerialSamplingModelParser.BDSS + "."
+                + BirthDeathSerialSamplingModelParser.PSI,
+                "Birth-Death rate of sampling taxa through time", PriorScaleType.NONE,
+                0.05, 0.0, 100.0);
+        createNonNegativeParameterUniformPrior(BirthDeathSerialSamplingModelParser.BDSS + "."
+                + BirthDeathSerialSamplingModelParser.ORIGIN,
+                "Birth-Death the time of the lineage originated (must > root height)", PriorScaleType.ORIGIN_SCALE,
+                1.0, 0.0, Parameter.UNIFORM_MAX_BOUND);
+//        createParameter(BirthDeathSerialSamplingModelParser.BDSS + "." + BirthDeathSerialSamplingModelParser.R,
+//                "Birth-Death the probabilty that a sampled individual continues being infectious after sample event",
+//                1.0); // fixed to 1
+//        createNonNegativeParameterUniformPrior(BirthDeathSerialSamplingModelParser.BDSS + "."
+//                + BirthDeathSerialSamplingModelParser.HAS_FINAL_SAMPLE,
+//                "Birth-Death the time in the past when the process starts with the first individual", PriorScaleType.NONE,
+//                80.0, 0.0, 1000.0);
+        createNonNegativeParameterUniformPrior(BirthDeathEpidemiologyModelParser.ORIGIN,
+                "The origin of the infection, x0 > tree.rootHeight", PriorScaleType.ORIGIN_SCALE,
+                1.0, 0.0, Parameter.UNIFORM_MAX_BOUND);
+        createParameterLognormalPrior(BirthDeathEpidemiologyModelParser.R0, "R0",
+                PriorScaleType.NONE, 2.0, 1.0, 1.25, 0.0, 0, Double.POSITIVE_INFINITY);
+        createNonNegativeParameterUniformPrior(BirthDeathEpidemiologyModelParser.RECOVERY_RATE,
+                "recoveryRate", PriorScaleType.NONE,
+                0.05, 0.0, 100.0);
+        createParameterBetaDistributionPrior(BirthDeathEpidemiologyModelParser.SAMPLING_PROBABILITY,
+                "samplingProbability",
+                0.01, 1.0, 1.0, 0.0);
 
         createScaleOperator("constant.popSize", demoTuning, demoWeights);
         createScaleOperator("exponential.popSize", demoTuning, demoWeights);
@@ -163,17 +196,39 @@ public class PartitionTreePrior extends PartitionOptions {
 
         // hack pass distribution in name
         createOperatorUsing2Parameters("demographic.popSize", "demographic.populationMeanDist", "", "demographic.popSize",
-        		"demographic.indicators", OperatorType.SAMPLE_NONACTIVE, 1, 5 * demoWeights);
+                "demographic.indicators", OperatorType.SAMPLE_NONACTIVE, 1, 5 * demoWeights);
         createOperatorUsing2Parameters("demographic.scaleActive", "demographic.scaleActive", "", "demographic.popSize",
-        		"demographic.indicators", OperatorType.SCALE_WITH_INDICATORS, 0.5, 2 * demoWeights);
+                "demographic.indicators", OperatorType.SCALE_WITH_INDICATORS, 0.5, 2 * demoWeights);
         createOperatorUsing2Parameters("gmrfGibbsOperator", "gmrfGibbsOperator", "Gibbs sampler for GMRF", "skyride.logPopSize",
-        		"skyride.precision", OperatorType.GMRF_GIBBS_OPERATOR, 2, 2);
+                "skyride.precision", OperatorType.GMRF_GIBBS_OPERATOR, 2, 2);
 
         createScaleOperator("yule.birthRate", demoTuning, demoWeights);
 
         createScaleOperator(BirthDeathModelParser.MEAN_GROWTH_RATE_PARAM_NAME, demoTuning, demoWeights);
         createScaleOperator(BirthDeathModelParser.RELATIVE_DEATH_RATE_PARAM_NAME, demoTuning, demoWeights);
-
+        createScaleOperator(BirthDeathModelParser.BIRTH_DEATH + "." + BirthDeathModelParser.SAMPLE_PROB, demoTuning, demoWeights);
+        createScaleOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+                + BirthDeathSerialSamplingModelParser.LAMBDA, demoTuning, 1);
+        createScaleOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+                + BirthDeathSerialSamplingModelParser.RELATIVE_MU, demoTuning, 1);
+        //Issue 656
+//        createScaleOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+//                + BirthDeathSerialSamplingModelParser.SAMPLE_PROBABILITY, demoTuning, 1);
+        createScaleOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+                + BirthDeathSerialSamplingModelParser.PSI, demoTuning, 1);   // todo random worl op ?
+        createScaleOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+                + BirthDeathSerialSamplingModelParser.ORIGIN, demoTuning, 1);
+//        createScaleOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+//                + BirthDeathSerialSamplingModelParser.R, demoTuning, 1);
+//        createScaleOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+//                + BirthDeathSerialSamplingModelParser.HAS_FINAL_SAMPLE, demoTuning, 1);
+//        createOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+//                + BirthDeathSerialSamplingModelParser.SAMPLE_BECOMES_NON_INFECTIOUS,
+//                OperatorType.RANDOM_WALK, 1.0, demoWeights);
+        createScaleOperator(BirthDeathEpidemiologyModelParser.ORIGIN, demoTuning, 1);
+        createScaleOperator(BirthDeathEpidemiologyModelParser.R0, demoTuning, 1);
+        createScaleOperator(BirthDeathEpidemiologyModelParser.RECOVERY_RATE, demoTuning, 1);
+        createScaleOperator(BirthDeathEpidemiologyModelParser.SAMPLING_PROBABILITY, demoTuning, 1);
     }
 
     /**
@@ -182,6 +237,7 @@ public class PartitionTreePrior extends PartitionOptions {
      * @param params the parameter list
      */
     public void selectParameters(List<Parameter> params) {
+        setAvgRootAndRate();
 
         if (nodeHeightPrior == TreePriorType.CONSTANT) {
             params.add(getParameter("constant.popSize"));
@@ -216,11 +272,34 @@ public class PartitionTreePrior extends PartitionOptions {
         } else if (nodeHeightPrior == TreePriorType.GMRF_SKYRIDE) {
 //            params.add(getParameter("skyride.popSize")); // force user to use GMRF, not allowed to change
             params.add(getParameter("skyride.precision"));
-        } else if (nodeHeightPrior == TreePriorType.YULE) {
+        } else if (nodeHeightPrior == TreePriorType.YULE || nodeHeightPrior == TreePriorType.YULE_CALIBRATION) {
             params.add(getParameter("yule.birthRate"));
-        } else if (nodeHeightPrior == TreePriorType.BIRTH_DEATH) {
+        } else if (nodeHeightPrior == TreePriorType.BIRTH_DEATH || nodeHeightPrior == TreePriorType.BIRTH_DEATH_INCOMPLETE_SAMPLING) {
             params.add(getParameter(BirthDeathModelParser.MEAN_GROWTH_RATE_PARAM_NAME));
             params.add(getParameter(BirthDeathModelParser.RELATIVE_DEATH_RATE_PARAM_NAME));
+            if (nodeHeightPrior == TreePriorType.BIRTH_DEATH_INCOMPLETE_SAMPLING)
+                params.add(getParameter(BirthDeathModelParser.BIRTH_DEATH + "." + BirthDeathModelParser.SAMPLE_PROB));
+
+        } else if (nodeHeightPrior == TreePriorType.BIRTH_DEATH_SERIAL_SAMPLING) {
+            params.add(getParameter(BirthDeathSerialSamplingModelParser.BDSS + "."
+                    + BirthDeathSerialSamplingModelParser.LAMBDA));
+            params.add(getParameter(BirthDeathSerialSamplingModelParser.BDSS + "."
+                    + BirthDeathSerialSamplingModelParser.RELATIVE_MU));
+            Parameter psi = getParameter(BirthDeathSerialSamplingModelParser.BDSS + "."
+                    + BirthDeathSerialSamplingModelParser.PSI);
+            if (options.maximumTipHeight > 0) psi.initial = MathUtils.round(1 / options.maximumTipHeight, 4);
+            params.add(psi);
+            params.add(getParameter(BirthDeathSerialSamplingModelParser.BDSS + "."
+                    + BirthDeathSerialSamplingModelParser.ORIGIN));
+//            params.add(getParameter(BirthDeathSerialSamplingModelParser.BDSS + "."
+//                    + BirthDeathSerialSamplingModelParser.SAMPLE_PROBABILITY));
+
+        } else if (nodeHeightPrior == TreePriorType.BIRTH_DEATH_BASIC_REPRODUCTIVE_NUMBER) {
+            params.add(getParameter(BirthDeathEpidemiologyModelParser.ORIGIN));
+            params.add(getParameter(BirthDeathEpidemiologyModelParser.R0));
+            params.add(getParameter(BirthDeathEpidemiologyModelParser.RECOVERY_RATE));
+            params.add(getParameter(BirthDeathEpidemiologyModelParser.SAMPLING_PROBABILITY));
+
         }
 
     }
@@ -267,11 +346,36 @@ public class PartitionTreePrior extends PartitionOptions {
             ops.add(getOperator("demographic.popSize"));
             ops.add(getOperator("demographic.indicators"));
             ops.add(getOperator("demographic.scaleActive"));
-        } else if (nodeHeightPrior == TreePriorType.YULE) {
+        } else if (nodeHeightPrior == TreePriorType.YULE || nodeHeightPrior == TreePriorType.YULE_CALIBRATION) {
             ops.add(getOperator("yule.birthRate"));
-        } else if (nodeHeightPrior == TreePriorType.BIRTH_DEATH) {
+        } else if (nodeHeightPrior == TreePriorType.BIRTH_DEATH || nodeHeightPrior == TreePriorType.BIRTH_DEATH_INCOMPLETE_SAMPLING) {
             ops.add(getOperator(BirthDeathModelParser.MEAN_GROWTH_RATE_PARAM_NAME));
             ops.add(getOperator(BirthDeathModelParser.RELATIVE_DEATH_RATE_PARAM_NAME));
+            if (nodeHeightPrior == TreePriorType.BIRTH_DEATH_INCOMPLETE_SAMPLING)
+                ops.add(getOperator(BirthDeathModelParser.BIRTH_DEATH + "." + BirthDeathModelParser.SAMPLE_PROB));
+        } else if (nodeHeightPrior == TreePriorType.BIRTH_DEATH_SERIAL_SAMPLING) {
+            ops.add(getOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+                    + BirthDeathSerialSamplingModelParser.LAMBDA));
+            ops.add(getOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+                    + BirthDeathSerialSamplingModelParser.RELATIVE_MU));
+            ops.add(getOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+                    + BirthDeathSerialSamplingModelParser.PSI));
+            ops.add(getOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+                    + BirthDeathSerialSamplingModelParser.ORIGIN));
+//            ops.add(getOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+//                    + BirthDeathSerialSamplingModelParser.SAMPLE_PROBABILITY));
+
+//            if (nodeHeightPrior == TreePriorType.BIRTH_DEATH_BASIC_REPRODUCTIVE_NUMBER) {
+//                ops.add(getOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+//                + BirthDeathSerialSamplingModelParser.R));
+//                ops.add(getOperator(BirthDeathSerialSamplingModelParser.BDSS + "."
+//                + BirthDeathSerialSamplingModelParser.HAS_FINAL_SAMPLE));
+//            }
+        } else if (nodeHeightPrior == TreePriorType.BIRTH_DEATH_BASIC_REPRODUCTIVE_NUMBER) {
+            ops.add(getOperator(BirthDeathEpidemiologyModelParser.ORIGIN));
+            ops.add(getOperator(BirthDeathEpidemiologyModelParser.R0));
+            ops.add(getOperator(BirthDeathEpidemiologyModelParser.RECOVERY_RATE));
+            ops.add(getOperator(BirthDeathEpidemiologyModelParser.SAMPLING_PROBABILITY));
         }
     }
 
@@ -288,14 +392,6 @@ public class PartitionTreePrior extends PartitionOptions {
 
     /////////////////////////////////////////////////////////////////////////
 
-    public PartitionTreeModel getTreeModel() {
-        return treeModel;
-    }
-
-//    public void setTreeModel(PartitionTreeModel treeModel) {
-//        this.treeModel = treeModel;
-//    }
-
     public TreePriorType getNodeHeightPrior() {
         return nodeHeightPrior;
     }
@@ -368,6 +464,14 @@ public class PartitionTreePrior extends PartitionOptions {
         this.populationSizeModel = populationSizeModel;
     }
 
+    public CalibrationPoints.CorrectionType getCalibCorrectionType() {
+        return calibCorrectionType;
+    }
+
+    public void setCalibCorrectionType(CalibrationPoints.CorrectionType calibCorrectionType) {
+        this.calibCorrectionType = calibCorrectionType;
+    }
+
     public BeautiOptions getOptions() {
         return options;
     }
