@@ -27,17 +27,19 @@ package dr.app.beauti.tipdatepanel;
 
 import dr.app.beauti.BeautiFrame;
 import dr.app.beauti.BeautiPanel;
-import dr.app.beauti.components.TipDateSamplingComponentOptions;
-import dr.app.beauti.enumTypes.TipDateSamplingType;
+import dr.app.beauti.components.tipdatesampling.TipDateSamplingComponentOptions;
 import dr.app.beauti.options.BeautiOptions;
+import dr.app.beauti.options.ClockModelGroup;
 import dr.app.beauti.options.DateGuesser;
+import dr.app.beauti.traitspanel.TraitValueDialog;
+import dr.app.beauti.types.TipDateSamplingType;
 import dr.app.beauti.util.PanelUtils;
+import dr.app.gui.table.DateCellEditor;
+import dr.app.gui.table.TableEditorStopper;
+import dr.app.gui.table.TableSorter;
 import dr.evolution.util.*;
 import dr.evoxml.util.DateUnitsType;
-import dr.app.gui.table.*;
 import jam.framework.Exportable;
-import jam.table.HeaderRenderer;
-import dr.app.gui.table.TableEditorStopper;
 import jam.table.TableRenderer;
 
 import javax.swing.*;
@@ -62,10 +64,12 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
      *
      */
     private static final long serialVersionUID = 5283922195494563924L;
+
     JScrollPane scrollPane = new JScrollPane();
     JTable dataTable = null;
     DataTableModel dataTableModel = null;
 
+    SetDatesAction setDatesAction = new SetDatesAction();
     ClearDatesAction clearDatesAction = new ClearDatesAction();
     GuessDatesAction guessDatesAction = new GuessDatesAction();
 
@@ -74,7 +78,8 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
     JComboBox unitsCombo = new JComboBox(EnumSet.range(DateUnitsType.YEARS, DateUnitsType.DAYS).toArray());
     JComboBox directionCombo = new JComboBox(EnumSet.range(DateUnitsType.FORWARDS, DateUnitsType.BACKWARDS).toArray());
 
-    JComboBox tipDateSamplingCombo = new JComboBox(new TipDateSamplingType[] { TipDateSamplingType.NO_SAMPLING, TipDateSamplingType.SAMPLE_INDIVIDUALLY});
+//    JComboBox tipDateSamplingCombo = new JComboBox( TipDateSamplingType.values() );
+    JComboBox tipDateSamplingCombo = new JComboBox(new TipDateSamplingType[] { TipDateSamplingType.NO_SAMPLING, TipDateSamplingType.SAMPLE_INDIVIDUALLY });
     JComboBox tipDateTaxonSetCombo = new JComboBox();
 
     BeautiFrame frame = null;
@@ -83,7 +88,8 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
 
     double[] heights = null;
 
-    GuessDatesDialog guessDatesDialog = null;
+    private GuessDatesDialog guessDatesDialog = null;
+    private DateValueDialog dateValueDialog = null;
 
     public TipDatesPanel(BeautiFrame parent) {
 
@@ -96,8 +102,8 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
         sorter.setTableHeader(dataTable.getTableHeader());
 
         dataTable.getTableHeader().setReorderingAllowed(false);
-        dataTable.getTableHeader().setDefaultRenderer(
-                new HeaderRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
+//        dataTable.getTableHeader().setDefaultRenderer(
+//                new HeaderRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
 
         dataTable.getColumnModel().getColumn(0).setCellRenderer(
                 new TableRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
@@ -137,9 +143,15 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
         JButton button = new JButton(guessDatesAction);
         PanelUtils.setupComponent(button);
         toolBar1.add(button);
+
+        button = new JButton(setDatesAction);
+        PanelUtils.setupComponent(button);
+        toolBar1.add(button);
+
         button = new JButton(clearDatesAction);
         PanelUtils.setupComponent(button);
         toolBar1.add(button);
+
         toolBar1.add(new JToolBar.Separator(new Dimension(12, 12)));
         final JLabel unitsLabel = new JLabel("Dates specified as ");
         toolBar1.add(unitsLabel);
@@ -193,6 +205,7 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
 
         clearDatesAction.setEnabled(false);
         guessDatesAction.setEnabled(false);
+        setDatesAction.setEnabled(false);
         directionCombo.setEnabled(false);
         unitsLabel.setEnabled(false);
         unitsCombo.setEnabled(false);
@@ -208,6 +221,7 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
                 boolean enabled = usingTipDates.isSelected();
                 clearDatesAction.setEnabled(enabled);
                 guessDatesAction.setEnabled(enabled);
+                setDatesAction.setEnabled(enabled);
                 unitsLabel.setEnabled(enabled);
                 unitsCombo.setEnabled(enabled);
                 directionCombo.setEnabled(enabled);
@@ -216,8 +230,6 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
                 tipDateSamplingCombo.setEnabled(enabled);
                 tipDateSamplingLabel.setEnabled(enabled);
 
-                frame.removeSpecifiedTreePrior(usingTipDates.isSelected());
-
                 if (options.taxonList != null) timeScaleChanged();
             }
         });
@@ -258,8 +270,10 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
 
         calculateHeights();
 
-        if (options.clockModelOptions.isTipCalibrated()) {
-            options.clockModelOptions.tipTimeCalibration();
+        if (options.clockModelOptions.isTipCalibrated()) { // todo correct?
+            for (ClockModelGroup clockModelGroup : options.clockModelOptions.getClockModelGroups()) {
+                options.clockModelOptions.tipTimeCalibration(clockModelGroup);
+            }
         }
 
         dataTableModel.fireTableDataChanged();
@@ -329,6 +343,55 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
         // nothing to do
     }
 
+    public void setDates() {
+        if (options.taxonList == null) { // validation of check empty taxonList
+            return;
+        }
+
+        int result;
+        do {
+            if (dateValueDialog == null) {
+                dateValueDialog = new DateValueDialog(frame);
+            }
+
+            int[] selRows = dataTable.getSelectedRows();
+
+            if (selRows.length > 0) {
+                dateValueDialog.setDescription("Set date values for selected taxa");
+            } else {
+                dateValueDialog.setDescription("Set date values for all taxa");
+            }
+
+            result = dateValueDialog.showDialog();
+
+            if (result == -1 || result == JOptionPane.CANCEL_OPTION) {
+                return;
+            }
+
+//            currentTrait.guessTrait = true; // ?? no use?
+            String value = dateValueDialog.getDateValue();
+
+            java.util.Date origin = new java.util.Date(0);
+            double d = Double.parseDouble(value);
+            Date date = Date.createTimeSinceOrigin(d, Units.Type.YEARS, origin);
+
+            if (selRows.length > 0) {
+                for (int row : selRows) {
+                    options.taxonList.getTaxon(row).setAttribute("date", date);
+                }
+            } else {
+                for (Taxon taxon : options.taxonList) {
+                    taxon.setAttribute("date", date);
+                }
+            }
+
+            // adjust the dates to the current timescale...
+            timeScaleChanged();
+
+           dataTableModel.fireTableDataChanged();
+        } while (result < 0);
+    }
+
     public void clearDates() {
         for (int i = 0; i < options.taxonList.getTaxonCount(); i++) {
             java.util.Date origin = new java.util.Date(0);
@@ -378,6 +441,22 @@ public class TipDatesPanel extends BeautiPanel implements Exportable {
         dataTableModel.fireTableDataChanged();
     }
 
+    public class SetDatesAction extends AbstractAction {
+        /**
+         *
+         */
+        private static final long serialVersionUID = -7281309694753868635L;
+
+        public SetDatesAction() {
+            super("Set Dates");
+            setToolTipText("Use this tool to set sampling date values from selected taxa");
+        }
+
+        public void actionPerformed(ActionEvent ae) {
+            setDates();
+        }
+    }
+
     public class ClearDatesAction extends AbstractAction {
         /**
          *
