@@ -28,17 +28,17 @@ package dr.app.beauti.traitspanel;
 import dr.app.beauti.BeautiFrame;
 import dr.app.beauti.BeautiPanel;
 import dr.app.beauti.ComboBoxRenderer;
+import dr.app.beauti.datapanel.DataPanel;
 import dr.app.beauti.options.BeautiOptions;
 import dr.app.beauti.options.TraitData;
 import dr.app.beauti.options.TraitGuesser;
 import dr.app.beauti.util.PanelUtils;
-import dr.evolution.util.Taxa;
-import dr.evolution.util.Taxon;
 import dr.app.gui.table.TableEditorStopper;
 import dr.app.gui.table.TableSorter;
+import dr.evolution.util.Taxa;
+import dr.evolution.util.Taxon;
 import jam.framework.Exportable;
 import jam.panels.ActionPanel;
-import jam.table.HeaderRenderer;
 import jam.table.TableRenderer;
 
 import javax.swing.*;
@@ -49,6 +49,7 @@ import javax.swing.table.AbstractTableModel;
 import javax.swing.table.TableColumn;
 import java.awt.*;
 import java.awt.event.ActionEvent;
+import java.util.ArrayList;
 
 /**
  * @author Andrew Rambaut
@@ -56,9 +57,6 @@ import java.awt.event.ActionEvent;
  */
 public class TraitsPanel extends BeautiPanel implements Exportable {
 
-    /**
-     *
-     */
     private static final long serialVersionUID = 5283922195494563924L;
 
     private static final int MINIMUM_TABLE_WIDTH = 140;
@@ -68,9 +66,12 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
             "file. Taxa should be in the first column and the trait names<br>" +
             "in the first row</html>";
     private static final String GUESS_TRAIT_VALUES_TOOLTIP = "<html>This attempts to extract values for this trait that are<br>" +
-            "encoded in the names of the taxa.</html>";
+            "encoded in the names of the selected taxa.</html>";
+    private static final String SET_TRAIT_VALUES_TOOLTIP = "<html>This sets values for this trait for all<br>" +
+            "the selected taxa.</html>";
     private static final String CLEAR_TRAIT_VALUES_TOOLTIP = "<html>This clears all the values currently assigned to taxa for<br>" +
             "this trait.</html>";
+    private static final String CREATE_TRAIT_PARTITIONS_TOOLTIP = "<html>Create a data partition for the selected traits.</html>";
 
     public final JTable traitsTable;
     private final TraitsTableModel traitsTableModel;
@@ -79,19 +80,26 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
     private final DataTableModel dataTableModel;
 
     private final BeautiFrame frame;
+    private final DataPanel dataPanel;
 
     private BeautiOptions options = null;
 
     private TraitData currentTrait = null; // current trait
 
     private CreateTraitDialog createTraitDialog = null;
-//    private GuessTraitDialog guessTraitDialog = null;
+    private GuessTraitDialog guessTraitDialog = null;
+    private TraitValueDialog traitValueDialog = null;
 
     AddTraitAction addTraitAction = new AddTraitAction();
+    Action importTraitsAction;
+    CreateTraitPartitionAction createTraitPartitionAction = new CreateTraitPartitionAction();
+    GuessTraitsAction guessTraitsAction = new GuessTraitsAction();
+    SetValueAction setValueAction = new SetValueAction();
 
-    public TraitsPanel(BeautiFrame parent, Action importTraitsAction) {
+    public TraitsPanel(BeautiFrame parent, DataPanel dataPanel, Action importTraitsAction) {
 
         this.frame = parent;
+        this.dataPanel = dataPanel;
 
         traitsTableModel = new TraitsTableModel();
 //        TableSorter sorter = new TableSorter(traitsTableModel);
@@ -101,8 +109,8 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
 
         traitsTable.getTableHeader().setReorderingAllowed(false);
         traitsTable.getTableHeader().setResizingAllowed(false);
-        traitsTable.getTableHeader().setDefaultRenderer(
-                new HeaderRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
+//        traitsTable.getTableHeader().setDefaultRenderer(
+//                new HeaderRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
 
         TableColumn col = traitsTable.getColumnModel().getColumn(1);
         ComboBoxRenderer comboBoxRenderer = new ComboBoxRenderer(TraitData.TraitType.values());
@@ -111,7 +119,7 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
 
         TableEditorStopper.ensureEditingStopWhenTableLosesFocus(traitsTable);
 
-        traitsTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
+        traitsTable.getSelectionModel().setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
         traitsTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
             public void valueChanged(ListSelectionEvent evt) {
                 traitSelectionChanged();
@@ -130,8 +138,8 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
         sorter.setTableHeader(dataTable.getTableHeader());
 
         dataTable.getTableHeader().setReorderingAllowed(false);
-        dataTable.getTableHeader().setDefaultRenderer(
-                new HeaderRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
+//        dataTable.getTableHeader().setDefaultRenderer(
+//                new HeaderRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
 
         dataTable.getColumnModel().getColumn(0).setCellRenderer(
                 new TableRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
@@ -163,12 +171,12 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
 
         JButton button;
 
-        // removed this button as there is already a "+" button at the bottom of the table.
-//        button = new JButton(addTraitAction);
-//        PanelUtils.setupComponent(button);
-//        button.setToolTipText(ADD_TRAITS_TOOLTIP);
-//        toolBar1.add(button);
+        button = new JButton(addTraitAction);
+        PanelUtils.setupComponent(button);
+        button.setToolTipText(ADD_TRAITS_TOOLTIP);
+        toolBar1.add(button);
 
+        this.importTraitsAction = importTraitsAction;
         button = new JButton(importTraitsAction);
         PanelUtils.setupComponent(button);
         button.setToolTipText(IMPORT_TRAITS_TOOLTIP);
@@ -176,13 +184,24 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
 
         toolBar1.add(new JToolBar.Separator(new Dimension(12, 12)));
 
-        button = new JButton(new GuessTraitsAction());
+        button = new JButton(guessTraitsAction);
         PanelUtils.setupComponent(button);
         button.setToolTipText(GUESS_TRAIT_VALUES_TOOLTIP);
         toolBar1.add(button);
-        button = new JButton(new ClearTraitAction());
+
+        button = new JButton(setValueAction);
         PanelUtils.setupComponent(button);
-        button.setToolTipText(CLEAR_TRAIT_VALUES_TOOLTIP);
+        button.setToolTipText(SET_TRAIT_VALUES_TOOLTIP);
+        toolBar1.add(button);
+
+        // Don't see the need for a clear values button
+//        button = new JButton(new ClearTraitAction());
+//        PanelUtils.setupComponent(button);
+//        button.setToolTipText(CLEAR_TRAIT_VALUES_TOOLTIP);
+//        toolBar1.add(button);
+        button = new JButton(createTraitPartitionAction);
+        PanelUtils.setupComponent(button);
+        button.setToolTipText(CREATE_TRAIT_PARTITIONS_TOOLTIP);
         toolBar1.add(button);
 
         ActionPanel actionPanel1 = new ActionPanel(false);
@@ -225,6 +244,8 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
     public void setOptions(BeautiOptions options) {
         this.options = options;
 
+        updateButtons();
+
 //        int selRow = traitsTable.getSelectedRow();
 //        traitsTableModel.fireTableDataChanged();
 //
@@ -267,6 +288,12 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
                 frame.updateDiscreteTraitAnalysis();
             }
 
+//            if (selRow > 0) {
+//                traitsTable.getSelectionModel().setSelectionInterval(selRow-1, selRow-1);
+//            } else if (selRow == 0 && options.traitsOptions.traits.size() > 0) { // options.traitsOptions.traits.size() after remove
+//                traitsTable.getSelectionModel().setSelectionInterval(0, 0);
+//            }
+
             traitsTableModel.fireTableDataChanged();
             options.updatePartitionAllLinks();
             frame.setDirty();
@@ -276,15 +303,15 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
     private void traitSelectionChanged() {
         int selRow = traitsTable.getSelectedRow();
         if (selRow >= 0) {
-            currentTrait = options.getTraitsList().get(selRow);
-            traitsTable.getSelectionModel().setSelectionInterval(selRow, selRow);
+            currentTrait = options.traits.get(selRow);
+//            traitsTable.getSelectionModel().setSelectionInterval(selRow, selRow);
             removeTraitAction.setEnabled(true);
 //        } else {
 //            currentTrait = null;
 //            removeTraitAction.setEnabled(false);
         }
 
-        if (options.getTraitsList().size() <= 0) {
+        if (options.traits.size() <= 0) {
             currentTrait = null;
             removeTraitAction.setEnabled(false);
         }
@@ -292,6 +319,15 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
 //        traitsTableModel.fireTableDataChanged();
     }
 
+    private void updateButtons() { //TODO: better to merge updateButtons() fireTraitsChanged() traitSelectionChanged() into one
+        boolean hasData = options.hasData();
+
+        addTraitAction.setEnabled(hasData);
+        importTraitsAction.setEnabled(hasData);
+        createTraitPartitionAction.setEnabled(hasData && options.traits.size() > 0);
+        guessTraitsAction.setEnabled(hasData && options.traits.size() > 0);
+        setValueAction.setEnabled(hasData && options.traits.size() > 0);
+    }
 
     public void clearTraitValues(String traitName) {
         options.clearTraitValues(traitName);
@@ -300,71 +336,132 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
     }
 
     public void guessTrait() {
-        if (options.taxonList != null) { // validation of check empty taxonList
-//            TraitGuesser guesser = options.traitsOptions.cureentTraitGuesser;
-            if (currentTrait == null) {
-                if (!addTrait()) {
-                    return; // if addTrait() cancel then false
-                }
-            }
-            int result;
-            do {
-                TraitGuesser currentTraitGuesser = new TraitGuesser(currentTrait);
-                GuessTraitDialog guessTraitDialog = new GuessTraitDialog(frame, currentTraitGuesser);
-                result = guessTraitDialog.showDialog();
+        if (options.taxonList == null) { // validation of check empty taxonList
+            return;
+        }
 
-                if (result == -1 || result == JOptionPane.CANCEL_OPTION) {
-                    return;
+        if (currentTrait == null) {
+            if (!addTrait()) {
+                return; // if addTrait() cancel then false
+            }
+        }
+        int result;
+        do {
+            TraitGuesser currentTraitGuesser = new TraitGuesser(currentTrait);
+            if (guessTraitDialog == null) {
+                guessTraitDialog = new GuessTraitDialog(frame);
+            }
+            guessTraitDialog.setDescription("Extract values for trait '" + currentTrait + "' from taxa labels");
+            result = guessTraitDialog.showDialog();
+
+            if (result == -1 || result == JOptionPane.CANCEL_OPTION) {
+                return;
+            }
+
+            guessTraitDialog.setupGuesserFromDialog(currentTraitGuesser);
+
+            try {
+                int[] selRows = dataTable.getSelectedRows();
+                if (selRows.length > 0) {
+                    Taxa selectedTaxa = new Taxa();
+
+                    for (int row : selRows) {
+                        Taxon taxon = (Taxon) dataTable.getValueAt(row, 0);
+                        selectedTaxa.addTaxon(taxon);
+                    }
+                    currentTraitGuesser.guessTrait(selectedTaxa);
+                } else {
+                    currentTraitGuesser.guessTrait(options.taxonList);
                 }
 
+            } catch (IllegalArgumentException iae) {
+                JOptionPane.showMessageDialog(this, iae.getMessage(), "Unable to guess trait value", JOptionPane.ERROR_MESSAGE);
+                result = -1;
+            }
+
+            dataTableModel.fireTableDataChanged();
+        } while (result < 0);
+    }
+
+    public void setTraitValue() {
+        if (options.taxonList == null) { // validation of check empty taxonList
+            return;
+        }
+
+        int result;
+        do {
+            if (traitValueDialog == null) {
+                traitValueDialog = new TraitValueDialog(frame);
+            }
+
+            int[] selRows = dataTable.getSelectedRows();
+
+            if (selRows.length > 0) {
+                traitValueDialog.setDescription("Set values for trait '" + currentTrait + "' for selected taxa");
+            } else {
+                traitValueDialog.setDescription("Set values for trait '" + currentTrait + "' for all taxa");
+            }
+
+            result = traitValueDialog.showDialog();
+
+            if (result == -1 || result == JOptionPane.CANCEL_OPTION) {
+                return;
+            }
+
 //            currentTrait.guessTrait = true; // ?? no use?
-                guessTraitDialog.setupGuesser();
+            String value = traitValueDialog.getTraitValue();
 
-                try {
-                    int[] selRows = dataTable.getSelectedRows();
-                    if (selRows.length > 0) {
-                        Taxa selectedTaxa = new Taxa();
-
-                        for (int row : selRows) {
-                            Taxon taxon = (Taxon) dataTable.getValueAt(row, 0);
-                            selectedTaxa.addTaxon(taxon);
-                        }
-                        currentTraitGuesser.guessTrait(selectedTaxa);
-                    } else {
-                        currentTraitGuesser.guessTrait(options.taxonList);
+            try {
+                if (selRows.length > 0) {
+                    for (int row : selRows) {
+                        Taxon taxon = (Taxon) dataTable.getValueAt(row, 0);
+                        taxon.setAttribute(currentTrait.getName(), value);
+                    }
+                } else {
+                    for (Taxon taxon : options.taxonList) {
+                        taxon.setAttribute(currentTrait.getName(), value);
                     }
-
-                } catch (IllegalArgumentException iae) {
-                    JOptionPane.showMessageDialog(this, iae.getMessage(), "Unable to guess trait value", JOptionPane.ERROR_MESSAGE);
-                    result = -1;
                 }
 
-                dataTableModel.fireTableDataChanged();
-            } while (result < 0);
-        } else {
-            JOptionPane.showMessageDialog(this, "No taxa loaded yet, please import Alignment file.",
-                    "No taxa loaded", JOptionPane.ERROR_MESSAGE);
-        }
+            } catch (IllegalArgumentException iae) {
+                JOptionPane.showMessageDialog(this, iae.getMessage(), "Unable to guess trait value", JOptionPane.ERROR_MESSAGE);
+                result = -1;
+            }
+
+            dataTableModel.fireTableDataChanged();
+        } while (result < 0);
     }
 
     public boolean addTrait() {
-        boolean isAdd = addTrait("Untitled");
-        // http://code.google.com/p/beast-mcmc/issues/detail?id=388
-        if (options.containTrait(TraitData.TRAIT_SPECIES)) {
-            JOptionPane.showMessageDialog(frame, "Keyword \"species\" has been reserved for *BEAST !" +
-                 "\nPlease use a different trait name.", "Illegal Argument Exception", JOptionPane.ERROR_MESSAGE);
-            options.removeTrait(TraitData.TRAIT_SPECIES);
-//            options.useStarBEAST = false;
-            traitsTableModel.fireTableDataChanged();
-            dataTableModel.fireTableDataChanged();
-            return false;
-        }
+        return addTrait("Untitled");
 
-        return isAdd;
+        // The 'species' trait doesn't need to be a reserved keyword since the StarBEAST button was added
+        // http://code.google.com/p/beast-mcmc/issues/detail?id=388
+//        if (options.traitExists(TraitData.TRAIT_SPECIES)) {
+//            JOptionPane.showMessageDialog(frame, "Keyword \"species\" has been reserved for *BEAST !" +
+//                    "\nPlease use a different trait name.", "Illegal Argument Exception", JOptionPane.ERROR_MESSAGE);
+//            options.removeTrait(TraitData.TRAIT_SPECIES);
+////            options.useStarBEAST = false;
+//            traitsTableModel.fireTableDataChanged();
+//            dataTableModel.fireTableDataChanged();
+//            return false;
+//        }
+//
+//        return isAdd;
     }
 
     public boolean addTrait(String traitName) {
-        createTraitDialog = new CreateTraitDialog(frame, traitName);
+        return addTrait(null, traitName, false);
+    }
+
+    public boolean addTrait(String message, String traitName, boolean isSpeciesTrait) {
+        if (createTraitDialog == null) {
+            createTraitDialog = new CreateTraitDialog(frame);
+        }
+
+        createTraitDialog.setSpeciesTrait(isSpeciesTrait);
+        createTraitDialog.setTraitName(traitName);
+        createTraitDialog.setMessage(message);
 
         int result = createTraitDialog.showDialog();
         if (result == JOptionPane.OK_OPTION) {
@@ -379,7 +476,12 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
             // user is overwriting an existing trait
             addTrait(newTrait);
 
+            if (createTraitDialog.createTraitPartition()) {
+                options.createPartitionForTraits(name, newTrait);
+            }
+
             fireTraitsChanged();
+            updateButtons();
 //            traitsTableModel.fireTableDataChanged();
 //            dataTableModel.fireTableDataChanged();
 
@@ -389,7 +491,24 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
             // called by guessTraits() anyway.
 //            guessTrait();
         } else if (result == CreateTraitDialog.OK_IMPORT) {
-            return frame.doImportTraits();
+            boolean done = frame.doImportTraits();
+            if (done) {
+                if (isSpeciesTrait) {
+                    // check that we did indeed import a 'species' trait
+                    if (!options.traitExists(TraitData.TRAIT_SPECIES)) {
+                        JOptionPane.showMessageDialog(this,
+                                "The imported trait file didn't contain a trait\n" +
+                                        "called '" + TraitData.TRAIT_SPECIES + "', required for *BEAST.\n" +
+                                        "Please edit it or select a different file.",
+                                "Reserved trait name",
+                                JOptionPane.WARNING_MESSAGE);
+
+                        return false;
+                    }
+                }
+                updateButtons();
+            }
+            return done;
         } else if (result == JOptionPane.CANCEL_OPTION) {
             return false;
         }
@@ -402,34 +521,29 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
         traitsTable.getSelectionModel().setSelectionInterval(selRow, selRow);
     }
 
-    public void removeTrait() {
+    private void removeTrait() {
         int selRow = traitsTable.getSelectedRow();
-        options.removeTrait(traitsTable.getValueAt(selRow, 0).toString());
+        removeTrait(traitsTable.getValueAt(selRow, 0).toString());
+    }
 
-        if (currentTrait != null) {
-            clearTraitValues(currentTrait.getName()); // Clear trait values
-//            if (currentTrait.getName().equalsIgnoreCase(TraitData.Traits.TRAIT_SPECIES.toString())) {
-//                frame.removeSepciesAnalysis();
-//            } else
-            if (currentTrait.getTraitType() == TraitData.TraitType.DISCRETE) {
-                frame.updateDiscreteTraitAnalysis();
-            }
-
-//            if (selRow > 0) {
-//                traitsTable.getSelectionModel().setSelectionInterval(selRow-1, selRow-1);
-//            } else if (selRow == 0 && options.traitsOptions.traits.size() > 0) { // options.traitsOptions.traits.size() after remove
-//                traitsTable.getSelectionModel().setSelectionInterval(0, 0);
-//            }
+    public void removeTrait(String traitName) {
+        if (options.useStarBEAST && traitName.equalsIgnoreCase(TraitData.TRAIT_SPECIES)) {
+            JOptionPane.showMessageDialog(this, "The trait named '" + traitName + "' is being used by *BEAST.\nTurn *BEAST off before deleting this trait.", "Trait in use", JOptionPane.ERROR_MESSAGE);
+            return;
         }
+        TraitData traitData = options.getTrait(traitName);
+        if (options.getTraitPartitions(traitData).size() > 0) {
+            JOptionPane.showMessageDialog(this, "The trait named '" + traitName + "' is being used in a partition.\nRemove the partition before deleting this trait.", "Trait in use", JOptionPane.ERROR_MESSAGE);
+            return;
+        }
+        options.removeTrait(traitName);
 
+        updateButtons();
         fireTraitsChanged();
         traitSelectionChanged();
     }
 
     public class ClearTraitAction extends AbstractAction {
-        /**
-         *
-         */
         private static final long serialVersionUID = -7281309694753868635L;
 
         public ClearTraitAction() {
@@ -443,9 +557,6 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
     }
 
     public class GuessTraitsAction extends AbstractAction {
-        /**
-         *
-         */
         private static final long serialVersionUID = 8514706149822252033L;
 
         public GuessTraitsAction() {
@@ -476,6 +587,18 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
         }
     };
 
+    public class SetValueAction extends AbstractAction {
+
+        public SetValueAction() {
+            super("Set trait values");
+            setToolTipText("Use this button to set the trait values of selected taxa");
+        }
+
+        public void actionPerformed(ActionEvent ae) {
+            setTraitValue();
+        }
+    }
+
 
     class TraitsTableModel extends AbstractTableModel {
 
@@ -491,17 +614,15 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
 
         public int getRowCount() {
             if (options == null) return 0;
-            if (options.getTraitsList() == null) return 0;
-
-            return options.getTraitsList().size();
+            return options.traits.size();
         }
 
         public Object getValueAt(int row, int col) {
             switch (col) {
                 case 0:
-                    return options.getTraitsList().get(row).getName();
+                    return options.traits.get(row).getName();
                 case 1:
-                    return options.getTraitsList().get(row).getTraitType();
+                    return options.traits.get(row).getTraitType();
             }
             return null;
         }
@@ -509,19 +630,24 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
         public void setValueAt(Object aValue, int row, int col) {
             switch (col) {
                 case 0:
-                    options.getTraitsList().get(row).setName(aValue.toString());
+                    String oldName = options.traits.get(row).getName();
+                    options.traits.get(row).setName(aValue.toString());
+                    Object value;
+                    for (Taxon t : options.taxonList) {
+                        value = t.getAttribute(oldName);
+                        t.setAttribute(aValue.toString(), value);
+                        // cannot remvoe attribute in Attributable inteface
+                    }
                     fireTraitsChanged();
                     break;
                 case 1:
-                    options.getTraitsList().get(row).setTraitType((TraitData.TraitType) aValue);
+                    options.traits.get(row).setTraitType((TraitData.TraitType) aValue);
                     break;
             }
         }
 
         public boolean isCellEditable(int row, int col) {
-//            return !(options.getTraitsList().get(row).getName().equalsIgnoreCase(TraitData.Traits.TRAIT_SPECIES.toString())
-//                    || options.getTraitsList().get(row).getName().equalsIgnoreCase(TraitData.Traits.TRAIT_LOCATIONS.toString()));
-            return col == 0;
+            return !options.useStarBEAST || !options.traits.get(row).getName().equalsIgnoreCase(TraitData.TRAIT_SPECIES.toString());
         }
 
         public String getColumnName(int column) {
@@ -557,9 +683,6 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
 
     class DataTableModel extends AbstractTableModel {
 
-        /**
-         *
-         */
         private static final long serialVersionUID = -6707994233020715574L;
         String[] columnNames = {"Taxon", "Value"};
 
@@ -649,4 +772,53 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
             return buffer.toString();
         }
     }
+
+    public class CreateTraitPartitionAction extends AbstractAction {
+        public CreateTraitPartitionAction() {
+            super("Create partition from trait ...");
+            setToolTipText("Create a data partition from a trait. Traits can be defined in the Traits panel.");
+        }
+
+        public void actionPerformed(ActionEvent ae) {
+            int[] selRows = traitsTable.getSelectedRows();
+            java.util.List<TraitData> traits = new ArrayList<TraitData>();
+            int discreteCount = 0;
+            int continuousCount = 0;
+            for (int row : selRows) {
+                TraitData trait = options.traits.get(row);
+                traits.add(trait);
+
+                if (trait.getTraitType() == TraitData.TraitType.DISCRETE) {
+                    discreteCount ++;
+                }
+                if (trait.getTraitType() == TraitData.TraitType.CONTINUOUS) {
+                    continuousCount ++;
+                }
+            }
+
+            boolean success = false;
+            if (discreteCount > 0) {
+                if (continuousCount > 0)  {
+                    JOptionPane.showMessageDialog(TraitsPanel.this, "Don't mix discrete and continuous traits when creating partition(s).", "Mixed Trait Types", JOptionPane.ERROR_MESSAGE);
+                    return;
+                }
+
+                // with discrete traits, create a separate partition for each
+                for (TraitData trait : traits) {
+                    java.util.List<TraitData> singleTrait = new ArrayList<TraitData>();
+                    singleTrait.add(trait);
+                    if (dataPanel.createFromTraits(singleTrait)) {
+                        success = true;
+                    }
+                }
+            } else {
+                // with
+                success = dataPanel.createFromTraits(traits);
+            }
+            if (success) {
+                frame.switchToPanel(BeautiFrame.DATA_PARTITIONS);
+            }
+        }
+    }
+
 }
\ No newline at end of file
