@@ -27,15 +27,11 @@ package dr.app.beauti.treespanel;
 
 import dr.app.beauti.BeautiFrame;
 import dr.app.beauti.BeautiPanel;
-import dr.app.beauti.enumTypes.TreePriorType;
-import dr.app.beauti.options.BeautiOptions;
-import dr.app.beauti.options.PartitionData;
-import dr.app.beauti.options.PartitionTreeModel;
-import dr.app.beauti.options.PartitionTreePrior;
+import dr.app.beauti.options.*;
+import dr.app.beauti.types.TreePriorType;
 import dr.app.gui.table.TableEditorStopper;
+import dr.evolution.datatype.Microsatellite;
 import jam.framework.Exportable;
-import jam.panels.OptionsPanel;
-import jam.table.HeaderRenderer;
 import jam.table.TableRenderer;
 
 import javax.swing.*;
@@ -50,6 +46,7 @@ import java.awt.*;
 import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 
 
@@ -71,7 +68,7 @@ public class TreesPanel extends BeautiPanel implements Exportable {
 //    private TreeDisplayPanel treeDisplayPanel;
 
     private BeautiFrame frame = null;
-    private BeautiOptions options = null;
+    public BeautiOptions options = null;
 
     private JTable treesTable = null;
     private TreesTableModel treesTableModel = null;
@@ -83,17 +80,12 @@ public class TreesPanel extends BeautiPanel implements Exportable {
     public JCheckBox linkTreePriorCheck = new JCheckBox("Link tree prior for all trees");
 
     JPanel treeModelPanelParent;
-    //    private OptionsPanel currentTreeModel = new OptionsPanel();
     public PartitionTreeModel currentTreeModel = null;
     TitledBorder treeModelBorder;
     Map<PartitionTreeModel, PartitionTreeModelPanel> treeModelPanels = new HashMap<PartitionTreeModel, PartitionTreeModelPanel>();
-    //  private OptionsPanel treePriorPanel = new OptionsPanel();
     JPanel treePriorPanelParent;
-    //    public PartitionTreePrior currentTreePrior = null;
     TitledBorder treePriorBorder;
-    Map<PartitionTreePrior, OptionsPanel> treePriorPanels = new HashMap<PartitionTreePrior, OptionsPanel>();
-
-    private boolean isCheckedTipDate = false;
+    Map<PartitionTreePrior, PartitionTreePriorPanel> treePriorPanels = new HashMap<PartitionTreePrior, PartitionTreePriorPanel>();
 
     public TreesPanel(BeautiFrame parent, Action removeTreeAction) {
         super();
@@ -104,8 +96,8 @@ public class TreesPanel extends BeautiPanel implements Exportable {
 
         treesTable.getTableHeader().setReorderingAllowed(false);
         treesTable.getTableHeader().setResizingAllowed(false);
-        treesTable.getTableHeader().setDefaultRenderer(
-                new HeaderRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
+//        treesTable.getTableHeader().setDefaultRenderer(
+//                new HeaderRenderer(SwingConstants.LEFT, new Insets(0, 4, 0, 4)));
 
         final TableColumnModel model = treesTable.getColumnModel();
         final TableColumn tableColumn0 = model.getColumn(0);
@@ -144,12 +136,6 @@ public class TreesPanel extends BeautiPanel implements Exportable {
         scrollPane2.setBorder(null);
         scrollPane2.getViewport().setOpaque(false);
 
-        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, panel1, scrollPane2);
-        splitPane.setDividerLocation(180);
-        splitPane.setContinuousLayout(true);
-        splitPane.setBorder(BorderFactory.createEmptyBorder());
-        splitPane.setOpaque(false);
-
         treePriorPanelParent = new JPanel(new FlowLayout(FlowLayout.LEFT));
         treePriorPanelParent.setOpaque(false);
         treePriorBorder = new TitledBorder("Tree Prior");
@@ -157,7 +143,7 @@ public class TreesPanel extends BeautiPanel implements Exportable {
 
         JPanel panel3 = new JPanel(new FlowLayout(FlowLayout.LEFT));
         panel3.setOpaque(false);
-        linkTreePriorCheck.setEnabled(true);
+        linkTreePriorCheck.setEnabled(false);
         linkTreePriorCheck.setSelected(true);
         linkTreePriorCheck.addItemListener(new ItemListener() {
             public void itemStateChanged(ItemEvent ev) {
@@ -169,42 +155,33 @@ public class TreesPanel extends BeautiPanel implements Exportable {
 
         JPanel panel4 = new JPanel(new BorderLayout());
         panel4.setOpaque(false);
-        panel4.add(panel3, BorderLayout.NORTH);
-        panel4.add(treePriorPanelParent, BorderLayout.CENTER);
+        panel4.add(treePriorPanelParent, BorderLayout.NORTH);
+        panel4.add(scrollPane2, BorderLayout.CENTER);
+
+        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, panel1, panel4);
+        splitPane.setDividerLocation(180);
+        splitPane.setContinuousLayout(true);
+        splitPane.setBorder(BorderFactory.createEmptyBorder());
+        splitPane.setOpaque(false);
 
         setOpaque(false);
         setLayout(new BorderLayout(0, 0));
         setBorder(new BorderUIResource.EmptyBorderUIResource(new Insets(12, 12, 12, 12)));
 
-        add(panel4, BorderLayout.NORTH);
+        add(panel3, BorderLayout.NORTH);
         add(splitPane, BorderLayout.CENTER);
     }
 
     public void updateShareSameTreePriorChanged() {
-//    	options.shareSameTreePrior = linkTreePriorCheck.isSelected();
-//    	fireShareSameTreePriorChanged ();
-
         if (linkTreePriorCheck.isSelected()) {
             options.linkTreePriors(currentTreeModel.getPartitionTreePrior());
         } else {
-            options.unLinkTreePriors();
-//    		currentTreePrior = currentTreeModel.getPartitionTreePrior();
+            options.unLinkTreePriors(currentTreeModel);
         }
-//    	setCurrentModelAndPrior(currentTreeModel);
-        updateTreePriorBorder();
-    }
+        setCurrentModelAndPrior(currentTreeModel); // this is important to make panel refreshing
 
-//    private void fireShareSameTreePriorChanged() {
-//    	linkTreePriorCheck.setSelected(options.shareSameTreePrior);
-//    	if (options.shareSameTreePrior) {
-//    		options.activedSameTreePrior = currentTreePrior;
-//    		// keep previous prior for reuse
-//    	} else {
-//    		// reuse previous prior
-//    		setCurrentModelAndPrior(currentTreeModel);
-//    	}
-//    	updateTreePriorBorder();
-//    }
+        fireTreePriorsChanged();
+    }
 
     private void updateTreePriorBorder() {
         if (options.useStarBEAST) {
@@ -217,46 +194,60 @@ public class TreesPanel extends BeautiPanel implements Exportable {
         repaint();
     }
 
-    private void fireTreePriorsChanged() {
+    public void fireTreePriorsChanged() {
         options.updatePartitionAllLinks();
         frame.setDirty();
     }
 
     public void updatePriorPanelForSpeciesAnalysis() {
-        linkTreePriorCheck.setSelected(true);
+        if (options.useStarBEAST) linkTreePriorCheck.setSelected(true); // not to set false if non-*BEAST
         updateShareSameTreePriorChanged();
 
         if (currentTreeModel.getPartitionTreePrior() != null) treePriorPanelParent.removeAll();
 
         if (options.getPartitionTreePriors().size() > 0) {
-            OptionsPanel p;
+
             if (options.useStarBEAST) {
-                linkTreePriorCheck.setEnabled(false);
 
                 options.getPartitionTreePriors().get(0).setNodeHeightPrior(TreePriorType.SPECIES_YULE);
 
-                options.clockModelOptions.fixRateOfFirstClockPartition(); // fix 1st partition
-
-                p = new SpeciesTreesPanel(options.getPartitionTreePriors().get(0));
+                List<ClockModelGroup> groupList = options.clockModelOptions.getClockModelGroups();// all data partitions
+                for (ClockModelGroup clockModelGroup : groupList) { // todo correct?
+                    options.clockModelOptions.fixRateOfFirstClockPartition(clockModelGroup); // fix 1st partition
+                }
 
             } else {
-                linkTreePriorCheck.setEnabled(true);
+//                if (options.hasData() && options.contains(Microsatellite.INSTANCE)) {
+//                    linkTreePriorCheck.setEnabled(false);
+//                } else {
+//                    linkTreePriorCheck.setEnabled(true);
+//                }
 
                 options.getPartitionTreePriors().get(0).setNodeHeightPrior(TreePriorType.CONSTANT);
-
-                p = new PartitionTreePriorPanel(options.getPartitionTreePriors().get(0), this);
-
             }
 
+            PartitionTreePriorPanel p = new PartitionTreePriorPanel(options.getPartitionTreePriors().get(0), this);
+
             treePriorPanels.put(options.getPartitionTreePriors().get(0), p);
-            updateTreePriorBorder();
+
+            for (PartitionTreeModel model : options.getPartitionTreeModels()) {
+                if (model != null && treeModelPanels.get(model) != null) treeModelPanels.get(model).setupPanel();
+
+            }
             treePriorPanelParent.add(p);
         }
+        updateTreePriorBorder();
         repaint();
     }
 
     private void selectionChanged() {
         int selRow = treesTable.getSelectedRow();
+
+        if (selRow >= options.getPartitionTreeModels().size()) {
+            selRow = 0;
+            treesTable.getSelectionModel().setSelectionInterval(selRow, selRow);
+        }
+
         if (selRow >= 0) {
             PartitionTreeModel ptm = options.getPartitionTreeModels().get(selRow);
             setCurrentModelAndPrior(ptm);
@@ -289,7 +280,7 @@ public class TreesPanel extends BeautiPanel implements Exportable {
 
             PartitionTreePrior prior = model.getPartitionTreePrior();
 
-            OptionsPanel panel1 = treePriorPanels.get(prior);
+            PartitionTreePriorPanel panel1 = treePriorPanels.get(prior);
             if (panel1 == null) {
                 panel1 = new PartitionTreePriorPanel(prior, this);
                 treePriorPanels.put(prior, panel1);
@@ -305,11 +296,6 @@ public class TreesPanel extends BeautiPanel implements Exportable {
         }
     }
 
-
-    public void setCheckedTipDate(boolean isCheckedTipDate) {
-        this.isCheckedTipDate = isCheckedTipDate;
-    }
-
     private void resetPanel() {
         if (!options.hasData()) {
             currentTreeModel = null;
@@ -326,6 +312,13 @@ public class TreesPanel extends BeautiPanel implements Exportable {
         }
     }
 
+    public void updateLinkTreePriorEnablility() {
+        boolean selected = !(options.getPartitionTreeModels().size() < 2
+                || options.contains(Microsatellite.INSTANCE) || options.useStarBEAST);
+
+        linkTreePriorCheck.setEnabled(selected);
+    }
+
     public void setOptions(BeautiOptions options) {
         this.options = options;
 
@@ -333,34 +326,25 @@ public class TreesPanel extends BeautiPanel implements Exportable {
 
         settingOptions = true;
 
-        linkTreePriorCheck.setEnabled(options.getPartitionTreeModels().size() > 1);
+        updateLinkTreePriorEnablility();
         linkTreePriorCheck.setSelected(options.isShareSameTreePrior()); // important
 
-        for (PartitionTreeModel model : treeModelPanels.keySet()) {
-            if (model != null) {
-                treeModelPanels.get(model).setOptions();
-            }
+        for (PartitionTreeModel model : options.getPartitionTreeModels()) {
+            if (model != null && treeModelPanels.get(model) != null) treeModelPanels.get(model).setupPanel();
         }
 
         for (PartitionTreePrior prior : options.getPartitionTreePriors()) {
-            if (options.useStarBEAST) {
-                linkTreePriorCheck.setEnabled(false);
-                SpeciesTreesPanel ptpp = (SpeciesTreesPanel) treePriorPanels.get(prior);
-                if (ptpp != null) {
-                    ptpp.setOptions();
-                }
-            } else {
-                linkTreePriorCheck.setEnabled(true);
-                PartitionTreePriorPanel ptpp = (PartitionTreePriorPanel) treePriorPanels.get(prior);
-                if (ptpp != null) {
-                    ptpp.setOptions();
+            PartitionTreePriorPanel ptpp = treePriorPanels.get(prior);
+            if (ptpp != null) {
+                ptpp.setTreePriorChoices(options.useStarBEAST, options.getPartitionTreeModels().size() > 1,
+                        options.clockModelOptions.isTipCalibrated());
+                // setTreePriorChoices should be always before setOptions
+                ptpp.setOptions();
+//                    if (options.contains(Microsatellite.INSTANCE)) {
+//                        ptpp.setMicrosatelliteTreePrior();
+//                    } else
 
-                    if (isCheckedTipDate) {
-                        ptpp.removeCertainPriorFromTreePriorCombo();
-                    } else {
-                        ptpp.recoveryTreePriorCombo();
-                    }
-                }
+                ptpp.repaint();
             }
         }
 
@@ -395,16 +379,9 @@ public class TreesPanel extends BeautiPanel implements Exportable {
 //     	}
 
         for (PartitionTreePrior prior : options.getPartitionTreePriors()) {
-            if (options.useStarBEAST) {
-                SpeciesTreesPanel ptpp = (SpeciesTreesPanel) treePriorPanels.get(prior);
-                if (ptpp != null) {
-                    ptpp.getOptions();
-                }
-            } else {
-                PartitionTreePriorPanel ptpp = (PartitionTreePriorPanel) treePriorPanels.get(prior);
-                if (ptpp != null) {
-                    ptpp.getOptions();
-                }
+            PartitionTreePriorPanel ptpp = treePriorPanels.get(prior);
+            if (ptpp != null) {
+                ptpp.getOptions();
             }
         }
     }
@@ -420,7 +397,7 @@ public class TreesPanel extends BeautiPanel implements Exportable {
 
     private boolean isUsed(int row) {
         PartitionTreeModel model = options.getPartitionTreeModels().get(row);
-        for (PartitionData partition : options.dataPartitions) {
+        for (AbstractPartitionData partition : options.dataPartitions) {
             if (partition.getPartitionTreeModel() == model) {
                 return true;
             }
