@@ -29,7 +29,10 @@ import dr.stats.Variate;
 
 import java.awt.*;
 import java.awt.geom.*;
-import java.util.*;
+import java.util.Collection;
+import java.util.HashSet;
+import java.util.List;
+import java.util.Set;
 
 /**
  * Plot.java
@@ -68,12 +71,12 @@ public interface Plot {
     /**
      * Set data
      */
-    void setData(double[] xData, double[] yData);
+    void setData(List<Double> xData, List<Double> yData);
 
     /**
      * Set data
      */
-    void setData(Variate xData, Variate yData);
+    void setData(Variate.N xData, Variate.N yData);
 
     /**
      * Set line style
@@ -189,12 +192,12 @@ public interface Plot {
     public abstract class AbstractPlot implements Plot {
 
         protected Axis xAxis, yAxis;
-        protected Variate xData = null;
-        protected Variate yData = null;
+        protected Variate.N xData = null;
+        protected Variate.N yData = null;
 
         protected Shape mark;
 
-        protected Stroke lineStroke = new BasicStroke(2.0f);
+        protected Stroke lineStroke = new BasicStroke(1.5f);
         protected Paint linePaint = Color.black;
 
         protected Stroke markStroke = new BasicStroke(0.5f);
@@ -218,7 +221,7 @@ public interface Plot {
         /**
          * Constructor
          */
-        public AbstractPlot(Variate xData, Variate yData) {
+        public AbstractPlot(Variate.N xData, Variate.N yData) {
             setData(xData, yData);
         }
 
@@ -226,7 +229,7 @@ public interface Plot {
         /**
          * Constructor
          */
-        public AbstractPlot(double[] xData, double[] yData) {
+        public AbstractPlot(List<Double> xData, List<Double> yData) {
             setData(xData, yData);
         }
 
@@ -234,9 +237,9 @@ public interface Plot {
         /**
          * Set data
          */
-        public void setData(double[] xData, double[] yData) {
-            Variate.Double xd = new Variate.Double(xData);
-            Variate.Double yd = new Variate.Double(yData);
+        public void setData(List<Double> xData, List<Double> yData) {
+            Variate.D xd = new Variate.D(xData);
+            Variate.D yd = new Variate.D(yData);
 
             this.xData = xd;
             this.yData = yd;
@@ -245,7 +248,7 @@ public interface Plot {
         /**
          * Set data
          */
-        public void setData(Variate xData, Variate yData) {
+        public void setData(Variate.N xData, Variate.N yData) {
             this.xData = xData;
             this.yData = yData;
         }
@@ -275,14 +278,14 @@ public interface Plot {
                     double minValue = java.lang.Double.POSITIVE_INFINITY;
 
                     for (int i = 0; i < xData.getCount(); i++) {
-                        double value = xData.get(i);
+                        double value = (Double) xData.get(i);
                         if (value > 0.0 && value < minValue)
                             minValue = value;
                     }
 
-                    xAxis.addRange(minValue, xData.getMax());
+                    xAxis.addRange(minValue, (Double) xData.getMax());
                 } else {
-                    xAxis.addRange(xData.getMin(), xData.getMax());
+                    xAxis.addRange((Double) xData.getMin(), (Double) xData.getMax());
                 }
             }
             if (yData != null) {
@@ -290,14 +293,14 @@ public interface Plot {
                     double minValue = java.lang.Double.POSITIVE_INFINITY;
 
                     for (int i = 0; i < yData.getCount(); i++) {
-                        double value = yData.get(i);
+                        double value = (Double) yData.get(i);
                         if (value > 0.0 && value < minValue)
                             minValue = value;
                     }
 
-                    yAxis.addRange(minValue, yData.getMax());
+                    yAxis.addRange(minValue, (Double) yData.getMax());
                 } else {
-                    yAxis.addRange(yData.getMin(), yData.getMax());
+                    yAxis.addRange((Double) yData.getMin(), (Double) yData.getMax());
                 }
             }
         }
@@ -525,7 +528,7 @@ public interface Plot {
         /**
          * Paint data series
          */
-        abstract protected void paintData(Graphics2D g2, Variate xData, Variate yData);
+        abstract protected void paintData(Graphics2D g2, Variate.N xData, Variate.N yData);
 
         /**
          * A point on the plot has been clicked
@@ -552,8 +555,8 @@ public interface Plot {
             double y1 = untransformY(dragRectangle.getY());
 
             for (int i = 0; i < xData.getCount(); i ++) {
-                double x = xData.get(i);
-                double y = yData.get(i);
+                double x = (Double) xData.get(i);
+                double y = (Double) yData.get(i);
 
                 if (x >= x0 && x <= x1 && y >= y0 && y <= y1) {
                     selectedPoints.add(i);
