@@ -25,15 +25,23 @@
 
 package dr.app.pathogen;
 
-import dr.app.gui.util.LongTask;
-import dr.evolution.tree.*;
 import dr.app.gui.chart.*;
+import dr.app.gui.util.LongTask;
+import dr.evolution.tree.NodeRef;
+import dr.evolution.tree.Tree;
+import dr.math.MathUtils;
 import dr.stats.DiscreteStatistics;
 import dr.stats.Regression;
 import dr.stats.Variate;
 import dr.util.NumberFormatter;
+import figtree.panel.FigTreePanel;
+import figtree.treeviewer.TreePaneSelector;
+import figtree.treeviewer.TreeSelectionListener;
 import jam.framework.Exportable;
 import jam.table.TableRenderer;
+import jebl.evolution.graphs.Node;
+import jebl.evolution.taxa.Taxon;
+import jebl.evolution.trees.RootedTree;
 
 import javax.swing.*;
 import javax.swing.plaf.BorderUIResource;
@@ -45,13 +53,7 @@ import java.io.PrintWriter;
 import java.io.Writer;
 import java.text.DecimalFormat;
 import java.util.*;
-
-import figtree.panel.FigTreePanel;
-import figtree.treeviewer.TreePaneSelector;
-import figtree.treeviewer.TreeSelectionListener;
-import jebl.evolution.trees.RootedTree;
-import jebl.evolution.taxa.Taxon;
-import jebl.evolution.graphs.Node;
+import java.util.List;
 
 /**
  * @author Andrew Rambaut
@@ -70,18 +72,30 @@ public class TreesPanel extends JPanel implements Exportable {
     PathogenFrame frame = null;
     JTabbedPane tabbedPane = new JTabbedPane();
     JTextArea textArea = new JTextArea();
+    JCheckBox showMRCACheck = new JCheckBox("Show ancestor traces");
 
     //    JTreeDisplay treePanel;
     FigTreePanel treePanel;
+
     JChartPanel rootToTipPanel;
     JChart rootToTipChart;
     ScatterPlot rootToTipPlot;
+
+    private static final boolean SHOW_NODE_DENSITY = true;
+    JChartPanel nodeDensityPanel;
+    JChart nodeDensityChart;
+    ScatterPlot nodeDensityPlot;
+
     JChartPanel residualPanel;
     JChart residualChart;
     ScatterPlot residualPlot;
 
+    ParentPlot mrcaPlot;
+
     Map<Node, Integer> pointMap = new HashMap<Node, Integer>();
 
+    Set<Integer> selectedPoints = new HashSet<Integer>();
+
     private boolean bestFittingRoot;
     private TemporalRooting.RootingFunction rootingFunction;
     private TemporalRooting temporalRooting = null;
@@ -106,7 +120,7 @@ public class TreesPanel extends JPanel implements Exportable {
         Box controlPanel1 = new Box(BoxLayout.PAGE_AXIS);
         controlPanel1.setOpaque(false);
 
-        JPanel panel3 = new JPanel(new BorderLayout(0,0));
+        JPanel panel3 = new JPanel(new BorderLayout(0, 0));
         panel3.setOpaque(false);
         rootingCheck = new JCheckBox("Best-fitting root");
         panel3.add(rootingCheck, BorderLayout.CENTER);
@@ -144,9 +158,12 @@ public class TreesPanel extends JPanel implements Exportable {
         ChartSelector selector1 = new ChartSelector(rootToTipChart);
 
         rootToTipPanel = new JChartPanel(rootToTipChart, "", "time", "divergence");
-        rootToTipPanel.setOpaque(false);
+        JPanel panel = new JPanel(new BorderLayout());
+        panel.add(rootToTipPanel, BorderLayout.CENTER);
+        panel.add(showMRCACheck, BorderLayout.SOUTH);
+        panel.setOpaque(false);
 
-        tabbedPane.add("Root-to-tip", rootToTipPanel);
+        tabbedPane.add("Root-to-tip", panel);
 
         residualChart = new JChart(new LinearAxis(), new LinearAxis(Axis.AT_ZERO, Axis.AT_MINOR_TICK));
 
@@ -164,6 +181,19 @@ public class TreesPanel extends JPanel implements Exportable {
         panel2.add(tabbedPane, BorderLayout.CENTER);
 //        panel2.add(textArea, BorderLayout.SOUTH);
 
+        if (SHOW_NODE_DENSITY) {
+            nodeDensityChart = new JChart(new LinearAxis(), new LinearAxis(Axis.AT_ZERO, Axis.AT_MINOR_TICK));
+            nodeDensityPanel = new JChartPanel(nodeDensityChart, "", "time", "node density");
+            JPanel panel4 = new JPanel(new BorderLayout());
+            panel4.add(nodeDensityPanel, BorderLayout.CENTER);
+            panel4.setOpaque(false);
+
+            ChartSelector selector3 = new ChartSelector(nodeDensityChart);
+
+            tabbedPane.add("Node density", panel4);
+        }
+
+
         JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, panel1, panel2);
         splitPane.setDividerLocation(220);
         splitPane.setContinuousLayout(true);
@@ -178,38 +208,82 @@ public class TreesPanel extends JPanel implements Exportable {
 
         rootingCheck.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent e) {
-                setBestFittingRoot(rootingCheck.isSelected(), (TemporalRooting.RootingFunction)rootingFunctionCombo.getSelectedItem());
+                setBestFittingRoot(rootingCheck.isSelected(), (TemporalRooting.RootingFunction) rootingFunctionCombo.getSelectedItem());
             }
         });
 
         rootingFunctionCombo.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent e) {
-                setBestFittingRoot(rootingCheck.isSelected(), (TemporalRooting.RootingFunction)rootingFunctionCombo.getSelectedItem());
+                setBestFittingRoot(rootingCheck.isSelected(), (TemporalRooting.RootingFunction) rootingFunctionCombo.getSelectedItem());
             }
         });
 
+        showMRCACheck.addActionListener(new ActionListener() {
+            public void actionPerformed(ActionEvent e) {
+                setupPanel();
+            }
+        });
+
+
         setTree(tree);
     }
 
+
     private void treeSelectionChanged() {
         if (rootToTipPlot != null) {
             Set<Node> selectedTips = treePanel.getTreeViewer().getSelectedTips();
-            Set<Integer> selectedPoints = new HashSet<Integer>();
+            selectedPoints = new HashSet<Integer>();
             for (Node node : selectedTips) {
                 selectedPoints.add(pointMap.get(node));
             }
             rootToTipPlot.setSelectedPoints(selectedPoints);
             residualPlot.setSelectedPoints(selectedPoints);
+
+            if (SHOW_NODE_DENSITY) {
+                nodeDensityPlot.setSelectedPoints(selectedPoints);
+            }
+
+            selectMRCA();
         }
     }
 
     private void plotSelectionChanged(final Set<Integer> selectedPoints) {
+        this.selectedPoints = selectedPoints;
         Set<String> selectedTaxa = new HashSet<String>();
         for (Integer i : selectedPoints) {
             selectedTaxa.add(tree.getTaxon(i).toString());
         }
 
         treePanel.getTreeViewer().selectTaxa(selectedTaxa);
+
+        selectMRCA();
+    }
+
+    private void selectMRCA() {
+        if (mrcaPlot == null) return;
+
+        if (selectedPoints != null && selectedPoints.size() > 0) {
+
+            Set<String> selectedTaxa = new HashSet<String>();
+            for (Integer i : selectedPoints) {
+                selectedTaxa.add(tree.getTaxon(i).toString());
+            }
+
+            Regression r = temporalRooting.getRootToTipRegression(currentTree);
+            NodeRef mrca = Tree.Utils.getCommonAncestorNode(currentTree, selectedTaxa);
+            double mrcaDistance1 = temporalRooting.getRootToTipDistance(currentTree, mrca);
+            double mrcaTime1 = r.getX(mrcaDistance1);
+            if (tree.isExternal(mrca)) {
+                mrca = tree.getParent(mrca);
+            }
+            double mrcaDistance = temporalRooting.getRootToTipDistance(currentTree, mrca);
+            double mrcaTime = r.getX(mrcaDistance);
+
+            mrcaPlot.setSelectedPoints(selectedPoints, mrcaTime, mrcaDistance);
+        } else {
+            mrcaPlot.clearSelection();
+        }
+        repaint();
     }
 
     public void timeScaleChanged() {
@@ -222,7 +296,7 @@ public class TreesPanel extends JPanel implements Exportable {
     }
 
     public JComponent getExportableComponent() {
-        return (JComponent)tabbedPane.getSelectedComponent();
+        return (JComponent) tabbedPane.getSelectedComponent();
     }
 
     public void setTree(Tree tree) {
@@ -300,21 +374,28 @@ public class TreesPanel extends JPanel implements Exportable {
             RootedTree jtree = Tree.Utils.asJeblTree(currentTree);
 
             if (temporalRooting.isContemporaneous()) {
-                double values[] = temporalRooting.getRootToTipDistances(currentTree);
+                double[] dv = temporalRooting.getRootToTipDistances(currentTree);
+
+                List<Double> values = new ArrayList<Double>();
+                for (double d : dv) {
+                    values.add(d);
+                }
 
                 rootToTipChart.removeAllPlots();
                 NumericalDensityPlot dp = new NumericalDensityPlot(values, 20, null);
-                dp.setLineColor(new Color(9,70,15));
+                dp.setLineColor(new Color(9, 70, 15));
 
-                double yOffset = dp.getYData().getMax() / 2;
-                double[] dummyValues = new double[values.length];
-                for (int i = 0; i < dummyValues.length; i++) {
-                    dummyValues[i] = yOffset;
+                double yOffset = (Double) dp.getYData().getMax() / 2;
+                List<Double> dummyValues = new ArrayList<Double>();
+                for (int i = 0; i < values.size(); i++) {
+                    // add a random y offset to give some visual spread
+                    double y = MathUtils.nextGaussian() * ((Double) dp.getYData().getMax() * 0.05);
+                    dummyValues.add(yOffset + y);
                 }
 
                 rootToTipPlot = new ScatterPlot(values, dummyValues);
-                rootToTipPlot.setMarkStyle(Plot.CIRCLE_MARK, 5, new BasicStroke(0.5F), new Color(44,44,44), new Color(249,202,105));
-                rootToTipPlot.setHilightedMarkStyle(new BasicStroke(0.5F), new Color(44,44,44), UIManager.getColor("List.selectionBackground"));
+                rootToTipPlot.setMarkStyle(Plot.CIRCLE_MARK, 5, new BasicStroke(0.5F), new Color(44, 44, 44), new Color(249, 202, 105));
+                rootToTipPlot.setHilightedMarkStyle(new BasicStroke(0.5F), new Color(44, 44, 44), UIManager.getColor("List.selectionBackground"));
                 rootToTipPlot.addListener(new Plot.Adaptor() {
                     public void selectionChanged(final Set<Integer> selectedPoints) {
                         plotSelectionChanged(selectedPoints);
@@ -329,12 +410,15 @@ public class TreesPanel extends JPanel implements Exportable {
                 residualChart.removeAllPlots();
 
                 sb.append(", contemporaneous tips");
-                sb.append(", mean root-tip distance: " + nf.format(DiscreteStatistics.mean(values)));
-                sb.append(", coefficient of variation: " + nf.format(DiscreteStatistics.stdev(values) / DiscreteStatistics.mean(values)));
-                sb.append(", stdev: " + nf.format(DiscreteStatistics.stdev(values)));
-                sb.append(", variance: " + nf.format(DiscreteStatistics.variance(values)));
+                sb.append(", mean root-tip distance: " + nf.format(DiscreteStatistics.mean(dv)));
+                sb.append(", coefficient of variation: " + nf.format(DiscreteStatistics.stdev(dv) / DiscreteStatistics.mean(dv)));
+                sb.append(", stdev: " + nf.format(DiscreteStatistics.stdev(dv)));
+                sb.append(", variance: " + nf.format(DiscreteStatistics.variance(dv)));
+
+                showMRCACheck.setVisible(false);
             } else {
                 Regression r = temporalRooting.getRootToTipRegression(currentTree);
+
                 double[] residuals = temporalRooting.getRootToTipResiduals(currentTree, r);
                 pointMap.clear();
                 for (int i = 0; i < currentTree.getExternalNodeCount(); i++) {
@@ -346,39 +430,64 @@ public class TreesPanel extends JPanel implements Exportable {
                 }
 
                 rootToTipChart.removeAllPlots();
+
+                if (showMRCACheck.isSelected()) {
+                    double[] dv = temporalRooting.getParentRootToTipDistances(currentTree);
+
+                    List<Double> parentDistances = new ArrayList<Double>();
+                    for (int i = 0; i < dv.length; i++) {
+                        parentDistances.add(i, dv[i]);
+                    }
+
+                    List<Double> parentTimes = new ArrayList<Double>();
+                    for (int i = 0; i < parentDistances.size(); i++) {
+                        parentTimes.add(i, r.getX(parentDistances.get(i)));
+                    }
+                    mrcaPlot = new ParentPlot(r.getXData(), r.getYData(), parentTimes, parentDistances);
+                    mrcaPlot.setLineColor(new Color(105, 202, 105));
+                    mrcaPlot.setLineStroke(new BasicStroke(0.5F));
+
+                    rootToTipChart.addPlot(mrcaPlot);
+                }
+
                 rootToTipPlot = new ScatterPlot(r.getXData(), r.getYData());
                 rootToTipPlot.addListener(new Plot.Adaptor() {
                     public void selectionChanged(final Set<Integer> selectedPoints) {
                         plotSelectionChanged(selectedPoints);
                     }
                 });
-                rootToTipPlot.setMarkStyle(Plot.CIRCLE_MARK, 5, new BasicStroke(0.5F), new Color(44,44,44), new Color(249,202,105));
-                rootToTipPlot.setHilightedMarkStyle(new BasicStroke(0.5F), new Color(44,44,44), UIManager.getColor("List.selectionBackground"));
+                rootToTipPlot.setMarkStyle(Plot.CIRCLE_MARK, 5, new BasicStroke(0.5F), new Color(44, 44, 44), new Color(249, 202, 105));
+                rootToTipPlot.setHilightedMarkStyle(new BasicStroke(0.5F), new Color(44, 44, 44), UIManager.getColor("List.selectionBackground"));
+
                 rootToTipChart.addPlot(rootToTipPlot);
+
                 rootToTipChart.addPlot(new RegressionPlot(r));
-                rootToTipChart.getXAxis().addRange(r.getXIntercept(), r.getXData().getMax());
+
+                rootToTipChart.getXAxis().addRange(r.getXIntercept(), (Double) r.getXData().getMax());
                 rootToTipPanel.setXAxisTitle("time");
                 rootToTipPanel.setYAxisTitle("root-to-tip divergence");
 
                 residualChart.removeAllPlots();
-                Variate values = r.getYResidualData();
+                Variate.D values = (Variate.D) r.getYResidualData();
                 NumericalDensityPlot dp = new NumericalDensityPlot(values, 20);
-                dp.setLineColor(new Color(103,128,144));
+                dp.setLineColor(new Color(103, 128, 144));
 
-                double yOffset = dp.getYData().getMax() / 2;
-                double[] dummyValues = new double[values.getCount()];
+                double yOffset = (Double) dp.getYData().getMax() / 2;
+                Double[] dummyValues = new Double[values.getCount()];
                 for (int i = 0; i < dummyValues.length; i++) {
-                    dummyValues[i] = yOffset;
+                    // add a random y offset to give some visual spread
+                    double y = MathUtils.nextGaussian() * ((Double) dp.getYData().getMax() * 0.05);
+                    dummyValues[i] = yOffset + y;
                 }
-                Variate yOffsetValues = new Variate.Double(dummyValues);
+                Variate.D yOffsetValues = new Variate.D(dummyValues);
                 residualPlot = new ScatterPlot(values, yOffsetValues);
                 residualPlot.addListener(new Plot.Adaptor() {
                     public void selectionChanged(final Set<Integer> selectedPoints) {
                         plotSelectionChanged(selectedPoints);
                     }
                 });
-                residualPlot.setMarkStyle(Plot.CIRCLE_MARK, 5, new BasicStroke(0.5F), new Color(44,44,44), new Color(249,202,105));
-                residualPlot.setHilightedMarkStyle(new BasicStroke(0.5F), new Color(44,44,44), UIManager.getColor("List.selectionBackground"));
+                residualPlot.setMarkStyle(Plot.CIRCLE_MARK, 5, new BasicStroke(0.5F), new Color(44, 44, 44), new Color(249, 202, 105));
+                residualPlot.setHilightedMarkStyle(new BasicStroke(0.5F), new Color(44, 44, 44), UIManager.getColor("List.selectionBackground"));
 
                 residualChart.addPlot(residualPlot);
                 residualChart.addPlot(dp);
@@ -396,12 +505,35 @@ public class TreesPanel extends JPanel implements Exportable {
 //                residualPanel.setXAxisTitle("residual");
 //                residualPanel.setYAxisTitle("proportion");
 
+                if (SHOW_NODE_DENSITY) {
+                    Regression r2 = temporalRooting.getNodeDensityRegression(currentTree);
+                    nodeDensityChart.removeAllPlots();
+                    nodeDensityPlot = new ScatterPlot(r2.getXData(), r2.getYData());
+                    nodeDensityPlot.addListener(new Plot.Adaptor() {
+                        public void selectionChanged(final Set<Integer> selectedPoints) {
+                            plotSelectionChanged(selectedPoints);
+                        }
+                    });
+                    nodeDensityPlot.setMarkStyle(Plot.CIRCLE_MARK, 5, new BasicStroke(0.5F), new Color(44, 44, 44), new Color(249, 202, 105));
+                    nodeDensityPlot.setHilightedMarkStyle(new BasicStroke(0.5F), new Color(44, 44, 44), UIManager.getColor("List.selectionBackground"));
+
+                    nodeDensityChart.addPlot(nodeDensityPlot);
+
+                    nodeDensityChart.addPlot(new RegressionPlot(r2));
+
+                    nodeDensityChart.getXAxis().addRange(r2.getXIntercept(), (Double) r2.getXData().getMax());
+                    nodeDensityPanel.setXAxisTitle("time");
+                    nodeDensityPanel.setYAxisTitle("node density");
+                }
+
                 sb.append(", dated tips");
                 sb.append(", date range: " + nf.format(temporalRooting.getDateRange()));
                 sb.append(", slope (rate): " + nf.format(r.getGradient()));
                 sb.append(", x-intercept (TMRCA): " + nf.format(r.getXIntercept()));
                 sb.append(", corr. coeff: " + nf.format(r.getCorrelationCoefficient()));
                 sb.append(", R^2: " + nf.format(r.getRSquared()));
+
+                showMRCACheck.setVisible(true);
             }
 
             treePanel.setTree(jtree);
@@ -525,7 +657,7 @@ public class TreesPanel extends JPanel implements Exportable {
 
                 switch (row) {
                     case 0:
-                        value =DiscreteStatistics.mean(values);
+                        value = DiscreteStatistics.mean(values);
                         break;
                     case 1:
                         value = DiscreteStatistics.stdev(values) / DiscreteStatistics.mean(values);
