@@ -63,8 +63,6 @@ public class TreeAnnotator {
     private final static Version version = new BeastVersion();
 
     private final static boolean USE_R = false;
-    private final static String HPD_2D_STRING = "80";
-    private final static double HPD_2D = 0.80;
 
     private static boolean forceIntegerToDiscrete = false;
 
@@ -87,7 +85,8 @@ public class TreeAnnotator {
     enum HeightsSummary {
         MEDIAN_HEIGHTS("Median heights"),
         MEAN_HEIGHTS("Mean heights"),
-        KEEP_HEIGHTS("Keep target heights");
+        KEEP_HEIGHTS("Keep target heights"),
+        CA_HEIGHTS("Common Ancestor heights");
 
         String desc;
 
@@ -100,6 +99,7 @@ public class TreeAnnotator {
         }
     }
 
+
     // Messages to stderr, output to stdout
     private static PrintStream progressStream = System.err;
 
@@ -110,6 +110,7 @@ public class TreeAnnotator {
     public TreeAnnotator(final int burnin,
                          HeightsSummary heightsOption,
                          double posteriorLimit,
+                         double hpd2D,
                          Target targetOption,
                          String targetTreeFileName,
                          String inputFileName,
@@ -117,6 +118,7 @@ public class TreeAnnotator {
     ) throws IOException {
 
         this.posteriorLimit = posteriorLimit;
+        this.hpd2D = hpd2D;
 
         attributeNames.add("height");
         attributeNames.add("length");
@@ -276,8 +278,12 @@ public class TreeAnnotator {
 
         try {
             cladeSystem.annotateTree(targetTree, targetTree.getRoot(), null, heightsOption);
+
+            if( heightsOption == HeightsSummary.CA_HEIGHTS ) {
+                setTreeHeightsByCA(targetTree, inputFileName, burnin);
+            }
         } catch (Exception e) {
-            System.err.println("Error to annotate tree: " + e.getMessage() + "\nPlease check the tree log file format.");
+            System.err.println("Error annotating tree: " + e.getMessage() + "\nPlease check the tree log file format.");
             return;
         }
 
@@ -307,6 +313,11 @@ public class TreeAnnotator {
                 }
             }
         }
+
+        for (TreeAnnotationPlugin plugin : plugins) {
+            Set<String> claimed = plugin.setAttributeNames(attributeNames);
+            attributeNames.removeAll(claimed);
+        }
     }
 
     private Tree summarizeTrees(int burnin, CladeSystem cladeSystem, String inputFileName,
@@ -479,38 +490,43 @@ public class TreeAnnotator {
                 int i = 0;
                 Object[] values = new Object[attributeNames.size()];
                 for (String attributeName : attributeNames) {
-                    Object value;
-                    if (attributeName.equals("height")) {
-                        value = tree.getNodeHeight(node);
-                    } else if (attributeName.equals("length")) {
-                        value = tree.getBranchLength(node);
-                    } else if (attributeName.equals(location1Attribute)) {
-                        // If this is one of the two specified bivariate location names then
-                        // merge this and the other one into a single array.
-                        Object value1 = tree.getNodeAttribute(node, attributeName);
-                        Object value2 = tree.getNodeAttribute(node, location2Attribute);
+                    boolean processed = false;
 
-                        value = new Object[]{value1, value2};
-                    } else if (attributeName.equals(location2Attribute)) {
-                        // do nothing - already dealt with this...
-                        value = null;
-                    } else {
-                        value = tree.getNodeAttribute(node, attributeName);
-                        if (value instanceof String && ((String) value).startsWith("\"")) {
-                            value = ((String) value).replaceAll("\"", "");
+                    if (!processed) {
+                        Object value;
+                        if (attributeName.equals("height")) {
+                            value = tree.getNodeHeight(node);
+                        } else if (attributeName.equals("length")) {
+                            value = tree.getBranchLength(node);
+// AR - we deal with this once everything                             
+//                        } else if (attributeName.equals(location1Attribute)) {
+//                            // If this is one of the two specified bivariate location names then
+//                            // merge this and the other one into a single array.
+//                            Object value1 = tree.getNodeAttribute(node, attributeName);
+//                            Object value2 = tree.getNodeAttribute(node, location2Attribute);
+//
+//                            value = new Object[]{value1, value2};
+//                        } else if (attributeName.equals(location2Attribute)) {
+//                            // do nothing - already dealt with this...
+//                            value = null;
+                        } else {
+                            value = tree.getNodeAttribute(node, attributeName);
+                            if (value instanceof String && ((String) value).startsWith("\"")) {
+                                value = ((String) value).replaceAll("\"", "");
+                            }
                         }
+
+                        //if (value == null) {
+                        //    progressStream.println("attribute " + attributeNames[i] + " is null.");
+                        //}
+
+                        values[i] = value;
                     }
-
-                    //if (value == null) {
-                    //    progressStream.println("attribute " + attributeNames[i] + " is null.");
-                    //}
-
-                    values[i] = value;
                     i++;
                 }
                 clade.attributeValues.add(values);
 
-                //progressStream.println(clade + " " + clade.getCount());
+                //progressStream.println(clade + " " + clade.getValuesSize());
                 clade.setCount(clade.getCount() + 1);
             }
         }
@@ -596,7 +612,7 @@ public class TreeAnnotator {
             return clade.getCredibility();
         }
 
-        public void annotateTree(MutableTree tree, NodeRef node, BitSet bits, HeightsSummary heightsOption) throws Exception {
+        public void annotateTree(MutableTree tree, NodeRef node, BitSet bits, HeightsSummary heightsOption) {
 
             BitSet bits2 = new BitSet();
 
@@ -729,53 +745,68 @@ public class TreeAnnotator {
                         }
 
                         if (!filter) {
-                            if (!isDiscrete) {
-                                if (!isDoubleArray)
-                                    annotateMeanAttribute(tree, node, attributeName, values);
-                                else {
+                            boolean processed = false;
+                            for (TreeAnnotationPlugin plugin : plugins) {
+                                if (plugin.handleAttribute(tree, node, attributeName, values)) {
+                                    processed = true;
+                                }
+                            }
+
+                            if (!processed) {
+                                if (!isDiscrete) {
+                                    if (!isDoubleArray)
+                                        annotateMeanAttribute(tree, node, attributeName, values);
+                                    else {
+                                        for (int k = 0; k < lenArray; k++) {
+                                            annotateMeanAttribute(tree, node, attributeName + (k + 1), valuesArray[k]);
+                                        }
+                                    }
+                                } else {
+                                    annotateModeAttribute(tree, node, attributeName, hashMap);
+                                    annotateFrequencyAttribute(tree, node, attributeName, hashMap);
+                                }
+                                if (!isBoolean && minValue < maxValue && !isDiscrete && !isDoubleArray) {
+                                    // Basically, if it is a boolean (0, 1) then we don't need the distribution information
+                                    // Likewise if it doesn't vary.
+                                    annotateMedianAttribute(tree, node, attributeName + "_median", values);
+                                    annotateHPDAttribute(tree, node, attributeName + "_95%_HPD", 0.95, values);
+                                    annotateRangeAttribute(tree, node, attributeName + "_range", values);
+                                }
+
+                                if (isDoubleArray) {
+                                    String name = attributeName;
+                                    // todo
+//                                    if (name.equals(location1Attribute)) {
+//                                        name = locationOutputAttribute;
+//                                    }
+                                    boolean want2d = processBivariateAttributes && lenArray == 2;
+                                    if (name.equals("dmv")) {  // terrible hack
+                                        want2d = false;
+                                    }
                                     for (int k = 0; k < lenArray; k++) {
-                                        annotateMeanAttribute(tree, node, attributeName + (k + 1), valuesArray[k]);
+                                        if (minValueArray[k] < maxValueArray[k]) {
+                                            annotateMedianAttribute(tree, node, name + (k + 1) + "_median", valuesArray[k]);
+                                            annotateRangeAttribute(tree, node, name + (k + 1) + "_range", valuesArray[k]);
+                                            if (!want2d)
+                                                annotateHPDAttribute(tree, node, name + (k + 1) + "_95%_HPD", 0.95, valuesArray[k]);
+                                        }
                                     }
-                                }
-                            } else {
-                                annotateModeAttribute(tree, node, attributeName, hashMap);
-                                annotateFrequencyAttribute(tree, node, attributeName, hashMap);
-                            }
-                            if (!isBoolean && minValue < maxValue && !isDiscrete && !isDoubleArray) {
-                                // Basically, if it is a boolean (0, 1) then we don't need the distribution information
-                                // Likewise if it doesn't vary.
-                                annotateMedianAttribute(tree, node, attributeName + "_median", values);
-                                annotateHPDAttribute(tree, node, attributeName + "_95%_HPD", 0.95, values);
-                                annotateRangeAttribute(tree, node, attributeName + "_range", values);
-                            }
-                            if (isDoubleArray) {
-                                String name = attributeName;
-                                if (name.equals(location1Attribute)) {
-                                    name = locationOutputAttribute;
-                                }
-                                for (int k = 0; k < lenArray; k++) {
-                                    if (minValueArray[k] < maxValueArray[k]) {
-                                        annotateMedianAttribute(tree, node, name + (k + 1) + "_median", valuesArray[k]);
-                                        annotateRangeAttribute(tree, node, name + (k + 1) + "_range", valuesArray[k]);
-                                        if (!processBivariateAttributes || lenArray != 2)
-                                            annotateHPDAttribute(tree, node, name + (k + 1) + "_95%_HPD", 0.95, valuesArray[k]);
+                                    // 2D contours
+                                    if (want2d) {
+
+                                        boolean variationInFirst = (minValueArray[0] < maxValueArray[0]);
+                                        boolean variationInSecond = (minValueArray[1] < maxValueArray[1]);
+
+                                        if (variationInFirst && !variationInSecond)
+                                            annotateHPDAttribute(tree, node, name + "1" + "_95%_HPD", 0.95, valuesArray[0]);
+
+                                        if (variationInSecond && !variationInFirst)
+                                            annotateHPDAttribute(tree, node, name + "2" + "_95%_HPD", 0.95, valuesArray[1]);
+
+                                        if (variationInFirst && variationInSecond)
+                                            annotate2DHPDAttribute(tree, node, name, "_" + (int) (100 * hpd2D) + "%HPD", hpd2D, valuesArray);
                                     }
                                 }
-                                // 2D contours
-                                if (processBivariateAttributes && lenArray == 2) {
-
-                                    boolean variationInFirst = (minValueArray[0] < maxValueArray[0]);
-                                    boolean variationInSecond = (minValueArray[1] < maxValueArray[1]);
-
-                                    if (variationInFirst && !variationInSecond)
-                                        annotateHPDAttribute(tree, node, name + "1" + "_95%_HPD", 0.95, valuesArray[0]);
-
-                                    if (variationInSecond && !variationInFirst)
-                                        annotateHPDAttribute(tree, node, name + "2" + "_95%_HPD", 0.95, valuesArray[1]);
-
-                                    if (variationInFirst && variationInSecond)
-                                        annotate2DHPDAttribute(tree, node, name, "_" + HPD_2D_STRING + "%HPD", HPD_2D, valuesArray);
-                                }
                             }
                         }
                     }
@@ -976,7 +1007,10 @@ public class TreeAnnotator {
 
 
 //                KernelDensityEstimator2D kde = new KernelDensityEstimator2D(values[0], values[1], N);
-                ContourMaker kde = new ContourWithSynder(values[0], values[1], N);
+                //ContourMaker kde = new ContourWithSynder(values[0], values[1], N);
+                boolean bandwidthLimit = false;
+
+                ContourMaker kde = new ContourWithSynder(values[0], values[1], bandwidthLimit);
 
                 ContourPath[] paths = kde.getContourPaths(hpd);
 
@@ -1046,6 +1080,29 @@ public class TreeAnnotator {
 
         }
 
+        // Get tree clades as bitSets on target taxa
+        // codes is an array of existing BitSet objects, which are reused
+
+        void getTreeCladeCodes(Tree tree, BitSet[] codes) {
+            getTreeCladeCodes(tree, tree.getRoot(), codes);
+        }
+
+        int getTreeCladeCodes(Tree tree, NodeRef node, BitSet[] codes) {
+            final int inode = node.getNumber();
+            codes[inode].clear();
+            if (tree.isExternal(node)) {
+                int index = taxonList.getTaxonIndex(tree.getNodeTaxon(node).getId());
+                codes[inode].set(index);
+            } else {
+                for (int i = 0; i < tree.getChildCount(node); i++) {
+                    final NodeRef child = tree.getChild(node, i);
+                    final int childIndex = getTreeCladeCodes(tree, child, codes);
+
+                    codes[inode].or(codes[childIndex]);
+                }
+            }
+            return inode;
+        }
 
         class Clade {
             public Clade(BitSet bits) {
@@ -1106,6 +1163,9 @@ public class TreeAnnotator {
     int totalTrees = 0;
     int totalTreesUsed = 0;
     double posteriorLimit = 0.0;
+    double hpd2D = 0.80;
+
+    private final List<TreeAnnotationPlugin> plugins = new ArrayList<TreeAnnotationPlugin>();
 
     Set<String> attributeNames = new HashSet<String>();
     TaxonList taxa = null;
@@ -1218,6 +1278,7 @@ public class TreeAnnotator {
 
             int burnin = dialog.getBurnin();
             double posteriorLimit = dialog.getPosteriorLimit();
+            double hpd2D = 0.80;
             Target targetOption = dialog.getTargetOption();
             HeightsSummary heightsOption = dialog.getHeightsOption();
 
@@ -1243,6 +1304,7 @@ public class TreeAnnotator {
                 new TreeAnnotator(burnin,
                         heightsOption,
                         posteriorLimit,
+                        hpd2D,
                         targetOption,
                         targetTreeFileName,
                         inputFileName,
@@ -1267,13 +1329,14 @@ public class TreeAnnotator {
         Arguments arguments = new Arguments(
                 new Arguments.Option[]{
                         //new Arguments.StringOption("target", new String[] { "maxclade", "maxtree" }, false, "an option of 'maxclade' or 'maxtree'"),
-                        new Arguments.StringOption("heights", new String[]{"keep", "median", "mean"}, false,
-                                "an option of 'keep' (default), 'median' or 'mean'"),
+                        new Arguments.StringOption("heights", new String[]{"keep", "median", "mean", "ca"}, false,
+                                "an option of 'keep' (default), 'median', 'mean' or 'ca'"),
                         new Arguments.IntegerOption("burnin", "the number of states to be considered as 'burn-in'"),
                         new Arguments.RealOption("limit", "the minimum posterior probability for a node to be annotated"),
                         new Arguments.StringOption("target", "target_file_name", "specifies a user target tree to be annotated"),
                         new Arguments.Option("help", "option to print this message"),
-                        new Arguments.Option("forceDiscrete", "forces integer traits to be treated as discrete traits.")
+                        new Arguments.Option("forceDiscrete", "forces integer traits to be treated as discrete traits."),
+                        new Arguments.RealOption("hpd2D", "the HPD interval to be used for the bivariate traits")
                 });
 
         try {
@@ -1301,6 +1364,8 @@ public class TreeAnnotator {
                 heights = HeightsSummary.MEAN_HEIGHTS;
             } else if (value.equalsIgnoreCase("median")) {
                 heights = HeightsSummary.MEDIAN_HEIGHTS;
+            } else if (value.equalsIgnoreCase("ca")) {
+                heights = HeightsSummary.CA_HEIGHTS;
             }
         }
 
@@ -1314,6 +1379,11 @@ public class TreeAnnotator {
             posteriorLimit = arguments.getRealOption("limit");
         }
 
+        double hpd2D = 0.80;
+        if (arguments.hasOption("hpd2D")) {
+            hpd2D = arguments.getRealOption("hpd2D");
+        }
+
         Target target = Target.MAX_CLADE_CREDIBILITY;
         if (arguments.hasOption("target")) {
             target = Target.USER_TARGET_TREE;
@@ -1337,9 +1407,106 @@ public class TreeAnnotator {
             }
         }
 
-        new TreeAnnotator(burnin, heights, posteriorLimit, target, targetTreeFileName, inputFileName, outputFileName);
+        new TreeAnnotator(burnin, heights, posteriorLimit, hpd2D, target, targetTreeFileName, inputFileName, outputFileName);
 
         System.exit(0);
     }
+
+    /**
+     * @author Andrew Rambaut
+     * @version $Id$
+     */
+    public static interface TreeAnnotationPlugin {
+        Set<String> setAttributeNames(Set<String> attributeNames);
+
+        boolean handleAttribute(Tree tree, NodeRef node, String attributeName, double[] values);
+    }
+
+    // very inefficient, but Java wonderful bitset has no subset op
+    // perhaps using bit iterator would be faster, I can't br bothered.
+
+    static boolean isSubSet(BitSet x, BitSet y) {
+        y = (BitSet) y.clone();
+        y.and(x);
+        return y.equals(x);
+    }
+
+    boolean setTreeHeightsByCA(MutableTree targetTree, final String inputFileName, final int burnin)
+            throws IOException, Importer.ImportException {
+        progressStream.println("Setting node heights...");
+        progressStream.println("0              25             50             75            100");
+        progressStream.println("|--------------|--------------|--------------|--------------|");
+
+        int reportStepSize = totalTrees / 60;
+        if (reportStepSize < 1) reportStepSize = 1;
+
+        final FileReader fileReader = new FileReader(inputFileName);
+        final NexusImporter importer = new NexusImporter(fileReader);
+
+        // this call increments the clade counts and it shouldn't
+        // this is remedied with removeClades call after while loop below
+        CladeSystem cladeSystem = new CladeSystem(targetTree);
+        final int nClades = cladeSystem.getCladeMap().size();
+
+        // allocate posterior tree nodes order once
+        int[] postOrderList = new int[nClades];
+        BitSet[] ctarget = new BitSet[nClades];
+        BitSet[] ctree = new BitSet[nClades];
+
+        for (int k = 0; k < nClades; ++k) {
+            ctarget[k] = new BitSet();
+            ctree[k] = new BitSet();
+        }
+
+        cladeSystem.getTreeCladeCodes(targetTree, ctarget);
+
+        // temp collecting heights inside loop allocated once
+        double[] hs = new double[nClades];
+
+        // heights total sum from posterior trees
+        double[] ths = new double[nClades];
+
+        totalTreesUsed = 0;
+
+        int counter = 0;
+        while (importer.hasTree()) {
+            final Tree tree = importer.importNextTree();
+
+            if (counter >= burnin) {
+                Tree.Utils.preOrderTraversalList(tree, postOrderList);
+                cladeSystem.getTreeCladeCodes(tree, ctree);
+                for (int k = 0; k < nClades; ++k) {
+                    int j = postOrderList[k];
+                    for (int i = 0; i < nClades; ++i) {
+                        if( isSubSet(ctarget[i], ctree[j]) ) {
+                            hs[i] = tree.getNodeHeight(tree.getNode(j));
+                        }
+                    }
+                }
+                for (int k = 0; k < nClades; ++k) {
+                    ths[k] += hs[k];
+                }
+                totalTreesUsed += 1;
+            }
+            if (counter > 0 && counter % reportStepSize == 0) {
+                progressStream.print("*");
+                progressStream.flush();
+            }
+            counter++;
+
+        }
+        cladeSystem.removeClades(targetTree, targetTree.getRoot(), true);
+        for (int k = 0; k < nClades; ++k) {
+            ths[k] /= totalTreesUsed;
+            final NodeRef node = targetTree.getNode(k);
+            targetTree.setNodeHeight(node, ths[k]);
+        }
+        fileReader.close();
+
+        progressStream.println();
+        progressStream.println();
+
+        return true;
+    }
 }
 
