@@ -26,9 +26,13 @@
 package dr.app.tracer.analysis;
 
 import dr.app.gui.components.RealNumberField;
+import dr.app.gui.components.WholeNumberField;
+import dr.app.gui.util.LongTask;
 import dr.inference.trace.TraceDistribution;
 import dr.inference.trace.TraceList;
 import dr.stats.Variate;
+import jam.framework.DocumentFrame;
+import jam.panels.OptionsPanel;
 import jebl.evolution.coalescent.IntervalList;
 import jebl.evolution.coalescent.Intervals;
 import jebl.evolution.io.ImportException;
@@ -36,10 +40,6 @@ import jebl.evolution.io.NewickImporter;
 import jebl.evolution.io.NexusImporter;
 import jebl.evolution.io.TreeImporter;
 import jebl.evolution.trees.RootedTree;
-import dr.app.gui.components.WholeNumberField;
-import jam.framework.DocumentFrame;
-import jam.panels.OptionsPanel;
-import dr.app.gui.util.LongTask;
 
 import javax.swing.*;
 import javax.swing.border.EmptyBorder;
@@ -51,7 +51,9 @@ import java.awt.event.ActionListener;
 import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileReader;
+import java.util.ArrayList;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Set;
 
 public class GMRFSkyrideDialog {
@@ -424,7 +426,7 @@ public class GMRFSkyrideDialog {
 
         int stateCount;
 
-        Double[][] popSizes;
+        ArrayList<ArrayList> popSizes;
 
         private int lengthOfTask = 0;
         private int current = 0;
@@ -446,9 +448,9 @@ public class GMRFSkyrideDialog {
 
             stateCount = traceList.getStateCount();
 
-            popSizes = new Double[popSizeCount][stateCount];
+            popSizes = new ArrayList<ArrayList>();
             for (int i = 0; i < popSizeCount; i++) {
-                traceList.getValues(firstPopSize + i, popSizes[i]);
+                popSizes.add(new ArrayList(traceList.getValues(firstPopSize + i)));
             }
         }
 
@@ -470,10 +472,10 @@ public class GMRFSkyrideDialog {
 
         public Object doWork() {
 
-            Double[] heights = new Double[stateCount];
-            traceList.getValues(traceList.getTraceIndex(rootHeightTrace), heights);
+            List heights = traceList.getValues(traceList.getTraceIndex(rootHeightTrace));
 
-            TraceDistribution distribution = new TraceDistribution(heights, traceList.getStepSize());
+            TraceDistribution distribution = new TraceDistribution(heights,
+                    traceList.getTrace(traceList.getTraceIndex(rootHeightTrace)).getTraceType(), traceList.getStepSize());
 
             double timeMean = distribution.getMean();
             double timeMedian = distribution.getMedian();
@@ -552,6 +554,25 @@ public class GMRFSkyrideDialog {
                     importer = new NewickImporter(reader, false);
                 }
 
+                int treeTotalStates = importer.importTrees().size();
+                int logTotalStates = traceList.getStateCount() + traceList.getBurninStateCount();
+
+                if (treeTotalStates != logTotalStates) {
+                    throw new IllegalArgumentException("BEAST log states (" + logTotalStates
+                            + ") does not match tree log states (" + treeTotalStates + ")"
+                            + "\nPlease check both log files.");
+                }
+
+                // importer.importTrees() makes point to the end of file, and reader.mark(?) not working for large file
+                reader = new BufferedReader(new FileReader(treeFile));
+
+                line = reader.readLine();
+                if (line.toUpperCase().startsWith("#NEXUS")) {
+                    importer = new NexusImporter(reader);
+                } else {
+                    importer = new NewickImporter(reader, false);
+                }
+
                 int burnin = traceList.getBurnIn();
                 int skip = burnin / traceList.getStepSize();
                 int state = 0;
@@ -607,7 +628,7 @@ public class GMRFSkyrideDialog {
                     ex.printStackTrace(System.out);
                 }
 
-                Variate[] bins = new Variate[binCount];
+                Variate.D[] bins = new Variate.D[binCount];
                 double height;
                 if (ageOfYoungest > 0.0) {
                     height = ageOfYoungest - maxTime;
@@ -617,7 +638,7 @@ public class GMRFSkyrideDialog {
 
 
                 for (int k = 0; k < binCount; k++) {
-                    bins[k] = new Variate.Double();
+                    bins[k] = new Variate.D();
 
                     if (height >= 0.0 && height <= maxHeight) {
                         for (state = 0; state < stateCount; state++) {
@@ -644,11 +665,11 @@ public class GMRFSkyrideDialog {
                     current += 1;
                 }
 
-                Variate xData = new Variate.Double();
-                Variate yDataMean = new Variate.Double();
-                Variate yDataMedian = new Variate.Double();
-                Variate yDataUpper = new Variate.Double();
-                Variate yDataLower = new Variate.Double();
+                Variate.D xData = new Variate.D();
+                Variate.D yDataMean = new Variate.D();
+                Variate.D yDataMedian = new Variate.D();
+                Variate.D yDataUpper = new Variate.D();
+                Variate.D yDataLower = new Variate.D();
 
                 double t;
                 if (ageOfYoungest > 0.0) {
@@ -656,7 +677,7 @@ public class GMRFSkyrideDialog {
                 } else {
                     t = minTime;
                 }
-                for (Variate bin : bins) {
+                for (Variate.D bin : bins) {
                     xData.add(t);
                     if (bin.getCount() > 0) {
                         yDataMean.add(bin.getMean());
@@ -686,6 +707,10 @@ public class GMRFSkyrideDialog {
                 JOptionPane.showMessageDialog(frame, "Error reading file: " + ioe.getMessage(),
                         "Error reading file",
                         JOptionPane.ERROR_MESSAGE);
+            } catch (IllegalArgumentException ile) {
+                JOptionPane.showMessageDialog(frame, ile.getMessage(),
+                            "Invalid log file",
+                            JOptionPane.ERROR_MESSAGE);
             } catch (Exception ex) {
                 JOptionPane.showMessageDialog(frame, "Fatal exception (email the authors):" + ex.getMessage(),
                         "Fatal exception",
@@ -697,7 +722,7 @@ public class GMRFSkyrideDialog {
         }
 
         private double getPopSize(int index, int state) {
-            return Math.exp(popSizes[index][state]);
+            return Math.exp((Double) popSizes.get(index).get(state));
         }
     }
 }
\ No newline at end of file
