@@ -26,19 +26,19 @@
 package dr.app.tracer.analysis;
 
 import dr.app.gui.components.RealNumberField;
+import dr.app.gui.components.WholeNumberField;
+import dr.app.gui.util.LongTask;
 import dr.inference.trace.TraceDistribution;
 import dr.inference.trace.TraceList;
 import dr.stats.Variate;
+import jam.framework.DocumentFrame;
+import jam.panels.OptionsPanel;
 import jebl.evolution.graphs.Node;
 import jebl.evolution.io.ImportException;
 import jebl.evolution.io.NewickImporter;
 import jebl.evolution.io.NexusImporter;
 import jebl.evolution.io.TreeImporter;
 import jebl.evolution.trees.RootedTree;
-import dr.app.gui.components.WholeNumberField;
-import jam.framework.DocumentFrame;
-import jam.panels.OptionsPanel;
-import dr.app.gui.util.LongTask;
 
 import javax.swing.*;
 import javax.swing.border.EmptyBorder;
@@ -50,6 +50,7 @@ import java.awt.event.ActionListener;
 import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileReader;
+import java.util.List;
 
 public class TraitThroughTimeDialog {
 
@@ -368,10 +369,10 @@ public class TraitThroughTimeDialog {
 
         public Object doWork() {
 
-            Double[] heights = new Double[stateCount];
-            traceList.getValues(traceList.getTraceIndex(rootHeightTrace), heights);
+            List heights = traceList.getValues(traceList.getTraceIndex(rootHeightTrace));
 
-            TraceDistribution distribution = new TraceDistribution(heights, traceList.getStepSize());
+            TraceDistribution distribution = new TraceDistribution(heights,
+                    traceList.getTrace(traceList.getTraceIndex(rootHeightTrace)).getTraceType(), traceList.getStepSize());
 
             double timeMean = distribution.getMean();
             double timeMedian = distribution.getMedian();
@@ -461,9 +462,9 @@ public class TraitThroughTimeDialog {
 
                 current = 0;
 
-                Variate[] bins = new Variate[binCount];
+                Variate.D[] bins = new Variate.D[binCount];
                 for (int k = 0; k < binCount; k++) {
-                    bins[k] = new Variate.Double();
+                    bins[k] = new Variate.D();
                 }
 
                 double startHeight;
@@ -524,11 +525,11 @@ public class TraitThroughTimeDialog {
                     ex.printStackTrace(System.out);
                 }
 
-                Variate xData = new Variate.Double();
-                Variate yDataMean = new Variate.Double();
-                Variate yDataMedian = new Variate.Double();
-                Variate yDataUpper = new Variate.Double();
-                Variate yDataLower = new Variate.Double();
+                Variate.D xData = new Variate.D();
+                Variate.D yDataMean = new Variate.D();
+                Variate.D yDataMedian = new Variate.D();
+                Variate.D yDataUpper = new Variate.D();
+                Variate.D yDataLower = new Variate.D();
 
                 double t;
                 if (ageOfYoungest > 0.0) {
@@ -536,7 +537,7 @@ public class TraitThroughTimeDialog {
                 } else {
                     t = minTime;
                 }
-                for (Variate bin : bins) {
+                for (Variate.D bin : bins) {
                     xData.add(t);
                     if (bin.getCount() > 0) {
                         yDataMean.add(bin.getMean());
