@@ -13,6 +13,8 @@ public interface AnalysisMenuHandler {
 
 	Action getBayesianSkylineAction();
 
+    Action getExtendedBayesianSkylineAction();
+
     Action getGMRFSkyrideAction();
 
 	Action getLineagesThroughTimeAction();
@@ -25,7 +27,11 @@ public interface AnalysisMenuHandler {
 
 	Action getAddBayesianSkylineAction();
 
+    Action getAddExtendedBayesianSkylineAction();
+
 	Action getAddTimeDensityAction();
 
 	Action getBayesFactorsAction();
+
+    Action getConditionalPosteriorDistAction();
 }
