@@ -26,14 +26,19 @@
 package dr.app.tracer.traces;
 
 import dr.app.gui.chart.*;
-import dr.inference.trace.*;
+import dr.inference.trace.Trace;
+import dr.inference.trace.TraceCorrelation;
+import dr.inference.trace.TraceFactory;
+import dr.inference.trace.TraceList;
 import dr.stats.Variate;
 import jam.framework.Exportable;
 
 import javax.swing.*;
 import java.awt.*;
 import java.awt.event.ActionEvent;
+import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 
 /**
@@ -44,9 +49,13 @@ import java.util.Map;
  * @version $Id: DensityPanel.java,v 1.3 2006/11/29 09:54:30 rambaut Exp $
  */
 public class DensityPanel extends JPanel implements Exportable {
-    public static int COLOUR_BY_TRACE = 0;
-    public static int COLOUR_BY_FILE = 1;
-    public static int COLOUR_BY_ALL = 2;
+    private static final int DEFAULT_KDE_BINS = 5000;
+
+    public static enum ColourByOptions {
+        COLOUR_BY_TRACE,
+        COLOUR_BY_FILE,
+        COLOUR_BY_ALL
+    };
 
     private static final Paint[] paints = new Paint[]{
             Color.BLACK,
@@ -61,24 +70,39 @@ public class DensityPanel extends JPanel implements Exportable {
             Color.DARK_GRAY
     };
 
-    private ChartSetupDialog chartSetupDialog = null;
+    private class Settings {
+        ChartSetupDialog chartSetupDialog = null;
+        KDESetupDialog kdeSetupDialog = null;
+        int minimumBins = 100;
+        boolean showKDE = true;
+        boolean showHistogram = false;
+        boolean drawSolid = true;
+        boolean relativeDensity = false;
+        int barCount = 0;
+        int legendAlignment = 0;
+        ColourByOptions colourBy = ColourByOptions.COLOUR_BY_TRACE;
+    }
 
-//    private JChart densityChart = new JChart(new LinearAxis(Axis.AT_MAJOR_TICK, Axis.AT_MAJOR_TICK), new LinearAxis());
-    protected DiscreteJChart densityChart = new DiscreteJChart(new LinearAxis(Axis.AT_MAJOR_TICK, Axis.AT_MAJOR_TICK), new LinearAxis());
+    private Settings currentSettings = new Settings();
+    private Map<String, Settings> settingsMap = new HashMap<String, Settings>();
+
+    private JChart densityChart = new JChart(new LinearAxis(Axis.AT_MAJOR_TICK, Axis.AT_MAJOR_TICK), new LinearAxis());
+
+    // as far as I can see DiscreteJChart is superfluous (discrete stats use the CategoryDensityPlot):
+//    protected DiscreteJChart densityChart = new DiscreteJChart(new LinearAxis(Axis.AT_MAJOR_TICK_PLUS, Axis.AT_MAJOR_TICK), new LinearAxis());
 
     protected JChartPanel chartPanel = new JChartPanel(densityChart, null, "", "");
 
-    protected int minimumBins = 100;
     protected JLabel labelBins;
     protected JComboBox binsCombo = new JComboBox(
             new Integer[]{10, 20, 50, 100, 200, 500, 1000});
 
-    private JCheckBox kdeCheckBox = new JCheckBox("KDE");
-    private JButton kdeSetupButton = new JButton("Settings...");
-    private KDESetupDialog kdeSetupDialog = null;
+    private JComboBox displayCombo = new JComboBox(
+            new String[]{"KDE", "Histogram", "Both"}
+    );
 
-    private boolean showKDE = false;
-    private int barCount = 0;
+//    private JCheckBox kdeCheckBox = new JCheckBox("KDE");
+//    private JButton kdeSetupButton = new JButton("Settings...");
 
     protected JCheckBox relativeDensityCheckBox = new JCheckBox("Relative density");
     private JCheckBox solidCheckBox = new JCheckBox("Fill plot");
@@ -92,9 +116,7 @@ public class DensityPanel extends JPanel implements Exportable {
     private JButton chartSetupButton = new JButton("Axes...");
     private JLabel messageLabel = new JLabel("No data loaded");
 
-    private int colourBy = COLOUR_BY_TRACE;
-
-    private Class traceType = null;
+    private TraceFactory.TraceType traceType = null;
 
     private final JFrame frame;
 
@@ -128,18 +150,27 @@ public class DensityPanel extends JPanel implements Exportable {
         chartSetupButton.setFont(UIManager.getFont("SmallSystemFont"));
         toolBar.add(chartSetupButton);
 
+        JLabel label = new JLabel("Display:");
+        label.setFont(UIManager.getFont("SmallSystemFont"));
+        label.setLabelFor(displayCombo);
+        toolBar.add(label);
+        displayCombo.setFont(UIManager.getFont("SmallSystemFont"));
+        displayCombo.setOpaque(false);
+        toolBar.add(displayCombo);
+
         binsCombo.setFont(UIManager.getFont("SmallSystemFont"));
         binsCombo.setOpaque(false);
-        binsCombo.setSelectedItem(minimumBins);
+        binsCombo.setSelectedItem(currentSettings.minimumBins);
         labelBins = new JLabel("Bins:");
         labelBins.setFont(UIManager.getFont("SmallSystemFont"));
         labelBins.setLabelFor(binsCombo);
         toolBar.add(labelBins);
         toolBar.add(binsCombo);
 
-        relativeDensityCheckBox.setOpaque(false);
-        relativeDensityCheckBox.setFont(UIManager.getFont("SmallSystemFont"));
-        toolBar.add(relativeDensityCheckBox);
+        // KDE's don' do this at present so just taking up space on the toolbar...
+//        relativeDensityCheckBox.setOpaque(false);
+//        relativeDensityCheckBox.setFont(UIManager.getFont("SmallSystemFont"));
+//        toolBar.add(relativeDensityCheckBox);
 
         // Probably don't need this as an option - takes up space and
         // solid (translucent) plots look cool...
@@ -149,7 +180,7 @@ public class DensityPanel extends JPanel implements Exportable {
 //		toolBar.add(solidCheckBox);
 
         toolBar.add(new JToolBar.Separator(new Dimension(8, 8)));
-        JLabel label = new JLabel("Legend:");
+        label = new JLabel("Legend:");
         label.setFont(UIManager.getFont("SmallSystemFont"));
         label.setLabelFor(legendCombo);
         toolBar.add(label);
@@ -168,36 +199,48 @@ public class DensityPanel extends JPanel implements Exportable {
 
         toolBar.add(new JToolBar.Separator(new Dimension(8, 8)));
 
-        kdeCheckBox.setFont(UIManager.getFont("SmallSystemFont"));
-        toolBar.add(kdeCheckBox);
-
-        kdeSetupButton.putClientProperty(
-                "Quaqua.Button.style", "placard"
-        );
-        kdeSetupButton.setFont(UIManager.getFont("SmallSystemFont"));
-        toolBar.add(kdeSetupButton);
-
-        kdeSetupButton.setEnabled(kdeCheckBox.isSelected());
+//        kdeCheckBox.setFont(UIManager.getFont("SmallSystemFont"));
+//        toolBar.add(kdeCheckBox);
+//
+//        kdeSetupButton.putClientProperty(
+//                "Quaqua.Button.style", "placard"
+//        );
+//        kdeSetupButton.setFont(UIManager.getFont("SmallSystemFont"));
+//        toolBar.add(kdeSetupButton);
+//
+//        kdeSetupButton.setEnabled(kdeCheckBox.isSelected());
 
         chartSetupButton.addActionListener(
                 new java.awt.event.ActionListener() {
                     public void actionPerformed(ActionEvent actionEvent) {
-                        if (chartSetupDialog == null) {
-                            chartSetupDialog = new ChartSetupDialog(frame, true, false,
+                        if (currentSettings.chartSetupDialog == null) {
+                            currentSettings.chartSetupDialog = new ChartSetupDialog(frame, true, false,
                                     Axis.AT_MAJOR_TICK, Axis.AT_MAJOR_TICK, Axis.AT_ZERO, Axis.AT_MAJOR_TICK);
                         }
 
-                        chartSetupDialog.showDialog(densityChart);
+                        currentSettings.chartSetupDialog.showDialog(densityChart);
                         validate();
                         repaint();
                     }
                 }
         );
 
+        displayCombo.addItemListener(
+                new java.awt.event.ItemListener() {
+                    public void itemStateChanged(java.awt.event.ItemEvent ev) {
+                        currentSettings.showHistogram = displayCombo.getSelectedIndex() >= 1;
+                        currentSettings.showKDE = displayCombo.getSelectedIndex() != 1;
+
+                        binsCombo.setEnabled(currentSettings.showHistogram);
+                        setupTraces();
+                    }
+                }
+        );
+
         binsCombo.addItemListener(
                 new java.awt.event.ItemListener() {
                     public void itemStateChanged(java.awt.event.ItemEvent ev) {
-                        minimumBins = (Integer) binsCombo.getSelectedItem();
+                        currentSettings.minimumBins = (Integer) binsCombo.getSelectedItem();
                         setupTraces();
                     }
                 }
@@ -206,12 +249,8 @@ public class DensityPanel extends JPanel implements Exportable {
         relativeDensityCheckBox.addItemListener(
                 new java.awt.event.ItemListener() {
                     public void itemStateChanged(java.awt.event.ItemEvent ev) {
-                        for (int i = 0; i < densityChart.getPlotCount(); i++) {
-                            ((NumericalDensityPlot) densityChart.getPlot(i)).setRelativeDensity(relativeDensityCheckBox.isSelected());
-                        }
-                        densityChart.recalibrate();
-                        validate();
-                        repaint();
+                        currentSettings.relativeDensity = relativeDensityCheckBox.isSelected();
+                        setupTraces();
                     }
                 }
         );
@@ -219,12 +258,8 @@ public class DensityPanel extends JPanel implements Exportable {
         solidCheckBox.addItemListener(
                 new java.awt.event.ItemListener() {
                     public void itemStateChanged(java.awt.event.ItemEvent ev) {
-                        for (int i = 0; i < densityChart.getPlotCount(); i++) {
-                            ((NumericalDensityPlot) densityChart.getPlot(i)).setSolid(solidCheckBox.isSelected());
-                        }
-                        densityChart.recalibrate();
-                        validate();
-                        repaint();
+                        currentSettings.drawSolid = solidCheckBox.isSelected();
+                        setupTraces();
                     }
                 }
         );
@@ -232,37 +267,8 @@ public class DensityPanel extends JPanel implements Exportable {
         legendCombo.addItemListener(
                 new java.awt.event.ItemListener() {
                     public void itemStateChanged(java.awt.event.ItemEvent ev) {
-                        switch (legendCombo.getSelectedIndex()) {
-                            case 0:
-                                break;
-                            case 1:
-                                densityChart.setLegendAlignment(SwingConstants.NORTH_WEST);
-                                break;
-                            case 2:
-                                densityChart.setLegendAlignment(SwingConstants.NORTH);
-                                break;
-                            case 3:
-                                densityChart.setLegendAlignment(SwingConstants.NORTH_EAST);
-                                break;
-                            case 4:
-                                densityChart.setLegendAlignment(SwingConstants.WEST);
-                                break;
-                            case 5:
-                                densityChart.setLegendAlignment(SwingConstants.EAST);
-                                break;
-                            case 6:
-                                densityChart.setLegendAlignment(SwingConstants.SOUTH_WEST);
-                                break;
-                            case 7:
-                                densityChart.setLegendAlignment(SwingConstants.SOUTH);
-                                break;
-                            case 8:
-                                densityChart.setLegendAlignment(SwingConstants.SOUTH_EAST);
-                                break;
-                        }
-                        densityChart.setShowLegend(legendCombo.getSelectedIndex() != 0);
-                        validate();
-                        repaint();
+                        currentSettings.legendAlignment = legendCombo.getSelectedIndex();
+                        setupTraces();
                     }
                 }
         );
@@ -270,34 +276,34 @@ public class DensityPanel extends JPanel implements Exportable {
         colourByCombo.addItemListener(
                 new java.awt.event.ItemListener() {
                     public void itemStateChanged(java.awt.event.ItemEvent ev) {
-                        colourBy = colourByCombo.getSelectedIndex();
+                        currentSettings.colourBy = ColourByOptions.values()[colourByCombo.getSelectedIndex()];
                         setupTraces();
                     }
                 }
         );
 
-        kdeCheckBox.addItemListener(
-                new java.awt.event.ItemListener() {
-                    public void itemStateChanged(java.awt.event.ItemEvent ev) {
-                        showKDE = kdeCheckBox.isSelected();
-                        kdeSetupButton.setEnabled(showKDE);
-                        setupTraces();
-                    }
-                }
-        );
-        kdeSetupButton.addActionListener(
-                new java.awt.event.ActionListener() {
-                    public void actionPerformed(ActionEvent actionEvent) {
-                        if (kdeSetupDialog == null) {
-                            kdeSetupDialog = new KDESetupDialog(frame, true, false,
-                                    Axis.AT_MAJOR_TICK, Axis.AT_MAJOR_TICK, Axis.AT_ZERO, Axis.AT_MAJOR_TICK);
-                        }
-
-                        kdeSetupDialog.showDialog(densityChart);
-                        setupTraces();
-                    }
-                }
-        );
+//        kdeCheckBox.addItemListener(
+//                new java.awt.event.ItemListener() {
+//                    public void itemStateChanged(java.awt.event.ItemEvent ev) {
+//                        currentSettings.showKDE = kdeCheckBox.isSelected();
+//                        kdeSetupButton.setEnabled(currentSettings.showKDE);
+//                        setupTraces();
+//                    }
+//                }
+//        );
+//        kdeSetupButton.addActionListener(
+//                new java.awt.event.ActionListener() {
+//                    public void actionPerformed(ActionEvent actionEvent) {
+//                        if (currentSettings.kdeSetupDialog == null) {
+//                            currentSettings.kdeSetupDialog = new KDESetupDialog(frame, true, false,
+//                                    Axis.AT_MAJOR_TICK, Axis.AT_MAJOR_TICK, Axis.AT_ZERO, Axis.AT_MAJOR_TICK);
+//                        }
+//
+//                        currentSettings.kdeSetupDialog.showDialog(densityChart);
+//                        setupTraces();
+//                    }
+//                }
+//        );
 
         return toolBar;
     }
@@ -309,9 +315,38 @@ public class DensityPanel extends JPanel implements Exportable {
         this.traceLists = traceLists;
         this.traceNames = traceNames;
 
+        if (traceNames.size() > 0) {
+            // find the first settings for the one of the selected traces...
+            Settings settings = null;
+
+            for (String name : traceNames) {
+                settings = settingsMap.get(name);
+                if (settings != null) {
+                    break;
+                }
+            }
+            if (settings == null) {
+                // if none of the traces have settings yet, create and store one for the
+                // first selected trace
+                settings = new Settings();
+                settingsMap.put(traceNames.get(0), settings);
+            }
+            currentSettings = settings;
+        }
+
+        displayCombo.setSelectedIndex(currentSettings.showHistogram && currentSettings.showKDE ? 2 : (currentSettings.showKDE ? 0 : 1));
+        binsCombo.setEnabled(currentSettings.showHistogram);
+
+        binsCombo.setSelectedItem(currentSettings.minimumBins);
+        relativeDensityCheckBox.setSelected(currentSettings.relativeDensity);
+        legendCombo.setSelectedIndex(currentSettings.legendAlignment);
+        colourByCombo.setSelectedIndex(currentSettings.colourBy.ordinal());
+//        kdeCheckBox.setSelected(currentSettings.showKDE);
+//        kdeSetupButton.setEnabled(currentSettings.showKDE);
+
         traceType = null;
 
-        barCount = 0;
+//        barCount = 0;
         for (TraceList tl : traceLists) {
             for (String traceName : traceNames) {
                 int traceIndex = tl.getTraceIndex(traceName);
@@ -333,60 +368,42 @@ public class DensityPanel extends JPanel implements Exportable {
         }
 
         // only enable controls relevent to continuous densities...
-        relativeDensityCheckBox.setEnabled(traceType == Double.class);
-        labelBins.setEnabled(traceType == Double.class);
-        binsCombo.setEnabled(traceType == Double.class);
-        kdeCheckBox.setEnabled(traceType == Double.class);
-        kdeSetupButton.setEnabled(traceType == Double.class);
+        displayCombo.setEnabled(traceType == TraceFactory.TraceType.DOUBLE);
+        relativeDensityCheckBox.setEnabled(traceType == TraceFactory.TraceType.DOUBLE);
+        labelBins.setEnabled(traceType == TraceFactory.TraceType.DOUBLE);
+        binsCombo.setEnabled(traceType == TraceFactory.TraceType.DOUBLE);
+//        kdeCheckBox.setEnabled(traceType == TraceFactory.TraceType.DOUBLE);
+//        kdeSetupButton.setEnabled(traceType == TraceFactory.TraceType.DOUBLE);
 
         setupTraces();
     }
 
     protected Plot setupDensityPlot(TraceList tl, int traceIndex, TraceCorrelation td) {
-        Double values[] = new Double[tl.getStateCount()];
-        tl.getValues(traceIndex, values);
-        boolean[] selected = new boolean[tl.getStateCount()];
-        tl.getSelected(traceIndex, selected);
-
-        FrequencyPlot plot = new NumericalDensityPlot(Trace.arrayConvert(values, selected), minimumBins, td);
-
+        List values = tl.getValues(traceIndex);
+        NumericalDensityPlot plot = new NumericalDensityPlot(values, currentSettings.minimumBins, td);
         return plot;
     }
 
     protected Plot setupKDEPlot(TraceList tl, int traceIndex, TraceCorrelation td) {
-        Double values[] = new Double[tl.getStateCount()];
-        tl.getValues(traceIndex, values);
-        boolean[] selected = new boolean[tl.getStateCount()];
-        tl.getSelected(traceIndex, selected);
-
-        Plot plot = new KDENumericalDensityPlot(Trace.arrayConvert(values, selected), minimumBins, td);
-
+        List values = tl.getValues(traceIndex);
+        Plot plot = new KDENumericalDensityPlot(values, DEFAULT_KDE_BINS, td);
         return plot;
     }
 
     protected Plot setupIntegerPlot(TraceList tl, int traceIndex, TraceCorrelation td, int barCount, int barId) {
-        Integer values[] = new Integer[tl.getStateCount()];
-        tl.getValues(traceIndex, values);
-        boolean[] selected = new boolean[tl.getStateCount()];
-        tl.getSelected(traceIndex, selected);
-
-        CategoryDensityPlot plot = new CategoryDensityPlot(Trace.arrayConvert(values, selected), -1, td, barCount, barId);
-
+        List values = tl.getValues(traceIndex);
+        CategoryDensityPlot plot = new CategoryDensityPlot(values, -1, td, barCount, barId);
         return plot;
     }
 
     protected Plot setupCategoryPlot(TraceList tl, int traceIndex, TraceCorrelation td, Map<Integer, String> categoryDataMap, int barCount, int barId) {
-        String initValues[] = new String[tl.getStateCount()];
-        tl.getValues(traceIndex, initValues);
-        boolean[] selected = new boolean[tl.getStateCount()];
-        tl.getSelected(traceIndex, selected);
+        List values = tl.getValues(traceIndex);
 
-        String[] values = Trace.arrayConvert(initValues, selected);
-
-        int[] intData = new int[values.length];
-        for (int v = 0; v < values.length; v++) {
-            intData[v] = td.credSet.getIndex(values[v]);
-            categoryDataMap.put(intData[v], values[v]);
+        List<Double> intData = new ArrayList<Double>();
+        for (int v = 0; v < values.size(); v++) {
+            int index = td.getIndex(values.get(v).toString());
+            intData.add(v, (double) index);
+            categoryDataMap.put(index, values.get(v).toString());
         }
 
         CategoryDensityPlot plot = new CategoryDensityPlot(intData, -1, td, barCount, barId);
@@ -426,14 +443,22 @@ public class DensityPanel extends JPanel implements Exportable {
                     }
 
                     Map<Integer, String> categoryDataMap = new HashMap<Integer, String>();
-                    if (trace.getTraceType() == Double.class) {
-                        plot = setupDensityPlot(tl, traceIndex, td);
+                    if (trace.getTraceType() == TraceFactory.TraceType.DOUBLE) {
+                        if (currentSettings.showHistogram) {
+                            plot = setupDensityPlot(tl, traceIndex, td);
+                            ((NumericalDensityPlot)plot).setRelativeDensity(currentSettings.relativeDensity);
+                            ((NumericalDensityPlot)plot).setPointsOnly(currentSettings.showKDE);
+                        } else {
+                            plot = null;
+                        }
 
-                        if (showKDE) {
-                            ((NumericalDensityPlot)plot).setSolid(false);
-                            ((NumericalDensityPlot)plot).setLineStroke(null);
-                            ((NumericalDensityPlot)plot).setMarkStyle(Plot.POINT_MARK, 1, new BasicStroke(0.5f),
-                                    Color.black, Color.black);
+                        if (currentSettings.showKDE) {
+                            if (plot != null) {
+                                ((NumericalDensityPlot)plot).setSolid(false);
+                                ((NumericalDensityPlot)plot).setLineStroke(null);
+                                ((NumericalDensityPlot)plot).setMarkStyle(Plot.POINT_MARK, 1, new BasicStroke(0.5f),
+                                        Color.black, Color.black);
+                            }
 
                             Plot plot2 = setupKDEPlot(tl, traceIndex, td);
                             plot2.setName(name + " KDE");
@@ -444,44 +469,79 @@ public class DensityPanel extends JPanel implements Exportable {
                             }
                             densityChart.addPlot(plot2);
                         }
-                    } else if (trace.getTraceType() == Integer.class) {
 
-                        plot = setupIntegerPlot(tl, traceIndex, td, barCount, barId);
+                    } else if (trace.getTraceType() == TraceFactory.TraceType.INTEGER) {
+
+                        plot = setupIntegerPlot(tl, traceIndex, td, currentSettings.barCount, barId);
                         barId++;
 
-                    } else if (trace.getTraceType() == String.class) {
+                    } else if (trace.getTraceType() == TraceFactory.TraceType.STRING) {
 
-                        plot = setupCategoryPlot(tl, traceIndex, td, categoryDataMap, barCount, barId);
+                        plot = setupCategoryPlot(tl, traceIndex, td, categoryDataMap, currentSettings.barCount, barId);
                         barId++;
 
                     } else {
                         throw new RuntimeException("Trace type is not recognized: " + trace.getTraceType());
                     }
 
+                    if (plot != null) {
+                        plot.setName(name);
+                        if (tl instanceof CombinedTraces) {
+                            plot.setLineStyle(new BasicStroke(2.0f), paints[i]);
+                        } else {
+                            plot.setLineStyle(new BasicStroke(1.0f), paints[i]);
+                        }
 
-                    plot.setName(name);
-                    if (tl instanceof CombinedTraces) {
-                        plot.setLineStyle(new BasicStroke(2.0f), paints[i]);
-                    } else {
-                        plot.setLineStyle(new BasicStroke(1.0f), paints[i]);
+                        densityChart.addPlot(plot);
                     }
-
-                    densityChart.addPlot(plot);
-
-                    if (colourBy == COLOUR_BY_TRACE || colourBy == COLOUR_BY_ALL) {
+                    if (currentSettings.colourBy == ColourByOptions.COLOUR_BY_TRACE || currentSettings.colourBy == ColourByOptions.COLOUR_BY_ALL) {
                         i++;
                     }
                     if (i == paints.length) i = 0;
                 }
             }
-            if (colourBy == COLOUR_BY_FILE) {
+            if (currentSettings.colourBy == ColourByOptions.COLOUR_BY_FILE) {
                 i++;
-            } else if (colourBy == COLOUR_BY_TRACE) {
+            } else if (currentSettings.colourBy == ColourByOptions.COLOUR_BY_TRACE) {
                 i = 0;
             }
             if (i == paints.length) i = 0;
         }
 
+        switch (currentSettings.legendAlignment) {
+            case 0:
+                break;
+            case 1:
+                densityChart.setLegendAlignment(SwingConstants.NORTH_WEST);
+                break;
+            case 2:
+                densityChart.setLegendAlignment(SwingConstants.NORTH);
+                break;
+            case 3:
+                densityChart.setLegendAlignment(SwingConstants.NORTH_EAST);
+                break;
+            case 4:
+                densityChart.setLegendAlignment(SwingConstants.WEST);
+                break;
+            case 5:
+                densityChart.setLegendAlignment(SwingConstants.EAST);
+                break;
+            case 6:
+                densityChart.setLegendAlignment(SwingConstants.SOUTH_WEST);
+                break;
+            case 7:
+                densityChart.setLegendAlignment(SwingConstants.SOUTH);
+                break;
+            case 8:
+                densityChart.setLegendAlignment(SwingConstants.SOUTH_EAST);
+                break;
+        }
+        densityChart.setShowLegend(currentSettings.legendAlignment != 0);
+
+        if (currentSettings.chartSetupDialog != null) {
+            currentSettings.chartSetupDialog.applySettings(densityChart);
+        }
+
         if (traceLists.length == 1) {
             chartPanel.setXAxisTitle(traceLists[0].getName());
         } else if (traceNames.size() == 1) {
@@ -490,9 +550,9 @@ public class DensityPanel extends JPanel implements Exportable {
             chartPanel.setXAxisTitle("Multiple Traces");
         }
 
-        if (traceType == Double.class) {
+        if (traceType == TraceFactory.TraceType.DOUBLE) {
             chartPanel.setYAxisTitle("Density");
-            densityChart.setXAxis(false, new HashMap<Integer, String>());// make HashMap empty
+//            densityChart.setXAxis(false, new HashMap<Integer, String>());// make HashMap empty
         } else {
             chartPanel.setYAxisTitle("Probability");
         }
