@@ -228,7 +228,7 @@ public class JointDensityPanel extends JPanel implements Exportable {
 
         messageLabel.setText("");
 
-        if (td1.getTraceType() != TraceFactory.TraceType.CONTINUOUS && td2.getTraceType() != TraceFactory.TraceType.CONTINUOUS) {
+        if (td1.getTraceType() != TraceFactory.TraceType.DOUBLE && td2.getTraceType() != TraceFactory.TraceType.DOUBLE) {
             chartPanel.remove(correlationChart);
             chartPanel.add(tableScrollPane, "Table");
 
@@ -238,8 +238,8 @@ public class JointDensityPanel extends JPanel implements Exportable {
             cateTableProbTypeCombo.setVisible(true);
             defaultNumberFormatCheckBox.setVisible(true);
 
-            Object[] rowNames = td1.credSet.getRange().toArray();
-            Object[] colNames = td2.credSet.getRange().toArray();
+            Object[] rowNames = td1.getRange().toArray();
+            Object[] colNames = td2.getRange().toArray();
             double[][] data = categoricalPlot(td1, td2);
 
             tableScrollPane.setTable(rowNames, colNames, data, defaultNumberFormatCheckBox.isSelected());
@@ -251,7 +251,7 @@ public class JointDensityPanel extends JPanel implements Exportable {
             cateTableProbTypeCombo.setVisible(false);
             defaultNumberFormatCheckBox.setVisible(false);
 
-            if (td1.getTraceType() == TraceFactory.TraceType.CATEGORY) {
+            if (td1.getTraceType() == TraceFactory.TraceType.STRING) {
                 mixedCategoricalPlot(td1, false); // isFirstTraceListNumerical
 
                 sampleCheckBox.setVisible(false);
@@ -259,7 +259,7 @@ public class JointDensityPanel extends JPanel implements Exportable {
                 translucencyCheckBox.setVisible(false);
 
 
-            } else if (td2.getTraceType() == TraceFactory.TraceType.CATEGORY) {
+            } else if (td2.getTraceType() == TraceFactory.TraceType.STRING) {
                 mixedCategoricalPlot(td2, true); // isFirstTraceListNumerical
 
                 sampleCheckBox.setVisible(false);
@@ -287,7 +287,7 @@ public class JointDensityPanel extends JPanel implements Exportable {
 
     private void mixedCategoricalPlot(TraceDistribution td, boolean isFirstTraceListNumerical) {
         correlationChart.setXAxis(new DiscreteAxis(true, true));
-        List<String> categoryValues = td.credSet.getRange();
+        List<String> categoryValues = td.getRange();
         Map<String, TraceDistribution> categoryTdMap = new HashMap<String, TraceDistribution>();
 
         if (categoryValues == null || categoryValues.size() < 1) return;
@@ -300,65 +300,30 @@ public class JointDensityPanel extends JPanel implements Exportable {
         double samples1[] = new double[sampleSize];
         int k = 0;
 
-        boolean[] selected;
-        if (td.getTraceType() == TraceFactory.TraceType.INTEGER) {
-            Integer values[] = new Integer[maxCount];
-            selected = new boolean[maxCount];
-            if (isFirstTraceListNumerical) {
-                tl1.getValues(traceIndex1, values);
-                tl1.getSelected(traceIndex1, selected);
-            } else {
-                tl2.getValues(traceIndex2, values);
-                tl2.getSelected(traceIndex2, selected);
-            }
-
-            for (int i = 0; i < sampleSize; i++) {
-                if (selected[k]) {
-                    samples1[i] = (double) values[k];
-                } else {
-                    samples1[i] = Double.NaN;
-                }
-                k += minCount / sampleSize;
-            }
+        List values;
+        if (isFirstTraceListNumerical) {
+            values = tl1.getValues(traceIndex1);
         } else {
-            Double values[] = new Double[maxCount];
-            selected = new boolean[maxCount];
-            if (isFirstTraceListNumerical) {
-                tl1.getValues(traceIndex1, values);
-                tl1.getSelected(traceIndex1, selected);
-            } else {
-                tl2.getValues(traceIndex2, values);
-                tl2.getSelected(traceIndex2, selected);
-            }
-            for (int i = 0; i < sampleSize; i++) {
-                if (selected[k]) {
-                    samples1[i] = values[k];
-                } else {
-                    samples1[i] = Double.NaN;
-                }
-                k += minCount / sampleSize;
-            }
+            values = tl2.getValues(traceIndex2);
         }
 
+        for (int i = 0; i < sampleSize; i++) {
+            samples1[i] = ((Number) values.get(k)).doubleValue();
+            k += minCount / sampleSize;
+        }
+
+
         String samples2[] = new String[sampleSize];
         k = 0;
 
-        String values[] = new String[maxCount];
-        selected = new boolean[maxCount];
+        List values2;
         if (isFirstTraceListNumerical) {
-            tl2.getValues(traceIndex2, values);
-            tl2.getSelected(traceIndex2, selected);
+            values2 = tl2.getValues(traceIndex2);
         } else {
-            tl1.getValues(traceIndex1, values);
-            tl1.getSelected(traceIndex1, selected);
+            values2 = tl1.getValues(traceIndex1);
         }
         for (int i = 0; i < sampleSize; i++) {
-            if (selected[k]) {
-                samples2[i] = values[k];
-            } else {
-                samples2[i] = "";
-            }
-
+            samples2[i] = values2.get(k).toString();
             k += minCount / sampleSize;
         }
 
@@ -372,7 +337,7 @@ public class JointDensityPanel extends JPanel implements Exportable {
                 }
             }
 
-            TraceDistribution categoryTd = new TraceDistribution(sepValues[i].toArray());
+            TraceDistribution categoryTd = new TraceDistribution(sepValues[i], TraceFactory.TraceType.DOUBLE); // todo ?
             categoryTdMap.put(categoryValues.get(i), categoryTd);
         }
 
@@ -383,8 +348,8 @@ public class JointDensityPanel extends JPanel implements Exportable {
     }
 
     private double[][] categoricalPlot(TraceDistribution td1, TraceDistribution td2) {
-        List<String> rowNames = td1.credSet.getRange();
-        List<String> colNames = td2.credSet.getRange();
+        List<String> rowNames = td1.getRange();
+        List<String> colNames = td2.getRange();
 
         double[][] data = new double[rowNames.size()][colNames.size()];
 
@@ -398,70 +363,29 @@ public class JointDensityPanel extends JPanel implements Exportable {
         String samples1[] = new String[sampleSize];
         int k = 0;
 
-        boolean[] selected;
-        if (td1.getTraceType() == TraceFactory.TraceType.INTEGER) {
-            Integer values[] = new Integer[maxCount];
-            tl1.getValues(traceIndex1, values);
-            selected = new boolean[maxCount];
-            tl1.getSelected(traceIndex1, selected);
-
-            for (int i = 0; i < sampleSize; i++) {
-                if (selected[k]) {
-                    samples1[i] = values[k].toString();
-                } else {
-                    samples1[i] = "";
-                }
-
-                k += minCount / sampleSize; // = 1 for non-continous vs non-continous
-            }
-        } else {
-            String values[] = new String[maxCount];
-            tl1.getValues(traceIndex1, values);
-            selected = new boolean[maxCount];
-            tl1.getSelected(traceIndex1, selected);
-
-            for (int i = 0; i < sampleSize; i++) {
-                if (selected[k]) {
-                    samples1[i] = values[k];
-                } else {
-                    samples1[i] = "";
-                }
-
-                k += minCount / sampleSize;
+        List values = tl1.getValues(traceIndex1);
+        TraceFactory.TraceType type = tl1.getTrace(traceIndex1).getTraceType();
+        for (int i = 0; i < sampleSize; i++) {
+            if (type == TraceFactory.TraceType.INTEGER) { // as Integer is stored as Double in Trace
+                samples1[i] = Integer.toString( ((Number) values.get(k)).intValue() );
+            } else {
+                samples1[i] = values.get(k).toString();
             }
+            k += minCount / sampleSize; // = 1 for non-continous vs non-continous
         }
 
         String samples2[] = new String[sampleSize];
         k = 0;
 
-        if (td2.getTraceType() == TraceFactory.TraceType.INTEGER) {
-            Integer values[] = new Integer[maxCount];
-            tl2.getValues(traceIndex2, values);
-            selected = new boolean[maxCount];
-            tl2.getSelected(traceIndex2, selected);
-            for (int i = 0; i < sampleSize; i++) {
-                if (selected[k]) {
-                    samples2[i] = values[k].toString();
-                } else {
-                    samples2[i] = "";
-                }
-                k += minCount / sampleSize;
-            }
-        } else {
-            String values[] = new String[maxCount];
-            tl2.getValues(traceIndex2, values);
-            selected = new boolean[maxCount];
-            tl2.getSelected(traceIndex2, selected);
-
-            for (int i = 0; i < sampleSize; i++) {
-                if (selected[k]) {
-                    samples2[i] = values[k];
-                } else {
-                    samples2[i] = "";
-                }
-
-                k += minCount / sampleSize;
+        values = tl2.getValues(traceIndex2);
+        type = tl2.getTrace(traceIndex2).getTraceType();
+        for (int i = 0; i < sampleSize; i++) {
+            if (type == TraceFactory.TraceType.INTEGER) { // as Integer is stored as Double in Trace
+                samples2[i] = Integer.toString( ((Number) values.get(k)).intValue() );
+            } else {
+                samples2[i] = values.get(k).toString();
             }
+            k += minCount / sampleSize;
         }
 
         // calculate count
@@ -531,82 +455,35 @@ public class JointDensityPanel extends JPanel implements Exportable {
             }
         }
 
-        double samples1[] = new double[sampleSize];
         int k = 0;
-
-        boolean[] selected;
         if (td1.getTraceType() == TraceFactory.TraceType.INTEGER) {
             correlationChart.setXAxis(new DiscreteAxis(true, true));
-
-            Integer values[] = new Integer[maxCount];
-            tl1.getValues(traceIndex1, values);
-            selected = new boolean[maxCount];
-            tl1.getSelected(traceIndex1, selected);
-
-            for (int i = 0; i < sampleSize; i++) {
-                if (selected[k]) {
-                    samples1[i] = (double) values[k];
-                } else {
-                    samples1[i] = Double.NaN;
-                }
-                k += minCount / sampleSize;
-            }
         } else {
             correlationChart.setXAxis(new LinearAxis());
+        }
+        List values = tl1.getValues(traceIndex1);
 
-            Double values[] = new Double[maxCount];
-            tl1.getValues(traceIndex1, values);
-            selected = new boolean[maxCount];
-            tl1.getSelected(traceIndex1, selected);
-
-            for (int i = 0; i < sampleSize; i++) {
-                if (selected[k]) {
-                    samples1[i] = values[k];
-                } else {
-                    samples1[i] = Double.NaN;
-                }
-                k += minCount / sampleSize;
-            }
+        List<Double> samples1 = new ArrayList<Double>();
+        for (int i = 0; i < sampleSize; i++) {
+            samples1.add(i, ((Number) values.get(k)).doubleValue());
+            k += minCount / sampleSize;
         }
 
-        double samples2[] = new double[sampleSize];
         k = 0;
-
         if (td2.getTraceType() == TraceFactory.TraceType.INTEGER) {
             correlationChart.setYAxis(new DiscreteAxis(true, true));
-
-            Integer values[] = new Integer[maxCount];
-            tl2.getValues(traceIndex2, values);
-            selected = new boolean[maxCount];
-            tl2.getSelected(traceIndex2, selected);
-
-            for (int i = 0; i < sampleSize; i++) {
-                if (selected[k]) {
-                    samples2[i] = (double) values[k];
-                } else {
-                    samples2[i] = Double.NaN;
-                }
-                k += minCount / sampleSize;
-            }
         } else {
             correlationChart.setYAxis(new LinearAxis());
+        }
+        values = tl2.getValues(traceIndex2);
 
-            Double values[] = new Double[maxCount];
-            tl2.getValues(traceIndex2, values);
-            selected = new boolean[maxCount];
-            tl2.getSelected(traceIndex2, selected);
-
-            for (int i = 0; i < sampleSize; i++) {
-                if (selected[k]) {
-                    samples2[i] = values[k];
-                } else {
-                    samples2[i] = Double.NaN;
-                }
-                k += minCount / sampleSize;
-            }
+        List<Double> samples2 = new ArrayList<Double>();
+        for (int i = 0; i < sampleSize; i++) {
+            samples2.add(i, ((Number) values.get(k)).doubleValue());
+            k += minCount / sampleSize;
         }
 
-        ScatterPlot plot = new ScatterPlot(removeNaN(samples1), removeNaN(samples2));
+        ScatterPlot plot = new ScatterPlot(samples1, samples2);
         plot.setMarkStyle(pointsCheckBox.isSelected() ? Plot.POINT_MARK : Plot.CIRCLE_MARK, pointsCheckBox.isSelected() ? 1.0 : 3.0,
                 new BasicStroke(2.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER),
                 new Color(16, 16, 64, translucencyCheckBox.isSelected() ? 32 : 255),
@@ -614,22 +491,22 @@ public class JointDensityPanel extends JPanel implements Exportable {
         correlationChart.addPlot(plot);
     }
 
-    private double[] removeNaN(double[] sample) {
-        List<Double> selectedValuesList = new ArrayList<Double>();
-
-        for (int i = 0; i < sample.length; i++) {
-            if (sample[i] != Double.NaN) {
-                selectedValuesList.add(sample[i]);
-            }
-        }
-
-        double[] dest = new double[selectedValuesList.size()];
-        for (int i = 0; i < dest.length; i++) {
-            dest[i] = selectedValuesList.get(i).doubleValue();
-        }
-
-        return dest;
-    }
+//    private double[] removeNaN(double[] sample) {
+//        List<Double> selectedValuesList = new ArrayList<Double>();
+//
+//        for (int i = 0; i < sample.length; i++) {
+//            if (sample[i] != Double.NaN) {
+//                selectedValuesList.add(sample[i]);
+//            }
+//        }
+//
+//        double[] dest = new double[selectedValuesList.size()];
+//        for (int i = 0; i < dest.length; i++) {
+//            dest[i] = selectedValuesList.get(i).doubleValue();
+//        }
+//
+//        return dest;
+//    }
 
     public JComponent getExportableComponent() {
         return chartPanel;
