@@ -1,7 +1,7 @@
 /*
  * NexusImporter.java
  *
- * Copyright (C) 2002-2010 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -329,8 +329,8 @@ public class NexusImporter extends Importer implements SequenceImporter, TreeImp
 
                 NexusImporter.NexusBlock block = findNextBlock();
 
-                if (block == NexusImporter.TAXA_BLOCK) {
-
+                if (block == NexusImporter.TAXA_BLOCK && taxonList[0] == null) {
+                    // only read the taxon list if one hasn't been set already...
                     taxonList[0] = readTaxaBlock();
 
                 } else if (block == NexusImporter.TREES_BLOCK) {
@@ -543,7 +543,7 @@ public class NexusImporter extends Importer implements SequenceImporter, TreeImp
 
                 for (i = 0; i < taxonCount; i++) {
 
-                    String token = readToken();
+                    String token = readToken().trim();
 
                     Sequence sequence;
 
@@ -556,7 +556,7 @@ public class NexusImporter extends Importer implements SequenceImporter, TreeImp
                         Taxon taxon;
 
                         if (taxonList != null) {
-                            int index = taxonList.getTaxonIndex(token);
+                            int index = taxonList.getTaxonIndex(token.trim());
                             if (index == -1) {
                                 // taxon not found in taxon list...
                                 // ...perhaps it is a numerical taxon reference?
@@ -565,7 +565,7 @@ public class NexusImporter extends Importer implements SequenceImporter, TreeImp
                                 taxon = taxonList.getTaxon(index);
                             }
                         } else {
-                            taxon = new Taxon(token);
+                            taxon = new Taxon(token.trim());
                         }
 
                         sequence.setTaxon(taxon);
@@ -618,7 +618,7 @@ public class NexusImporter extends Importer implements SequenceImporter, TreeImp
         } else {
 
             for (i = 0; i < taxonCount; i++) {
-                String token = readToken();
+                String token = readToken().trim();
 
                 Sequence sequence = new Sequence();
                 sequence.setDataType(dataType);
@@ -685,8 +685,8 @@ public class NexusImporter extends Importer implements SequenceImporter, TreeImp
         Taxa taxa = new Taxa();
 
         do {
-            String name = readToken(";");
-            if (name.trim().length() > 0) {
+            String name = readToken(";").trim();
+            if (name.length() > 0) {
                 Taxon taxon = new Taxon(name);
                 taxa.addTaxon(taxon);
             }
@@ -700,8 +700,8 @@ public class NexusImporter extends Importer implements SequenceImporter, TreeImp
 
         int duplicateTaxon = TaxonList.Utils.findDuplicateTaxon(taxa);
         if (duplicateTaxon >= 0)
-            throw new IllegalArgumentException("Find duplicate taxon name: " + taxa.getTaxon(duplicateTaxon).getId() +
-                      "!\nAll taxon names should be unique.");
+            throw new IllegalArgumentException("Tree contains duplicate taxon name: " + taxa.getTaxon(duplicateTaxon).getId() +
+                    "!\nAll taxon names should be unique.");
 
         return taxa;
     }
@@ -1246,6 +1246,19 @@ public class NexusImporter extends Importer implements SequenceImporter, TreeImp
     }
 
     static void parseMetaCommentPairs(String meta, Attributable item) throws Importer.BadFormatException {
+        if (meta.startsWith("B ")) {
+            // a MrBayes annotation
+            String[] parts = meta.split(" ");
+            if (parts.length == 3 && parts[1].length() > 0 && parts[2].length() > 0) {
+                item.setAttribute(parts[1], parseValue(parts[2]));
+            } else if (parts.length == 2 && parts[1].length() > 0) {
+                item.setAttribute(parts[1], Boolean.TRUE);
+            } else {
+                throw new Importer.BadFormatException("Badly formatted attribute: '" + meta + "'");
+            }
+            return;
+        }
+
         // This regex should match key=value pairs, separated by commas
         // This can match the following types of meta comment pairs:
         // value=number, value="string", value={item1, item2, item3}
