@@ -540,6 +540,8 @@ public interface Tree extends TaxonList, Units, Identifiable, Attributable {
                 }
                 if (!ignore.contains(id)) {
                     leafCount[0] = 1;
+                } else {
+                    leafCount[0] = 0;
                 }
                 return false;
             }
@@ -572,6 +574,31 @@ public interface Tree extends TaxonList, Units, Identifiable, Attributable {
             return false;
         }
 
+        public static NodeRef getCommonAncestor(Tree tree, NodeRef n1, NodeRef n2) {
+            while( n1 != n2 ) {
+                if( tree.getNodeHeight(n1) < tree.getNodeHeight(n2) ) {
+                    n1 = tree.getParent(n1);
+                } else {
+                    n2 = tree.getParent(n2);
+                }
+            }
+            return n1;
+        }
+
+        // A lightweight version for finding the most recent common ancestor of a group of taxa.
+        // return the node-ref of the MRCA.
+
+        // would be nice to use nodeRef's, but they are not preserved :(
+        public static NodeRef getCommonAncestor(Tree tree, int[] nodes) {
+            NodeRef cur = tree.getNode(nodes[0]);
+
+            for(int k = 1; k < nodes.length; ++k) {
+                cur = getCommonAncestor(tree, cur, tree.getNode(nodes[k]));
+            }
+            return cur;
+        }
+
+
         /**
          * @param tree
          * @param range
@@ -772,6 +799,32 @@ public interface Tree extends TaxonList, Units, Identifiable, Attributable {
             }
         }
 
+        // populate  postOrderList with node numbers in posorder: parent before children.
+        //  postOrderList is pre-allocated with the right size
+        public static void preOrderTraversalList(Tree tree, int[] postOrderList) {
+
+            final int nodeCount = tree.getNodeCount();
+            if (postOrderList.length != nodeCount) {
+                throw new IllegalArgumentException("Illegal list length");
+            }
+            postOrderList[0] = tree.getRoot().getNumber();
+            preOrderTraversalList(tree, 0, postOrderList);
+        }
+
+        static int preOrderTraversalList(Tree tree, int idx, int[] postOrderList) {
+            final NodeRef node = tree.getNode(postOrderList[idx]);
+            for(int i = 0; i < tree.getChildCount(node); ++i) {
+                final NodeRef child = tree.getChild(node, i);
+                idx += 1;
+                postOrderList[idx] = child.getNumber();
+                if( ! tree.isExternal(child) ) {
+                  idx = preOrderTraversalList(tree, idx, postOrderList);
+                }
+            }
+            return idx;
+        }
+
+
         /**
          * determine postorder successor of a node
          *
@@ -931,7 +984,14 @@ public interface Tree extends TaxonList, Units, Identifiable, Attributable {
                         buffer.append((k + 1));
                     }
                 } else {
-                    buffer.append(tree.getTaxonId(node.getNumber()));
+                    String label = tree.getTaxonId(node.getNumber());
+                    if (label.contains(" ") || label.contains(":") || label.contains(";") || label.contains(",")) {
+                        buffer.append("\"");
+                        buffer.append(label);
+                        buffer.append("\"");
+                    } else {
+                        buffer.append(label);
+                    }
                 }
             } else {
                 buffer.append("(");
