@@ -155,8 +155,9 @@ public class ARGSubtreeSlideOperator extends AbstractCoercableOperator {
 				if (tree.isRoot(newChild)) {
 
 					if (true) {
+                        tree.endTreeEdit();
 						try {
-							tree.endTreeEdit();
+                            tree.checkTreeIsValid();
 						} catch (MutableTree.InvalidTreeException e) {
 							e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
 						}
@@ -211,8 +212,9 @@ public class ARGSubtreeSlideOperator extends AbstractCoercableOperator {
 
 				//               System.err.println("Intermediate slide up:\n"+tree.toGraphString());
 
+                tree.endTreeEdit();
 				try {
-					tree.endTreeEdit();
+                    tree.checkTreeIsValid();
 				} catch (MutableTree.InvalidTreeException ite) {
 					throw new RuntimeException(ite.toString());
 				}
@@ -270,7 +272,7 @@ public class ARGSubtreeSlideOperator extends AbstractCoercableOperator {
 //                	System.err.println("Grabbing random child.");
 //                	logq -= Math.log(2); // TODO check ratio
 //                }
-//                //NodeRef oops 
+//                //NodeRef oops
 				//newParent =
 
 //                logq += Math.log(possibleDestinations);
@@ -344,8 +346,9 @@ public class ARGSubtreeSlideOperator extends AbstractCoercableOperator {
 //				System.err.println("newHeight = " + newHeight);
 				//              System.err.println("After slide down:\n"+tree.toGraphString());
 
+                tree.endTreeEdit();
 				try {
-					tree.endTreeEdit();
+					tree.checkTreeIsValid();
 				} catch (MutableTree.InvalidTreeException ite) {
 					throw new RuntimeException(ite.toString());
 				}
@@ -387,7 +390,7 @@ public class ARGSubtreeSlideOperator extends AbstractCoercableOperator {
                 tree.setNodeTrait(j, tmp);
             }
 
-        } 
+        }
 */
 
 		// todo fix
@@ -470,7 +473,7 @@ public class ARGSubtreeSlideOperator extends AbstractCoercableOperator {
 //        double thisHeight = tree.getNodeHeight(node);
 //        double height0 = tree.getNodeHeight(parent0);
 //        double height1 = tree.getNodeHeight(parent1);
-//       
+//
 //        if( (height0 < height) && (height1 < height) ) return 0;
 //        int count = 0;
 //        if( (thisHeight < height) && (height0 >= height) ) {
@@ -495,7 +498,7 @@ public class ARGSubtreeSlideOperator extends AbstractCoercableOperator {
 //        }
 //        if( count > 0 )
 //        	return count;
-//        
+//
 //        return intersectingEdges(tree, tree.getChild(node,0), height, directChildren);
 //    }
 
