@@ -8,7 +8,6 @@
  */
 package dr.evomodel.arg.operators;
 
-import dr.evolution.tree.MutableTree;
 import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
 import dr.evomodel.arg.ARGModel;
@@ -257,12 +256,13 @@ public class ObsoleteARGNewEventOperator extends AbstractCoercableOperator {
         }
 
         arg.pushTreeSizeChangedEvent();
-        try {
-            arg.endTreeEdit();
-        } catch (MutableTree.InvalidTreeException ite) {
-            throw new RuntimeException(ite.toString() + "\n" + arg.toString()
-                    + "\n" + Tree.Utils.uniqueNewick(arg, arg.getRoot()));
-        }
+        arg.endTreeEdit();
+//        try {
+//            arg.checkTreeIsValid();
+//        } catch (MutableTree.InvalidTreeException ite) {
+//            throw new RuntimeException(ite.toString() + "\n" + arg.toString()
+//                    + "\n" + Tree.Utils.uniqueNewick(arg, arg.getRoot()));
+//        }
 
 //	    double d1 = findPotentialAttachmentPoints(reverseBifurcationHeight,null);
 //	    double d2 = findPotentialAttachmentPoints(reverseReassortmentHeight,null);
@@ -293,7 +293,7 @@ public class ObsoleteARGNewEventOperator extends AbstractCoercableOperator {
     private void checkAllHeights() {
         int len = arg.getInternalNodeCount();
         System.err.println("# internal nodes = " + len);
-        int n = internalNodeParameters.getNumberOfParameters();
+        int n = internalNodeParameters.getParameterCount();
         System.err.println("VSCP (" + n + ")");
         for (int i = 0; i < n; i++) {
             System.err.println(internalNodeParameters.getParameterValue(i));
@@ -514,12 +514,13 @@ public class ObsoleteARGNewEventOperator extends AbstractCoercableOperator {
 
         arg.pushTreeSizeChangedEvent();
 
-        try {
-            arg.endTreeEdit();
-        } catch (MutableTree.InvalidTreeException ite) {
-            throw new RuntimeException(ite.toString() + "\n" + arg.toString()
-                    + "\n" + Tree.Utils.uniqueNewick(arg, arg.getRoot()));
-        }
+        arg.endTreeEdit();
+//        try {
+//            arg.checkTreeIsValid();
+//        } catch (MutableTree.InvalidTreeException ite) {
+//            throw new RuntimeException(ite.toString() + "\n" + arg.toString()
+//                    + "\n" + Tree.Utils.uniqueNewick(arg, arg.getRoot()));
+//        }
 
 //	    double d1 = findPotentialNodesToRemove(null);
 //	    System.err.printf("d1 = %5.4f\n",d1);
