@@ -1,7 +1,7 @@
 /*
- * SkylineLikelihood.java
+ * GMRFSkyrideLikelihood.java
  *
- * Copyright (C) 2002-2006 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2011 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -25,7 +25,6 @@
 
 package dr.evomodel.coalescent;
 
-
 import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
 import dr.evomodel.tree.TreeModel;
@@ -87,6 +86,10 @@ public class GMRFSkyrideLikelihood extends OldAbstractCoalescentLikelihood {
 		super(GMRFSkyrideLikelihoodParser.SKYLINE_LIKELIHOOD);
 	}
 
+    public GMRFSkyrideLikelihood(String name) {
+		super(name);
+	}
+
 	public GMRFSkyrideLikelihood(Tree tree, Parameter popParameter, Parameter groupParameter, Parameter precParameter,
 	                             Parameter lambda, Parameter beta, MatrixParameter dMatrix,
 	                             boolean timeAwareSmoothing, boolean rescaleByRootHeight) {
@@ -393,7 +396,7 @@ public class GMRFSkyrideLikelihood extends OldAbstractCoalescentLikelihood {
 	 * given a demographic model.
      * @return coalescent part of density
 	 */
-	private double calculateLogCoalescentLikelihood() {
+	protected double calculateLogCoalescentLikelihood() {
 
 		if (!intervalsKnown) {
 			// intervalsKnown -> false when handleModelChanged event occurs in super.
@@ -414,7 +417,7 @@ public class GMRFSkyrideLikelihood extends OldAbstractCoalescentLikelihood {
 		return currentLike;// + LogNormalDistribution.logPdf(Math.exp(popSizeParameter.getParameterValue(coalescentIntervals.length - 1)), mu, sigma);
 	}
 
-    private double calculateLogFieldLikelihood() {
+    protected double calculateLogFieldLikelihood() {
 
         if (!intervalsKnown) {
             // intervalsKnown -> false when handleModelChanged event occurs in super.
