@@ -4,6 +4,7 @@ import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
 import dr.evolution.tree.TreeTrait;
 import dr.evolution.tree.TreeTraitProvider;
+import dr.evolution.util.Taxon;
 import dr.evomodel.branchratemodel.BranchRateModel;
 import dr.evomodel.tree.TreeModel;
 import dr.evomodelxml.treelikelihood.TreeTraitParserUtilities;
@@ -13,6 +14,7 @@ import dr.inference.loggers.NumberColumn;
 import dr.inference.model.*;
 import dr.math.distributions.MultivariateDistribution;
 import dr.math.distributions.MultivariateNormalDistribution;
+import dr.util.Author;
 import dr.util.Citable;
 import dr.util.Citation;
 import dr.util.CommonCitations;
@@ -46,6 +48,9 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
     public static final String RECIPROCAL_RATES = "reciprocalRates";
     public static final String PRIOR_SAMPLE_SIZE = "priorSampleSize";
     public static final String RANDOM_SAMPLE = "randomSample";
+    public static final String IGNORE_PHYLOGENY = "ignorePhylogeny";
+    public static final String ASCERTAINMENT = "ascertainedTaxon";
+    public static final String EXCHANGEABLE_TIPS = "exchangeableTips";
 
     public AbstractMultivariateTraitLikelihood(String traitName,
                                                TreeModel treeModel,
@@ -59,6 +64,23 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                                                Model samplingDensity,
                                                boolean reportAsMultivariate,
                                                boolean reciprocalRates) {
+        this(traitName, treeModel, diffusionModel, traitParameter, null, missingIndices, cacheBranches,
+                scaleByTime, useTreeLength, rateModel, samplingDensity, reportAsMultivariate, reciprocalRates);
+    }
+
+    public AbstractMultivariateTraitLikelihood(String traitName,
+                                               TreeModel treeModel,
+                                               MultivariateDiffusionModel diffusionModel,
+                                               CompoundParameter traitParameter,
+                                               Parameter deltaParameter,
+                                               List<Integer> missingIndices,
+                                               boolean cacheBranches,
+                                               boolean scaleByTime,
+                                               boolean useTreeLength,
+                                               BranchRateModel rateModel,
+                                               Model samplingDensity,
+                                               boolean reportAsMultivariate,
+                                               boolean reciprocalRates) {
 
         super(TRAIT_LIKELIHOOD);
 
@@ -71,6 +93,13 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
         addModel(treeModel);
         addModel(diffusionModel);
 
+        this.deltaParameter = deltaParameter;
+        if (deltaParameter != null) {
+            addVariable(deltaParameter);
+        }
+
+
+
         if (rateModel != null) {
             hasRateModel = true;
             addModel(rateModel);
@@ -97,6 +126,19 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
         this.useTreeLength = useTreeLength;
         this.reciprocalRates = reciprocalRates;
 
+        dimTrait = diffusionModel.getPrecisionmatrix().length;
+        dim = traitParameter != null ? traitParameter.getParameter(0).getDimension() : 0;
+        numData = dim / dimTrait;
+
+        if (dim % dimTrait != 0)
+            throw new RuntimeException("dim is not divisible by dimTrait");
+
+        recalculateTreeLength();                           
+        printInformtion();
+
+    }
+
+    protected void printInformtion() {
         StringBuffer sb = new StringBuffer("Creating multivariate diffusion model:\n");
         sb.append("\tTrait: ").append(traitName).append("\n");
         sb.append("\tDiffusion process: ").append(diffusionModel.getId()).append("\n");
@@ -115,11 +157,31 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
         sb.append("\tPlease cite:\n");
         sb.append(Citable.Utils.getCitationString(this));
 
-        Logger.getLogger("dr.evomodel").info(sb.toString());
 
-        recalculateTreeLength();
+        sb.append("\n\tDiffusion dimension   : ").append(dimTrait).append("\n");
+        sb.append(  "\tNumber of observations: ").append(numData).append("\n");
+        Logger.getLogger("dr.evomodel").info(sb.toString());
     }
 
+    private static Citable TraitAscertainmentCitation = new Citable() {//} implements Citable {
+
+        public List<Citation> getCitations() {
+            List<Citation> list = new ArrayList<Citation>();
+            list.add(
+                    new Citation(
+                            new Author[]{
+                                    new Author("MA", "Suchard"),
+                                    new Author("J", "Novembre"),
+                                    new Author("B", "von Holdt"),
+                                    new Author("G", "Cybis"),
+                            },
+                            Citation.Status.IN_PREPARATION
+                    )
+            );
+            return list;
+        }
+    };
+
     public List<Citation> getCitations() {
         List<Citation> citations = new ArrayList<Citation>();
         citations.add(
@@ -134,6 +196,20 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
         return traitParameter;
     }
 
+    public void setAscertainedTaxon(Taxon taxon) {
+        ascertainedTaxonIndex = treeModel.getTaxonIndex(taxon);
+        if (ascertainedTaxonIndex == -1) {
+            throw new RuntimeException("Taxon " + taxon.getId() + " is not in tree " + treeModel.getId());
+        }
+        doAscertainmentCorrect = true;
+        StringBuilder sb = new StringBuilder("Enabling ascertainment correction for multivariate trait model: ");
+        sb.append(getId()).append("\n");
+        sb.append("\tTaxon: ").append(taxon.getId()).append("\n");
+        sb.append("\tPlease cite:\n");
+        sb.append(Citable.Utils.getCitationString(TraitAscertainmentCitation));
+        Logger.getLogger("dr.evomodel").info(sb.toString());
+    }
+
     public double getRescaledBranchLength(NodeRef node) {
 
         double length = treeModel.getBranchLength(node);
@@ -146,8 +222,13 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
             }
         }
 
-        if (scaleByTime)
-            return length / treeLength;
+        if (scaleByTime) {
+            length /= treeLength;
+        }
+
+        if (deltaParameter != null && treeModel.isExternal(node)) {
+            length += deltaParameter.getParameterValue(0);
+        }
 
         return length;
     }
@@ -175,7 +256,10 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
         else if (model == treeModel) {
             if (object instanceof TreeModel.TreeChangedEvent) {
                 TreeModel.TreeChangedEvent event = (TreeModel.TreeChangedEvent) object;
-                if (event.isHeightChanged()) {
+                if (event.isTreeChanged()) {
+                    recalculateTreeLength();
+                    updateAllNodes();
+                } else  if (event.isHeightChanged()) {
                     recalculateTreeLength();
                     if (useTreeLength || (scaleByTime && treeModel.isRoot(event.getNode())))
                         updateAllNodes();
@@ -192,18 +276,18 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                         updateNodeAndChildren(event.getNode());
                     }
                 } else {
-                    throw new RuntimeException("Unexpected TreeModel TreeChangedEvent occuring in AbstractMultivariateTraitLikelihood");
+                    throw new RuntimeException("Unexpected TreeModel TreeChangedEvent occurring in AbstractMultivariateTraitLikelihood");
                 }
             } else if (object instanceof Parameter) {
                 // Ignoring
             } else {
-                throw new RuntimeException("Unexpected TreeModel event occurring in AbstractMultivariateTraitLikelihood");
+                throw new RuntimeException("Unexpected object throwing events in AbstractMultivariateTraitLikelihood");
             }
         } else if (model == rateModel) {
             if (index == -1) {
                 updateAllNodes();
             } else {
-                if (((Parameter) object).getDimension() == 2 * (treeModel.getNodeCount() - 1))
+                if (object == null || ((Parameter) object).getDimension() == 2 * (treeModel.getNodeCount() - 1))
                     updateNode(treeModel.getNode(index)); // This is a branch specific update
                 else
                     updateAllNodes(); // Probably an epoch model
@@ -254,6 +338,13 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
     // **************************************************************
 
     protected void handleVariableChangedEvent(Variable variable, int index, Parameter.ChangeType type) {
+        if (variable == deltaParameter) {
+            likelihoodKnown = false;
+        }
+
+        if (variable == traitParameter) {
+            likelihoodKnown = false;
+        }
 
         // All parameter changes are handled first by the treeModel
         if (!cacheBranches)
@@ -331,11 +422,18 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
     public final double getLogLikelihood() {
         if (!likelihoodKnown) {
             logLikelihood = calculateLogLikelihood();
+            if (doAscertainmentCorrect) {
+                double correction = calculateAscertainmentCorrection(ascertainedTaxonIndex);
+//                System.err.println("Correction = " + correction);
+                logLikelihood -= correction;
+            }
             likelihoodKnown = true;
         }
         return logLikelihood;
     }
 
+    protected abstract double calculateAscertainmentCorrection(int taxonIndex);
+
     public abstract double getLogDataLikelihood();
 
     public void makeDirty() {
@@ -408,6 +506,14 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
         return null;
     }
 
+    public final int getNumData() {
+        return numData;
+    }
+
+    public final int getDimTrait() {
+        return dimTrait;
+    }
+
     protected double[] getRootNodeTrait() {
         return treeModel.getMultivariateNodeTrait(treeModel.getRoot(), traitName);
     }
@@ -446,7 +552,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
             boolean useTreeLength = xo.getAttribute(USE_TREE_LENGTH, false);
             boolean scaleByTime = xo.getAttribute(SCALE_BY_TIME, false);
             boolean reciprocalRates = xo.getAttribute(RECIPROCAL_RATES, false);
-            boolean reportAsMultivariate = xo.getAttribute(REPORT_MULTIVARIATE, false);
+            boolean reportAsMultivariate = xo.getAttribute(REPORT_MULTIVARIATE, true);
 
             BranchRateModel rateModel = (BranchRateModel) xo.getChild(BranchRateModel.class);
 
@@ -454,7 +560,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
             String traitName = TreeTraitParserUtilities.DEFAULT_TRAIT_NAME;
 
             TreeTraitParserUtilities.TraitsAndMissingIndices returnValue =
-                utilities.parseTraitsFromTaxonAttributes(xo, traitName, treeModel, integrate);
+                    utilities.parseTraitsFromTaxonAttributes(xo, traitName, treeModel, integrate);
             CompoundParameter traitParameter = returnValue.traitParameter;
             List<Integer> missingIndices = returnValue.missingIndices;
             traitName = returnValue.traitName;
@@ -466,6 +572,11 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                 samplingDensity = (Model) cxo.getChild(Model.class);
             }
 
+            Parameter deltaParameter = null;
+            if (xo.hasChildNamed("delta")) {
+                XMLObject cxo = xo.getChild("delta");
+                deltaParameter = (Parameter) cxo.getChild(Parameter.class);
+            }
             AbstractMultivariateTraitLikelihood like;
 
             if (integrate) {
@@ -490,6 +601,7 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                         throw new XMLParseException("Must specify a conjugate or multivariate normal root prior");
                     }
 
+                    boolean ignorePhylogeny = xo.getAttribute(IGNORE_PHYLOGENY, false);
 
                     Parameter meanParameter = (Parameter) cxo.getChild(MultivariateDistributionLikelihood.MVN_MEAN)
                             .getChild(Parameter.class);
@@ -503,10 +615,19 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                     double[] mean = meanParameter.getParameterValues();
                     double pseudoObservations = sampleSizeParameter.getParameterValue(0);
 
-                    like = new FullyConjugateMultivariateTraitLikelihood(traitName, treeModel, diffusionModel,
-                            traitParameter, missingIndices, cacheBranches,
-                            scaleByTime, useTreeLength, rateModel, samplingDensity, reportAsMultivariate,
-                            mean, pseudoObservations, reciprocalRates);
+                    if (ignorePhylogeny) {
+                        boolean exchangeableTips = xo.getAttribute(EXCHANGEABLE_TIPS, true);
+
+                        like = new NonPhylogeneticMultivariateTraitLikelihood(traitName, treeModel, diffusionModel,
+                                traitParameter, deltaParameter, missingIndices, cacheBranches,
+                                scaleByTime, useTreeLength, rateModel, samplingDensity, reportAsMultivariate,
+                                mean, pseudoObservations, reciprocalRates, exchangeableTips);
+                    } else {
+                        like = new FullyConjugateMultivariateTraitLikelihood(traitName, treeModel, diffusionModel,
+                                traitParameter, deltaParameter, missingIndices, cacheBranches,
+                                scaleByTime, useTreeLength, rateModel, samplingDensity, reportAsMultivariate,
+                                mean, pseudoObservations, reciprocalRates);
+                    }
                 }
             } else {
 
@@ -530,6 +651,16 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                 like.check(check);
             }
 
+            if (xo.hasChildNamed(ASCERTAINMENT)) {
+                XMLObject cxo = xo.getChild(ASCERTAINMENT);
+                Taxon taxon = (Taxon) cxo.getChild(Taxon.class);
+                if (!integrate) {
+                    throw new XMLParseException("Ascertainment correction is currently only implemented" +
+                        " for integrated multivariate trait likelihood models");
+                }
+                like.setAscertainedTaxon(taxon);
+            }
+
             return like;
         }
 
@@ -551,16 +682,22 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                 new ElementRule(TreeTraitParserUtilities.TRAIT_PARAMETER, new XMLSyntaxRule[]{
                         new ElementRule(Parameter.class)
                 }),
+                new ElementRule("delta", new XMLSyntaxRule[]{
+                        new ElementRule(Parameter.class)
+                }, true),
                 AttributeRule.newBooleanRule(INTEGRATE, true),
 //                new XORRule(
-                        new ElementRule(MultivariateDistributionLikelihood.class, true),
-                        new ElementRule(CONJUGATE_ROOT_PRIOR, new XMLSyntaxRule[]{
-                                new ElementRule(MultivariateDistributionLikelihood.MVN_MEAN,
-                                        new XMLSyntaxRule[]{new ElementRule(Parameter.class)}),
-                                new ElementRule(PRIOR_SAMPLE_SIZE,
-                                        new XMLSyntaxRule[]{new ElementRule(Parameter.class)}),
-                        }, true),
+                new ElementRule(MultivariateDistributionLikelihood.class, true),
+                new ElementRule(CONJUGATE_ROOT_PRIOR, new XMLSyntaxRule[]{
+                        new ElementRule(MultivariateDistributionLikelihood.MVN_MEAN,
+                                new XMLSyntaxRule[]{new ElementRule(Parameter.class)}),
+                        new ElementRule(PRIOR_SAMPLE_SIZE,
+                                new XMLSyntaxRule[]{new ElementRule(Parameter.class)}),
+                }, true),
 //                        true),
+                new ElementRule(ASCERTAINMENT, new XMLSyntaxRule[] {
+                        new ElementRule(Taxon.class)
+                }, true),
                 new ElementRule(MultivariateDiffusionModel.class),
                 new ElementRule(TreeModel.class),
                 new ElementRule(BranchRateModel.class, true),
@@ -569,7 +706,10 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
                 AttributeRule.newBooleanRule(USE_TREE_LENGTH, true),
                 AttributeRule.newBooleanRule(SCALE_BY_TIME, true),
                 AttributeRule.newBooleanRule(RECIPROCAL_RATES, true),
+                AttributeRule.newBooleanRule(CACHE_BRANCHES, true),
                 AttributeRule.newIntegerRule(RANDOM_SAMPLE, true),
+                AttributeRule.newBooleanRule(IGNORE_PHYLOGENY, true),
+                AttributeRule.newBooleanRule(EXCHANGEABLE_TIPS, true),
                 new ElementRule(Parameter.class, true),
                 TreeTraitParserUtilities.randomizeRules(true),
                 TreeTraitParserUtilities.jitterRules(true),
@@ -612,5 +752,15 @@ public abstract class AbstractMultivariateTraitLikelihood extends AbstractModelL
     protected double[] storedCachedLogLikelihood;
     protected boolean[] validLogLikelihoods;
     protected boolean[] storedValidLogLikelihoods;
+
+    private final Parameter deltaParameter;
+
+    private boolean doAscertainmentCorrect = false;
+    private int ascertainedTaxonIndex;
+
+    protected int numData;
+    protected int dimTrait;
+    protected int dim;
+
 }
 
