@@ -4,8 +4,11 @@ import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
 import dr.evomodel.branchratemodel.BranchRateModel;
 import dr.evomodel.tree.TreeModel;
-import dr.inference.model.*;
 import dr.inference.loggers.LogColumn;
+import dr.inference.model.CompoundParameter;
+import dr.inference.model.Model;
+import dr.inference.model.Parameter;
+import dr.inference.model.Variable;
 import dr.math.distributions.MultivariateNormalDistribution;
 import dr.math.distributions.WishartSufficientStatistics;
 import dr.math.matrixAlgebra.Matrix;
@@ -38,16 +41,24 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
                                                  boolean reportAsMultivariate,
                                                  boolean reciprocalRates) {
 
-        super(traitName, treeModel, diffusionModel, traitParameter, missingIndices, cacheBranches, scaleByTime,
+        this(traitName, treeModel, diffusionModel, traitParameter, null, missingIndices, cacheBranches, scaleByTime,
+                useTreeLength, rateModel, samplingDensity, reportAsMultivariate, reciprocalRates);
+    }
+
+    public IntegratedMultivariateTraitLikelihood(String traitName,
+                                                 TreeModel treeModel,
+                                                 MultivariateDiffusionModel diffusionModel,
+                                                 CompoundParameter traitParameter,
+                                                 Parameter deltaParameter,
+                                                 List<Integer> missingIndices,
+                                                 boolean cacheBranches, boolean scaleByTime, boolean useTreeLength,
+                                                 BranchRateModel rateModel, Model samplingDensity,
+                                                 boolean reportAsMultivariate,
+                                                 boolean reciprocalRates) {
+
+        super(traitName, treeModel, diffusionModel, traitParameter, deltaParameter, missingIndices, cacheBranches, scaleByTime,
                 useTreeLength, rateModel, samplingDensity, reportAsMultivariate, reciprocalRates);
 
-        dimTrait = diffusionModel.getPrecisionmatrix().length;
-        dim = traitParameter.getParameter(0).getDimension();
-
-        if (dim % dimTrait != 0)
-            throw new RuntimeException("dim is not divisible by dimTrait");
-
-        numData = dim / dimTrait;
 
         meanCache = new double[dim * treeModel.getNodeCount()];
         drawnStates = new double[dim * treeModel.getNodeCount()];
@@ -74,10 +85,6 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
         setTipDataValuesForAllNodes(missingIndices);
 
-        StringBuffer sb = new StringBuffer();
-        sb.append("\tDiffusion dimension: ").append(dimTrait).append("\n");
-        sb.append("\tNumber of observations: ").append(numData).append("\n");
-        Logger.getLogger("dr.evomodel").info(sb.toString());
     }
 
     private void setTipDataValuesForAllNodes(List<Integer> missingIndices) {
@@ -103,6 +110,9 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
         // Set tip data values
         int index = node.getNumber();
         double[] traitValue = traitParameter.getParameter(index).getParameterValues();
+        if (traitValue.length < dim) {
+            throw new RuntimeException("The trait parameter for the tip with index, " + index + ", is too short");
+        }
         System.arraycopy(traitValue, 0, meanCache, dim * index, dim);
         missing[index] = false;
     }
@@ -640,22 +650,26 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
             }
         }
 
-        if (!treeModel.isExternal(node)) {
+        if (peel() && !treeModel.isExternal(node)) {
             preOrderTraverseSample(treeModel, treeModel.getChild(node, 0), thisIndex, treePrecision, treeVariance);
             preOrderTraverseSample(treeModel, treeModel.getChild(node, 1), thisIndex, treePrecision, treeVariance);
         }
     }
 
+    protected boolean peel() {
+        return true;
+    }
+
     public LogColumn[] getColumns() {
         return new LogColumn[]{
                 new LikelihoodColumn(getId())};
     }
 
-    private boolean areStatesRedrawn = false;
+    protected boolean areStatesRedrawn = false;
 
     protected double[] meanCache;
     protected double[] upperPrecisionCache;
-    private double[] lowerPrecisionCache;
+    protected double[] lowerPrecisionCache;
     private double[] logRemainderDensityCache;
 
     protected boolean[] missing;
@@ -667,10 +681,6 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
     private double[] drawnStates;
 
-    protected int numData;
-    protected int dimTrait;
-    protected int dim;
-
     protected final boolean integrateRoot = true; // Set to false if conditioning on root value (not fully implemented)
     protected static boolean DEBUG = false;
     protected static boolean DEBUG_PREORDER = false;
@@ -683,4 +693,5 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
     protected double[] Ay;
     protected double[][] tmpM;
     protected double[] tmp2;
+
 }
