@@ -1,7 +1,7 @@
 /*
  * TreeModel.java
  *
- * Copyright (C) 2002-2006 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -50,6 +50,8 @@ public class TreeModel extends AbstractModel implements MutableTree {
 
     public static final String TREE_MODEL = "treeModel";
 
+    private static final boolean TEST_NODE_BOUNDS = false;
+
     public TreeModel(String name) {
         super(name);
         nodeCount = 0;
@@ -83,7 +85,7 @@ public class TreeModel extends AbstractModel implements MutableTree {
         // any zero branches.
         MutableTree.Utils.correctHeightsForTips(binaryTree);
 
-        // clone the node structure (this will create the individual parameters
+        // clone the node structure (this will create the individual parameters)
         Node node = new Node(binaryTree, binaryTree.getRoot());
 
         internalNodeCount = binaryTree.getInternalNodeCount();
@@ -127,11 +129,19 @@ public class TreeModel extends AbstractModel implements MutableTree {
     }
 
 
+    boolean heightBoundsSetup = false;
+
     public void setupHeightBounds() {
 
+        if (heightBoundsSetup) {
+            throw new IllegalArgumentException("Node height bounds set up twice");
+        }
+
         for (int i = 0; i < nodeCount; i++) {
             nodes[i].setupHeightBounds();
         }
+
+        heightBoundsSetup = true;
     }
 
     /**
@@ -248,8 +258,8 @@ public class TreeModel extends AbstractModel implements MutableTree {
             return parameter == node.traitParameters.get(name);
         }
 
-        public boolean areAllInternalHeightsChanged(){
-            if(parameter != null){
+        public boolean areAllInternalHeightsChanged() {
+            if (parameter != null) {
                 return parameter == node.heightParameter && index == CHANGE_IN_ALL_INTERNAL_NODES;
             }
             return false;
@@ -461,7 +471,7 @@ public class TreeModel extends AbstractModel implements MutableTree {
         root = (Node) newRoot;
 
         // We shouldn't need this because the addChild will already have fired appropriate events.
-         pushTreeChangedEvent(root);
+        pushTreeChangedEvent(root);
     }
 
     public void addChild(NodeRef p, NodeRef c) {
@@ -502,7 +512,7 @@ public class TreeModel extends AbstractModel implements MutableTree {
         return false;
     }
 
-    public void endTreeEdit() throws MutableTree.InvalidTreeException {
+    public void endTreeEdit() {
         if (!inEdit) throw new RuntimeException("Not in edit transaction mode!");
 
         inEdit = false;
@@ -511,9 +521,11 @@ public class TreeModel extends AbstractModel implements MutableTree {
             swapParameterObjects(oldRoot, root);
         }
 
-        for (Node node : nodes) {
-            if (!node.heightParameter.isWithinBounds()) {
-                throw new InvalidTreeException("height parameter out of bounds");
+        if (TEST_NODE_BOUNDS) {
+            try {
+                checkTreeIsValid();
+            } catch (InvalidTreeException ite) {
+                throw new RuntimeException(ite.getMessage());
             }
         }
 
@@ -523,19 +535,12 @@ public class TreeModel extends AbstractModel implements MutableTree {
         treeChangedEvents.clear();
     }
 
-    public void endTreeEditUnsafe() throws MutableTree.InvalidTreeException {
-        if (!inEdit) throw new RuntimeException("Not in edit transaction mode!");
-
-        inEdit = false;
-
-        if (root != oldRoot) {
-            swapParameterObjects(oldRoot, root);
+    public void checkTreeIsValid() throws MutableTree.InvalidTreeException {
+        for (Node node : nodes) {
+            if (!node.heightParameter.isWithinBounds()) {
+                throw new InvalidTreeException("height parameter out of bounds");
+            }
         }
-
-        for (TreeChangedEvent treeChangedEvent : treeChangedEvents) {
-            listenerHelper.fireModelChanged(this, treeChangedEvent);
-        }
-        treeChangedEvents.clear();
     }
 
     public void setNodeHeight(NodeRef n, double height) {
@@ -877,13 +882,20 @@ public class TreeModel extends AbstractModel implements MutableTree {
             if (node.heightParameter == parameter) {
                 return node;
             }
-            if (hasRates && node.rateParameter == parameter) {
-                return node;
+        }
+
+        if (hasRates) {
+            for (Node node : nodes) {
+                if (node.rateParameter == parameter) {
+                    return node;
+                }
             }
         }
-        for (Node node : nodes) {
-            if (hasTraits && node.traitParameters.containsValue(parameter)) {
-                return node;
+        if (hasTraits) {
+            for (Node node : nodes) {
+                if (node.traitParameters.containsValue(parameter)) {
+                    return node;
+                }
             }
         }
         throw new RuntimeException("Parameter not found in any nodes:" + parameter.getId() + " " + parameter.hashCode());
@@ -1073,7 +1085,6 @@ public class TreeModel extends AbstractModel implements MutableTree {
             n1.rateParameter.setParameterValueQuietly(0, rate1);
             n2.rateParameter.setParameterValueQuietly(0, rate2);
         }
-
     }
 
     // **************************************************************
