@@ -31,6 +31,7 @@ import dr.evolution.alignment.SitePatterns;
 import dr.evolution.datatype.DataType;
 import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
+import dr.evolution.util.Taxon;
 import dr.evolution.util.TaxonList;
 import dr.evomodel.branchratemodel.BranchRateModel;
 import dr.evomodel.branchratemodel.DefaultBranchRateModel;
@@ -59,7 +60,7 @@ public class TreeLikelihood extends AbstractTreeLikelihood {
                           TreeModel treeModel,
                           SiteModel siteModel,
                           BranchRateModel branchRateModel,
-                          TipPartialsModel tipPartialsModel,
+                          TipStatesModel tipStatesModel,
                           boolean useAmbiguities,
                           boolean allowMissingTaxa,
                           boolean storePartials,
@@ -77,7 +78,7 @@ public class TreeLikelihood extends AbstractTreeLikelihood {
             this.frequencyModel = siteModel.getFrequencyModel();
             addModel(frequencyModel);
 
-            this.tipPartialsModel = tipPartialsModel;
+            this.tipStatesModel = tipStatesModel;
 
             integrateAcrossCategories = siteModel.integrateAcrossCategories();
 
@@ -108,9 +109,15 @@ public class TreeLikelihood extends AbstractTreeLikelihood {
                         likelihoodCore = new AminoAcidLikelihoodCore();
                     }
 
-                } else if (dataType instanceof dr.evolution.datatype.Codons) {
                     // The codon core was out of date and did nothing more than the general core...
-                    likelihoodCore = new GeneralLikelihoodCore(patternList.getStateCount());
+                } else if (dataType instanceof dr.evolution.datatype.Codons) {
+                    if (!forceJavaCore && NativeGeneralLikelihoodCore.isAvailable()) {
+                        coreName = "native general";
+                        likelihoodCore = new NativeGeneralLikelihoodCore(patternList.getStateCount());
+                    } else {
+                        coreName = "Java general";
+                        likelihoodCore = new GeneralLikelihoodCore(patternList.getStateCount());
+                    }
                     useAmbiguities = true;
                 } else {
                     if (!forceJavaCore && NativeGeneralLikelihoodCore.isAvailable()) {
@@ -147,8 +154,8 @@ public class TreeLikelihood extends AbstractTreeLikelihood {
             int extNodeCount = treeModel.getExternalNodeCount();
             int intNodeCount = treeModel.getInternalNodeCount();
 
-            if (tipPartialsModel != null) {
-                tipPartialsModel.setTree(treeModel);
+            if (tipStatesModel != null) {
+                tipStatesModel.setTree(treeModel);
 
                 tipPartials = new double[patternCount * stateCount];
 
@@ -162,11 +169,11 @@ public class TreeLikelihood extends AbstractTreeLikelihood {
                                 ", is not found in patternList, " + patternList.getId());
                     }
 
-                    tipPartialsModel.setStates(patternList, index, i, id);
+                    tipStatesModel.setStates(patternList, index, i, id);
                     likelihoodCore.createNodePartials(i);
                 }
 
-                addModel(tipPartialsModel);
+                addModel(tipStatesModel);
                 //useAmbiguities = true;
             } else {
                 for (int i = 0; i < extNodeCount; i++) {
@@ -253,9 +260,14 @@ public class TreeLikelihood extends AbstractTreeLikelihood {
 
             updateAllNodes();
 
-        } else if (model == tipPartialsModel) {
-
-            updateAllNodes();
+        } else if (model == tipStatesModel) {
+        	if(object instanceof Taxon)
+        	{
+        		for(int i=0; i<treeModel.getNodeCount(); i++)
+        			if(treeModel.getNodeTaxon(treeModel.getNode(i))!=null && treeModel.getNodeTaxon(treeModel.getNode(i)).getId().equalsIgnoreCase(((Taxon)object).getId()))
+        				updateNode(treeModel.getNode(i));
+        	}else
+        		updateAllNodes();
 
         } else if (model instanceof SiteModel) {
 
@@ -324,12 +336,12 @@ public class TreeLikelihood extends AbstractTreeLikelihood {
             }
         }
 
-        if (tipPartialsModel != null) {
+        if (tipStatesModel != null) {
             int extNodeCount = treeModel.getExternalNodeCount();
             for (int index = 0; index < extNodeCount; index++) {
                 if (updateNode[index]) {
                     likelihoodCore.setNodePartialsForUpdate(index);
-                    tipPartialsModel.getTipPartials(index, tipPartials);
+                    tipStatesModel.getTipPartials(index, tipPartials);
                     likelihoodCore.setCurrentNodePartials(index, tipPartials);
                 }
             }
@@ -392,7 +404,7 @@ public class TreeLikelihood extends AbstractTreeLikelihood {
     @param patternLogProbs log pattern probabilities
     @return the log total probability for a pattern.
     */
-    protected double getAscertainmentCorrection(double[] patternLogProbs) {               
+    protected double getAscertainmentCorrection(double[] patternLogProbs) {
         if (patternList instanceof AscertainedSitePatterns) {
             return ((AscertainedSitePatterns) patternList).getAscertainmentCorrection(patternLogProbs);
         } else {
@@ -589,7 +601,7 @@ public class TreeLikelihood extends AbstractTreeLikelihood {
     /**
      * the tip partials model
      */
-    private final TipPartialsModel tipPartialsModel;
+    private final TipStatesModel tipStatesModel;
 
     private final boolean storePartials;
 
