@@ -1,7 +1,7 @@
 /*
  * MarkovChain.java
  *
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -56,7 +56,7 @@ public final class MarkovChain {
     private boolean pleaseStop = false;
     private boolean isStopped = false;
     private double bestScore, currentScore, initialScore;
-    private int currentLength;
+    private long currentLength;
 
     private boolean useCoercion = true;
 
@@ -100,12 +100,12 @@ public final class MarkovChain {
      *               <p/>
      *               param onTheFlyOperatorWeights
      */
-    public int runChain(int length, boolean disableCoerce /*,int onTheFlyOperatorWeights*/) {
+    public long runChain(long length, boolean disableCoerce /*,int onTheFlyOperatorWeights*/) {
 
         likelihood.makeDirty();
         currentScore = evaluate(likelihood, prior);
 
-        int currentState = currentLength;
+        long currentState = currentLength;
 
         final Model currentModel = likelihood.getModel();
 
@@ -131,14 +131,24 @@ public final class MarkovChain {
             if (likelihood instanceof CompoundLikelihood) {
                 message += ": " + ((CompoundLikelihood) likelihood).getDiagnosis();
             } else {
-                message += "!";
+                message += ".";
+            }
+            throw new IllegalArgumentException(message);
+        } else if (currentScore == Double.POSITIVE_INFINITY || Double.isNaN(currentScore)) {
+            String message = "A likelihood returned with a numerical error";
+            if (likelihood instanceof CompoundLikelihood) {
+                message += ": " + ((CompoundLikelihood) likelihood).getDiagnosis();
+            } else {
+                message += ".";
             }
             throw new IllegalArgumentException(message);
         }
 
-        pleaseStop = false;
+            pleaseStop = false;
         isStopped = false;
 
+        String diagnostic = "";
+
         //int otfcounter = onTheFlyOperatorWeights > 0 ? onTheFlyOperatorWeights : 0;
 
         double[] logr = {0.0};
@@ -226,7 +236,21 @@ public final class MarkovChain {
 
                 // assert Profiler.stopProfile("Evaluate");
 
+                if (score == Double.POSITIVE_INFINITY || Double.isNaN(score)) {
+                    if (likelihood instanceof CompoundLikelihood) {
+                        Logger.getLogger("error").severe("A likelihood returned with a numerical error:\n" +
+                                ((CompoundLikelihood)likelihood).getDiagnosis());
+                    } else {
+                        Logger.getLogger("error").severe("A likelihood returned with a numerical error.");
+                    }
+
+                    // If the user has chosen to ignore this error then we transform it
+                    // to a negative infinity so the state is rejected.
+                    score = Double.NEGATIVE_INFINITY;
+                }
+
                 if (usingFullEvaluation) {
+
                     // This is a test that the state is correctly restored. The
                     // restored state is fully evaluated and the likelihood compared with
                     // that before the operation was made.
@@ -273,7 +297,11 @@ public final class MarkovChain {
 //                    }
 //                }
 
-                oldScore = score; // for the usingFullEvaluation test
+                if (usingFullEvaluation) {
+                    oldScore = score; // for the usingFullEvaluation test
+                    diagnostic = likelihood instanceof CompoundLikelihood ?
+                            ((CompoundLikelihood)likelihood).getDiagnosis() : "";
+                }
             } else {
                 if (DEBUG) {
                     System.out.println("** Move rejected: new score = " + score
@@ -293,25 +321,23 @@ public final class MarkovChain {
                 // restored state is fully evaluated and the likelihood compared with
                 // that before the operation was made.
 
-                final String d1 = likelihood instanceof CompoundLikelihood ?
-                        ((CompoundLikelihood)likelihood).getDiagnosis() : "";
                 likelihood.makeDirty();
                 final double testScore = evaluate(likelihood, prior);
 
+                final String d2 = likelihood instanceof CompoundLikelihood ?
+                        ((CompoundLikelihood)likelihood).getDiagnosis() : "";
+
                 if (Math.abs(testScore - oldScore) > EVALUATION_TEST_THRESHOLD) {
 
+
                     final Logger logger = Logger.getLogger("error");
                     logger.severe("State was not correctly restored after reject step.\n"
                             + "Likelihood before: " + oldScore
                             + " Likelihood after: " + testScore
                             + "\n" + "Operator: " + mcmcOperator
-                            + " " + mcmcOperator.getOperatorName());
-                    if( d1.length() > 0 ) {
-                        logger.severe(d1);
-                        final String d2 = ((CompoundLikelihood)likelihood).getDiagnosis();
-                        logger.severe(d1);
-                        logger.severe(d2);
-                    }
+                            + " " + mcmcOperator.getOperatorName()
+                            + (diagnostic.length() > 0 ? "\n\nDetails\nBefore: " + diagnostic + "\nAfter: " + d2 : "")
+                    );
                     fullEvaluationError = true;
                 }
             }
@@ -331,7 +357,7 @@ public final class MarkovChain {
                     if (fullEvaluationError) {
                         // If there has been an error then stop with an error
                         throw new RuntimeException(
-                                "One or more evaluation errors occured during the test phase of this\n" +
+                                "One or more evaluation errors occurred during the test phase of this\n" +
                                         "run. These errors imply critical errors which may produce incorrect\n" +
                                         "results.");
                     }
@@ -429,11 +455,11 @@ public final class MarkovChain {
         return bestScore;
     }
 
-    public int getCurrentLength() {
+    public long getCurrentLength() {
         return currentLength;
     }
 
-    public void setCurrentLength(int currentLength) {
+    public void setCurrentLength(long currentLength) {
         this.currentLength = currentLength;
     }
 
@@ -514,20 +540,20 @@ public final class MarkovChain {
         listeners.remove(listener);
     }
 
-    private void fireBestModel(int state, Model bestModel) {
+    private void fireBestModel(long state, Model bestModel) {
 
         for (MarkovChainListener listener : listeners) {
             listener.bestState(state, bestModel);
         }
     }
 
-    private void fireCurrentModel(int state, Model currentModel) {
+    private void fireCurrentModel(long state, Model currentModel) {
         for (MarkovChainListener listener : listeners) {
             listener.currentState(state, currentModel);
         }
     }
 
-    private void fireFinished(int chainLength) {
+    private void fireFinished(long chainLength) {
 
         for (MarkovChainListener listener : listeners) {
             listener.finished(chainLength);
