@@ -49,7 +49,7 @@ import org.apache.commons.math.distribution.BetaDistributionImpl;
  */
 public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
 
-    public MarginalLikelihoodEstimator(String id, int chainLength, int burninLength, int pathSteps,
+    public MarginalLikelihoodEstimator(String id, int chainLength, int burninLength, int pathSteps, double fixedRunValue,
 //                                       boolean linear, boolean lacing,
 PathScheme scheme,
 PathLikelihood pathLikelihood,
@@ -61,9 +61,10 @@ MCLogger logger) {
         this.pathSteps = pathSteps;
         this.scheme = scheme;
         this.schedule = schedule;
-        // depricated
-//        this.linear = (scheme == PathScheme.LINEAR);
-//        this.lacing = false; // Was not such a good idea
+        this.fixedRunValue = fixedRunValue;
+        // deprecated
+        // this.linear = (scheme == PathScheme.LINEAR);
+        // this.lacing = false; // Was not such a good idea
 
         this.burninLength = burninLength;
 
@@ -95,7 +96,7 @@ MCLogger logger) {
         for (pathParameter = scheme.nextPathParameter(); pathParameter >= 0; pathParameter = scheme.nextPathParameter()) {
             pathLikelihood.setPathParameter(pathParameter);
             reportIteration(pathParameter, chainLength, burnin);
-            int cl = mc.getCurrentLength();
+            long cl = mc.getCurrentLength();
             mc.setCurrentLength(0);
             mc.runChain(burnin, false/*, 0*/);
             mc.setCurrentLength(cl);
@@ -120,20 +121,82 @@ MCLogger logger) {
         abstract double nextPathParameter();
     }
 
+    public class FixedThetaRun extends Integrator {
+    	private double value;
+
+    	public FixedThetaRun(double value) {
+    		super(1);
+    		this.value = value;
+    	}
+
+    	double nextPathParameter() {
+    		if (step == 0) {
+    			step++;
+    			return value;
+    		} else {
+    			return -1.0;
+    		}
+    	}
+
+    }
+
     public class LinearIntegrator extends Integrator {
         public LinearIntegrator(int pathSteps) {
             super(pathSteps);
         }
 
         double nextPathParameter() {
-            if (step > pathSteps)
+            if (step > pathSteps) {
                 return -1;
-            double pathParameter = 1.0 - step / (pathSteps - 1);
+            }
+            double pathParameter = 1.0 - (double)step / (double)(pathSteps - 1);
             step = step + 1;
             return pathParameter;
         }
     }
 
+    public class SigmoidIntegrator extends Integrator {
+    	private double alpha;
+
+    	public SigmoidIntegrator(double alpha, int pathSteps) {
+    		super(pathSteps);
+    		this.alpha = alpha;
+    	}
+
+    	double nextPathParameter() {
+    		if (step == 0) {
+    			step++;
+    			return 1.0;
+    		} else if (step == pathSteps) {
+    			step++;
+    			return 0.0;
+    		} else if (step > pathSteps) {
+    			return -1.0;
+    		} else {
+    			double xvalue = ((pathSteps - step)/((double)pathSteps)) - 0.5;
+    			step++;
+    			return Math.exp(alpha*xvalue)/(Math.exp(alpha*xvalue) + Math.exp(-alpha*xvalue));
+    		}
+    	}
+    }
+
+    public class BetaQuantileIntegrator extends Integrator {
+    	private double alpha;
+
+    	public BetaQuantileIntegrator(double alpha, int pathSteps) {
+    		super(pathSteps);
+    		this.alpha = alpha;
+    	}
+
+    	double nextPathParameter() {
+    		if (step > pathSteps)
+                return -1;
+    		double result = Math.pow((pathSteps - step)/((double)pathSteps), 1.0/alpha);
+    		step++;
+    		return result;
+    	}
+    }
+
     public class BetaIntegrator extends Integrator {
         private BetaDistributionImpl betaDistribution;
 
@@ -172,7 +235,7 @@ MCLogger logger) {
             if (step > pathSteps) {
                 return -1;
             }
-            if (step == pathSteps - 1) {
+            if (step == pathSteps) { //pathSteps instead of pathSteps - 1
                 step += 1;
                 return 0;
             }
@@ -182,23 +245,23 @@ MCLogger logger) {
         }
     }
 
-    public void linearIntegration() {
+    /*public void linearIntegration() {
         setDefaultBurnin();
         mc.setCurrentLength(0);
         for (int step = 0; step < pathSteps; step++) {
             pathLikelihood.setPathParameter(pathParameter);
             reportIteration(pathParameter, chainLength, burnin);
-            mc.runChain(chainLength + burnin, false/*, 0*/);
+            //mc.runChain(chainLength + burnin, false, 0);
+            mc.runChain(chainLength + burnin, false);
             pathParameter -= pathDelta;
-
         }
-
         pathLikelihood.setPathParameter(0.0);
         reportIteration(pathParameter, chainLength, burnin);
-        mc.runChain(chainLength + burnin, false/*, 0*/);
-    }
+        //mc.runChain(chainLength + burnin, false, 0);
+        mc.runChain(chainLength + burnin, false);
+    }*/
 
-    public void betaIntegration(double alpha, double beta) {
+    /*public void betaIntegration(double alpha, double beta) {
         setDefaultBurnin();
         mc.setCurrentLength(0);
 
@@ -219,22 +282,23 @@ MCLogger logger) {
             }
             pathLikelihood.setPathParameter(pathParameter);
             reportIteration(pathParameter, chainLength, burnin);
-            mc.runChain(chainLength + burnin, false/*, 0*/);
+            //mc.runChain(chainLength + burnin, false, 0);
+            mc.runChain(chainLength + burnin, false);
             (new OperatorAnalysisPrinter(schedule)).showOperatorAnalysis(System.out);
             ((CombinedOperatorSchedule) schedule).reset();
         }
-    }
+    }*/
 
-    private void reportIteration(double pathParameter, int cl, int burn) {
-        System.out.println("Attempting theta = " + pathParameter + " for " + cl + " iterations + " + burn + " burnin.");
+    private void reportIteration(double pathParameter, long chainLength, long burnin) {
+        System.out.println("Attempting theta = " + pathParameter + " for " + chainLength + " iterations + " + burnin + " burnin.");
     }
 
     public void run() {
 
         logger.startLogging();
         mc.addMarkovChainListener(chainListener);
-/*
-        switch (scheme) {
+
+        /*switch (scheme) {
             case LINEAR:
                 linearIntegration();
                 break;
@@ -249,9 +313,12 @@ MCLogger logger) {
                 break;
             default:
                 throw new RuntimeException("Illegal path scheme");
-        }    */
+        }*/
 
         switch (scheme) {
+        	case FIXED:
+        		integrate(new FixedThetaRun(fixedRunValue));
+        		break;
             case LINEAR:
                 integrate(new LinearIntegrator(pathSteps));
                 break;
@@ -264,6 +331,12 @@ MCLogger logger) {
             case BETA:
                 integrate(new BetaIntegrator(alphaFactor, betaFactor, pathSteps));
                 break;
+            case BETA_QUANTILE:
+            	integrate(new BetaQuantileIntegrator(alphaFactor, pathSteps));
+            	break;
+            case SIGMOID:
+            	integrate(new SigmoidIntegrator(alphaFactor, pathSteps));
+            	break;
             default:
                 throw new RuntimeException("Illegal path scheme");
         }
@@ -279,7 +352,7 @@ MCLogger logger) {
         /**
          * Called to update the current model keepEvery states.
          */
-        public void currentState(int state, Model currentModel) {
+        public void currentState(long state, Model currentModel) {
 
             currentState = state;
 
@@ -291,14 +364,14 @@ MCLogger logger) {
         /**
          * Called when a new new best posterior state is found.
          */
-        public void bestState(int state, Model bestModel) {
+        public void bestState(long state, Model bestModel) {
             currentState = state;
         }
 
         /**
          * cleans up when the chain finishes (possibly early).
          */
-        public void finished(int chainLength) {
+        public void finished(long chainLength) {
             currentState = chainLength;
             (new OperatorAnalysisPrinter(schedule)).showOperatorAnalysis(System.out);
 //            logger.log(currentState);
@@ -363,10 +436,14 @@ MCLogger logger) {
             if (xo.hasAttribute(PRERUN)) {
                 prerunLength = xo.getIntegerAttribute(PRERUN);
             }
+            double fixedRunValue = -1.0;
+            if (xo.hasAttribute(FIXED_VALUE)) {
+            	fixedRunValue = xo.getDoubleAttribute(FIXED_VALUE);
+            }
 
-            // deciprated
+            // deprecated
             boolean linear = xo.getAttribute(LINEAR, true);
-//            boolean lacing = xo.getAttribute(LACING,false);
+            // boolean lacing = xo.getAttribute(LACING,false);
             PathScheme scheme;
             if (linear) {
                 scheme = PathScheme.LINEAR;
@@ -375,7 +452,7 @@ MCLogger logger) {
             }
 
             // new approach
-            if (xo.hasAttribute(PATH_SCHEME)) { // change to: getAttribute once depricated approach removed
+            if (xo.hasAttribute(PATH_SCHEME)) { // change to: getAttribute once deprecated approach removed
                 scheme = PathScheme.parseFromString(xo.getAttribute(PATH_SCHEME, PathScheme.LINEAR.getText()));
             }
 
@@ -407,7 +484,7 @@ MCLogger logger) {
             }
 
             MarginalLikelihoodEstimator mle = new MarginalLikelihoodEstimator(MARGINAL_LIKELIHOOD_ESTIMATOR, chainLength,
-                    burninLength, pathSteps, scheme, pathLikelihood, os, logger);
+                    burninLength, pathSteps, fixedRunValue, scheme, pathLikelihood, os, logger);
 
             if (!xo.getAttribute(SPAWN, true))
                 mle.setSpawnable(false);
@@ -420,18 +497,24 @@ MCLogger logger) {
                 mle.setBetaFactor(xo.getAttribute(BETA, 0.5));
             }
 
-            String alphaBetaText = "(";
+            String alphaBetaText = "";
             if (scheme == PathScheme.ONE_SIDED_BETA) {
-                alphaBetaText += "1," + mle.getBetaFactor() + ")";
+                alphaBetaText += "(1," + mle.getBetaFactor() + ")";
             } else if (scheme == PathScheme.BETA) {
-                alphaBetaText += mle.getAlphaFactor() + "," + mle.getBetaFactor() + ")";
+                alphaBetaText += "(" + mle.getAlphaFactor() + "," + mle.getBetaFactor() + ")";
+            } else if (scheme == PathScheme.BETA_QUANTILE) {
+            	alphaBetaText += "(" + mle.getAlphaFactor() + ")";
+            } else if (scheme == PathScheme.SIGMOID) {
+            	alphaBetaText += "(" + mle.getAlphaFactor() + ")";
             }
             java.util.logging.Logger.getLogger("dr.inference").info("\nCreating the Marginal Likelihood Estimator chain:" +
                     "\n  chainLength=" + chainLength +
                     "\n  pathSteps=" + pathSteps +
                     "\n  pathScheme=" + scheme.getText() + alphaBetaText +
                     "\n  If you use these results, please cite:" +
-                    "\n    Alekseyenko, Rambaut, Lemey and Suchard (in preparation)");
+                    "\n    Guy Baele, Philippe Lemey, Trevor Bedford, Andrew Rambaut, Marc A. Suchard, and Alexander V. Alekseyenko." +
+                    "\n    2012. Improving the accuracy of demographic and molecular clock model comparison while accommodating " +
+                    "\n          phylogenetic uncertainty. Mol. Biol. Evol. (in press).");
             return mle;
         }
 
@@ -460,6 +543,7 @@ MCLogger logger) {
                 AttributeRule.newBooleanRule(LACING, true),
                 AttributeRule.newBooleanRule(SPAWN, true),
                 AttributeRule.newStringRule(PATH_SCHEME, true),
+                AttributeRule.newDoubleRule(FIXED_VALUE, true),
                 AttributeRule.newDoubleRule(ALPHA, true),
                 AttributeRule.newDoubleRule(BETA, true),
                 new ElementRule(MCMC,
@@ -480,10 +564,13 @@ MCLogger logger) {
     }
 
     enum PathScheme {
+    	FIXED("fixed"),
         LINEAR("linear"),
         GEOMETRIC("geometric"),
         BETA("beta"),
-        ONE_SIDED_BETA("oneSidedBeta");
+        ONE_SIDED_BETA("oneSidedBeta"),
+        BETA_QUANTILE("betaQuantile"),
+        SIGMOID("sigmoid");
 
         PathScheme(String text) {
             this.text = text;
@@ -515,17 +602,18 @@ MCLogger logger) {
 
     private String id = null;
 
-    private int currentState;
-    private final int chainLength;
+    private long currentState;
+    private final long chainLength;
 
-    private int burnin;
-    private final int burninLength;
+    private long burnin;
+    private final long burninLength;
     private int pathSteps;
     //    private final boolean linear;
     //    private final boolean lacing;
     private final PathScheme scheme;
     private double alphaFactor = 0.5;
     private double betaFactor = 0.5;
+    private double fixedRunValue = -1.0;
     private final double pathDelta;
     private double pathParameter;
 
@@ -536,12 +624,14 @@ MCLogger logger) {
     public static final String MARGINAL_LIKELIHOOD_ESTIMATOR = "marginalLikelihoodEstimator";
     public static final String CHAIN_LENGTH = "chainLength";
     public static final String PATH_STEPS = "pathSteps";
+    public static final String FIXED = "fixed";
     public static final String LINEAR = "linear";
     public static final String LACING = "lacing";
     public static final String SPAWN = "spawn";
     public static final String BURNIN = "burnin";
     public static final String MCMC = "samplers";
     public static final String PATH_SCHEME = "pathScheme";
+    public static final String FIXED_VALUE = "fixedValue";
     public static final String ALPHA = "alpha";
     public static final String BETA = "beta";
     public static final String PRERUN = "prerun";
