@@ -29,7 +29,10 @@ import dr.inference.parallel.MPIServices;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.HashSet;
+import java.util.Set;
 
 /**
  * Represents a multi-dimensional continuous parameter.
@@ -238,6 +241,21 @@ public interface Parameter extends Statistic, Variable<Double> {
             return getParameterValue(dim);
         }
 
+        @Override
+        public String getDimensionName(int dim) {
+            if (dimensionNames == null) {
+                return super.getDimensionName(dim);
+            }
+            return dimensionNames[dim];
+        }
+
+        public final void setDimensionNames(String[] names) {
+            if (names != null && names.length != getDimension()) {
+                throw new IllegalArgumentException("Length of dimension name array doesn't match the number of dimensions");
+            }
+            dimensionNames = names;
+        }
+
         public void setDimension(int dim) {
             throw new UnsupportedOperationException();
         }
@@ -329,7 +347,7 @@ public interface Parameter extends Statistic, Variable<Double> {
         /**
          * @return the size of this variable - i.e. the length of the vector
          */
-        public final int getSize() {
+        public int getSize() {
             return getDimension();
         }
 
@@ -418,6 +436,8 @@ public interface Parameter extends Statistic, Variable<Double> {
         private boolean isValid = true;
 
         private ArrayList<VariableListener> listeners;
+
+        private String[] dimensionNames = null;
     }
 
 
@@ -483,9 +503,16 @@ public interface Parameter extends Statistic, Variable<Double> {
 
         public void addBounds(Bounds<Double> boundary) {
             if (bounds == null) {
-                bounds = new IntersectionBounds(getDimension());
+                bounds = boundary;
+            } else {
+                if (!(bounds instanceof IntersectionBounds)) {
+                    IntersectionBounds newBounds = new IntersectionBounds(getDimension());
+                    newBounds.addBounds(bounds);
+                    bounds = newBounds;
+                }
+
+                ((IntersectionBounds)bounds).addBounds(boundary);
             }
-            bounds.addBounds(boundary);
 
             // can't change dimension after bounds are added!
             //hasBeenStored = true;
@@ -499,6 +526,10 @@ public interface Parameter extends Statistic, Variable<Double> {
             return values.length;
         }
 
+        public final int getSize() {
+            return getDimension();
+        }
+
         public final double getParameterValue(int i) {
             return values[i];
         }
@@ -688,7 +719,7 @@ public interface Parameter extends Statistic, Variable<Double> {
 
         // same as !storedValues && !bounds
         //private boolean hasBeenStored = false;
-        private IntersectionBounds bounds = null;
+        private Bounds<Double> bounds = null;
 
         public void addBounds(double lower, double upper) {
             addBounds(new DefaultBounds(upper, lower, getDimension()));
@@ -742,6 +773,10 @@ public interface Parameter extends Statistic, Variable<Double> {
             return uppers.length;
         }
 
+        public boolean isConstant() {
+            return true;
+        }
+
         private final double[] uppers, lowers;
     }
 
