@@ -29,6 +29,7 @@ import dr.inference.distribution.DistributionLikelihood;
 import dr.inference.model.Likelihood;
 import dr.inference.model.Statistic;
 import dr.math.distributions.*;
+import dr.util.DataTable;
 import dr.xml.*;
 
 /**
@@ -43,12 +44,14 @@ public class PriorParsers {
     public static final String INVGAMMA_PRIOR = "invgammaPrior";
     public static final String INVGAMMA_PRIOR_CORRECT = "inverseGammaPrior";
     public static final String LAPLACE_PRIOR = "laplacePrior";
+    public static final String BETA_PRIOR = "betaPrior";
     public static final String UPPER = "upper";
     public static final String LOWER = "lower";
     public static final String MEAN = "mean";
     public static final String MEAN_IN_REAL_SPACE = "meanInRealSpace";
     public static final String STDEV = "stdev";
     public static final String SHAPE = "shape";
+    public static final String SHAPEB = "shapeB";
     public static final String SCALE = "scale";
     public static final String OFFSET = "offset";
     public static final String UNINFORMATIVE = "uninformative";
@@ -67,7 +70,11 @@ public class PriorParsers {
             double lower = xo.getDoubleAttribute(LOWER);
             double upper = xo.getDoubleAttribute(UPPER);
 
-            DistributionLikelihood likelihood = new DistributionLikelihood(new UniformDistribution(lower, upper));
+            if (lower == Double.NEGATIVE_INFINITY || upper == Double.POSITIVE_INFINITY)
+                throw new XMLParseException("Uniform prior " + xo.getName() + " cannot take a bound at infinity, " +
+                        "because it returns 1/(high-low) = 1/inf");
+
+            DistributionLikelihood likelihood = new DistributionLikelihood(new UniformDistribution(lower, upper), true);
             for (int j = 0; j < xo.getChildCount(); j++) {
                 if (xo.getChild(j) instanceof Statistic) {
                     likelihood.addData((Statistic) xo.getChild(j));
@@ -109,10 +116,16 @@ public class PriorParsers {
 
         public Object parseXMLObject(XMLObject xo) throws XMLParseException {
 
-            final double mean = xo.getDoubleAttribute(MEAN);
+            double scale;
+
+            if (xo.hasAttribute(SCALE)) {
+                scale = xo.getDoubleAttribute(SCALE);
+            } else {
+                scale = xo.getDoubleAttribute(MEAN);
+            }
             final double offset = xo.hasAttribute(OFFSET) ? xo.getDoubleAttribute(OFFSET) : 0.0;
 
-            DistributionLikelihood likelihood = new DistributionLikelihood(new ExponentialDistribution(1.0 / mean), offset);
+            DistributionLikelihood likelihood = new DistributionLikelihood(new ExponentialDistribution(1.0 / scale), offset);
             for (int j = 0; j < xo.getChildCount(); j++) {
                 if (xo.getChild(j) instanceof Statistic) {
                     likelihood.addData((Statistic) xo.getChild(j));
@@ -129,7 +142,10 @@ public class PriorParsers {
         }
 
         private final XMLSyntaxRule[] rules = {
-                AttributeRule.newDoubleRule(MEAN),
+                new XORRule(
+                        AttributeRule.newDoubleRule(SCALE),
+                        AttributeRule.newDoubleRule(MEAN)
+                ),
                 AttributeRule.newDoubleRule(OFFSET, true),
                 new ElementRule(Statistic.class, 1, Integer.MAX_VALUE)
         };
@@ -269,7 +285,7 @@ public class PriorParsers {
             }
 
             final DistributionLikelihood likelihood = new DistributionLikelihood(new LogNormalDistribution(mean, stdev), offset);
-            
+
             for (int j = 0; j < xo.getChildCount(); j++) {
                 if (xo.getChild(j) instanceof Statistic) {
                     likelihood.addData((Statistic) xo.getChild(j));
@@ -338,7 +354,7 @@ public class PriorParsers {
                 AttributeRule.newDoubleRule(SHAPE),
                 AttributeRule.newDoubleRule(SCALE),
                 AttributeRule.newDoubleRule(OFFSET, true),
-               // AttributeRule.newBooleanRule(UNINFORMATIVE, true),
+                // AttributeRule.newBooleanRule(UNINFORMATIVE, true),
                 new ElementRule(Statistic.class, 1, Integer.MAX_VALUE)
         };
 
@@ -446,4 +462,51 @@ public class PriorParsers {
         }
     };
 
+    /**
+     * A special parser that reads a convenient short form of priors on parameters.
+     */
+    public static XMLObjectParser BETA_PRIOR_PARSER = new AbstractXMLObjectParser() {
+
+        public String getParserName() {
+            return BETA_PRIOR;
+        }
+
+        public Object parseXMLObject(XMLObject xo) throws XMLParseException {
+
+            final double shape = xo.getDoubleAttribute(SHAPE);
+            final double shapeB = xo.getDoubleAttribute(SHAPEB);
+            final double offset = xo.getAttribute(OFFSET, 0.0);
+
+            DistributionLikelihood likelihood = new DistributionLikelihood(new BetaDistribution(shape, shapeB), offset);
+            for (int j = 0; j < xo.getChildCount(); j++) {
+                if (xo.getChild(j) instanceof Statistic) {
+                    likelihood.addData((Statistic) xo.getChild(j));
+                } else {
+                    throw new XMLParseException("illegal element in " + xo.getName() + " element");
+                }
+            }
+
+            return likelihood;
+        }
+
+        public XMLSyntaxRule[] getSyntaxRules() {
+            return rules;
+        }
+
+        private final XMLSyntaxRule[] rules = {
+                AttributeRule.newDoubleRule(SHAPE),
+                AttributeRule.newDoubleRule(SHAPEB),
+                AttributeRule.newDoubleRule(OFFSET, true),
+                new ElementRule(Statistic.class, 1, Integer.MAX_VALUE)
+        };
+
+        public String getParserDescription() {
+            return "Calculates the prior probability of some data under a given beta distribution.";
+        }
+
+        public Class getReturnType() {
+            return Likelihood.class;
+        }
+    };
+
 }
