@@ -1,11 +1,10 @@
 package dr.inferencexml.model;
 
+import dr.inference.model.DefaultModel;
 import dr.inference.model.DummyLikelihood;
 import dr.inference.model.Model;
-import dr.xml.AbstractXMLObjectParser;
-import dr.xml.ElementRule;
-import dr.xml.XMLObject;
-import dr.xml.XMLSyntaxRule;
+import dr.inference.model.Parameter;
+import dr.xml.*;
 
 /**
  * Reads a distribution likelihood from a DOM Document element.
@@ -19,8 +18,15 @@ public class DummyLikelihoodParser extends AbstractXMLObjectParser {
     public Object parseXMLObject(XMLObject xo) {
 
         Model model = (Model)xo.getChild(Model.class);
+        Parameter parameter = (Parameter)xo.getChild(Parameter.class);
+
+        if (model == null) {
+            model = new DefaultModel();
+        }
         final DummyLikelihood likelihood = new DummyLikelihood(model);
 
+        ((DefaultModel)model).addVariable(parameter);
+
         return likelihood;
     }
 
@@ -37,6 +43,9 @@ public class DummyLikelihoodParser extends AbstractXMLObjectParser {
     public XMLSyntaxRule[] getSyntaxRules() { return rules; }
 
     private final XMLSyntaxRule[] rules = {
-        new ElementRule(Model.class, "A model element")
+            new XORRule(
+                    new ElementRule(Model.class, "A model element"),
+                    new ElementRule(Parameter.class, "A parameter")
+            )
     };
 }
