@@ -55,7 +55,7 @@ public class XMLObject {
 
        this(obj.element);
        nativeObject = ((List)obj.getNativeObject()).get(index);
-   }  
+   }
 
     /**
      * @return the number of children this XMLObject has.
@@ -291,6 +291,13 @@ public class XMLObject {
         return getInteger(getAndTest(name));
     }
 
+    /**
+     * @return the named attribute as a long integer.
+     */
+    public long getLongIntegerAttribute(String name) throws XMLParseException {
+        return getLongInteger(getAndTest(name));
+    }
+
     /**
      * @return the named attribute as a string.
      */
@@ -346,7 +353,7 @@ public class XMLObject {
         }
     }
 
-    final static String ID = "id";
+    public final static String ID = "id";
 
     public boolean hasId() {
         return hasAttribute(ID);
@@ -530,6 +537,18 @@ public class XMLObject {
         }
     }
 
+    /**
+     * @return the object as an integer if possible
+     */
+    private long getLongInteger(Object obj) throws XMLParseException {
+        if (obj instanceof Number) return ((Number) obj).longValue();
+        try {
+            return Long.parseLong((String) obj);
+        } catch (NumberFormatException e) {
+            throw new XMLParseException("Expected long integer, got " + obj);
+        }
+    }
+
     /**
      * @return the object as a string if possible
      */
@@ -557,7 +576,7 @@ public class XMLObject {
             }
             return strings;
         }
-        throw new XMLParseException("Expected array of double precision numbers, but got " + obj);
+        throw new XMLParseException("Expected array of strings, but got " + obj);
     }
 
     /**
