@@ -1,7 +1,7 @@
 /*
  * XMLParser.java
  *
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -12,10 +12,10 @@
  * published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
- * BEAST is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with BEAST; if not, write to the
@@ -33,6 +33,9 @@ import dr.util.FileHelpers;
 import dr.util.Identifiable;
 import org.w3c.dom.*;
 import org.xml.sax.InputSource;
+import org.xml.sax.SAXException;
+import org.xml.sax.SAXParseException;
+import org.xml.sax.helpers.DefaultHandler;
 
 import java.io.*;
 import java.util.*;
@@ -123,9 +126,9 @@ public class XMLParser {
      */
     public Object parse(Reader reader, Class target)
             throws java.io.IOException,
-                   org.xml.sax.SAXException,
-                   dr.xml.XMLParseException,
-                   javax.xml.parsers.ParserConfigurationException {
+            org.xml.sax.SAXException,
+            dr.xml.XMLParseException,
+            javax.xml.parsers.ParserConfigurationException {
 
         InputSource in = new InputSource(reader);
         javax.xml.parsers.DocumentBuilderFactory documentBuilderFactory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
@@ -146,14 +149,15 @@ public class XMLParser {
 
     public ObjectStore parse(Reader reader, boolean run)
             throws java.io.IOException,
-                   org.xml.sax.SAXException,
-                   dr.xml.XMLParseException,
-                   javax.xml.parsers.ParserConfigurationException {
+            org.xml.sax.SAXException,
+            dr.xml.XMLParseException,
+            javax.xml.parsers.ParserConfigurationException {
 
         InputSource in = new InputSource(reader);
         javax.xml.parsers.DocumentBuilderFactory documentBuilderFactory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
 
         javax.xml.parsers.DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
+        documentBuilder.setErrorHandler(new MyErrorHandler());
         Document document = documentBuilder.parse(in);
 
         Element e = document.getDocumentElement();
@@ -169,6 +173,31 @@ public class XMLParser {
         return objectStore;
     }
 
+    private class MyErrorHandler extends DefaultHandler {
+        public void warning(SAXParseException e) throws SAXException {
+            System.out.println("Warning: ");
+            printInfo(e);
+        }
+
+        public void error(SAXParseException e) throws SAXException {
+            System.out.println("Error: ");
+            printInfo(e);
+        }
+
+        public void fatalError(SAXParseException e) throws SAXException {
+            System.out.println("Fatal error: ");
+            printInfo(e);
+        }
+
+        private void printInfo(SAXParseException e) {
+//            System.out.println("\tPublic ID: " + e.getPublicId());
+//            System.out.println("\tSystem ID: " + e.getSystemId());
+            System.out.println("\tLine number: " + e.getLineNumber());
+            System.out.println("\tColumn number: " + e.getColumnNumber());
+            System.out.println("\tError message: " + e.getMessage());
+        }
+    }
+
     public XMLObject getRoot() {
         return root;
     }
@@ -181,10 +210,10 @@ public class XMLParser {
 
             String idref = e.getAttribute(IDREF);
 
-            if(e.hasAttribute("index")){
+            if (e.hasAttribute("index")) {
                 index = Integer.parseInt(e.getAttribute("index"));
             }
-            if ((e.getAttributes().getLength() > 1 || e.getChildNodes().getLength() > 1)&& index == -1) {
+            if ((e.getAttributes().getLength() > 1 || e.getChildNodes().getLength() > 1) && index == -1) {
                 throw new XMLParseException("Object with idref=" + idref + " must not have other content or attributes (or perhaps it was not intended to be a reference?).");
             }
 
@@ -225,7 +254,6 @@ public class XMLParser {
             if (verbose) System.out.println("  Restoring idref=" + idref);
 
 
-
             return new Reference(restoredXMLObject);
 
         } else {
@@ -293,11 +321,11 @@ public class XMLParser {
                 }
 
                 if (obj instanceof Likelihood) {
-                    Likelihood.FULL_LIKELIHOOD_SET.add((Likelihood)obj);
+                    Likelihood.FULL_LIKELIHOOD_SET.add((Likelihood) obj);
                 } else if (obj instanceof Model) {
-                    Model.FULL_MODEL_SET.add((Model)obj);
+                    Model.FULL_MODEL_SET.add((Model) obj);
                 } else if (obj instanceof Parameter) {
-                    Parameter.FULL_PARAMETER_SET.add((Parameter)obj);
+                    Parameter.FULL_PARAMETER_SET.add((Parameter) obj);
                 }
 
                 xo.setNativeObject(obj);
@@ -341,6 +369,52 @@ public class XMLParser {
         }
     }
 
+    public static FileReader getFileReader(XMLObject xo, String attributeName) throws XMLParseException {
+        if (xo.hasAttribute(attributeName)) {
+            final File inFile = getFileHandle(xo, attributeName);
+            try {
+                return new FileReader(inFile);
+            } catch (FileNotFoundException e) {
+                throw new XMLParseException("Input file " + inFile.getName()
+                        + " was not found in the working directory");
+            }
+        }
+        throw new XMLParseException("Error reading input file in " + xo.getId());
+    }
+
+
+    /**
+     * Get filename and path from BEAST XML object
+     *
+     * @param xo
+     * @return
+     */
+    private static File getFileHandle(XMLObject xo, String attributeName) throws XMLParseException {
+        String fileName = xo.getStringAttribute(attributeName);
+
+        // Check to see if a filename prefix has been specified, check it doesn't contain directory
+        // separator characters and then prefix it.
+        final String fileNamePrefix = System.getProperty("file.name.prefix");
+        final String fileSeparator = System.getProperty("file.separator");
+        if (fileNamePrefix != null) {
+            if (fileNamePrefix.trim().length() == 0 || fileNamePrefix.contains(fileSeparator)) {
+                throw new XMLParseException("The specified file name prefix is illegal.");
+            }
+        }
+
+        final String fileRankPostfix = System.getProperty("mpi.rank.postfix");
+        if (fileRankPostfix != null) {
+            if (fileName.endsWith(".log")) {
+                fileName = fileName.substring(0, fileName.length() - 4) + fileRankPostfix + ".log";
+            }
+            if (fileName.endsWith(".trees")) {
+                fileName = fileName.substring(0, fileName.length() - 6) + fileRankPostfix + ".trees";
+            }
+        }
+
+        return FileHelpers.getFile(fileName, fileNamePrefix);
+    }
+
     /**
      * Allow a file relative to beast xml file with a prefix of ./
      *
@@ -349,41 +423,15 @@ public class XMLParser {
      * @return Print writer from fileName attribute in the given XMLObject
      * @throws XMLParseException if file can't be created for some reason
      */
+
     public static PrintWriter getFilePrintWriter(XMLObject xo, String parserName) throws XMLParseException {
+        return getFilePrintWriter(xo, parserName, FileHelpers.FILE_NAME);
+    }
 
-        if (xo.hasAttribute(FileHelpers.FILE_NAME)) {
-
-            final String fileName = xo.getStringAttribute(FileHelpers.FILE_NAME);
-
-            // Check to see if a filename prefix has been specified, check it doesn't contain directory
-            // separator characters and then prefix it.
-            final String fileNamePrefix = System.getProperty("file.name.prefix");
-            final String fileSeparator = System.getProperty("file.separator");
-            if (fileNamePrefix != null) {
-                if (fileNamePrefix.trim().length()==0 || fileNamePrefix.contains(fileSeparator)) {
-                    throw new XMLParseException("The specified file name prefix is illegal.");
-                }
-            }
-
-            final File logFile = FileHelpers.getFile(fileName, fileNamePrefix);
-
-            boolean allowOverwrite = false;
-
-            if (xo.hasAttribute(LoggerParser.ALLOW_OVERWRITE_LOG)) {
-                allowOverwrite = xo.getBooleanAttribute(LoggerParser.ALLOW_OVERWRITE_LOG);
-            }
-
-            // override with a runtime set System Property
-            if (System.getProperty("log.allow.overwrite") != null) {
-                allowOverwrite = Boolean.parseBoolean(System.getProperty("log.allow.overwrite", "false"));
-            }
-
-            if (logFile.exists() && !allowOverwrite) {
-                throw new XMLParseException("\nThe log file " + fileName + " already exists in the working directory." +
-                        "\nTo allow it to be overwritten, use the '-overwrite' command line option when running" +
-                        "\nBEAST or select the option in the Run Options dialog box as appropriate.");
-            }
+    public static PrintWriter getFilePrintWriter(XMLObject xo, String parserName, String attributeName) throws XMLParseException {
 
+        if (xo.hasAttribute(attributeName)) {
+            File logFile = getLogFile(xo, attributeName);
 
             try {
                 return new PrintWriter(new FileOutputStream(logFile));
@@ -396,6 +444,29 @@ public class XMLParser {
         return new PrintWriter(System.out);
     }
 
+    public static File getLogFile(XMLObject xo, String attributeName) throws XMLParseException {
+        final File logFile = getFileHandle(xo, attributeName);
+        boolean allowOverwrite = false;
+
+        if (xo.hasAttribute(LoggerParser.ALLOW_OVERWRITE_LOG)) {
+            allowOverwrite = xo.getBooleanAttribute(LoggerParser.ALLOW_OVERWRITE_LOG);
+        }
+
+        // override with a runtime set System Property
+        if (System.getProperty("log.allow.overwrite") != null) {
+            allowOverwrite = Boolean.parseBoolean(System.getProperty("log.allow.overwrite", "false"));
+        }
+
+        if (logFile.exists() && !allowOverwrite) {
+            throw new XMLParseException("\nThe log file " + logFile.getName() + " already exists in the working directory." +
+                    "\nTo allow it to be overwritten, use the '-overwrite' command line option when running" +
+                    "\nBEAST or select the option in the Run Options dialog box as appropriate.");
+        }
+
+
+        return logFile;
+    }
+
 
     public class ArrayParser extends AbstractXMLObjectParser {
 
